@@ -15,7 +15,6 @@
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
-#include "preferences.h"
 
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
@@ -25,6 +24,7 @@ extern int any_ison P_((void));
 extern int listing_ison P_((void));
 extern int plot_ison P_((void));
 extern int srch_ison P_((void));
+extern void av_cursor P_((Cursor c));
 extern void c_cursor P_((Cursor c));
 extern void c_update P_((Now *np, int force));
 extern void db_cursor P_((Cursor c));
@@ -41,6 +41,7 @@ extern void e_selection_mode P_((int whether));
 extern void e_update P_((Now *np, int force));
 extern void fs_cursor P_((Cursor c));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void hzn_cursor P_((Cursor c));
 extern void jm_cursor P_((Cursor c));
 extern void jm_newdb P_((int appended));
 extern void jm_selection_mode P_((int whether));
@@ -62,6 +63,9 @@ extern void marsm_newdb P_((int appended));
 extern void marsm_selection_mode P_((int whether));
 extern void marsm_update P_((Now *np, int how_much));
 extern void msg_cursor P_((Cursor c));
+extern void ng_cursor P_((Cursor c));
+extern void ng_update P_((Now *np, int force));
+extern void ng_newobj P_((int appended));
 extern void obj_cursor P_((Cursor c));
 extern void obj_newdb P_((int appended));
 extern void obj_update P_((Now *np, int howmuch));
@@ -70,11 +74,14 @@ extern void plt_log P_((char *name, double value));
 extern void plt_selection P_((char *name));
 extern void pm_cursor P_((Cursor c));
 extern void XPS_cursor P_((Cursor c));
+extern void sah_cursor P_((Cursor c));
 extern void sf_cursor P_((Cursor c));
+extern void sl_cursor P_((Cursor c));
 extern void sm_cursor P_((Cursor c));
 extern void sm_newdb P_((int appended));
 extern void sm_selection_mode P_((int whether));
 extern void sm_update P_((Now *np, int how_much));
+extern void sr_cursor P_((Cursor c));
 extern void srch_cursor P_((Cursor c));
 extern void srch_log P_((char *name, double value));
 extern void srch_selection P_((char *name));
@@ -87,12 +94,14 @@ extern void sv_cursor P_((Cursor c));
 extern void sv_newdb P_((int appended));
 extern void sv_newobj P_((int dbidx));
 extern void sv_update P_((Now *np, int how_much));
+extern void svh_cursor P_((Cursor c));
 extern void svf_cursor P_((Cursor c));
 extern void um_cursor P_((Cursor c));
 extern void um_newdb P_((int appended));
 extern void um_selection_mode P_((int whether));
 extern void um_update P_((Now *np, int how_much));
 extern void v_cursor P_((Cursor c));
+extern void wt_cursor P_((Cursor c));
 
 /* called to set or unset the watch cursor on all menus.
  * allow for nested requests.
@@ -118,11 +127,14 @@ int want;
 	    c = (Cursor)0;
 	}
 
+	XPS_cursor(c);
+	av_cursor(c);
 	c_cursor(c);
 	db_cursor(c);
 	dm_cursor(c);
 	e_cursor(c);
 	fs_cursor(c);
+	hzn_cursor(c);
 	jm_cursor(c);
 	lst_cursor(c);
 	m_cursor(c);
@@ -130,18 +142,23 @@ int want;
 	mars_cursor(c);
 	marsm_cursor(c);
 	msg_cursor(c);
+	ng_cursor(c);
 	obj_cursor(c);
 	plt_cursor(c);
 	pm_cursor(c);
-	XPS_cursor(c);
+	sah_cursor(c);
 	sf_cursor(c);
+	sl_cursor(c);
 	sm_cursor(c);
+	sr_cursor(c);
 	srch_cursor(c);
 	ss_cursor(c);
 	sv_cursor(c);
 	svf_cursor(c);
+	svh_cursor(c);
 	um_cursor(c);
 	v_cursor(c);
+	wt_cursor(c);
  
 	XFlush (XtD);
 	XmUpdateDisplay (toplevel_w);
@@ -156,6 +173,7 @@ int how_much;
 	dm_update (np, how_much);
 	mars_update (np, how_much);
 	marsm_update (np, how_much);
+	ng_update (np, how_much);
 	e_update (np, how_much);
 	jm_update (np, how_much);
 	sm_update (np, how_much);
@@ -179,6 +197,7 @@ int dbidx;
 	db_newobj(dbidx);
 	dm_newobj(dbidx);
 	e_newobj (dbidx);
+	ng_newobj (dbidx);
 	ss_newobj(dbidx);
 	sv_newobj(dbidx);
 	all_update (mm_get_now(), 1);
@@ -275,4 +294,4 @@ any_ison()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: broadcast.c,v $ $Date: 1999/02/18 22:58:11 $ $Revision: 1.4 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: broadcast.c,v $ $Date: 2000/10/08 08:10:08 $ $Revision: 1.12 $ $Name:  $"};
