@@ -40,14 +40,16 @@ extern Colormap xe_cm;
 extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
 extern Obj *db_scan P_((DBScan *sp));
-extern char *getXRes P_((char *name, char *def));
+extern char *getPrivateDir P_((void));
 extern char *syserrstr P_((void));
 extern double atod P_((char *buf));
 extern int confirm P_((void));
 extern int db_n P_((void));
 extern int existsh P_((char *filename));
+extern int isUp P_((Widget w));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
+extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
 extern void fs_pangle P_((char out[], double a));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void get_xmstring P_((Widget w, char *resource, char **txtp));
@@ -58,6 +60,7 @@ extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_point P_((Obj *op));
 extern void timestamp P_((Now *np, Widget w));
 extern void watch_cursor P_((int want));
@@ -88,10 +91,10 @@ typedef struct {
     float sep;		/* separation, rads (float to save memory) */
 } Pair;
 
-static void c_create_form P_((void));
+static void c_create_shell P_((void));
 static void c_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_helpon_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void c_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_flist_cb P_((Widget w, XtPointer client, XtPointer call));
 static void c_actlist_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -119,7 +122,7 @@ static void make_flist P_((char *name, char *how));
 static void write_flist P_((FILE *fp));
 
 
-static Widget cform_w;		/* main Form dialog */
+static Widget cshell_w;		/* main shell */
 static Widget sep_w;		/* text widget to hold max sep */
 static Widget mag_w;		/* text widget to hold mag limit */
 static Widget timestmp_w;	/* label for time stamp */
@@ -129,17 +132,18 @@ static Widget flist_w;		/* the file list dialog */
 static Widget wander_w;		/* toggle to omit pairs of fixed objs*/
 static Widget autoup_w;		/* toggle to automatically redo on updates */
 
+static char closecategory[] = "Close pairs"; 	/* Save category */
 
+/* bring up Close pairs shell, creating if first time */
 void
 c_manage()
 {
-	if (!cform_w)
-	    c_create_form();
-
-	if (XtIsManaged(cform_w))
-	    XtUnmanageChild (cform_w);
-	else
-	    XtManageChild (cform_w);
+	if (!cshell_w) {
+	    c_create_shell();
+	    c_create_flist_w ();
+	}
+	XtPopup (cshell_w, XtGrabNone);
+	set_something (cshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* called when time has advanced.
@@ -151,8 +155,7 @@ c_update (np, force)
 Now *np;
 int force;
 {
-	if (!cform_w || !XtIsManaged(cform_w)
-					|| !XmToggleButtonGetState (autoup_w))
+	if (!isUp(cshell_w) || !XmToggleButtonGetState (autoup_w))
 	    return;
 
 	do_search();
@@ -166,8 +169,8 @@ Cursor c;
 {
 	Window win;
 
-	if (cform_w && (win = XtWindow(cform_w)) != 0) {
-	    Display *dsp = XtDisplay(cform_w);
+	if (cshell_w && (win = XtWindow(cshell_w)) != 0) {
+	    Display *dsp = XtDisplay(cshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -175,9 +178,9 @@ Cursor c;
 	}
 }
 
-/* create a form dialog to allow user to work with the list. */
+/* create a form in a shell to allow user to work with the list. */
 static void
-c_create_form ()
+c_create_shell ()
 {
 	typedef struct {
 	    char *label;	/* what goes on the help label */
@@ -192,31 +195,34 @@ c_create_form ()
 	Widget mb_w, pd_w, cb_w;
 	Widget s_w;
 	XmString str;
+	Widget cform_w;
 	Widget w;
 	Arg args[20];
 	int n;
 	int i;
 	
-	/* create outter form dialog */
+	/* create outter shell and form */
 
 	n = 0;
-	XtSetArg(args[n], XmNautoUnmanage, False);  n++;
-	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
-	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	cform_w = XmCreateFormDialog(toplevel_w, "CloseObjs", args, n);
-	set_something (cform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (cform_w, XmNmapCallback, prompt_map_cb, NULL);
-	XtAddCallback (cform_w, XmNunmapCallback, c_unmap_cb, NULL);
-	XtAddCallback (cform_w, XmNhelpCallback, c_help_cb, NULL);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
-	n = 0;
 	XtSetArg (args[n], XmNtitle, "xephem Close Pairs"); n++;
-	XtSetValues (XtParent(cform_w), args, n);
+	XtSetArg (args[n], XmNiconName, "Close"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	cshell_w = XtCreatePopupShell ("CloseObjs", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (cshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (cshell_w, XmNpopdownCallback, c_popdown_cb, NULL);
+	sr_reg (cshell_w, "XEphem*CloseObjs.width", closecategory, 0);
+	sr_reg (cshell_w, "XEphem*CloseObjs.height", closecategory, 0);
+	sr_reg (cshell_w, "XEphem*CloseObjs.x", closecategory, 0);
+	sr_reg (cshell_w, "XEphem*CloseObjs.y", closecategory, 0);
+
+	n = 0;
+	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
+	cform_w = XmCreateForm(cshell_w, "CloseSh", args, n);
+	XtAddCallback (cform_w, XmNhelpCallback, c_help_cb, NULL);
+	XtManageChild (cform_w);
 
 	/* create the menu bar across the top */
 
@@ -262,6 +268,7 @@ c_create_form ()
 	    set_xmstring(autoup_w, XmNlabelString, "Auto update");
 	    wtip (autoup_w, "When on, perform a new scan after each Main menu Update");
 	    XtManageChild (autoup_w);
+	    sr_reg (autoup_w, NULL, closecategory, 1);
 
 	    /* make the sky point button */
 
@@ -292,7 +299,7 @@ c_create_form ()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Close", args, n);
 	    XtAddCallback (w, XmNactivateCallback, c_close_cb, 0);
-	    wtip (w, "Close this dialog");
+	    wtip (w, "Close this window");
 	    XtManageChild (w);
 
 	/* make the options pulldown */
@@ -315,6 +322,7 @@ c_create_form ()
 	    set_xmstring(wander_w, XmNlabelString, "Omit fixed pairs");
 	    wtip (wander_w, "When on, both objects in a pair can not be of type FIXED");
 	    XtManageChild (wander_w);
+	    sr_reg (wander_w, NULL, closecategory, 1);
 
 	/* make the help pulldown */
 
@@ -366,7 +374,7 @@ c_create_form ()
 	XtSetArg (args[n], XmNleftPosition, 53); n++;
 	XtSetArg (args[n], XmNcolumns, 10); n++;
 	sep_w = XmCreateTextField (cform_w, "Sep", args, n);
-	wtip (sep_w, "Enter desired minimum separation");
+	wtip (sep_w, "Enter desired max separation to list");
 	XtManageChild (sep_w);
 
 	/* make the limiting mag label and entry field */
@@ -437,7 +445,9 @@ c_create_form ()
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNbottomWidget, timestmp_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftOffset, 10); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightOffset, 10); n++;
 	XtSetArg (args[n], XmNselectionPolicy, XmBROWSE_SELECT); n++;
 	XtSetArg (args[n], XmNlistSizePolicy, XmCONSTANT); n++;
 	list_w = XmCreateScrolledList (cform_w, "List", args, n);
@@ -483,24 +493,18 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if (!flist_w)
-	    c_create_flist_w();
-
-	if (XtIsManaged(flist_w))
-	    XtUnmanageChild (flist_w);
-	else
-	    XtManageChild (flist_w);
+	XtManageChild (flist_w);
 }
 
-/* called when the main dialog is unmapped */
+/* called when our toplevel shell is popped down */
 /* ARGSUSED */
 static void
-c_unmap_cb (w, client, call)
+c_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if (flist_w && XtIsManaged(flist_w))
+	if (XtIsManaged(flist_w))
 	    XtUnmanageChild (flist_w);
 }
 
@@ -512,8 +516,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let the unmap do the rest of the work */
-	XtUnmanageChild (cform_w);
+	/* let the popdown do the rest of the work */
+	XtPopdown (cshell_w);
 }
 
 /* called when a list item is double-clicked */
@@ -1115,22 +1119,21 @@ double sep;
 static void
 c_create_flist_w()
 {
-	char fname[1024];
 	Arg args[20];
+	Widget tw;
 	int n;
 
-	(void) sprintf (fname,"%s/closelist.txt", getXRes("PrivateDir","work"));
-
 	n = 0;
-	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	XtSetArg(args[n], XmNmarginWidth, 10);  n++;
 	XtSetArg(args[n], XmNmarginHeight, 10);  n++;
-	flist_w = XmCreatePromptDialog (toplevel_w, "CloseList", args,n);
+	flist_w = XmCreatePromptDialog (cshell_w, "CloseList", args,n);
 	set_something (flist_w, XmNcolormap, (XtArgVal)xe_cm);
 	set_xmstring (flist_w, XmNdialogTitle, "xephem Close list");
 	set_xmstring (flist_w, XmNselectionLabelString, "File name:");
-	set_xmstring (flist_w, XmNtextString, fname);
+	tw = XmSelectionBoxGetChild(flist_w, XmDIALOG_TEXT);
+	defaultTextFN (tw, 1, getPrivateDir(), "closelist.txt");
+	sr_reg (tw, NULL, closecategory, 1);
 	XtUnmanageChild (XmSelectionBoxGetChild(flist_w, XmDIALOG_HELP_BUTTON));
 	XtAddCallback (flist_w, XmNokCallback, c_flistok_cb, NULL);
 	XtAddCallback (flist_w, XmNmapCallback, prompt_map_cb, NULL);
@@ -1249,4 +1252,4 @@ FILE *fp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: closemenu.c,v $ $Date: 1998/04/27 21:51:49 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: closemenu.c,v $ $Date: 2000/07/20 04:16:29 $ $Revision: 1.8 $ $Name:  $"};
