@@ -27,6 +27,7 @@
 #include "astro.h"
 #include "circum.h"
 #include "preferences.h"
+#include "dm.h"
 
 extern Widget	toplevel_w;
 extern Colormap xe_cm;
@@ -35,14 +36,18 @@ extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern char *cns_name P_((int id));
-extern char *getXRes P_((char *name, char *def));
+extern char *getPrivateDir P_((void));
+extern char *mm_getsite P_((void));
 extern char *syserrstr P_((void));
 extern double atod P_((char *buf));
 extern int any_ison P_((void));
 extern int confirm P_((void));
 extern int cns_pick P_((double ra, double dec, double e));
 extern int existsh P_((char *filename));
+extern int isUp P_((Widget shell));
 extern void db_update P_((Obj *op));
+extern void buttonAsButton P_((Widget w, int whether));
+extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
 extern void dm_update P_((Now *np, int how_much));
 extern void f_prdec P_((Widget w, double a));
 extern void f_dm_angle P_((Widget w, double a));
@@ -55,6 +60,7 @@ extern void f_showit P_((Widget w, char *s));
 extern void f_string P_((Widget w, char *s));
 extern void f_time P_((Widget w, double t));
 extern void fs_mtime P_((char out[], double t));
+extern void fs_sexa P_((char buf[], double a, int w, int fracbase));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void get_xmstring P_((Widget w, char *resource, char **txtp));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
@@ -65,6 +71,7 @@ extern void register_selection P_((char *name));
 extern void riset_cir P_((Now *np, Obj *op, double dis, RiseSet *rp));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void timestamp P_((Now *np, Widget w));
 extern void watch_cursor P_((int want));
 extern void wtip P_((Widget w, char *tip));
@@ -75,14 +82,15 @@ typedef enum {
     MISC_COL, RISET_COL, SEP_COL
 } ColType;
 
-void dm_create_form P_((void));
+void dm_create_shell P_((void));
+void dm_setup_cb P_((Widget w, XtPointer client, XtPointer call));
+void dm_separation P_((Obj *p, Obj *q, double *sp));
 
 static void dm_set_buttons P_((int whether));
 static void dm_create_table P_((Widget parent));
 static void dm_activate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void dm_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void dm_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void dm_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void dm_setup_cb P_((Widget w, XtPointer client, XtPointer call));
 static void dm_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void dm_compute P_((int r, int force, Now *np));
 static void dm_format P_((Now *np, Obj *op, RiseSet *rp, int c, Widget w));
@@ -90,7 +98,6 @@ static void dm_settags P_((void));
 static void dm_showtim P_((Now *np, Widget w, double t));
 static void dm_rs_hrsup P_((Now *np, Obj *op, Widget w, RiseSet *rp));
 static void show_constellation P_((Now *np, Obj *op, Widget w));
-static void dm_separation P_((Obj *p, Obj *q, Widget w));
 static void ds_create_selection P_((Widget parent));
 static void ds_setup_row_selections P_((void));
 static void ds_setup_col_selections P_((ColType type));
@@ -153,15 +160,9 @@ typedef enum {
     OK, APPLY, CANCEL, HELP
 } DSCtrls;
 
-/* identifiers for each entry in col[]. these must match the order therein.
+/* column details.
+ * N.B. these must match the DMCol order
  */
-enum {
-    CONSTEL_ID, RA_ID, HA_ID, DEC_ID, AZ_ID, ALT_ID, ZEN_ID, Z_ID, HLONG_ID,
-    HLAT_ID, GLONG_ID, GLAT_ID, EDST_ID, ELGHT_ID, SDST_ID, SLGHT_ID,
-    ELONG_ID, SIZE_ID, VMAG_ID, PHS_ID, RSTIME_ID, RSAZ_ID, TRTIME_ID,
-    TRALT_ID, SETTIME_ID, SETAZ_ID, HRSUP_ID
-};
-
 static ColHdr col[] = {
     {MISC_COL,	"Cns",	  0, "Constellation"},
     {MISC_COL,	"RA",	  0, "Right Ascension (to Main's settings)"},
@@ -171,20 +172,22 @@ static ColHdr col[] = {
     {MISC_COL,	"Alt",	  0, "Angle above horizon"},
     {MISC_COL,	"Zenith", 0, "Angle down from zenith"},
     {MISC_COL,	"Air",	  0, "Air mass"},
-    {MISC_COL,	"HeLong", 0, "Heliocentric longitude"},
+    {MISC_COL,	"Size",	  0, "Angular diameter, arc seconds"},
+    {MISC_COL,	"VMag",	  0, "Apparent magnitude"},
+    {MISC_COL,	"Phase",  0, "Percent illumination seen from Earth"},
     {MISC_COL,	"HeLat",  0, "Heliocentric latitude"},
-    {MISC_COL,	"GLong",  0, "Galactic longitude"},
+    {MISC_COL,	"HeLong", 0, "Heliocentric longitude"},
     {MISC_COL,	"GLat",	  0, "Galactic latitude"},
+    {MISC_COL,	"GLong",  0, "Galactic longitude"},
+    {MISC_COL,	"EcLat",  0, "Geocentric Ecliptic latitude"},
+    {MISC_COL,	"EcLong", 0, "Geocentric Ecliptic longitude"},
     {MISC_COL,	"EaDst",  0, "Distance from Earth, AU (moon is km)"},
     {MISC_COL,	"EaLght", 0, "Light travel time from Earth"},
     {MISC_COL,	"SnDst",  0, "Distance from Sun, AU"},
     {MISC_COL,	"SnLght", 0, "Light travel time from Sun"},
     {MISC_COL,	"Elong",  0, "Elongation: angular degrees from Sun, +E"},
-    {MISC_COL,	"Size",	  0, "Angular diameter, arc seconds"},
-    {MISC_COL,	"VMag",	  0, "Apparent magnitude"},
-    {MISC_COL,	"Phase",  0, "Percent illumination seen from Earth"},
-    {RISET_COL,	"RiseTm", 0, "Rise time, today"},
-    {RISET_COL,	"RiseAz", 0, "Rise azimuth, today"},
+    {RISET_COL,	"RisTm",  0, "Rise time, today"},
+    {RISET_COL,	"RisAz",  0, "Rise azimuth, today"},
     {RISET_COL,	"TrnTm",  0, "Transit time, today"},
     {RISET_COL,	"TrnAlt", 0, "Transit altitude, today"},
     {RISET_COL,	"SetTm",  0, "Set time, today"},
@@ -209,7 +212,7 @@ static ColHdr col[] = {
 #define	NC	XtNumber(col)
 
 static Widget t_w[NR][NC];	/* pushbuttons within table */
-static Widget dataform_w;	/* the overall table form */
+static Widget datashell_w;	/* the overall table shell */
 static Widget table_w;		/* the overall RowColumn table */
 static Widget corner_w;		/* upper left corner of table */
 static Widget hdrcol_w;		/* RowColumn for first column */
@@ -227,23 +230,27 @@ static Widget flist_w;		/* file list dialog widget */
 
 static int dm_selecting;	/* set while our fields are being selected */
 
+/* Save categories */
+static char dtcategory[] = "Data Table";
+static char dtrowscategory[] = "Data Table -- Rows";
+static char dtrscategory[] = "Data Table -- Rise/Set";
+static char dtmisccategory[] = "Data Table -- Miscellaneous";
+static char dtsepcategory[] = "Data Table -- Separations";
+
 /* called when the data menu is activated via the main menu pulldown.
- * if never called before, create all the widgets form;
- * otherwise, just toggle whether the form is managed.
+ * if never called before, create all the widgets.
  */
 void
 dm_manage ()
 {
-	if (!dataform_w)
-	    dm_create_form();
+	if (!datashell_w)
+	    dm_create_shell();
 	
-	if (XtIsManaged(dataform_w))
-	    XtUnmanageChild (dataform_w);
-	else {
-	    dm_update (mm_get_now(), 1);
-	    dm_set_buttons (dm_selecting);
-	    XtManageChild (dataform_w);
-	}
+	dm_update (mm_get_now(), 1);
+	dm_set_buttons (dm_selecting);
+	XtPopup (datashell_w, XtGrabNone);
+	set_something (datashell_w, XmNiconic, (XtArgVal)False);
+
 }
 
 /* user-defined object dbidx has changed.
@@ -260,8 +267,8 @@ int dbidx;
 	int i, c;
 
 	/* might get called before we have been managed the first time */
-	if (!dataform_w)
-	    dm_create_form();
+	if (!datashell_w)
+	    dm_create_shell();
 
 	for (i = 0; i < NR; i++)
 	    if (row[i].dbidx == dbidx) {
@@ -315,10 +322,8 @@ int dbidx;
 
 #if 0
 	/* come on up and show the new user obj */
-	if (!XtIsManaged(dataform_w)) {
-	    dm_set_buttons (dm_selecting);
-	    XtManageChild (dataform_w);
-	}
+	if (!datashell_w)
+	    dm_manage ();
 #endif
 }
 
@@ -332,9 +337,9 @@ int how_much;
 {
 	int i;
 
-	if (!dataform_w)
+	if (!datashell_w)
 	    return;
-	if (!XtIsManaged(dataform_w) && !any_ison() && !how_much)
+	if (!isUp(datashell_w) && !any_ison() && !how_much)
 	    return;
 
 	watch_cursor (1);
@@ -361,9 +366,12 @@ void
 dm_selection_mode (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	dm_selecting += whether ? 1 : -1;
+	if (whether)
+	    dm_selecting++;
+	else if (dm_selecting > 0)
+	    --dm_selecting;
 
-	if (dataform_w && XtIsManaged(dataform_w))
+	if (datashell_w)
 	    if ((whether && dm_selecting == 1)     /* first one to want on */
 		|| (!whether && dm_selecting == 0) /* last one to want off */)
 		dm_set_buttons (whether);
@@ -372,7 +380,7 @@ int whether;	/* whether setting up for plotting or for not plotting */
 /* return the user's horizon displacement, in rads.
  * + means less sky.
  */
-double
+static double
 dm_hzndep()
 {
 	char *str = XmTextFieldGetString (adprefr_w);
@@ -394,8 +402,8 @@ RiseSet *rsp;
 	double dis;	/* rads apparent horizon is above true */
 
 	/* might get called before we have been managed the first time */
-	if (!dataform_w)
-	    dm_create_form();
+	if (!datashell_w)
+	    dm_create_shell();
 
 	/* user's displacement correction */
 	dis = -dm_hzndep();
@@ -414,8 +422,8 @@ Cursor c;
 {
 	Window win;
 
-	if (dataform_w && (win = XtWindow(dataform_w)) != 0) {
-	    Display *dsp = XtDisplay(dataform_w);
+	if (datashell_w && (win = XtWindow(datashell_w)) != 0) {
+	    Display *dsp = XtDisplay(datashell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -431,32 +439,86 @@ Cursor c;
 	}
 }
 
+/* given a DMCol, fill str[] with its column heading.
+ * return 0 if column is on, else -1
+ */
+int
+dm_colHeader (c, str)
+DMCol c;
+char str[];
+{
+	if (!col[c].on)
+	    return (-1);
+	if (c >= SEP_SUN_ID && c <= SEP_OBJZ_ID) {
+	    Obj *op = db_basic (col[c].dbidx);
+	    strcpy (str, op->o_type == UNDEFOBJ ? "??" : op->o_name);
+	} else {
+	    strcpy (str, col[c].name);
+	}
+	return (0);
+}
+
+/* given circumstances and DMCol, fill str[] with its string value.
+ * return 0 if column is on, else -1 (but always fill in str).
+ */
+int
+dm_colFormat (np, op, rp, c, str)
+Now *np;
+Obj *op;
+RiseSet *rp;
+DMCol c;
+char str[];
+{
+	static Widget w;
+	char *lstr;
+
+	/* we get the value round-about via a widget because dm_format() can
+	 * not be written to return a string (because it must use the funcs
+	 * that call field_log())
+	 */
+	if (!w)
+	    w = XmCreateLabel (toplevel_w, "DMFT", NULL, 0);
+	dm_format (np, op, rp, c, w);
+	get_xmstring (w, XmNlabelString, &lstr);
+	(void) strcpy (str, lstr);
+	XtFree (lstr);
+
+	return (col[c].on ? 0 : -1);
+}
+
+/* N.B. might be called early if preloading certain .edb files
+ */
 void
-dm_create_form()
+dm_create_shell()
 {
 	Widget ctlrc_w, w;
 	Widget mb_w, cb_w, pd_w;
+	Widget dform_w;
 	Arg args[20];
 	int n;
 
-	/* create the form */
+	/* benign if called > 1 */
+	if (datashell_w)
+	    return;
+
+	/* create the shell and main form */
 	n = 0;
 	XtSetArg (args[n], XmNallowShellResize, True); n++;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNallowOverlap, False); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	dataform_w = XmCreateFormDialog (toplevel_w, "Data", args, n);
-	set_something (dataform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (dataform_w, XmNhelpCallback, dm_help_cb, 0);
-	XtAddCallback (dataform_w, XmNunmapCallback, dm_unmap_cb, 0);
+	XtSetArg (args[n], XmNtitle, "xephem Data Table"); n++;
+	XtSetArg (args[n], XmNiconName, "Data"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	datashell_w = XtCreatePopupShell ("Data", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (datashell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (datashell_w, XmNpopdownCallback, dm_popdown_cb, 0);
+	sr_reg (datashell_w, "XEphem*Data.x", dtcategory, 0);
+	sr_reg (datashell_w, "XEphem*Data.y", dtcategory, 0);
 
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem General Data Table"); n++;
-	XtSetValues (XtParent(dataform_w), args, n);
+	dform_w = XmCreateForm (datashell_w, "DTForm", args, n);
+	XtAddCallback (dform_w, XmNhelpCallback, dm_help_cb, 0);
+	XtManageChild (dform_w);
 
 	/* make the menubar */
 
@@ -464,7 +526,7 @@ dm_create_form()
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	mb_w = XmCreateMenuBar (dataform_w, "MB", args, n);
+	mb_w = XmCreateMenuBar (dform_w, "MB", args, n);
 	XtManageChild (mb_w);
 
 	    /* make the Control pulldown */
@@ -528,7 +590,7 @@ dm_create_form()
 	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
 	XtSetArg (args[n], XmNpacking, XmPACK_TIGHT); n++;
 	XtSetArg (args[n], XmNentryAlignment, XmALIGNMENT_CENTER); n++;
-	ctlrc_w = XmCreateRowColumn (dataform_w, "DataTblRC", args, n);
+	ctlrc_w = XmCreateRowColumn (dform_w, "DataTblRC", args, n);
 	XtManageChild (ctlrc_w);
 
 	    /* make the limb and centric indicators in frames.
@@ -568,7 +630,7 @@ dm_create_form()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
 	XtSetArg (args[n], XmNpacking, XmPACK_TIGHT); n++;
-	table_w = XmCreateRowColumn (dataform_w, "DataTable", args, n);
+	table_w = XmCreateRowColumn (dform_w, "DataTable", args, n);
 	XtManageChild (table_w);
 
 	dm_create_table (table_w);
@@ -576,10 +638,32 @@ dm_create_form()
 	/* create the selection dialog.
 	 * don't manage it yet but its state info is used right off.
 	 */
-	ds_create_selection(toplevel_w);
+	ds_create_selection(datashell_w);
 	ds_apply_selections();
 }
 
+/* callback to bring up the Data setup dialog */
+/* ARGSUSED */
+void
+dm_setup_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (!sel_w)
+	    dm_create_shell();
+
+	if (!XtIsManaged(sel_w)) {
+	    watch_cursor(1);
+	    ds_setup_row_selections();
+	    ds_setup_col_selections(MISC_COL);
+	    ds_setup_col_selections(RISET_COL);
+	    ds_setup_col_selections(SEP_COL);
+	    XtManageChild (sel_w);
+	    watch_cursor(0);
+	}
+}
+
 /* go through all the buttons and set whether they
  * should appear to look like buttons or just flat labels.
  */
@@ -587,76 +671,20 @@ static void
 dm_set_buttons (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
 	int r, c;
-	Arg *ap;
-	int na;
+	int i;
 
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     * also make the corner and headers look like labels forever.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    int n;
-	    int i;
+	/* do these always too in case user is trying a new color */
+	buttonAsButton (corner_w, 0);
+	for (i = 0; i < NR; i++)
+	    buttonAsButton (row[i].lw, 0);
+	for (i = 0; i < NC; i++)
+	    buttonAsButton (col[i].lw, 0);
 
-	    n = 0;
-            XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-            XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (corner_w, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	    XtSetValues (corner_w, ap, na);
-	    for (i = 0; i < NR; i++)
-		XtSetValues (row[i].lw, ap, na);
-	    for (i = 0; i < NC; i++)
-		XtSetValues (col[i].lw, ap, na);
-
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
 	for (r = 0; r < NR; r++)
 	    for (c = 0; c < NC; c++)
 		if (t_w[r][c])
-		    XtSetValues (t_w[r][c], ap, na);
+		    buttonAsButton (t_w[r][c], whether);
 }
 
 /* create the main data table - everything but the first column is unmanaged.
@@ -798,10 +826,10 @@ XtPointer call;
 	}
 }
 
-/* callback when main dialog is unmapped */
+/* callback when main dialog is popped down */
 static void
 /* ARGSUSED */
-dm_unmap_cb (w, client, call)
+dm_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
@@ -821,30 +849,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let unmap to the real work */
-	XtUnmanageChild (dataform_w);
-}
-
-/* callback from the Data table Setup button.
- */
-static void
-/* ARGSUSED */
-dm_setup_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	if (XtIsManaged(sel_w))
-	    XtUnmanageChild (sel_w);
-	else {
-	    watch_cursor(1);
-	    ds_setup_row_selections();
-	    ds_setup_col_selections(MISC_COL);
-	    ds_setup_col_selections(RISET_COL);
-	    ds_setup_col_selections(SEP_COL);
-	    XtManageChild (sel_w);
-	    watch_cursor(0);
-	}
+	/* let popdown do the real work */
+	XtPopdown (datashell_w);
 }
 
 /* callback from the Data table Help button
@@ -901,6 +907,7 @@ Widget w;
 {
 	static char me[] = "dm_format()";
 	double tmp;
+	Obj *bop;
 
 	switch (c) {
 	case CONSTEL_ID:
@@ -930,10 +937,8 @@ Widget w;
 	    f_pangle (w, PI/2 - op->s_alt);
 	    break;
 	case Z_ID: /* airmass */
-	    if (op->s_alt > 0.1)
-		f_double (w, "%4.2f", 1./sin(op->s_alt));
-	    else
-		f_double (w, "%4.0f", 99.0);
+	    airmass (op->s_alt, &tmp);
+	    f_double (w, "%5.2f", tmp);
 	    break;
 	case HLONG_ID:
 	    if (is_ssobj(op))
@@ -969,6 +974,18 @@ Widget w;
 	    f_pangle (w, glat);
 	    }
 	    break;
+	case ECLONG_ID: {
+	    double eclat, eclng;
+	    eq_ecl (mjd, op->s_gaera, op->s_gaedec, &eclat, &eclng);
+	    f_pangle (w, eclng);
+	    }
+	    break;
+	case ECLAT_ID: {
+	    double eclat, eclng;
+	    eq_ecl (mjd, op->s_gaera, op->s_gaedec, &eclat, &eclng);
+	    f_pangle (w, eclat);
+	    }
+	    break;
 	case EDST_ID:
 	    if (is_planet(op, MOON)) {
 		tmp = op->s_edist;
@@ -1029,7 +1046,7 @@ Widget w;
 	    }
 	    break;
 	case ELONG_ID:
-	    if (!is_planet(op, SUN))
+	    if (op->o_type != EARTHSAT)
 		f_double (w, "%6.1f", op->s_elong);
 	    else
 		f_string (w, "      ");
@@ -1146,11 +1163,17 @@ Widget w;
 		printf ("Bug: %s: col[%d].type = 0x%x\n", me, c, col[c].type);
 		exit (1);
 	    }
-	    if (op != db_basic(col[c].dbidx)) {
-		db_update(db_basic(col[c].dbidx));
-		dm_separation (op, db_basic(col[c].dbidx), w);
-	    } else
-		f_string (w, "     ");
+	    if (op != (bop= db_basic(col[c].dbidx)) && bop->o_type != UNDEFOBJ){
+		double sep;
+		db_update(bop);
+		dm_separation (op, bop, &sep);
+		f_pangle (w, sep);
+	    } else {
+		if (pref_get(PREF_DPYPREC) == PREF_LOPREC)
+		    f_string (w, "      ");
+		else
+		    f_string (w, "         ");
+	    }
 	    break;
 	}
 }
@@ -1260,14 +1283,13 @@ Widget w;
 	f_string(w, nm);
 }
 
-/* compute and display the separation between the two sky locations */
-static void
-dm_separation (p, q, w)
+/* compute the separation between the two sky locations */
+void
+dm_separation (p, q, sp)
 Obj *p, *q;
-Widget w;
+double *sp;
 {
 	double spy, cpy, px, qx, sqy, cqy;
-	double sep;
 
 	spy = sin (p->s_dec);
 	cpy = cos (p->s_dec);
@@ -1276,8 +1298,7 @@ Widget w;
 	sqy = sin (q->s_dec);
 	cqy = cos (q->s_dec);
 
-	sep = acos(spy*sqy + cpy*cqy*cos(px-qx));
-	f_pangle (w, sep);
+	*sp = acos(spy*sqy + cpy*cqy*cos(px-qx));
 }
 
 /* create the selections dialog */
@@ -1301,7 +1322,7 @@ Widget parent;
 	XmString str;
 	Widget rl_w, cl_w, arc_w;
 	Widget rf_w, mf_w, rsf_w, sf_w, ctlf_w, rowrc_w, mrc_w, rsrc_w, src_w;
-	Widget sep_w, w;
+	Widget w;
 	Widget rb_w;
 	int n;
 	int i;
@@ -1310,7 +1331,6 @@ Widget parent;
 	n = 0;
 	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNallowOverlap, False); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	sel_w = XmCreateFormDialog (parent, "DataSelection", args, n);
@@ -1353,21 +1373,13 @@ Widget parent;
 		XtManageChild (w);
 	    }
 
-	/* make a top separator */
-	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	sep_w = XmCreateSeparator (sel_w, "DSSep", args, n);
-	XtManageChild (sep_w);
-
 	/* make the Rows heading */
 
 	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, sep_w); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	XtSetArg (args[n], XmNmarginHeight, 4); n++;
 	rl_w = XmCreateLabel (sel_w, "Rows:", args, n);
 	XtManageChild (rl_w);
 
@@ -1436,17 +1448,19 @@ Widget parent;
 	    wtip (w, "Return each item in this column to its original state");
 	    XmStringFree (str);
 
+	    /* make 1 PB for each row */
 	    for (i = 0; i < NR; i++) {
 		Obj *op = db_basic(row[i].dbidx);
+		char *name = op->o_type != UNDEFOBJ ? op->o_name : "RowSelObj";
+
 		n = 0;
 		XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-		if (op->o_type != UNDEFOBJ) {
-		    w = row[i].sw = XmCreateToggleButton(rowrc_w, op->o_name,
-								    args, n);
+		w = row[i].sw = XmCreateToggleButton(rowrc_w, name, args, n);
+
+		if (op->o_type != UNDEFOBJ)
 		    XtManageChild (w);
-		} else
-		    w = row[i].sw = XmCreateToggleButton(rowrc_w, "RowSelObj",
-								    args, n);
+		if (row[i].dbidx <= MOON)
+		    sr_reg (w, NULL, dtrowscategory, 1);
 		if (row[i].tip)
 		    wtip (w, row[i].tip);
 	    }
@@ -1454,11 +1468,11 @@ Widget parent;
 	/* make the Columns heading */
 
 	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, sep_w); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNleftWidget, rf_w); n++;
 	XtSetArg (args[n], XmNleftOffset, 10); n++;
+	XtSetArg (args[n], XmNmarginHeight, 4); n++;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	cl_w = XmCreateLabel (sel_w, "Columns:", args, n);
 	XtManageChild (cl_w);
@@ -1613,6 +1627,7 @@ Widget parent;
 		n = 0;
 		limb_w = XmCreateToggleButton (rb_w, "Limb", args, n);
 		wtip (limb_w, "Compute Rise/Set events with respect to upper limb");
+		sr_reg (limb_w, NULL, dtrscategory, 1);
 		XtManageChild (limb_w);
 
 		n = 0;
@@ -1689,11 +1704,13 @@ Widget parent;
 	    case MISC_COL:
 		/* one of the misc columns */
 		w = col[i].sw = XmCreateToggleButton(mrc_w, col[i].name,args,n);
+		sr_reg (w, NULL, dtmisccategory, 1);
 		XtManageChild (w);
 		break;
 	    case RISET_COL:
 		/* one of the rise/set columns */
 		w = col[i].sw = XmCreateToggleButton(rsrc_w,col[i].name,args,n);
+		sr_reg (w, NULL, dtrscategory, 1);
 		XtManageChild (w);
 		break;
 	    case SEP_COL: {
@@ -1705,8 +1722,10 @@ Widget parent;
 		    XtManageChild (w);
 		} else
 		    w=col[i].sw= XmCreateToggleButton(src_w,"ColSelObj",args,n);
-		break;
+		if (col[i].dbidx <= MOON)
+		    sr_reg (w, NULL, dtsepcategory, 1);	/*not ObjXYZ*/
 		}
+		break;
 	    default:
 		printf ("Bug: %s: col[%d].type = 0x%x\n", me, i, col[i].type);
 		exit (1);
@@ -1978,22 +1997,19 @@ XtPointer call;
 static void
 dm_create_flist_w()
 {
-	char fname[1024];
 	Arg args[20];
 	int n;
 
-	(void) sprintf (fname, "%s/datatbl.txt", getXRes("PrivateDir","work"));
-
 	n = 0;
-	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
         XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	XtSetArg(args[n], XmNmarginWidth, 10);  n++;
 	XtSetArg(args[n], XmNmarginHeight, 10);  n++;
-	flist_w = XmCreatePromptDialog (toplevel_w, "DataTbl", args,n);
+	flist_w = XmCreatePromptDialog (datashell_w, "DataSel", args,n);
 	set_something (flist_w, XmNcolormap, (XtArgVal)xe_cm);
 	set_xmstring (flist_w, XmNdialogTitle, "xephem Data list");
 	set_xmstring (flist_w, XmNselectionLabelString, "File name:");
-	set_xmstring (flist_w, XmNtextString, fname);
+	defaultTextFN (XmSelectionBoxGetChild(flist_w, XmDIALOG_TEXT), 1,
+						getPrivateDir(), "datatbl.txt");
 	XtUnmanageChild (XmSelectionBoxGetChild(flist_w, XmDIALOG_HELP_BUTTON));
 	XtAddCallback (flist_w, XmNokCallback, dm_flistok_cb, NULL);
 	XtAddCallback (flist_w, XmNmapCallback, prompt_map_cb, NULL);
@@ -2080,9 +2096,7 @@ XtPointer call;
 	if (!flist_w)
 	    dm_create_flist_w();
 
-	if (XtIsManaged(flist_w))
-	    XtUnmanageChild (flist_w);
-	else
+	if (!XtIsManaged(flist_w))
 	    XtManageChild (flist_w);
 }
 
@@ -2104,6 +2118,7 @@ static void
 dm_list_get (buf)
 char buf[];
 {
+	Now *np = mm_get_now();
 	char *onerow[NC];
 	char *colhdrs[NC];
 	char *rowhdrs[NR];
@@ -2116,22 +2131,7 @@ char buf[];
 	/* init as legal string */
 	buf[0] = '\0';
 
-	/* print the banner */
-	if (XtIsManaged(XtParent(limbl_w))) {
-	    get_xmstring (limbl_w, XmNlabelString, &str);
-	    sprintf (buf+strlen(buf), "%s ", str);
-	    XtFree (str);
-	}
-	if (XtIsManaged(XtParent(centric_w))) {
-	    get_xmstring (centric_w, XmNlabelString, &str);
-	    sprintf (buf+strlen(buf), "%s ", str);
-	    XtFree (str);
-	}
-	get_xmstring (dt_w, XmNlabelString, &str);
-	sprintf (buf+strlen(buf), "%s:\n", str);
-	XtFree (str);
-
-	/* gather all the row headers to find longest */
+	/* malloc all the row headers to find longest */
 	maxrh = 0;
 	for (r = 0; r < NR; r++) {
 	    if (!row[r].on)
@@ -2142,7 +2142,7 @@ char buf[];
 		maxrh = l;
 	}
 
-	/* gather all col headers and entries in first row to get max column
+	/* malloc all col headers and entries in first row to get max column
 	 * widths.
 	 * N.B. this assumes all data rows in a given column are the same
 	 * length.
@@ -2158,7 +2158,7 @@ char buf[];
 		maxch[c] = l;
 	}
 
-	/* print the column headers, freeing as we go */
+	/* print the column headers, freeing strings as we go */
 	sprintf (buf+strlen(buf), "%*s", maxrh, "");
 	for (c = 0; c < NC; c++) {
 	    if (!col[c].on)
@@ -2169,7 +2169,7 @@ char buf[];
 	}
 	sprintf (buf+strlen(buf), "\n");
 
-	/* now print the table, freeing strings too as we go */
+	/* now print the table, freeing strings as we go */
 	for (r = 0; r < NR; r++) {
 	    if (!row[r].on)
 		continue;
@@ -2188,7 +2188,36 @@ char buf[];
 	    sprintf (buf+strlen(buf), "\n");
 	}
 	sprintf (buf+strlen(buf), "\n");
+
+	/* print the footer */
+	if (XtIsManaged(XtParent(limbl_w))) {
+	    get_xmstring (limbl_w, XmNlabelString, &str);
+	    sprintf (buf+strlen(buf), "%s ", str);
+	    XtFree (str);
+	}
+	if (XtIsManaged(XtParent(centric_w))) {
+	    get_xmstring (centric_w, XmNlabelString, &str);
+	    sprintf (buf+strlen(buf), "%s ", str);
+	    XtFree (str);
+	}
+	get_xmstring (dt_w, XmNlabelString, &str);
+	sprintf (buf+strlen(buf), "%s:\n", str);
+	XtFree (str);
+
+	/* add more details */
+	strcat (buf, "\n");
+	sprintf (buf+strlen(buf), " Lat: ");
+	fs_sexa (buf+strlen(buf), raddeg(lat), 4, 3600);
+	strcat (buf, "\n");
+	sprintf (buf+strlen(buf), "Long: ");
+	fs_sexa (buf+strlen(buf), -raddeg(lng), 4, 3600);
+	strcat (buf, "\n");
+	sprintf (buf+strlen(buf), "Elev: %gm\n", elev*ERAD);
+	sprintf (buf+strlen(buf), "Temp: %gC\n", temp);
+	sprintf (buf+strlen(buf), "Pres: %gmB\n", pressure);
+	if ((str = mm_getsite()) != NULL)
+	    sprintf (buf+strlen(buf), "Site: %s\n", str);
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: datamenu.c,v $ $Date: 1999/02/15 23:21:12 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: datamenu.c,v $ $Date: 2000/11/26 08:46:09 $ $Revision: 1.23 $ $Name:  $"};
