@@ -33,34 +33,42 @@ extern int close();
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
+#include "db.h"
 
-extern Widget toplevel_w;
 extern XtAppContext xe_app;
-#define XtD XtDisplay(toplevel_w)
-
 
 extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
+extern char *getPrivateDir P_((void));
 extern char *getShareDir P_((void));
 extern char *getXRes P_((char *name, char *def));
 extern char *syserrstr P_((void));
-extern int db_read P_((FILE *fp, int nodups));
 extern double delra P_((double dra));
 extern int openh P_((char *name, int flags, ...));
+extern int db_load1 P_((void));
 extern void all_newdb P_((int appended));
+extern void db_read P_((char *fn, int nodups));
 extern void db_update P_((Obj *op));
+extern void obj_set P_((Obj *op, int dbidx));
 extern void pm_set P_((int p));
+extern void setXRes P_((char *name, char *val));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
+
 extern void watch_cursor P_((int want));
 extern void xe_msg P_((char *msg, int app_modal));
 
-static int db_add P_((Obj *newop));
+static int db_objadd P_((Obj *newop));
 static void db_init P_((void));
 static void dbfifo_cb P_((XtPointer client, int *fdp, XtInputId *idp));
+static DBCat *db_catadd P_((char *filename));
 
 #define	MAXDBLINE	256	/* longest allowed db line */
 
 #define	DBFIFO_MSG	'!'	/* introduces a message line from the DBFIFO */
 
+/* Category for ours and dbmenu's resources in the Save system */
+char dbcategory[] = "Data Base";
+
 /* This counter is incremented when we want to mark all the Obj derived entries
  * as being out-of-date. This works because then each of the age's will be !=
  * db_age. This is to eliminate ever calling obj_cir() under the same
@@ -72,8 +80,11 @@ static ObjAge_t db_age = 1;	/* insure all objects are initially o-o-d */
 /* the "database".
  * one such struct per object type. the space for objects is malloced in
  *   seperate chunks of DBCHUNK to avoid needing big contiguous memory blocks.
+ *   the space is never freed, the total is just reduced and can be reused.
+ * N.B. deleting by catalog depends on all objects from a given catalog being
+ *   appended contiguously within their respective ObjType list.
  * N.B. because the number is fixed and known, we use static storage for the
- *   NOBJ Objects for the PLANET type; see db_init().
+ *   NOBJ Objects for the PLANET type, not malloced storage; see db_init().
  */
 #define	DBCHUNK	256	/* number we malloc more of at once; a power of two
 			 * might help the compiler optimize the divides and
@@ -83,16 +94,24 @@ typedef struct {
     char **dblist;	/* malloced list of malloced DBCHUNKS arrays */
     int nobj;		/* number of objects actually in use */
     int nmem;		/* total number of objects for which we have room */
-    int ncp;		/* nobj at time of last checkpoint */
     int size;		/* bytes per object */
 } DBMem;
 static DBMem db[NOBJTYPES];	/* this is the head for each object */
+
+/* return true if the database has been initialized */
 #define	DBINITED	(db[PLANET].dblist)	/* anything setup by db_init()*/
 
 /* macro that returns the address of an object given its type and index */
 #define	OBJP(t,n)	\
 		((Obj *)(db[t].dblist[(n)/DBCHUNK] + ((n)%DBCHUNK)*db[t].size))
 
+/* keep track of the catalogs.
+ * the array is kept sorted by name.
+ */
+static DBCat *dbcat;		/* malloced array, one for each loaded catalog*/
+static int ndbcat;		/* number of entries in dbcat[] */
+static char dbifres[] = "DBinitialFiles";	/* init files resource name */
+
 /* Size quantizer to keep objects allocated in the heap worst-case-aligned.
  * This assumes that double will be the largest primitive data type
  * on all systems.  Not elegant but gets it done for the moment.
@@ -128,23 +147,6 @@ db_n()
 	return (n);
 }
 
-/* return the total current number of objects as of the last checkpoint.
- * this includes the NOBJ basic objects.
- */
-int
-db_n_cp()
-{
-	DBMem *dmp;
-	int n;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (n = 0, dmp = db; dmp < &db[NOBJTYPES]; dmp++)
-	    n += dmp->ncp;
-	return (n);
-}
-
 /* given one of the basic ids in astro.h return pointer to its updated Obj in
  * the database.
  */
@@ -170,62 +172,58 @@ int id;
 
 /* load each file of objects listed in the DBinitialFiles resource
  * and inform all modules of the update.
+ * support leading ~ and / else assume in ShareDir.
  */
 void
 db_loadinitial()
 {
-	static char dbi[] = "DBinitialFiles";	/* init files resource name */
 	char *fns;		/* value of DBinitialFiles */
-	char *dir;		/* value of ShareDir */
 	char dbicpy[2048];	/* local copy of dir */
 	char *fnf[128];		/* ptrs into dbicpy[] at each ' ' */
-	char fn[1024];		/* each dir/filename */
-	char msg[256];
-	FILE *fp;
 	int nfn;
 	int i;
 
-	/* get the base directory name */
-	dir = getShareDir();
-
-	/* get the initial list of files */
-	fns = getXRes (dbi, "YBS.edb Messier.edb");
+	/* get the initial list of files, if any */
+	fns = getXRes (dbifres, NULL);
+	if (!fns)
+	    return;
 
 	/* work on a copy since we are about to break into fields */
-	dbicpy[sizeof(dbicpy)-1] = '\0';
-	(void) strncpy (dbicpy, fns, sizeof(dbicpy)-1);
+	(void) sprintf (dbicpy, "%.*s", (int)sizeof(dbicpy)-1, fns);
 	nfn = get_fields (dbicpy, ' ', fnf);
 	if (nfn > XtNumber(fnf)) {
 	    /* we exit because we've clobbered our stack by now!
 	     * TODO: pass the size of fnf to get_fields().
 	     */
-	    printf ("Too many entries in %s. Max is %d\n", dbi, XtNumber(fnf));
+	    printf ("Too many entries in %s. Max is %d\n", dbifres,
+							    XtNumber(fnf));
 	    exit (1);
 	}
 
-	/* open and read each file.
+	/* read each catalog.
 	 * N.B. get_fields() will return 1 even if there are no fields.
 	 */
-	for (i = 0; i < nfn && fnf[i][0] != '\0'; i++) {
-	    (void) sprintf (fn, "%.500s/catalogs/%.500s", dir, fnf[i]);
-	    fp = fopenh (fn, "r");
-	    if (!fp) {
-		(void) sprintf (msg, "Can not open %.150s: %.50s\n", fn,
-								syserrstr());
-		xe_msg (msg, 1);
-		continue;
-	    }
-	    /*
-	    printf ("Reading %s\n", fnf[i]);
-	    */
-	    if (db_read (fp, 1) < 0) {
-		(void) sprintf (msg, "Error reading `%.200s'", fn);
-		xe_msg (msg, 0);
-	    }
-	    (void) fclose (fp);
-	}
+	for (i = 0; i < nfn && fnf[i][0] != '\0'; i++)
+	    db_read (fnf[i], 1);
 
-	all_newdb(1);
+	/* all new */
+	all_newdb(0);
+}
+
+/* make the current set of databases the new default */
+static void
+db_setinitial()
+{
+	DBCat *dbcp;
+	char buf[2048];
+	int l;
+
+	l = 0;
+	buf[0] = '\0';
+	for (dbcp = dbcat; dbcp < &dbcat[ndbcat]; dbcp++)
+	    l += sprintf (buf+l, " %s", dbcp->name);
+
+	setXRes (dbifres, buf);
 }
 
 /* set our official copy of the given user-defined object to *op */
@@ -243,7 +241,78 @@ Obj *op;
 	}
 }
 
-/* mark all db objects as old
+/* search for a loaded catalog with the given name.
+ * if find it return poiter to DBCat, else return NULL.
+ */
+DBCat *
+db_catfind (name)
+char *name;
+{
+	DBCat *dbcp;
+	char *base;
+
+	/* find just the basename */
+	while (*name == ' ')
+	    name++;
+	for (base = name+strlen(name); base > name && base[-1] != '/'; --base)
+	    continue;
+
+	for (dbcp = dbcat; dbcp < &dbcat[ndbcat]; dbcp++)
+	    if (!strcmp (dbcp->name, base))
+		return (dbcp);
+	return (NULL);
+}
+
+/* delete the entries associated with deldbcp then remove from list.
+ */
+void
+db_catdel (deldbcp)
+DBCat *deldbcp;
+{
+	DBCat *dbcp;
+	int t, n;
+
+	/* sanity check deldbcp is in dbcat[] */
+	if (deldbcp < dbcat || deldbcp >= &dbcat[ndbcat]) {
+	    printf ("Bug! Bad catdel deldbcp: %d\n", deldbcp - dbcat);
+	    exit(1);
+	}
+
+	/* copy each Obj after the ones loaded by this catalog down over the
+	 * entries from this catalog, then decrement the total
+	 */
+	for (t = 0; t < NOBJTYPES; t++) {
+	    if ((n = deldbcp->n[t]) > 0) {
+		int src = deldbcp->start[t] + n;
+		int size = db[t].size;
+		int nobj = db[t].nobj;
+
+		/* copy separately as they might be in different chunks */
+		while (src < nobj) {
+		    memcpy (OBJP(t,src-n), OBJP(t,src), size);
+		    src++;
+		}
+		db[t].nobj -= n;
+	    }
+	}
+
+	/* adjust starts down by amounts in deldbcp if start after its start */
+	for (dbcp = dbcat; dbcp < &dbcat[ndbcat]; dbcp++)
+	    for (t = 0; t < NOBJTYPES; t++)
+		if (dbcp->start[t] > deldbcp->start[t])
+		    dbcp->start[t] -= deldbcp->n[t];
+
+	/* remove deldbcp from dbcat.
+	 * N.B. deldbcp not valid after this
+	 */
+	memcpy (deldbcp, deldbcp+1, (&dbcat[--ndbcat] - deldbcp)*sizeof(DBCat));
+
+	/* update GUI */
+	db_newcatmenu (dbcat, ndbcat);
+	db_setinitial();
+}
+
+/* mark all db objects as out-of-date
  */
 void
 db_invalidate()
@@ -366,64 +435,23 @@ db_del_all()
 	if (!DBINITED)
 	    db_init();
 
+	/* free memory for each type */
 	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++) {
 	    /* N.B. PLANET entries are fixed -- not malloced */
 	    if (dmp == &db[PLANET])
-		continue;
+		continue;	/* N.B. except planets! */
 	    db_free (dmp);
 	}
-}
 
-/* delete all objects back to the last checkpoint.
- * we don't actually free memory.
- */
-void
-db_del_cp ()
-{
-	DBMem *dmp;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++)
-	    dmp->nobj = dmp->ncp;
-}
-
-/* set the checkpoint to the current set of objects in the db.
- */
-void
-db_set_cp()
-{
-	DBMem *dmp;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++)
-	    dmp->ncp = dmp->nobj;
-}
-
-/* set the checkpoints back to 0.
- * N.B. never fiddle with PLANET's count.
- */
-void
-db_clr_cp()
-{
-	DBMem *dmp;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++) {
-	    /* N.B. PLANET entries are fixed -- not malloced */
-	    if (dmp != &db[PLANET])
-		dmp->ncp = 0;
-	}
+	/* no more catalogs */
+	ndbcat = 0;
+	db_newcatmenu (dbcat, ndbcat);
+	db_setinitial();
 }
 
 /* compare 2 pointers to MaxNm's in qsort fashion */
 static int
-name_cmpf (s1, s2)
+maxnm_cmpf (s1, s2)
 qsort_arg s1, s2;
 {
 	return (strcmp (*(MaxNm *)s1, *(MaxNm *)s2));
@@ -440,10 +468,6 @@ int nnames;
 {
 	int t, b, m, s;
 
-	/* check for no names at all */
-	if (nnames <= 0)
-	    return (-1);
-
 	/* binary search */
 	t = nnames - 1;
 	b = 0;
@@ -462,24 +486,62 @@ int nnames;
 }
 
 /* read the given database file into memory.
- * add to the existing list.
+ * add to the existing list of objects.
  * if nodups, skip objects whose name already appears in memory.
+ * add a new catalog entry, sorted by catalog name, update GUI.
  * stop gracefully if we run out of memory.
- * return 0 if all ok, else -1.
+ * keep operator informed.
+ * look in several places for file.
+ * if enabled and only one object in whole file, preload into an ObjXYZ.
  */
-int
-db_read (fp, nodups)
-FILE *fp;
+void
+db_read (fn, nodups)
+char *fn;
 int nodups;
 {
 	char buf[MAXDBLINE];
+	char fullfn[1024];
+	char msg[1024];
 	MaxNm *names = 0;
+	DBCat *dbcp;
 	int nnames = 0;
+	int nobjs;
+	Obj o;
+	char *base;
+	FILE *fp;
 	long len;
+	int fok;
 
 	if (!DBINITED)
 	    db_init();
 
+	/* skip any leading blanks */
+	while (*fn == ' ')
+	    fn++;
+
+	/* open the file.
+	 * try looking for fn in several places.
+	 */
+	fp = fopenh (fn, "r");
+	if (fp)
+	    goto ok;
+	sprintf (fullfn, "%s/%s", getPrivateDir(), fn);
+	fp = fopenh (fullfn, "r");
+	if (fp)
+	    goto ok;
+	sprintf (fullfn, "%s/catalogs/%s", getShareDir(), fn);
+	fp = fopenh (fullfn, "r");
+	if (fp)
+	    goto ok;
+	(void) sprintf (msg, "%s:\n%s", fn, syserrstr());
+	xe_msg (msg, 1);
+	return;
+
+	/* report just base of fn */
+    ok:
+	for (base = fn+strlen(fn); base > fn && base[-1] != '/'; --base)
+	    continue;
+
 	/* set up to run the progress meter based on file position */
 	(void) fseek (fp, 0L, SEEK_END);
 	len = ftell (fp);
@@ -487,41 +549,52 @@ int nodups;
 	pm_set (0);
 
 	if (nodups) {
-	    /* build sorted list of existing names. 
-	     * N.B. actually copy the names in case the real db moves.
-	     */
+	    /* build sorted list of existing names for faster searching */
+	    int dbn = db_n();
+	    int mask = ALLM & ~PLANETM;
 	    DBScan dbs;
-	    int mask = ALLM;
 	    Obj *op;
 
 	    nnames = 0;
-	    names = (MaxNm *) malloc (db_n() * sizeof(MaxNm));
+	    names = (MaxNm *) malloc (dbn*sizeof(MaxNm));
 	    if (!names) {
 		xe_msg ("No memory to check dups", 1);
-		return (-1);
+		fclose (fp);
+		return;
 	    }
-	    for (db_scaninit(&dbs,mask,NULL,0); (op = db_scan(&dbs)) != NULL; )
-		(void) strcpy (names[nnames++], op->o_name);
-	    if (nnames > db_n()) {
-		printf ("db_read: too many objects! %d %d\n", db_n(), nnames);
-		exit (1);
+	    for (db_scaninit(&dbs,mask,NULL,0); (op = db_scan(&dbs)) != NULL; ){
+		if (nnames < dbn) {
+		    (void) strcpy (names[nnames++], op->o_name);
+		} else {
+		    printf ("Bug! too many objects! %d %d\n", dbn, nnames);
+		    exit (1);
+		}
 	    }
 
-	    qsort ((void *)names, nnames, sizeof(MaxNm), name_cmpf);
+	    qsort ((void *)names, nnames, sizeof(MaxNm), maxnm_cmpf);
+	}
+
+	/* get a fresh catalog entry */
+	dbcp = db_catadd(base);
+	if (!dbcp) {
+	    xe_msg ("No memory for new catalog", 1);
+	    if (names)
+		free ((void *)names);
+	    fclose(fp);
+	    return;
 	}
 
 	/* read each line from the file and add good ones to the db */
+	nobjs = 0;
 	while (fgets (buf, sizeof(buf), fp)) {
 	    char whynot[128];
-	    Obj o;
 
 	    pm_set ((int)(ftell(fp)*100/len)); /* update progress meter */
 
 	    if (db_crack_line (buf, &o, whynot) < 0) {
 		if (whynot[0] != '\0') {
-		    char wholemsg[1024];
-		    (void) sprintf (wholemsg, "Bad edb: %s: %s", buf, whynot);
-		    xe_msg (wholemsg, 0);
+		    (void) sprintf (msg,"%s: Bad edb:\n %s:%s",base,buf,whynot);
+		    xe_msg (msg, 0);
 		}
 		continue;
 	    }
@@ -529,27 +602,68 @@ int nodups;
 	    if (o.o_type == PLANET)
 		continue;
 
-	    if (nodups) {
-		char c, nm[MAXNM];
-		int i;
+	    if (nodups && chk_name (o.o_name, names, nnames) == 0)
+		continue;
 
-		for (i = 0; i < (MAXNM-1) && (c = buf[i]) != ','; i++)
-		    nm[i] = c;
-		nm[i] = '\0';
-		if (chk_name (nm, names, nnames) == 0)
-		    continue;
+	    if (db_objadd (&o) < 0) {
+		xe_msg ("No more memory", 1);
+		fclose(fp);
+		db_catdel (dbcp);
+		if (names)
+		    free ((void *)names);
+		return;
 	    }
-
-	    if (db_add (&o) < 0)
-		break;
+	    dbcp->n[o.o_type]++;
+	    nobjs++;
 	}
 
-	/* finished with names */
+	/* clean up */
 	if (names)
 	    free ((void *)names);
+	fok = !ferror(fp);
+	fclose(fp);
 
-	/* should have hit eof -- if not, other trouble */
-	return (feof(fp) ? 0 : -1);
+	/* check for trouble */
+	if (!fok) {
+	    sprintf (msg, "%s:\n%s", base, syserrstr());
+	    xe_msg (msg, 1);
+	    db_catdel (dbcp);
+	    return;
+	}
+
+	/* check for noop */
+	if (nobjs == 0) {
+	    db_catdel (dbcp);
+	    if (nodups)
+		sprintf (msg, "%s contains no new data", base);
+	    else
+		sprintf (msg, "%s contains no data", base);
+	    xe_msg (msg, 1);
+	}
+
+	/* catalog GUI update is sufficient feedback if added ok */
+	db_newcatmenu (dbcat, ndbcat);
+	db_setinitial();
+
+	/* auto set to ObjXY or Z if exactly 1 in catalog */
+	if (nobjs == 1 && db_load1()) {
+	    static int uidx[] = {OBJX, OBJY, OBJZ};
+	    static char uobjn[] = "XYZ";
+	    int i;
+
+	    for (i = 0; i < XtNumber(uidx); i++) {
+		Obj *uop = db_basic(uidx[i]);
+		if (i == XtNumber(uidx)-1 || uop->o_type == UNDEFOBJ ||
+					    (uop->o_type == o.o_type &&
+					    !strcmp(uop->o_name, o.o_name))) {
+		    char buf[54];
+		    obj_set (&o, uidx[i]);
+		    sprintf (buf, "Assigned %s to Obj%c", o.o_name, uobjn[i]);
+		    xe_msg (buf, 1);
+		    break;
+		}
+	    }
+	}
 }
 
 /* assuming we can open it ok, connect the db fifo to a callback.
@@ -591,7 +705,7 @@ db_connect_fifo()
  * N.B we do *not* validate newop in any way.
  */
 static int
-db_add (newop)
+db_objadd (newop)
 Obj *newop;
 {
 	int t = newop->o_type;
@@ -605,17 +719,18 @@ Obj *newop;
 	    char **newdbl;
 	    char *newchk;
 
+	    /* grow list of chunks */
 	    newdbl = dmp->dblist ? (char **) realloc (dmp->dblist, newdblsz)
 				 : (char **) malloc (newdblsz);
 	    if (!newdbl)
 		return (-1);
 
+	    /* add 1 chunk */
 	    newchk = malloc (dmp->size * DBCHUNK);
 	    if (!newchk) {
 		free ((char *)newdbl);
 		return (-1);
 	    }
-
 	    newdbl[ndbl] = newchk;
 
 	    dmp->dblist = newdbl;
@@ -661,13 +776,19 @@ db_init()
 	db[PLANET].dblist = plan_dblist;
 	db[PLANET].nmem = NOBJ;
 	db[PLANET].nobj = NOBJ;
-	db[PLANET].ncp = NOBJ;
 	for (i = MERCURY; i <= MOON; i++) {
 	    Obj *op = OBJP (PLANET, i);
 	    op->o_type = PLANET;
 	    (void) strncpy (op->o_name, planet_names[i], sizeof(op->o_name)-1);
 	    op->pl.pl_code = i;
 	}
+
+	/* init the catalog. malloc something so we can use realloc now */
+	dbcat = (DBCat *) malloc (sizeof(DBCat));
+	ndbcat = 0;
+
+	/* register the initial files list */
+	sr_reg (0, dbifres, dbcategory, 0);
 }
 
 /* free all the memory associated with the given DBMem */
@@ -685,7 +806,6 @@ DBMem *dmp;
 
 	dmp->nobj = 0;
 	dmp->nmem = 0;
-	dmp->ncp = 0;
 }
 
 /* called whenever there is input readable from the db fifo.
@@ -744,7 +864,7 @@ XtInputId *idp;         /* pointer to input id */
 				(void) sprintf (msg,
 				    "Planet %s ignored from DBFIFO",o.o_name);
 				xe_msg (msg, 0);
-			    } else if (db_add (&o) < 0)
+			    } else if (db_objadd (&o) < 0)
 				xe_msg ("No more memory for DBFIFO", 0);
 			}
 		    }
@@ -787,5 +907,54 @@ XtInputId *idp;         /* pointer to input id */
 	}
 }
 
+/* compare 2 pointers to DBCat's by name in qsort fashion */
+static int
+db_catcmpf (d1, d2)
+qsort_arg d1, d2;
+{
+	return (strcmp (((DBCat *)d1)->name, ((DBCat *)d2)->name));
+}
+
+/* allocate a new DBCat in dbcat[] and init with name, sorted by name.
+ * name is already just the basename of a full path.
+ * return pointer if ok, else NULL if no more memory.
+ */
+static DBCat *
+db_catadd (name)
+char *name;
+{
+	DBCat *dbcp;
+	int i;
+
+	/* make room for another */
+	dbcp = (DBCat *) realloc (dbcat, (ndbcat+1)*sizeof(DBCat));
+	if (!dbcp)
+	    return (NULL);
+	dbcat = dbcp;
+	dbcp = &dbcat[ndbcat++];
+	(void) sprintf (dbcp->name, "%.*s", (int)sizeof(dbcp->name)-1, name);
+
+	/* init the type counters -- leave key to find after search */
+	for (i = 0; i < NOBJTYPES; i++) {
+	    dbcp->start[i] = db[i].nobj;
+	    dbcp->n[i] = 0;
+	}
+	dbcp->n[0] = -1;
+
+	/* sort by name */
+	qsort ((void *)dbcat, ndbcat, sizeof(DBCat), db_catcmpf);
+
+	/* find key again */
+	for (dbcp = dbcat; dbcp < &dbcat[ndbcat]; dbcp++)
+	    if (dbcp->n[0] == -1) {
+		dbcp->n[0] = 0;
+		return (dbcp);
+	    }
+
+	/* eh?? */
+	printf ("Bug! catalog disappeared after sorting: %s\n", name);
+	exit(1);
+}
+
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: db.c,v $ $Date: 1998/12/22 06:19:07 $ $Revision: 1.4 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: db.c,v $ $Date: 2000/09/12 04:53:13 $ $Revision: 1.16 $ $Name:  $"};
