@@ -184,7 +184,7 @@ static int LoadGIF(pinfo)
   aspect = NEXTBYTE;
   if (aspect) {
     if (!gif89) return(gifError(pinfo,"corrupt GIF file (screen descriptor)"));
-    else normaspect = (float) (aspect + 15) / 64.0;   /* gif89 aspect ratio */
+    else normaspect = (float)((aspect + 15) / 64.0);   /* gif89 aspect ratio */
   }
   
   
