@@ -611,11 +611,16 @@ FImage *fip;
 char *name;
 double *vp;
 {
-	char *rp;
+	char buf[32];
+	char *dp, *rp;
 
 	if (findFImageVar (fip, name, &rp) < 0)
 	    return (-1);
-	*vp = atof (rp+10);
+	memcpy (buf, rp+10, 30);
+	buf[30] = '\0';
+	if ((dp = strchr (buf,'D')) || (dp = strchr (buf,'d')))
+	    *dp = 'e';
+	*vp = atof (buf);
 	return (0);
 }
 
@@ -960,3 +965,6 @@ lendian()
 
 	return (U.s == 1);
 }
+
+/* For RCS Only -- Do Not Edit */
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: fits.c,v $ $Date: 1998/12/27 21:07:42 $ $Revision: 1.2 $ $Name:  $"};
