@@ -322,19 +322,14 @@ Now *np;
 	    (void) strcpy (buf, tznm);
 }
 
-/* set the XmNlabelString resource of the given widget to the date and time 
- * as given in the Now struct at *np.
- * avoid redrawing the string if it has not changed but don't use f_showit()
- *   because we want the time to always be updated even during movie loops.
- */
+/* fill buf[] with time stamp from np */
 void
-timestamp (np, w)
+fs_timestamp (np, stamp)
 Now *np;
-Widget w;
+char stamp[];
 {
-	char d[32], t[32], stamp[64];
 	double lmjd;
-	char *txtp;
+	char d[32], t[32];
 	char timezonename[32];
 	int tzpref = pref_get (PREF_ZONE);
 
@@ -347,7 +342,22 @@ Widget w;
 
 	fs_tz (timezonename, tzpref, np);
 	(void) sprintf (stamp, "%s %s %s", d, t, timezonename);
+}
 
+/* set the XmNlabelString resource of the given widget to the date and time 
+ * as given in the Now struct at *np.
+ * avoid redrawing the string if it has not changed but don't use f_showit()
+ *   because we want the time to always be updated even during movie loops.
+ */
+void
+timestamp (np, w)
+Now *np;
+Widget w;
+{
+	char stamp[64];
+	char *txtp;
+
+	fs_timestamp (np, stamp);
 	get_xmstring (w, XmNlabelString, &txtp);
 	if (strcmp (txtp, stamp)) {
 	    set_xmstring (w, XmNlabelString, stamp);
@@ -361,4 +371,4 @@ Widget w;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: formats.c,v $ $Date: 1998/02/26 05:08:10 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: formats.c,v $ $Date: 2000/05/04 23:53:48 $ $Revision: 1.2 $ $Name:  $"};
