@@ -32,18 +32,17 @@ extern int close();
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
+#include "db.h"
 #include "net.h"
 
 extern Widget toplevel_w;
 #define XtD     XtDisplay(toplevel_w)
-extern Colormap xe_cm;
 extern XtAppContext xe_app;
-
+extern Colormap xe_cm;
 
 #define	DUPSEP		degrad(4/3600.)	/* dup if < this close, rads */
 #define	DUPMAG		(3.0)		/* dup if mags differ < this much */
-#define	GSCFOVLIM	degrad(15.0)	/* max gsc fov we'll fetch */
-#define	USNOFOVLIM	degrad(10.0)	/* max usno fov we'll fetch */
+#define	FOVLIM		degrad(15.0)	/* max fov we'll fetch */
 
 extern Now *mm_get_now P_((void));
 extern char *getShareDir P_((void));
@@ -59,18 +58,21 @@ extern int gscesofetch P_((Now *np, double ra, double dec, double fov,
     double mag, ObjF **opp, int nop, char msg[]));
 extern int gscnetfetch P_((char *host, Now *np, double ra, double dec,
     double fov, double mag, ObjF **opp, int nop, char msg[]));
+extern int isUp P_((Widget w));
 extern int obj_cir P_((Now *np, Obj *op));
-extern int ppmchkfile P_((char *file, char *msg));
-extern int ppmfetch P_((char *file, Now *np, double ra, double dec,
-    double fov, double mag, int saohd, ObjF **opp, char *msg));
+extern int xe2chkfile P_((char *file, char *msg));
+extern int xe2fetch P_((char *file, Now *np, double ra, double dec,
+    double fov, double mag, ObjF **opp, char *msg));
 extern Obj *db_scan P_((DBScan *sp));  
 extern void XCheck P_((XtAppContext app));
+extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
+extern void sv_loadfs P_((int force));
 extern void watch_cursor P_((int want));
 extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
@@ -79,17 +81,17 @@ static int objfd_cmp P_((qsort_arg v1, qsort_arg v2));
 static int fs_nodbdups P_((ObjF *fop, int nfop, double ra, double dec,
     double fov));
 static int scanchk P_((Obj *mop, Obj *op, double cdec));
-static void fs_create_fsform P_((void));
+static void fs_create_fsshell P_((void));
 static int fs_save P_((void));
 static void fs_setup P_((void));
-static void test_cb P_((Widget w, XtPointer client, XtPointer call));
+static void apply_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ok_cb P_((Widget w, XtPointer client, XtPointer call));
 static void gsctb_cb P_((Widget w, XtPointer client, XtPointer call));
 static void pmtb_cb P_((Widget w, XtPointer client, XtPointer call));
 static void cancel_cb P_((Widget w, XtPointer client, XtPointer call));
 static void help_cb P_((Widget w, XtPointer client, XtPointer call));
 
-static Widget fsform_w;		/* the main form dialog */
+static Widget fsshell_w;	/* the main shell */
 static Widget cdtf_w, cdtb_w;	/* GSC CD-ROM text field and toggle button */
 static Widget nettf_w, nettb_w;	/* GSC network text field and toggle button */
 static Widget chtf_w, chtb_w;	/* GSC Cache text field and toggle button */
@@ -97,7 +99,6 @@ static Widget notf_w, notb_w;	/* USNO text field and toggle button */
 static Widget etb_w;		/* ESO toggle button */
 static Widget ppmtf_w, ppmtb_w;	/* PMM text field and toggle button */
 static Widget tyctf_w, tyctb_w;	/* Tycho text field and toggle button */
-static Widget saohdtb_w;	/* PM TB on when want SAO/HD names */
 static Widget nodupt_w;		/* no dups toggle button */
 
 /* TB codes */
@@ -118,7 +119,6 @@ static int ppm_on;		/* whether ppm is currently on */
 static char *ppm_fn;		/* ppm filename */
 static int tyc_on;		/* whether tycho is currently on */
 static char *tyc_fn;		/* tycho filename */
-static int wantsaohd;		/* whether we want to use SAO/HD names */
 static int nodup_on;		/* whether dup checking is on */
 
 /* call to set up without actually bringing up the menus.
@@ -126,8 +126,8 @@ static int nodup_on;		/* whether dup checking is on */
 void
 fs_create()
 {
-	if (!fsform_w) {
-	    fs_create_fsform();
+	if (!fsshell_w) {
+	    fs_create_fsshell();
 	    (void) fs_save();	/* confirming here is just annoying */
 	}
 }
@@ -136,13 +136,10 @@ void
 fs_manage()
 {
 	fs_create();
+	fs_setup();
 
-	if (XtIsManaged(fsform_w))
-	    XtUnmanageChild (fsform_w);
-	else {
-	    fs_setup();
-	    XtManageChild (fsform_w);
-	}
+	XtPopup (fsshell_w, XtGrabNone);
+	set_something (fsshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* call to fetch a set of ObjF from the field star sources.
@@ -165,7 +162,6 @@ ObjF **opp;	/* we set *opp to a malloced list of ObjF */
 	int nop = 0;
 	int s = 0;
 	char msg[256];
-	double fovlim;
 	int i;
 
 	if(!cd_on &&!ch_on &&!net_on &&!e_on &&!usno_on &&!ppm_on &&!tyc_on)
@@ -173,35 +169,41 @@ ObjF **opp;	/* we set *opp to a malloced list of ObjF */
 
 	watch_cursor (1);
 
+	/* clamp fov */
+	if (fov > FOVLIM)
+	    fov = FOVLIM;
+
 	/* the fetch tools all want and return J2000 values */
 	if (epoch == EOD)
 	    ap_as (np, J2000, &ra, &dec);
 	else
 	    precess (epoch, J2000, &ra, &dec);
 
-	/* get the PM catalos, if desired */
+	/* get the PM catalogs, if desired */
 	if (ppm_on) {
-	    nop = ppmfetch (ppm_fn, np, ra, dec, fov, mag, wantsaohd, &op, msg);
+	    nop = xe2fetch (ppm_fn, np, ra, dec, fov, mag, &op, msg);
 	    if (nop < 0)
 		ppm_on = 0;
-	} else if (tyc_on) {
-	    nop = ppmfetch (tyc_fn, np, ra, dec, fov, mag, wantsaohd, &op, msg);
+	}
+	if (tyc_on) {
+	    nop = xe2fetch (tyc_fn, np, ra, dec, fov, mag, &op, msg);
 	    if (nop < 0)
 		tyc_on = 0;
 	}
 
 	/* now add the GSC, if desired -- enforce max fov */
-	fovlim = fov > GSCFOVLIM ? GSCFOVLIM : fov;
 	if (net_on) {
-	    nop = gscnetfetch (net_host, np, ra, dec, fovlim, mag, &op,nop,msg);
+	    nop = gscnetfetch (net_host, np, ra, dec, fov, mag, &op,nop,msg);
 	    if (nop < 0)
 		net_on = 0;
-	} else if (e_on) {
-	    nop = gscesofetch (np, ra, dec, fovlim, mag, &op, nop, msg);
+	}
+	if (e_on) {
+	    nop = gscesofetch (np, ra, dec, fov, mag, &op, nop, msg);
 	    if (nop < 0)
 		e_on = 0;
-	} else if (cd_on || ch_on) {
-	    nop = GSCFetch (ra, dec, fovlim, mag, &op, nop, msg);
+	}
+	if (cd_on || ch_on) {
+	    nop = GSCFetch (ra, dec, fov, mag, &op, nop, msg);
 	    if (nop < 0)
 		cd_on = 0;
 	}
@@ -209,9 +211,8 @@ ObjF **opp;	/* we set *opp to a malloced list of ObjF */
 	    xe_msg (msg, 1);
 
 	/* now add USNO if desired -- enforce max fov */
-	fovlim = fov > USNOFOVLIM ? USNOFOVLIM : fov;
 	if (usno_on) {
-	    nop = USNOFetch (ra, dec, fovlim, mag, &op, nop, msg);
+	    nop = USNOFetch (ra, dec, fov, mag, &op, nop, msg);
 	    if (nop < 0)
 		usno_on = 0;
 	}
@@ -257,6 +258,13 @@ ObjF **opp;	/* we set *opp to a malloced list of ObjF */
 	return (s);
 }
 
+/* return 1 if any proper motion catalog is on enabled, else 0 */
+int
+fs_pmon()
+{
+	return (ppm_on || tyc_on);
+}
+
 /* called to put up or remove the watch cursor.  */
 void
 fs_cursor (c)
@@ -264,7 +272,7 @@ Cursor c;
 {
 	Window win;
 
-	if (fsform_w && (win = XtWindow(fsform_w)) != 0) {
+	if (fsshell_w && (win = XtWindow(fsshell_w)) != 0) {
 	    if (c)
 		XDefineCursor (XtDisplay(toplevel_w), win, c);
 	    else
@@ -421,31 +429,32 @@ double cdec;
 }
 
 static void
-fs_create_fsform()
+fs_create_fsshell()
 {
-	char fname[1024];
-	Widget f, w;
+	Widget fsform_w, f, w;
 	Arg args[20];
 	int n;
 
-	/* create form */
+	/* create outtern shell and form */
+
+	n = 0;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Field Stars setup"); n++;
+	XtSetArg (args[n], XmNiconName, "FStars"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	fsshell_w = XtCreatePopupShell ("FieldStars", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (fsshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	sr_reg (fsshell_w, "XEphem*FieldStars.x", dbcategory, 0);
+	sr_reg (fsshell_w, "XEphem*FieldStars.y", dbcategory, 0);
+
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	fsform_w = XmCreateFormDialog (toplevel_w, "FieldStars", args, n);
-	set_something (fsform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (fsform_w, XmNmapCallback, prompt_map_cb, NULL);
-        XtAddCallback (fsform_w, XmNhelpCallback, help_cb, NULL);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
-	n = 0;
-	XtSetArg (args[n], XmNtitle,"xephem Field Stars setup");n++;
-	XtSetValues (XtParent(fsform_w), args, n);
+	fsform_w = XmCreateForm (fsshell_w, "FSForm", args, n);
+	XtAddCallback (fsform_w, XmNhelpCallback, help_cb, NULL);
+	XtManageChild (fsform_w);
 
 	/* make the GSC section title */
 
@@ -467,6 +476,7 @@ fs_create_fsform()
 	    cdtf_w = XmCreateTextField (fsform_w, "GSCCDDirectory", args, n);
 	    wtip (cdtf_w, "Pathname to the root of the CDROM when mounted");
 	    XtManageChild (cdtf_w);
+	    sr_reg (cdtf_w, NULL, dbcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -483,19 +493,20 @@ fs_create_fsform()
 	    set_xmstring (cdtb_w, XmNlabelString, "ASP CDROM Directory:");
 	    wtip (cdtb_w, "Whether to use the ASP CDROM for GSC stars");
 	    XtManageChild (cdtb_w);
+	    sr_reg (cdtb_w, NULL, dbcategory, 1);
 
 	    /* make the GSC Cache toggle/text */
 
-	    (void) sprintf (fname, "%s/catalogs/gsc",  getShareDir());
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, cdtf_w); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
-	    XtSetArg (args[n], XmNvalue, fname); n++;
 	    chtf_w = XmCreateTextField (fsform_w, "GSCCacheDirectory", args, n);
+	    defaultTextFN (chtf_w, 0, getShareDir(), "catalogs/gsc");
 	    wtip (chtf_w, "Pathname to the GSC subdirectory");
 	    XtManageChild (chtf_w);
+	    sr_reg (chtf_w, NULL, dbcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -512,6 +523,7 @@ fs_create_fsform()
 	    set_xmstring (chtb_w, XmNlabelString,  "Local Cache Directory:");
 	    wtip (chtb_w, "Whether to use a local disk for GSC stars");
 	    XtManageChild (chtb_w);
+	    sr_reg (chtb_w, NULL, dbcategory, 1);
 
 	    /* make the GSC network toggle/text */
 
@@ -522,6 +534,7 @@ fs_create_fsform()
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
 	    nettf_w = XmCreateTextField (fsform_w, "XEphemdbdURL", args, n);
 	    wtip (nettf_w, "URL (/<host>/.../xephemdbd.pl) from which to fetch GSC stars");
+	    sr_reg (nettf_w, NULL, dbcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -537,6 +550,7 @@ fs_create_fsform()
 							    (XtPointer)NETTB);
 	    set_xmstring (nettb_w, XmNlabelString, "Internet to xephemdbd:");
 	    wtip (nettb_w,"Whether to use an Internet XEphem server for stars");
+	    sr_reg (nettb_w, NULL, dbcategory, 1);
 	    XtManageChild (nettb_w);
 	    XtManageChild (nettf_w);
 
@@ -554,6 +568,7 @@ fs_create_fsform()
 	    set_xmstring (etb_w, XmNlabelString, "Internet to ESO");
 	    wtip(etb_w,"Whether to use the Internet to ESO to fetch GSC stars");
 	    XtManageChild (etb_w);
+	    sr_reg (etb_w, NULL, dbcategory, 1);
 
 	/* make the USNO section title */
 
@@ -568,16 +583,17 @@ fs_create_fsform()
 
 	    /* make the USNO toggle/text */
 
-	    (void) sprintf (fname, "%s/catalogs/usno",  getShareDir());
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, w); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
-	    XtSetArg (args[n], XmNvalue, fname); n++;
 	    notf_w = XmCreateTextField (fsform_w, "USNODirectory", args, n);
-	    wtip (notf_w, "Pathname of A1.0, SA1.0 etc catalog series root directory");
+	    wtip (notf_w,
+		"Pathname of A1.0, SA1.0 etc catalog series root directory");
+	    defaultTextFN (notf_w, 0, getShareDir(), "catalogs/usno");
 	    XtManageChild (notf_w);
+	    sr_reg (notf_w, NULL, dbcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -592,6 +608,7 @@ fs_create_fsform()
 	    set_xmstring (notb_w, XmNlabelString, "Root directory: ");
 	    wtip (notb_w, "Whether to access a USNO catalog");
 	    XtManageChild (notb_w);
+	    sr_reg (notb_w, NULL, dbcategory, 1);
 
 	/* make the PM section title */
 
@@ -606,16 +623,16 @@ fs_create_fsform()
 
 	    /* make the PM toggle/text */
 
-	    (void) sprintf (fname, "%s/catalogs/ppm.xe",  getShareDir());
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, w); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
-	    XtSetArg (args[n], XmNvalue, fname); n++;
 	    ppmtf_w = XmCreateTextField (fsform_w, "PPMFilename", args, n);
-	    wtip (ppmtf_w, "Full pathname of ppm.xe file");
+	    wtip (ppmtf_w, "Full pathname of ppm.xe2 file");
+	    defaultTextFN (ppmtf_w, 0, getShareDir(), "catalogs/ppm.xe2");
 	    XtManageChild (ppmtf_w);
+	    sr_reg (ppmtf_w, NULL, dbcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -628,21 +645,22 @@ fs_create_fsform()
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    ppmtb_w = XmCreateToggleButton (fsform_w, "PPM", args, n);
 	    set_xmstring (ppmtb_w, XmNlabelString, "PPM catalog: ");
-	    wtip (ppmtb_w, "Whether to access the PPM catalog, ppm.xe");
+	    wtip (ppmtb_w, "Whether to access the PPM catalog, ppm.xe2");
 	    XtManageChild (ppmtb_w);
+	    sr_reg (ppmtb_w, NULL, dbcategory, 1);
 
 	    /* make the Tycho toggle/text */
 
-	    (void) sprintf (fname, "%s/catalogs/tycho.xe",  getShareDir());
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, ppmtf_w); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
-	    XtSetArg (args[n], XmNvalue, fname); n++;
 	    tyctf_w = XmCreateTextField (fsform_w, "TychoFilename", args, n);
-	    wtip (tyctf_w, "Full pathname of tycho.xe file");
+	    wtip (tyctf_w, "Full pathname of hiptyc2.xe2 file");
+	    defaultTextFN (tyctf_w, 0, getShareDir(), "catalogs/hiptyc2.xe2");
 	    XtManageChild (tyctf_w);
+	    sr_reg (tyctf_w, NULL, dbcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -654,9 +672,10 @@ fs_create_fsform()
 	    XtSetArg (args[n], XmNrightOffset, 10); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    tyctb_w = XmCreateToggleButton (fsform_w, "Tycho", args, n);
-	    set_xmstring (tyctb_w, XmNlabelString, "Tycho catalog: ");
-	    wtip (tyctb_w, "Whether to access the Tycho catalog, tycho.xe");
+	    set_xmstring (tyctb_w, XmNlabelString, "Hipparcos + Tycho2: ");
+	    wtip (tyctb_w, "Whether to access the combined Hipparcos and Tycho2 catalog");
 	    XtManageChild (tyctb_w);
+	    sr_reg (tyctb_w, NULL, dbcategory, 1);
 
 	    /* enforce and tie together as 1-of-2 pair */
 	    if(XmToggleButtonGetState(ppmtb_w) &&
@@ -669,32 +688,19 @@ fs_create_fsform()
 	    XtAddCallback (tyctb_w, XmNvalueChangedCallback, pmtb_cb,
 							    (XtPointer)ppmtb_w);
 
-	    /* make the HD/SAO name toggle */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNtopWidget, tyctf_w); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftOffset, 15); n++;
-	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	    saohdtb_w = XmCreateToggleButton (fsform_w, "SAOHD", args, n);
-	    set_xmstring (saohdtb_w, XmNlabelString,
-					"Use SAO or HD name where possible");
-	    wtip (saohdtb_w,
-		"Use SAO or HD designation instead of raw catalog if possible");
-	    XtManageChild (saohdtb_w);
-
 	    /* make the No dups toggle */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNtopWidget, saohdtb_w); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNleftOffset, 15); n++;
+	    XtSetArg (args[n], XmNtopWidget, tyctf_w); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 40); n++;
 	    nodupt_w = XmCreateToggleButton (fsform_w, "NoDups", args, n);
-	    set_xmstring (nodupt_w, XmNlabelString, "No duplicates");
-	    wtip (nodupt_w,"Skip objects which appear to already be in memory");
+	    set_xmstring (nodupt_w,XmNlabelString,"Skip likely duplicates");
+	    wtip (nodupt_w,
+	      "When loading, skip objects which are very similar to ones already in memory");
 	    XtManageChild (nodupt_w);
+	    sr_reg (nodupt_w, NULL, dbcategory, 1);
 
 	/* make the controls across the bottom under a separator */
 
@@ -722,7 +728,7 @@ fs_create_fsform()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 3); n++;
 	    w = XmCreatePushButton (f, "Ok", args, n);
-	    wtip (w, "Test the settings and close if ok");
+	    wtip (w, "Engage the settings and close if ok");
 	    XtAddCallback (w, XmNactivateCallback, ok_cb, NULL);
 	    XtManageChild (w);
 
@@ -731,9 +737,9 @@ fs_create_fsform()
 	    XtSetArg (args[n], XmNleftPosition, 4); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 6); n++;
-	    w = XmCreatePushButton (f, "Test", args, n);
-	    wtip (w, "Test the settings and remain up");
-	    XtAddCallback (w, XmNactivateCallback, test_cb, NULL);
+	    w = XmCreatePushButton (f, "Apply", args, n);
+	    wtip (w, "Engage the settings and remain up");
+	    XtAddCallback (w, XmNactivateCallback, apply_cb, NULL);
 	    XtManageChild (w);
 
 	    n = 0;
@@ -742,7 +748,7 @@ fs_create_fsform()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 9); n++;
 	    w = XmCreatePushButton (f, "Close", args, n);
-	    wtip (w, "Close this dialog without doing anything");
+	    wtip (w, "Close this window without doing anything");
 	    XtAddCallback (w, XmNactivateCallback, cancel_cb, NULL);
 	    XtManageChild (w);
 
@@ -781,14 +787,13 @@ fs_setup ()
 	if (usno_fn)
 	    XmTextFieldSetString (notf_w, usno_fn);
 
-	/* PM*/
+	/* PM */
 	XmToggleButtonSetState (ppmtb_w, ppm_on, False);
 	if (ppm_fn)
 	    XmTextFieldSetString (ppmtf_w, ppm_fn);
 	XmToggleButtonSetState (tyctb_w, tyc_on, False);
 	if (tyc_fn)
 	    XmTextFieldSetString (tyctf_w, tyc_fn);
-	XmToggleButtonSetState (saohdtb_w, wantsaohd, False);
 	XmToggleButtonSetState (nodupt_w, nodup_on, False);
 }
 
@@ -890,7 +895,7 @@ fs_save ()
 	if (ppm_fn)
 	    XtFree (ppm_fn);
 	ppm_fn = XmTextFieldGetString (ppmtf_w);
-	if (ppm_on && ppmchkfile (ppm_fn, msg) < 0) {
+	if (ppm_on && xe2chkfile (ppm_fn, msg) < 0) {
 	    xe_msg (msg, 1);
 	    ppm_on = 0;
 	    fs_setup();
@@ -902,15 +907,14 @@ fs_save ()
 	if (tyc_fn)
 	    XtFree (tyc_fn);
 	tyc_fn = XmTextFieldGetString (tyctf_w);
-	if (tyc_on && ppmchkfile (tyc_fn, msg) < 0) {
+	if (tyc_on && xe2chkfile (tyc_fn, msg) < 0) {
 	    xe_msg (msg, 1);
 	    tyc_on = 0;
 	    fs_setup();
 	    allok = 0;
 	}
 
-	/* PM options */
-	wantsaohd = XmToggleButtonGetState (saohdtb_w);
+	/* options */
 	nodup_on = XmToggleButtonGetState (nodupt_w);
 
 	watch_cursor (0);
@@ -921,13 +925,13 @@ fs_save ()
 /* called from Apply */
 /* ARGSUSED */
 static void
-test_cb (w, client, call)
+apply_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
 	if (fs_save() == 0)
-	    xe_msg ("All options appear to be operating normally", 1);
+	    sv_loadfs (1);
 }
 
 /* called from Ok */
@@ -938,8 +942,10 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if (fs_save() == 0)
-	    XtUnmanageChild (fsform_w);
+	if (fs_save() == 0) {
+	    sv_loadfs (1);
+	    XtPopdown (fsshell_w);
+	}
 }
 
 /* called from Ok */
@@ -951,7 +957,7 @@ XtPointer client;
 XtPointer call;
 {
 	/* outta here */
-	XtUnmanageChild (fsform_w);
+	XtPopdown (fsshell_w);
 }
 
 /* called from any of the CDROM, network or Cache toggle buttons.
@@ -1023,4 +1029,4 @@ XtPointer call;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: fsmenu.c,v $ $Date: 1999/03/03 17:35:41 $ $Revision: 1.5 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: fsmenu.c,v $ $Date: 2000/09/11 07:15:11 $ $Revision: 1.14 $ $Name:  $"};
