@@ -31,6 +31,7 @@ extern void *malloc(), *realloc();
 #include "circum.h"
 
 extern char *syserrstr P_((void));
+extern char *expand_home P_((char *path));
 extern int existsh P_((char *path));
 extern FILE *fopenh P_((char *name, char *how));
 extern void pm_set P_((int percent));
@@ -188,7 +189,7 @@ char msg[];
 	    (void) strcpy (r.dir, "");
 	    cacheBuildDir (fullpath, &r);
 	    if (existsh (fullpath) < 0) {
-		(void) sprintf (msg, "%s: %s", chp, syserrstr());
+		(void) sprintf (msg, "GSC Cache %s:\n%s", chp, syserrstr());
 		return (-1);
 	    }
 	}
@@ -492,7 +493,7 @@ static int
 mymkdir (path)
 char *path;
 {
-	return (mkdir (path, 0755));
+	return (mkdir (expand_home (path), 0755));
 }
 
 
@@ -707,10 +708,10 @@ GSCRegion *rp;
 	nsub = sm_reg[lgregno].nsub;
 
 	/* find the subdivision indices and origin */
-        nsmdec = nsub*(decLrg-decband);
+        nsmdec = (int)(nsub*(decLrg-decband));
 	rp->dec0 = 7.5 * (decband + (double)nsmdec/nsub);
 	rp->dec0 = degrad(rp->dec0);
-	nsmra = nsub*(raLrg-raband);
+	nsmra = (int)(nsub*(raLrg-raband));
 	rp->ra0 = (360.0/nlrg) * (raband + (double)nsmra/nsub);
 	rp->ra0 = degrad(rp->ra0);
 
@@ -748,7 +749,7 @@ GSCRegion *rp;
 
 	/* make sure it's a SIMPLE FITS file at least */
 	if (gscSimpleFits (rp->fp) < 0) {
-	    (void) sprintf (lmsg, "%s: not a SIMPLE FITS file", fn);
+	    (void) sprintf (lmsg, "%s:\nnot a SIMPLE FITS file", fn);
 	    fclose (rp->fp);
 	    rp->fp = NULL;
 	    return (-1);
@@ -760,7 +761,7 @@ GSCRegion *rp;
 	 */
 	for (n = 2, state = FINDNAXIS2; state != FOUNDTABLE; n++) {
 	    if (fread (buf, 1, FLL, rp->fp) != FLL) {
-		(void) sprintf (lmsg, "%s: unexpected EOF", fn);
+		(void) sprintf (lmsg, "%s:\nunexpected EOF", fn);
 		fclose (rp->fp);
 		rp->fp = NULL;
 		return (-1);
@@ -894,7 +895,7 @@ GSCEntry *ep;
 	(void) fread (buf, 1, 1, rp->fp);	/* multiflag (discarded)*/
 
 	if (feof(rp->fp) || ferror(rp->fp)) {
-	    fprintf (stderr, "%s: %d stars short\n", rp->file, rp->nrows);
+	    fprintf (stderr, "%s:\n%d stars short\n", rp->file, rp->nrows);
 	    return(-1);
 	}
 
@@ -950,7 +951,7 @@ GSCRegion *cp;
 	cp->fp = fopenh (fullpath, "rb");
 	if (!cp->fp) {
 	    if (nocdrom)
-		(void) sprintf (lmsg, "%s: %s", fullpath, syserrstr());
+		(void) sprintf (lmsg, "%s:\n%s", fullpath, syserrstr());
 	    return (-1);
 	}
 
@@ -968,8 +969,8 @@ GSCRegion *cp;
 	if (rp->south != cp->south || rp->id != cp->id
 					|| fabs(rp->ra0 - cp->ra0) > RAERR
 					|| fabs(rp->dec0 - cp->dec0) > DECERR) {
-	    (void) sprintf (lmsg, "%s: header mismatch", fullpath);
-	    fprintf (stderr, "%s: header mismatch\n", fullpath);
+	    (void) sprintf (lmsg, "%s:\nheader mismatch", fullpath);
+	    fprintf (stderr, "%s:\nheader mismatch\n", fullpath);
 	    fprintf (stderr, "  %d %d %g %g %d\n", 
 			    cp->id, cp->nrows, cp->ra0, cp->dec0, cp->south);
 	    fprintf (stderr, "  %d ?? %g %g %d\n",
@@ -1103,7 +1104,7 @@ GSCRegion *rp;
 	    char fullpath[1024];
 	    cacheBuildFilename (fullpath, rp);
 	    hdr[sizeof(hdr)-1] = '\0';
-	    (void) sprintf (lmsg, "%s: no header: %s", fullpath, hdr);
+	    (void) sprintf (lmsg, "%s:\nno header: %s", fullpath, hdr);
 	    return (-1);
 	}
 
@@ -1114,14 +1115,14 @@ GSCRegion *rp;
 	if (i != 6) {
 	    char fullpath[128];
 	    cacheBuildFilename (fullpath, rp);
-	    (void) sprintf (lmsg, "%s: bad header: %s", fullpath, hdr);
+	    (void) sprintf (lmsg, "%s:\nbad header: %s", fullpath, hdr);
 	    return (-1);
 	}
 
 	if (v > cacheversion) {
 	    char fullpath[128];
 	    cacheBuildFilename (fullpath, rp);
-	    (void) sprintf (lmsg, "%s: unsupported version: %g", fullpath, v);
+	    (void) sprintf (lmsg, "%s:\nunsupported version: %g", fullpath, v);
 	    return (-1);
 	}
 
@@ -1226,4 +1227,4 @@ GSCEntry *ep;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: gsc.c,v $ $Date: 1999/03/03 19:06:34 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: gsc.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.6 $ $Name:  $"};
