@@ -29,6 +29,8 @@ extern Widget toplevel_w;
 #define	XtD XtDisplay(toplevel_w)
 extern Colormap xe_cm;
 
+char helpcategory[] = "Help";			/* Save category */
+
 extern FILE *fopenh P_((char *name, char *how));
 extern char *getShareDir P_((void));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -37,6 +39,7 @@ extern void xe_msg P_((char *msg, int app_modal));
 
 static Widget hlp_create_dialog P_((char *tag));
 static void hlp_ok_cb P_((Widget w, XtPointer client, XtPointer call));
+static void hlp_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static FILE *hlp_openfile P_((char *tag));
 static int hlp_fillfromfile P_((char *tag, Widget txt_w, int l));
 static void hlp_fillfromstrings P_((char *msg[], int nmsg, Widget txt_w));
@@ -58,72 +61,81 @@ char *tag;	/* tag to look for in help file - also dialog title */
 char *deflt[];	/* help text to use if tag not found */
 int ndeflt;	/* number of strings in deflt[] */
 {
-	Widget txt_w, form_w;
+	Widget txt_w, sh_w;
 
 	txt_w = hlp_create_dialog (tag);
-	form_w = XtParent(XtParent(txt_w));
+
+	sh_w = txt_w;
+	for (sh_w = txt_w; !XtIsShell(sh_w); sh_w = XtParent(sh_w))
+	    continue;
 
 	if (hlp_fillfromfile(tag, txt_w, 0) < 0) {
 	    if (!deflt || ndeflt == 0) {
 		char buf[MAXLINE];
 		(void) sprintf (buf, "No help for %s", tag);
 		xe_msg (buf, 1);
-		/* destroy the shell, not just the form */
-		while (!XtIsWMShell(form_w))
-		    form_w = XtParent(form_w);
-		XtDestroyWidget (form_w);
+		XtDestroyWidget (sh_w);
 		return;
 	    } else
 		hlp_fillfromstrings(deflt, ndeflt, txt_w);
 	}
 
 	XmTextShowPosition (txt_w, (XmTextPosition)0);
-	XtManageChild (form_w);
-	/* everything gets destroyed when the Ok button is selected */
+	XtPopup (sh_w, XtGrabNone);
+
+	/* everything gets destroyed if shell is popped down */
 }
 
-/* create the help dialog with a scrolled text area and an Ok button.
- * return the text area widget; when ready to view, manage its parent's parent:
- *   dialog -> form -> scrolled_window -> text.
- * pass the dialog widget as the client parameter of the ok activate callback
- *   so it can destroy it.
+/* create the help window with a scrolled text area and an Ok button.
+ * return the text area widget; when ready to view, popup its 3rd parent:
+ *   toplevel -> form -> scrolled_window -> text.
+ * pass the toplevel shell as the client parameter of the ok activate callback
+ *   so it can pop it down; popdown callback then destroys the whole thing.
  */
 static Widget
 hlp_create_dialog (tag)
 char *tag;
 {
-	Widget hlp_w;
+	Widget f_w;
+	Widget sh_w;
 	Widget t_w, cb_w;
 	Arg args[20];
 	char title[MAXLINE];
 	int n;
 
-	/* make the help shell form-dialog widget */
+	/* make the help shell and form */
 
 	n = 0;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNfractionBase, 9); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	hlp_w = XmCreateFormDialog (toplevel_w, "Help", args, n);
-	set_something (hlp_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (hlp_w, XmNmapCallback, prompt_map_cb, NULL);
-
+	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	(void) sprintf (title, "xephem Help on %s", tag);
-	set_something (XtParent(hlp_w), XmNtitle, (XtArgVal)title);
+	XtSetArg (args[n], XmNtitle, title); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNiconName, "Help"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	sh_w = XtCreatePopupShell ("HelpWindow", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (sh_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (sh_w, XmNpopdownCallback, hlp_popdown_cb, 0);
+	/* Save handles us special because we create/destroy many */
+
+	n = 0;
+	XtSetArg (args[n], XmNmarginHeight, 10); n++;
+	XtSetArg (args[n], XmNmarginWidth, 10); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
+	XtSetArg (args[n], XmNfractionBase, 9); n++;
+	f_w = XmCreateForm (sh_w, "HelpF", args, n);
+	XtManageChild (f_w);
 
 	/* make the Ok button */
 
 	n = 0;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNbottomOffset, 10); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNleftPosition, 3); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 6); n++;
-	cb_w = XmCreatePushButton (hlp_w, "Ok", args, n);
-	XtAddCallback (cb_w, XmNactivateCallback, hlp_ok_cb,
-							(XtPointer)hlp_w);
+	cb_w = XmCreatePushButton (f_w, "Ok", args, n);
+	XtAddCallback (cb_w, XmNactivateCallback, hlp_ok_cb, (XtPointer)sh_w);
 	XtManageChild (cb_w);
 
 	/* make the scrolled text area to help the help text */
@@ -132,18 +144,34 @@ char *tag;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNbottomWidget, cb_w); n++;
-	XtSetArg (args[n], XmNbottomOffset, 10); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
 	XtSetArg (args[n], XmNeditable, False); n++;
 	XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
-	t_w = XmCreateScrolledText (hlp_w, "HelpText", args, n);
+	XtSetArg (args[n], XmNmarginHeight, 5); n++;
+	XtSetArg (args[n], XmNmarginWidth, 5); n++;
+	t_w = XmCreateScrolledText (f_w, "HelpText", args, n);
 	XtManageChild (t_w);
 
 	return (t_w);
 }
 
+/* called whenever a help shell is popped down.
+ */
+/* ARGSUSED */
+static void
+hlp_popdown_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XtDestroyWidget (w);
+}
+
+/* called on OK.
+ * client is shell to pop down.
+ */
 /* ARGSUSED */
 static void
 hlp_ok_cb (w, client, call)
@@ -151,12 +179,10 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	Widget d_w = (Widget) client;
+	Widget sh_w = (Widget) client;
 
-	/* destroy the shell, not just the form */
-	while (!XtIsWMShell(d_w))
-	    d_w = XtParent(d_w);
-	XtDestroyWidget (d_w);
+	XtPopdown (sh_w);
+	/* popdown callback will destroy */
 }
 
 /* open the help file and position at first line after we see "@tag\n".
@@ -261,4 +287,4 @@ Widget txt_w;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: helpmenu.c,v $ $Date: 1998/07/12 02:33:52 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: helpmenu.c,v $ $Date: 2000/07/24 16:16:10 $ $Revision: 1.4 $ $Name:  $"};
