@@ -26,7 +26,6 @@ extern int close();
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
-#include "preferences.h"
 
 /* look around for O_WRONLY. Older VMS had it in xlib.h; some VMS has it in
  * fcntl.h (but not all have this file); some now have it in file.h. geeze.
@@ -46,8 +45,7 @@ static int open_wfifores P_((char *fn, FILE **fpp));
 
 /* if path starts with `~' replace with $HOME.
  * we also remove any leading or trailing blanks.
- * N.B. string returned will always be a copy of path but caller should
- *   never free it.
+ * caller should save returned string before calling us again.
  */
 char *
 expand_home (path)
@@ -149,36 +147,6 @@ getShareDir()
 
 }
 
-/* add ., $HOME, $HOME/XEphem and $TELHOME/archive/config to X resource search
- * path ($HOME/XEphem because that is the default PrivateDir).
- */
-void
-addXPath()
-{
-	static char xuenv[] = "XUSERFILESEARCHPATH";
-	char *th = getenv ("TELHOME");
-	char *ho = getenv ("HOME");
-	char *xu = getenv (xuenv);
-	char *newenv;
-
-	if (!th && !ho)
-	    return;	/* oh well */
-
-	if (!ho)
-	    ho = ".";
-	if (!th)
-	    th = "/usr/local/telescope";
-	if (!xu)
-	    xu = "";
-	newenv = malloc (strlen(xuenv)+2*strlen(ho)+strlen(th)+strlen(xu)+50);
-	if (newenv) {
-	    (void) sprintf (newenv,
-		    "%s=./%%N:%s/%%N:%s/XEphem/%%N:%s/archive/config/%%N:%s",
-							xuenv, ho, ho, th, xu);
-	    putenv (newenv);
-	}
-}
-
 /* return a string for whatever is in errno right now.
  * I never would have imagined it would be so crazy to turn errno into a string!
  */
@@ -269,7 +237,7 @@ FILE **fpp;
 	/* open for write-only, non-blocking in case their's no reader.
 	 * this prevents clogged writes too.
 	 */
-	fd = openh (fn, O_WRONLY|NODELAY);
+	fd = openh (fn, O_WRONLY|NODELAY, 0);
 	if (fd < 0) {
 	    (void) sprintf (msg,"%s: %s", fn, syserrstr());
 	    xe_msg (msg, 1);
@@ -291,4 +259,4 @@ FILE **fpp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: homeio.c,v $ $Date: 1999/03/03 19:27:28 $ $Revision: 1.4 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: homeio.c,v $ $Date: 2000/06/20 05:49:09 $ $Revision: 1.8 $ $Name:  $"};
