@@ -7,6 +7,7 @@
 #if defined(__STDC__)
 #include <stdlib.h>
 #endif
+
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -36,34 +37,43 @@ extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
+extern XFontStruct * getXResFont P_((char *rn));
 extern char *getShareDir P_((void));
+extern char *syserrstr P_((void));
 extern double atod P_((char *buf));
 extern double delra P_((double dra));
 extern int any_ison P_((void));
 extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
     double mag, ObjF **opp));
 extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
+extern int gif2X P_((Display *dsp, Colormap cm, unsigned char gif[], int ngif,
+    int *wp, int *hp, unsigned char **gifpix, XColor xcols[256], char err[]));
+extern int isUp P_((Widget shell));
 extern int magdiam P_((int fmag, int magstp, double scale, double mag,
     double size));
 extern int read_bdl P_((FILE *fp, double jd, double *xp, double *yp,
     double *zp, char ynot[]));
 extern int skyloc_fifo P_((Obj *op));
+extern void buttonAsButton P_((Widget w, int whether));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void f_prdec P_((Widget w, double a));
 extern void f_double P_((Widget w, char *fmt, double f));
 extern void f_ra P_((Widget w, double ra));
+extern void freeXColors P_((Display *dsp, Colormap cm, XColor xcols[],
+    int nxcols));
 extern void fs_manage P_((void));
 extern void fs_prdec P_((char out[], double a));
 extern void fs_ra P_((char out[], double ra));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
+extern void jm_update P_((Now *np, int how_much));
 extern void mm_movie P_((double stepsz));
 extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
 extern void register_selection P_((char *name));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void jm_update P_((Now *np, int how_much));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
 extern void timestamp P_((Now *np, Widget w));
@@ -96,11 +106,11 @@ typedef enum {
 static ScreenObj *screenobj[SO_N];
 static int nscreenobj[SO_N];
 
-static void jm_create_form_w P_((void));
+static void jm_create_shell_w P_((void));
 static void jm_create_tvform_w P_((void));
 static void jm_create_jsform_w P_((void));
 static void jm_set_buttons P_((int whether));
-static void jm_set_a_button P_((Widget pbw, int whether));
+static void jm_sstats_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_sstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_scale_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -109,7 +119,7 @@ static void jt_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jt_track_size P_((void));
 static void jt_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
-static void jm_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void jm_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_movie_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -122,8 +132,6 @@ static void jm_locfifo_cb P_((Widget w, XtPointer client, XtPointer call));
 static void jm_grsl_cb P_((Widget w, XtPointer client, XtPointer call));
 static void j_calibline P_((Display *dsp, Drawable win, GC gc, int xc, int yc,
     char *tag, int tw, int th, int l));
-static void jgrs_draw P_((Obj *jop, Drawable dr, int rad, int jx, int jy, int
-    fliptb, int fliplr, double grsl, double cmlII));
 
 static void add_screenobj P_((SOIdx, Obj *op, int x, int y));
 static void reset_screenobj P_((SOIdx));
@@ -132,7 +140,7 @@ static ScreenObj *close_screenobj P_((SOIdx, int x, int y));
 static void sky_background P_((Drawable win, unsigned w, unsigned h, int fmag,
     double ra0, double dec0, double scale, double rad, int fliptb, int fliplr));
 
-static Widget jupform_w;	/* main form */
+static Widget jupshell_w;	/* main shell */
 static Widget jsform_w;		/* statistics form */
 static Widget jmframe_w;	/* main frame */
 static Widget jtform_w;		/* top-view form */
@@ -150,7 +158,6 @@ static Widget grsl_w;		/* TF of GRS Sys II longitude, degrees */
 static Pixmap jm_pm;		/* main pixmap */
 static Pixmap jt_pm;		/* top-view pixmap */
 static GC j_fgc, j_bgc, j_xgc;	/* various colors and operators */
-static GC jgrs_gc;      	/* gc to use for spot */
 static XFontStruct *j_fs;	/* font for labels */
 static int j_cw, j_ch;		/* size of label font */
 static int jm_selecting;	/* set while our fields are being selected */
@@ -160,7 +167,6 @@ static int flip_tb;		/* whether we want to flip top/bottom */
 static int flip_lr;		/* whether we want to flip left/right */
 static int skybkg;		/* whether we want sky background */
 static int topview;		/* whether we want the top view */
-static int jswasman;		/* whether statistics form was managed */
 
 static Widget jmpu_w;		/* main popup */
 static Widget jmpu_name_w;	/* popup name label */
@@ -197,28 +203,33 @@ static struct MoonNames {
 enum {CEV, CSV, CX, CY, CZ, CRA, CDEC, CMAG, CNum};	/* j_w column index */
 static Widget	j_w[NM][CNum];			/* the data display widgets */
 
+static char jupcategory[] = "Jupiter";		/* Save category */
+
+/* info about the gif we use to morph an image */
+static char jmapbasenm[] = "jupitermap.gif";	/* base file name */
+static XColor jmapxcols[256];			/* pixels and colors */
+static unsigned char *jmappix;			/* malloced wxh of gif pixels */
+#define	JMAPGRSX	125			/* gif x of GRS */
+#define	JMAPW		512			/* gif width */
+#define	JMAPH		256			/* gif height */
+static Pixel jbg_p;				/* background color */
+
 /* called when the jupiter menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, just get out there and do it!
  */
 void
 jm_manage ()
 {
-	if (!jupform_w) {
-	    jm_create_form_w();
+	if (!jupshell_w) {
+	    jm_create_shell_w();
 	    jm_create_jsform_w();
+	    jm_create_tvform_w();
 	}
 	
-	if (XtIsManaged(jupform_w))
-	    XtUnmanageChild (jupform_w);
-	else {
-	    XtManageChild (jupform_w);
-	    if (jswasman) {
-		XtManageChild (jsform_w);
-		jm_set_buttons(jm_selecting);
-	    }
-	    /* rely on expose to do the first draw */
-	}
+	XtPopup (jupshell_w, XtGrabNone);
+	set_something (jupshell_w, XmNiconic, (XtArgVal)False);
+	jm_set_buttons(jm_selecting);
 }
 
 /* called when the database has changed.
@@ -236,7 +247,7 @@ int appended;
 int
 jm_ison()
 {
-	return (jupform_w && XtIsManaged(jupform_w));
+	return (isUp(jupshell_w));
 }
 
 /* called by other menus as they want to hear from our buttons or not.
@@ -247,9 +258,12 @@ void
 jm_selection_mode (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	jm_selecting += whether ? 1 : -1;
+	if (whether)
+	    jm_selecting++;
+	else if (jm_selecting > 0)
+	    --jm_selecting;
 
-	if (jm_ison())
+	if (jupshell_w)
 	    if ((whether && jm_selecting == 1)     /* first one to want on */
 		|| (!whether && jm_selecting == 0) /* last one to want off */)
 		jm_set_buttons (whether);
@@ -262,8 +276,8 @@ Cursor c;
 {
 	Window win;
 
-	if (jupform_w && (win = XtWindow(jupform_w)) != 0) {
-	    Display *dsp = XtDisplay(jupform_w);
+	if (jupshell_w && (win = XtWindow(jupshell_w)) != 0) {
+	    Display *dsp = XtDisplay(jupshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -277,21 +291,29 @@ Cursor c;
 	    else
 		XUndefineCursor (dsp, win);
 	}
+
+	if (jtform_w && (win = XtWindow(jtform_w)) != 0) {
+	    Display *dsp = XtDisplay(jtform_w);
+	    if (c)
+		XDefineCursor (dsp, win, c);
+	    else
+		XUndefineCursor (dsp, win);
+	}
 }
 
-/* create the main form dialog */
+/* create the main shell */
 static void
-jm_create_form_w()
+jm_create_shell_w()
 {
 	typedef struct {
-	    char *name;		/* toggle button instance name */
+	    char *name;		/* toggle button instance name, or NULL */
 	    char *label;	/* label */
 	    int *flagp;		/* pointer to global flag, or NULL if none */
 	    Widget *wp;		/* widget to save, or NULL */
 	    char *tip;		/* widget tip */
 	} Option;
 	static Option options[] = {
-	    {"TopView",	   "Top view",		&topview, &topview_w,
+	    {NULL,	   "Top view",		&topview, &topview_w,
 	    	"When on, show view looking from high above N pole"},
 	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w,
 	    	"When on, sky will include database objects and Field Stars"},
@@ -316,28 +338,31 @@ jm_create_form_w()
 	};
 	Widget w;
 	Widget mb_w, pd_w, cb_w;
+	Widget jupform_w;
 	XmString str;
 	Arg args[20];
 	int n;
 	int i;
 
-	/* create form */
-	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	jupform_w = XmCreateFormDialog (toplevel_w, "Jupiter", args, n);
-	set_something (jupform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (jupform_w, XmNhelpCallback, jm_help_cb, 0);
-	XtAddCallback (jupform_w, XmNunmapCallback, jm_unmap_cb, 0);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
+	/* create shell and form */
 	n = 0;
 	XtSetArg (args[n], XmNtitle, "xephem Jupiter view"); n++;
-	XtSetValues (XtParent(jupform_w), args, n);
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNiconName, "Jupiter"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	jupshell_w = XtCreatePopupShell ("Jupiter", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (jupshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (jupshell_w, XmNpopdownCallback, jm_popdown_cb, 0);
+	sr_reg (jupshell_w, "XEphem*Jupiter.width", jupcategory, 0);
+	sr_reg (jupshell_w, "XEphem*Jupiter.height", jupcategory, 0);
+	sr_reg (jupshell_w, "XEphem*Jupiter.x", jupcategory, 0);
+	sr_reg (jupshell_w, "XEphem*Jupiter.y", jupcategory, 0);
+
+	n = 0;
+	jupform_w = XmCreateForm (jupshell_w, "JupiterF", args, n);
+	XtAddCallback (jupform_w, XmNhelpCallback, jm_help_cb, 0);
+	XtManageChild (jupform_w);
 
 	/* create the menu bar across the top */
 
@@ -364,7 +389,7 @@ jm_create_form_w()
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "FS", args, n);
-	    set_xmstring (w, XmNlabelString, "Setup Field Stars ...");
+	    set_xmstring (w, XmNlabelString, "Field Stars ...");
 	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
 	    wtip (w, "Define where GSC and PPM catalogs are to be found");
 	    XtManageChild (w);
@@ -418,7 +443,8 @@ jm_create_form_w()
 		XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 		XtSetArg (args[n], XmNmarginHeight, 0); n++;
 		XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
-		w = XmCreateToggleButton (pd_w, op->name, args, n);
+		w = XmCreateToggleButton (pd_w, op->name ? op->name : "JMTB",
+								    args, n);
 		XtAddCallback (w, XmNvalueChangedCallback, jm_option_cb, 
 						    (XtPointer)(op->flagp));
 		set_xmstring (w, XmNlabelString, op->label);
@@ -429,6 +455,8 @@ jm_create_form_w()
 		if (op->tip)
 		    wtip (w, op->tip);
 		XtManageChild (w);
+		if (op->name)
+		    sr_reg (w, NULL, jupcategory, 1);
 	    }
 
 	    /* add a separator */
@@ -504,6 +532,7 @@ jm_create_form_w()
 	XtAddCallback (scale_w, XmNvalueChangedCallback, jm_scale_cb, 0);
 	wtip (scale_w, "Zoom in and out");
 	XtManageChild (scale_w);
+	sr_reg (scale_w, NULL, jupcategory, 0);
 
 	/* make the limiting mag scale widget.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -526,6 +555,7 @@ jm_create_form_w()
 	XtAddCallback (limmag_w, XmNvalueChangedCallback, jm_scale_cb, 0);
 	wtip (limmag_w, "Adjust the brightness and limiting magnitude");
 	XtManageChild (limmag_w);
+	sr_reg (limmag_w, NULL, jupcategory, 0);
 
 	/* make a frame for the drawing area.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -595,7 +625,7 @@ jm_create_jsform_w()
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	jsform_w = XmCreateFormDialog (toplevel_w, "JupiterStats", args, n);
+	jsform_w = XmCreateFormDialog (jupshell_w, "JupiterStats", args, n);
 	set_something (jsform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -686,6 +716,7 @@ jm_create_jsform_w()
 	grsl_w = XmCreateTextField (jsform_w, "GRSSysIILong", args, n);
 	XtAddCallback (grsl_w, XmNactivateCallback, jm_grsl_cb, NULL);
 	XtManageChild (grsl_w);
+	sr_reg (grsl_w, NULL, jupcategory, 1);
 
 	/* make table title label */
 
@@ -724,7 +755,7 @@ jm_create_jsform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "JupTag", args, n); /*PB for sz */
 		set_xmstring (w, XmNlabelString, tag ? tag : " ");
-		jm_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -750,7 +781,7 @@ jm_create_jsform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "JupName", args, n); /*PB for sz*/
 		set_xmstring (w, XmNlabelString, mnames[i].full);
-		jm_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -835,7 +866,7 @@ jm_create_jsform_w()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 70); n++;
 	w = XmCreatePushButton (jsform_w, "Close", args, n);
-	XtAddCallback (w, XmNactivateCallback, jm_sstats_cb, 0);
+	XtAddCallback (w, XmNactivateCallback, jm_sstats_close_cb, 0);
 	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
@@ -852,7 +883,7 @@ jm_create_tvform_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	jtform_w = XmCreateFormDialog (toplevel_w, "JupiterTV", args, n);
+	jtform_w = XmCreateFormDialog (jupshell_w, "JupiterTV", args, n);
 	set_something (jtform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (jtform_w, XmNunmapCallback, jt_unmap_cb, 0);
 	XtAddCallback (jtform_w, XmNmapCallback, jt_map_cb, NULL);
@@ -893,82 +924,10 @@ int whether;	/* whether setting up for plotting or for not plotting */
 
 	for (i = 0; i < NM; i++) {
 	    for (j = 0; j < CNum; j++)
-		jm_set_a_button (j_w[i][j], whether);
+		buttonAsButton (j_w[i][j], whether);
 	}
-	jm_set_a_button (cmlI_w, whether);
-	jm_set_a_button (cmlII_w, whether);
-}
-
-/* set whether the given button looks like a label.
- */
-static void
-jm_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
-{
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    Widget tmpw;
-	    int n;
-
-	    n = 0;
-	    tmpw = XmCreatePushButton (jupform_w, "tmp", args, n);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-	    XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    XtDestroyWidget (tmpw);
-	     
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
+	buttonAsButton (cmlI_w, whether);
+	buttonAsButton (cmlII_w, whether);
 }
 
 /* callback when the ENTER key is typed on the GRS Long TF */
@@ -982,9 +941,18 @@ XtPointer call;
 	jm_update (mm_get_now(), 1);
 }
 
-/* callback when the Close button is activated on the stats menu or when
- * the More Info button is activated. they each do the same thing.
- */
+/* callback when the Close button is activated on the stats menu */
+/* ARGSUSED */
+static void
+jm_sstats_close_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XtUnmanageChild (jsform_w);
+}
+
+/* callback when the More Info button is activated */
 /* ARGSUSED */
 static void
 jm_sstats_cb (w, client, call)
@@ -992,12 +960,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((jswasman = XtIsManaged(jsform_w)) != 0)
-	    XtUnmanageChild (jsform_w);
-	else {
-	    XtManageChild (jsform_w);
-	    jm_set_buttons(jm_selecting);
-	}
+	XtManageChild (jsform_w);
+	jm_set_buttons(jm_selecting);
 }
 
 /* callback from any of the option buttons.
@@ -1103,7 +1067,7 @@ XtPointer call;
 	}
 
         XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!jt_pm || nx != last_nx || ny != last_ny) {
+	if (!jt_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (jt_pm)
 		XFreePixmap (dsp, jt_pm);
 	    jt_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1155,7 +1119,7 @@ jt_track_size()
 	n = 0;
 	XtSetArg (args[n], XmNx, &mfx); n++;
 	XtSetArg (args[n], XmNy, &mfy); n++;
-	XtGetValues (jupform_w, args, n);
+	XtGetValues (jupshell_w, args, n);
 	n = 0;
 	XtSetArg (args[n], XmNx, &mdx); n++;
 	XtSetArg (args[n], XmNy, &mdy); n++;
@@ -1181,24 +1145,29 @@ XtPointer call;
 	XmToggleButtonSetState (topview_w, False, True);
 }
 
-/* callback when main dialog is unmapped */
+/* callback when main shell is popped down */
 /* ARGSUSED */
 static void
-jm_unmap_cb (w, client, call)
+jm_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((jswasman = (jsform_w && XtIsManaged(jsform_w))) != 0)
-	    XtUnmanageChild (jsform_w);
+	XtUnmanageChild (jsform_w);
+	XtUnmanageChild (jtform_w);
+
+	/* free pixmap */
         if (jm_pm) {
 	    XFreePixmap (XtDisplay(jda_w), jm_pm);
 	    jm_pm = (Pixmap) NULL;
 	}
 
-	/* close top view */
-	if (jtform_w && XtIsManaged(jtform_w))
-	    XtUnmanageChild (jtform_w);
+	/* free image info */
+	if (jmappix) {
+	    free ((char *)jmappix);
+	    jmappix = 0;
+	    freeXColors (XtD, xe_cm, jmapxcols, 256);
+	}
 
 	/* free any field stars */
 	if (fstars) {
@@ -1219,8 +1188,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let unmap do all the work */
-	XtUnmanageChild (jupform_w);
+	/* let popdown do all the work */
+	XtPopdown (jupshell_w);
 }
 
 /* callback from the Movie button
@@ -1286,7 +1255,7 @@ XtPointer call;
 	}
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!jm_pm || nx != last_nx || ny != last_ny) {
+	if (!jm_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (jm_pm)
 		XFreePixmap (dsp, jm_pm);
 	    jm_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1592,10 +1561,10 @@ int how_much;
 	int i;
 
 	/* see if we should bother */
-	if (!jupform_w)
+	if (!jupshell_w)
 	    return;
 	wantstats = XtIsManaged(jsform_w) || any_ison() || how_much;
-	if (!XtIsManaged(jupform_w) && !wantstats)
+	if (!isUp(jupshell_w) && !wantstats)
 	    return;
 
 	watch_cursor (1);
@@ -1633,7 +1602,7 @@ int how_much;
 	    timestamp (np, sdt_w);
 	}
 
-	if (XtIsManaged(jupform_w)) {
+	if (isUp(jupshell_w)) {
 	    jm_draw_map (jda_w, jop, jupsize, cmlII, md);
 	    timestamp (np, dt_w);
 	}
@@ -1641,6 +1610,255 @@ int how_much;
 	watch_cursor (0);
 }
 
+/* called when basic resources change.
+ * rebuild and redraw.
+ */
+void
+jm_newres()
+{
+	if (!jupshell_w)
+	    return;
+	make_gcs (jda_w);
+	jm_update (mm_get_now(), 1);
+}
+
+/* read the jupiter map gif.
+ * harmless if called again.
+ * return 0 if ok else -1.
+ */
+static int
+jm_getgif(dsp)
+Display *dsp;
+{
+	unsigned char *gif;
+	int ngif;
+	char fn[256];
+	char buf[256];
+	int w, h;
+	FILE *fp;
+
+	/* benign if called again */
+	if (jmappix)
+	    return (0);
+
+	/* read in the gif */
+	sprintf (fn, "%s/auxil/%s",  getShareDir(), jmapbasenm);
+	fp = fopenh (fn, "r");
+	if (!fp) {
+	    sprintf (buf, "%s:\n%s", fn, syserrstr());
+	    xe_msg (buf, 1);
+	    return (-1);
+	}
+	if (fseek (fp, 0L, SEEK_END) < 0) {
+	    sprintf (buf, "%s:\nCan not seek", jmapbasenm);
+	    xe_msg (buf, 1);
+	    fclose (fp);
+	    return (-1);
+	}
+	ngif = ftell (fp);
+	gif = XtMalloc (ngif);
+	rewind (fp);
+	if (fread (gif, ngif, 1, fp) != 1) {
+	    sprintf (buf, "%s:\n%s", jmapbasenm, syserrstr());
+	    xe_msg (buf, 1);
+	    fclose (fp);
+	    return (-1);
+	}
+	fclose (fp);
+
+	/* explode */
+	if (gif2X (dsp, xe_cm, gif, ngif, &w, &h, &jmappix,
+						    jmapxcols, buf) < 0) {
+	    sprintf (fn, "%s:\n%s", jmapbasenm, buf);
+	    xe_msg (fn, 1);
+	    return (-1);
+	}
+
+	if (w != JMAPW || h != JMAPH) {
+	    sprintf (buf, "%s:\nincorrect file size", jmapbasenm);
+	    xe_msg (buf, 1);
+	    return (-1);
+	}
+
+	return (0);
+}
+
+/* fill win with front image of Jupiter at the given location, size and
+ * rotation.
+ * factoids: default orientation is E on right, S on top. So, GRS moves from
+ *   right to left. Spot is in S hemisphere. CML increases with time.
+ */
+static void
+jm_fimage (dsp, win, lx, ty, sz, grs)
+Display *dsp;		/* Display */
+Drawable win;		/* drawable */
+int lx, ty, sz;		/* left x, top y, size */
+double grs;		/* angle from center to grs, rads */
+{
+	Obj *jop = db_basic (JUPITER);
+	XImage *xip;
+	int depth;
+	int bpp;
+	int nbytes;
+	char *data;
+	int x0, szr, szr2, x, y;
+	double tvc, pvc, theta, phi, sa, ca;
+
+	/* get the gif */
+	if (jm_getgif(dsp) < 0)
+	    return;
+
+	/* create a working image */
+	get_something (jda_w, XmNdepth, (XtArgVal)&depth);
+	bpp = depth>=17 ? 32 : (depth >= 9 ? 16 : 8);
+	nbytes = sz*sz*bpp/8;
+	data = XtMalloc (nbytes);
+	xip = XCreateImage (dsp, DefaultVisual (dsp, DefaultScreen(dsp)),
+	    /* depth */         depth,
+	    /* format */        ZPixmap,
+	    /* offset */        0,
+	    /* data */          data,
+	    /* width */         sz,
+	    /* height */        sz,
+	    /* pad */           bpp,
+	    /* bpl */           0);
+	xip->bitmap_bit_order = LSBFirst;
+	xip->byte_order = LSBFirst;
+
+	/* sky-plane rotation */
+	tvc = PI/2.0 - jop->s_dec;
+	pvc = jop->s_ra;
+	theta = PI/2.0 - POLE_DEC;
+	phi = POLE_RA;
+	sa = sin(tvc)*sin(theta)*(cos(pvc)*sin(phi) - sin(pvc)*cos(phi));
+	ca = sqrt (1.0 - sa*sa);
+
+	/* morph. include effects of sky plane rotation,
+	 * x,y limb foreshortening and y latitude stretch.
+	 * TODO: limb darkening
+	 */
+	x0 = JMAPGRSX + (int)(grs/(2*PI)*JMAPW) + JMAPW;
+	szr = sz/2;
+	szr2 = szr*szr;
+	for (y = 0; y < sz; y++) {
+	    int imy = flip_tb ? y - szr : szr - y;
+	    for (x = 0; x < sz; x++) {
+		int imx = flip_lr ? szr - x : x - szr;
+		double imxp =  imx*ca + imy*sa;
+		double imyp = -imx*sa + imy*ca;
+		double imy2 = imyp*imyp;
+		double imr2 = imxp*imxp + imy2;
+		unsigned long p;
+
+		if (imr2 >= szr2)
+		    p = jbg_p;
+		else {
+		    double xmult = JMAPW*szr/((2*PI)*sqrt((double)szr2 - imy2));
+		    int xg = (x0 - (int)(xmult*asin((double)imx/szr)))%JMAPW;
+		    int yg = (3*JMAPH/2 +
+				(int)(JMAPH*asin((double)imyp/szr)/PI))%JMAPH;
+		    p = jmapxcols[(int)jmappix[yg*JMAPW+xg]].pixel;
+		}
+		XPutPixel (xip, x, y, p);
+	    }
+	}
+
+	/* copy to win */
+	XPutImage (dsp, win, j_fgc, xip, 0, 0, lx, ty, sz, sz);
+
+	/* done */
+	free ((void *)xip->data);
+	xip->data = NULL;
+	XDestroyImage (xip);
+}
+
+/* fill win with top image of Jupiter at the given location, size and
+ * rotation.
+ */
+static void
+jm_timage (dsp, win, lx, ty, sz, grs)
+Display *dsp;		/* Display */
+Drawable win;		/* drawable */
+int lx, ty, sz;		/* left x, top y, size */
+double grs;		/* angle from center to grs, rads */
+{
+	Obj *jop = db_basic (JUPITER);
+	XImage *xip;
+	int depth;
+	int bpp;
+	int nbytes;
+	char *data;
+	int x0, szr, szr2, x, y;
+	double xmult, ymult;
+	double tvc, pvc, theta, phi, sa, ca;
+
+	/* get the gif */
+	if (jm_getgif(dsp) < 0)
+	    return;
+
+	/* create a working image */
+	get_something (jda_w, XmNdepth, (XtArgVal)&depth);
+	bpp = depth>=17 ? 32 : (depth >= 9 ? 16 : 8);
+	nbytes = sz*sz*bpp/8;
+	data = XtMalloc (nbytes);
+	xip = XCreateImage (dsp, DefaultVisual (dsp, DefaultScreen(dsp)),
+	    /* depth */         depth,
+	    /* format */        ZPixmap,
+	    /* offset */        0,
+	    /* data */          data,
+	    /* width */         sz,
+	    /* height */        sz,
+	    /* pad */           bpp,
+	    /* bpl */           0);
+	xip->bitmap_bit_order = LSBFirst;
+	xip->byte_order = LSBFirst;
+
+	/* sky-plane rotation */
+	tvc = PI/2.0 - jop->s_dec;
+	pvc = jop->s_ra;
+	theta = PI/2.0 - POLE_DEC;
+	phi = POLE_RA;
+	sa = sin(tvc)*sin(theta)*(cos(pvc)*sin(phi) - sin(pvc)*cos(phi));
+	ca = sqrt (1.0 - sa*sa);
+
+	/* morph.
+	 * TODO: limb darkening, tilt
+	 */
+	x0 = JMAPGRSX + (int)(grs/(2*PI)*JMAPW) + JMAPW;
+	szr = sz/2;
+	szr2 = sz*sz/4;
+	xmult = JMAPW/(2*PI);
+	ymult = (double)JMAPH/sz;
+	for (y = 0; y < sz; y++) {
+	    double imy = szr - y;
+	    double imy2 = imy*imy;
+	    for (x = 0; x < sz; x++) {
+		double imx = flip_lr ? szr - x : x - szr;
+		double imr2 = imx*imx + imy2;
+		unsigned long p;
+
+		if (imr2 > szr2)
+		    p = jbg_p;
+		else {
+		    int xg = (x0 - (int)(xmult*atan2(imx,-imy)))%JMAPW;
+		    int yg = (int)(ymult*sqrt(imr2));
+		    if (!flip_tb)
+			yg = JMAPH - yg;
+		    p = jmapxcols[(int)jmappix[yg*JMAPW+xg]].pixel;
+		}
+		XPutPixel (xip, x, y, p);
+	    }
+	}
+
+	/* copy to win */
+	XPutImage (dsp, win, j_fgc, xip, 0, 0, lx, ty, sz, sz);
+
+	/* done */
+	free ((void *)xip->data);
+	xip->data = NULL;
+	XDestroyImage (xip);
+}
+
 /* given the loc of the moons, draw a nifty little picture.
  * also save resulting screen locs of everything in the screenobj array.
  * scale of the locations is in terms of jupiter radii == 1.
@@ -1671,8 +1889,12 @@ MoonData md[NM];
 	int sv;
 	int fmag;
 	char *grslstr;
+	double grs;
 	char c;
 	int x, y;
+	XPoint xp[5];
+	char buf[64];
+	int l;
 	unsigned int nx, ny, bw, d;
 	int ns = flip_tb ? -1 : 1;
 	int ew = flip_lr ? -1 : 1;
@@ -1709,19 +1931,14 @@ MoonData md[NM];
 	XDrawString(dsp, jm_pm, j_fgc, (nx-j_cw)/2-1, j_fs->ascent, &c, 1);
 
 	/* draw Jupiter in center with unit radius */
-	XPSFillArc(dsp, jm_pm, j_fgc, nx/2-scale1, ny/2-scale1, 2*scale1-1,
-						    2*scale1-1, 0, 360*64);
+	grslstr = XmTextFieldGetString (grsl_w);
+	grs = degrad(atod(grslstr))-cmlII;
+	XtFree (grslstr);
+	jm_fimage (dsp, jm_pm, nx/2-scale1, ny/2-scale1, 2*scale1-1, grs);
 	add_screenobj (SO_MAIN, jop, nx/2, ny/2);
 
-	/* draw the GRS */
-	grslstr = XmTextFieldGetString (grsl_w);
-	jgrs_draw (jop, jm_pm, scale1, nx/2, ny/2, flip_tb, flip_lr,
-						degrad(atod (grslstr)), cmlII);
-	XtFree (grslstr);
-
 	/* draw 1' calibration line */
-	if (tags)
-	    j_calibline (dsp, jm_pm, j_fgc, nx/2, 5*ny/6, "1'", j_cw*2, j_ch+4,
+	j_calibline (dsp, jm_pm, j_fgc, nx/2, 5*ny/6, "1'", j_cw*2, j_ch+4,
 				(int)MAPSCALE(degrad(1.0/60.0)/(jupsize/2)));
 
 	/* draw each moon
@@ -1746,14 +1963,14 @@ MoonData md[NM];
 	    if (diam == 1)
 		XDrawPoint(dsp, jm_pm, j_xgc, x, y);
 	    else
-		XPSFillArc (dsp, jm_pm, j_xgc, x-diam/2, y-diam/2, diam, diam,
+		XFillArc (dsp, jm_pm, j_xgc, x-diam/2, y-diam/2, diam, diam,
 								    0, 360*64);
 
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_MAIN, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1773,10 +1990,11 @@ MoonData md[NM];
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
 	XFillRectangle (dsp, jt_pm, j_bgc, 0, 0, nx, ny);
 
-	/* draw jupiter in center with unit radius */
-	i = ew*64*raddeg(asin(sin(degrad(jop->s_elong))/jop->s_sdist));
-	XPSFillArc(dsp, jt_pm, j_fgc, nx/2-scale1, ny/2-scale1, 2*scale1-1,
-						    2*scale1-1, -i, -180*64);
+	/* draw jupiter in center with unit radius, cover dark side */
+	jm_timage (dsp, jt_pm, nx/2-scale1, ny/2-scale1, 2*scale1-1, grs);
+	i = (int)(-ew*64*raddeg(asin(sin(degrad(jop->s_elong))/jop->s_sdist)));
+	XFillArc(dsp, jt_pm, j_bgc, nx/2-scale1-1, ny/2-scale1-1, 2*scale1+1,
+						    2*scale1+1, i, 180*64);
 	reset_screenobj(SO_TOP);
 	add_screenobj (SO_TOP, jop, nx/2, ny/2);
 
@@ -1803,14 +2021,14 @@ MoonData md[NM];
 	    if (diam == 1)
 		XDrawPoint (dsp, jt_pm, j_xgc, x, y);
 	    else
-		XPSFillArc (dsp, jt_pm, j_xgc, x-diam/2, y-diam/2, diam, diam,
+		XFillArc (dsp, jt_pm, j_xgc, x-diam/2, y-diam/2, diam, diam,
 								    0, 360*64);
 
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_TOP, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1818,34 +2036,37 @@ MoonData md[NM];
 					mnames[i].tag, strlen(mnames[i].tag));
 	}
 
-	/* draw label towards earth */
-	if (tags) {
-	    XPoint xp[5];
+	/* draw labels */
 
-	    XDrawString(dsp, jt_pm, j_fgc, nx/2, ny-j_ch, "Earth", 5);
-	    xp[0].x = nx/2 - 15; xp[0].y = ny - 2*j_ch;
-	    xp[1].x = 0;         xp[1].y = 3*j_ch/2;
-	    xp[2].x = -5;        xp[2].y = -3*j_ch/4;
-	    xp[3].x = 10;        xp[3].y = 0;
-	    xp[4].x = -5;        xp[4].y = 3*j_ch/4;
-	    XDrawLines (dsp, jt_pm, j_fgc, xp, 5, CoordModePrevious);
-	}
+	l = sprintf (buf, "%s Pole", flip_tb ? "North" : "South");
+	XDrawString(dsp, jt_pm, j_fgc, (nx-j_cw*l)/2, j_ch, buf, l);
+	XDrawString(dsp, jt_pm, j_fgc, nx/2, ny-j_ch, "to Earth", 8);
+	xp[0].x = nx/2 - 15; xp[0].y = ny - 2*j_ch;
+	xp[1].x = 0;         xp[1].y = 3*j_ch/2;
+	xp[2].x = -3;        xp[2].y = -3*j_ch/4;
+	xp[3].x = 6;         xp[3].y = 0;
+	xp[4].x = -3;        xp[4].y = 3*j_ch/4;
+	XDrawLines (dsp, jt_pm, j_fgc, xp, 5, CoordModePrevious);
 
 	XCopyArea (dsp, jt_pm, win, j_fgc, 0, 0, nx, ny, 0, 0);
 }
 
+/* make the colors and GCs.
+ * TODO: reclaim old stuff if called again.
+ */
 static void
 make_gcs (w)
 Widget w;
 {
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
+	XFontStruct *fsp;
 	XGCValues gcv;
 	unsigned int gcm;
-	Pixel fg, bg;
+	Pixel fg;
 
 	gcm = GCForeground;
-	get_something (w, XmNforeground, (XtArgVal)&fg);
+	get_color_resource (w, "jupiterColor", &fg);
 	gcv.foreground = fg;
 	j_fgc = XCreateGC (dsp, win, gcm, &gcv);
 	j_fs = XQueryFont (dsp, XGContextFromGC (j_fgc));
@@ -1853,21 +2074,16 @@ Widget w;
 	j_ch = j_fs->max_bounds.ascent + j_fs->max_bounds.descent;
 
 	gcm = GCForeground;
-	get_something (w, XmNbackground, (XtArgVal)&bg);
-	gcv.foreground = bg;
+	get_color_resource (w, "JupiterBackground", &jbg_p);
+	gcv.foreground = jbg_p;
 	j_bgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	gcm = GCForeground | GCFunction;
-	gcv.foreground = fg ^ bg;
+	gcm = GCForeground | GCFunction | GCFont;
+	fsp = getXResFont ("moonsFont");
+	gcv.foreground = fg ^ jbg_p;
 	gcv.function = GXxor;
+	gcv.font = fsp->fid;
 	j_xgc = XCreateGC (dsp, win, gcm, &gcv);
-
-	if (get_color_resource (toplevel_w, "JupiterGRSColor", &fg) < 0) {
-	    xe_msg ("Actually .. using Black", 0);
-	    fg = BlackPixel(XtD, DefaultScreen(XtD));
-	}
-	jgrs_gc = XCreateGC (XtD, XtWindow(toplevel_w), 0L, (XGCValues *)0);
-	XSetForeground (XtD, jgrs_gc, fg);
 }
 
 /* draw a calibration line l pixels long centered at [xc,yc] marked with tag
@@ -1947,8 +2163,8 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    ddec = dec0 - op->s_dec;	/* + S */
 	    if (fabs(ddec) > rad || delra(dra)*cdec0 > rad)
 		continue;
-	    dx = dra/scale;
-	    dy = ddec/scale;
+	    dx = (int)(dra/scale);
+	    dy = (int)(ddec/scale);
 	    x = fliplr ? (int)w/2-dx : (int)w/2+dx;
 	    y = fliptb ? (int)h/2+dy : (int)h/2-dy;
 
@@ -2015,60 +2231,6 @@ MoonData md[NM];	/* fill in RA and Dec */
 	}
 }
 
-/* draw the great red spot.
- * factoids: default orientation is E on right, S on top. So, GRS moves from
- *   right to left. Spot is in S hemisphere. CML increases with time.
- */
-static void
-jgrs_draw (jop, dr, rad, jx, jy, fliptb, fliplr, jgrs_lng, cmlII)
-Obj *jop;	/* jupiter info */
-Drawable dr;	/* drawable to use */
-int rad;	/* radius of jupiter, pixels */
-int jx, jy;	/* center of jupiter circle, dr coords */
-int fliptb;	/* whether image is flipped tb; default is S on top */
-int fliplr;	/* whether image is flipped lr; default is E on right */
-double jgrs_lng;/* longitude of GRS in Sys II coords, rads */
-double cmlII;	/* current central merdian longitude, system II, rads */
-{
-	int dx, dy;	/* GRS center, relative to jx,jy, without flipping */
-	int x, y;	/* GRS center, relative to dr, after flipping */
-	int r;		/* radius of GRS, pixels */
-	int h, w;	/* h is always r, but we flatten w at limbs */
-	double a;	/* angle from center to GRS going towards right (+E) */
-	double tvc, pvc, theta, phi;	/* for sky-plane rotation */
-	double dxp, dyp;
-	double ca, sa;
-
-	a = jgrs_lng - cmlII;
-	ca = cos(a);
-	if (ca < 0)
-	    return;	/* behind */
-
-	sa = sin(a);
-	r = rad/5;	/* too large for real but looks better */
-	h = 2*r;
-	w = 1.5*h*ca;	/* oval but forshortened near limbs */
-
-	dx = rad*sa;	/* E right */
-	dy = -r;	/* S up */
-
-	tvc = PI/2.0 - jop->s_dec;
-	pvc = jop->s_ra;
-	theta = PI/2.0 - POLE_DEC;
-	phi = POLE_RA;
-	sa = sin(tvc)*sin(theta)*(cos(pvc)*sin(phi) - sin(pvc)*cos(phi));
-	ca = sqrt (1.0 - sa*sa);
-
-	dxp =  dx*ca + dy*sa;
-	dyp = -dx*sa + dy*ca;
-
-	x = fliplr ? jx - dxp : jx + dxp;
-	y = fliptb ? jy - dyp : jy + dyp;
-
-	if (w > 0 && h > 0)
-	    XFillArc (XtD, dr, jgrs_gc, x-w/2, y-h/2, w, h, 0, 360*64);
-}
-
 #define	dsin(x)	sin(degrad(x))
 #define	dcos(x)	cos(degrad(x))
 
@@ -2296,4 +2458,4 @@ MoonData md[NM];        /* fill md[1..NM-1].x/y/z for each moon */
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: jupmenu.c,v $ $Date: 1999/10/25 21:01:55 $ $Revision: 1.9 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: jupmenu.c,v $ $Date: 2000/12/01 22:35:16 $ $Revision: 1.24 $ $Name:  $"};
