@@ -18,7 +18,7 @@
 #include <Xm/ToggleB.h>
 #include <Xm/Separator.h>
 #include <Xm/RowColumn.h>
-#include <Xm/Text.h>
+#include <Xm/TextF.h>
 
 #include "P_.h"
 
@@ -26,12 +26,14 @@ extern Widget toplevel_w;
 extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
-extern char *getXRes P_((char *name, char *def));
+extern char *getPrivateDir P_((void));
 extern char *syserrstr P_((void));
 extern int confirm P_((void));
 extern int existsh P_((char *filename));
+extern int isUp P_((Widget shell));
 extern int listing_ison P_((void));
 extern void all_selection_mode P_((int whether));
+extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
 extern void f_string P_((Widget w, char *s));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
@@ -39,11 +41,12 @@ extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     void (*func3)(void)));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
 
 static void lst_select P_((int whether));
-static void lst_create_form P_((void));
+static void lst_create_shell P_((void));
 static void lst_activate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void lst_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void lst_help_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -64,18 +67,19 @@ static void lst_hdr P_((void));
 #define MAXLSTSTR	32	/* longest string we can list */
 #define MAXFLDNAM	32	/* longest allowed field name */
 
-static Widget lstform_w;
-static Widget select_w, active_w, prompt_w, colhdr_w;
+static Widget lstshell_w;
+static Widget select_w, active_w, prompt_w, colhdr_w, latex_w;
 static Widget title_w, filename_w;
 static Widget table_w[MAXLSTFLDS];	/* row indeces follow.. */
 
 static FILE *lst_fp;            /* the listing file; == 0 means don't plot */
 static int lst_new;		/* 1 when open until first set are printed */
+static int lst_latex;		/* 1 when want latex format */
 
 
 /* lst_activate_cb client values. */
 typedef enum {
-    SELECT, ACTIVE, COLHDR
+    SELECT, ACTIVE, COLHDR, LATEX
 } Options;
 
 /* store the name and string value of each field to track.
@@ -89,20 +93,20 @@ typedef struct {
 static LstFld lstflds[MAXLSTFLDS];
 static int nlstflds;		/* number of lstflds[] in actual use */
 
+static char listcategory[] = "Tools -- List"; /* Save category */
+
 /* called when the list menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, go for it.
  */
 void
 lst_manage ()
 {
-	if (!lstform_w)
-	    lst_create_form();
+	if (!lstshell_w)
+	    lst_create_shell();
 	
-	if (XtIsManaged(lstform_w))
-	    XtUnmanageChild (lstform_w);
-	else
-	    XtManageChild (lstform_w);
+	XtPopup (lstshell_w, XtGrabNone);
+	set_something (lstshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* called by the other menus (data, etc) as their buttons are
@@ -115,9 +119,7 @@ char *name;
 	Widget tw;
 
 
-	if (!lstform_w
-		|| !XtIsManaged(lstform_w)
-		|| !XmToggleButtonGetState(select_w))
+	if (!isUp(lstshell_w) || !XmToggleButtonGetState(select_w))
 		    return;
 
 	tw = table_w[nlstflds];
@@ -164,8 +166,13 @@ listing()
 	    }
 
 	    /* now print the fields */
-	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++)
+	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++) {
 		(void) fprintf (lst_fp, "  %-*s", lp->l_w, lp->l_str);
+		if (lst_latex && lp < &lstflds[nlstflds-1])
+		    (void) fprintf (lst_fp, "&");
+	    }
+	    if (lst_latex)
+	    	(void) fprintf (lst_fp, "\\\\");
 	    (void) fprintf (lst_fp, "\n");
 	    fflush (lst_fp);	/* to allow monitoring */
 	}
@@ -184,8 +191,8 @@ Cursor c;
 {
 	Window win;
 
-	if (lstform_w && (win = XtWindow(lstform_w)) != 0) {
-	    Display *dsp = XtDisplay(lstform_w);
+	if (lstshell_w && (win = XtWindow(lstshell_w)) != 0) {
+	    Display *dsp = XtDisplay(lstshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -204,41 +211,52 @@ int whether;
 }
 
 static void
-lst_create_form()
+lst_create_shell()
 {
-	static struct {
+	typedef struct {
+	    int indent;		/* amount to indent, pixels */
+	    char *iname;	/* instance name, if Saveable */
 	    char *title;
 	    int cb_data;
 	    Widget *wp;
 	    char *tip;
-	} tbs[] = {
-	    {"Select fields", SELECT, &select_w,
+	} TButton;
+	static TButton tbs[] = {
+	    {0, NULL, "Select fields", SELECT, &select_w,
 		"When on, data fields eligible for listing are selectable buttons"},
-	    {"List to file", ACTIVE, &active_w,
+	    {0, NULL, "Create list file", ACTIVE, &active_w,
 		"When on, selected fields are written to the named file at each main Update"},
-	    {"Column Headings", COLHDR, &colhdr_w,
+	    {15, "Headings", "Include column headings", COLHDR, &colhdr_w,
 		"Whether file format will include a heading over each column"},
+	    {15, "LaTex", "List in LaTeX format", LATEX, &latex_w,
+		"Whether to separate columns with `&' and end lines with `\\\\'"},
 	};
 	XmString str;
 	Widget w, rc_w, f_w;
+	Widget lstform_w;
 	Arg args[20];
-	char fname[1024];
 	int i, n;
 
 	/* create form dialog */
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-        XtSetArg (args[n], XmNmarginHeight, 10); n++;
-	XtSetArg (args[n], XmNmarginWidth, 10); n++;
+	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	lstform_w = XmCreateFormDialog (toplevel_w, "List", args, n);
-	set_something (lstform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (lstform_w, XmNhelpCallback, lst_help_cb, 0);
+	XtSetArg (args[n], XmNtitle, "xephem Listing Control"); n++;
+	XtSetArg (args[n], XmNiconName, "List"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	lstshell_w = XtCreatePopupShell ("List", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (lstshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	sr_reg (lstshell_w, "XEphem*List.x", listcategory, 0);
+	sr_reg (lstshell_w, "XEphem*List.y", listcategory, 0);
 
 	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Listing Control"); n++;
-	XtSetValues (XtParent(lstform_w), args, n);
+	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
+        XtSetArg (args[n], XmNmarginHeight, 10); n++;
+	XtSetArg (args[n], XmNmarginWidth, 10); n++;
+	lstform_w = XmCreateForm(lstshell_w, "ListF", args, n);
+	XtAddCallback (lstform_w, XmNhelpCallback, lst_help_cb, 0);
+	XtManageChild (lstform_w);
 
 	/* make a RowColumn to hold everything */
 
@@ -260,19 +278,25 @@ lst_create_form()
 	/* make the control toggle buttons */
 
 	for (i = 0; i < XtNumber(tbs); i++) {
-	    str = XmStringCreate(tbs[i].title, XmSTRING_DEFAULT_CHARSET);
+	    TButton *tbp = &tbs[i];
+
+	    str = XmStringCreate(tbp->title, XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
+	    XtSetArg (args[n], XmNmarginWidth, tbp->indent); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateToggleButton(rc_w, "ListTB", args, n);
+	    w = XmCreateToggleButton(rc_w, tbp->iname ? tbp->iname : "ListTB",
+								    args, n);
 	    XmStringFree (str);
 	    XtAddCallback(w, XmNvalueChangedCallback, lst_activate_cb,
-						    (XtPointer)tbs[i].cb_data);
-	    if (tbs[i].wp)
-		*tbs[i].wp = w;
-	    if (tbs[i].tip)
-		wtip (w, tbs[i].tip);
+						    (XtPointer)tbp->cb_data);
+	    if (tbp->wp)
+		*tbp->wp = w;
+	    if (tbp->tip)
+		wtip (w, tbp->tip);
 	    XtManageChild (w);
+	    if (tbp->iname)
+		sr_reg (w, NULL, listcategory, 1);
 	}
 
 	/* create filename text area and its label */
@@ -285,12 +309,12 @@ lst_create_form()
 	XmStringFree (str);
 	XtManageChild (w);
 
-	(void) sprintf (fname, "%s/ephem.lst", getXRes ("PrivateDir", "work"));
 	n = 0;
-	XtSetArg (args[n], XmNvalue, fname); n++;
-	filename_w = XmCreateText (rc_w, "Filename", args, n);
+	filename_w = XmCreateTextField (rc_w, "Filename", args, n);
+	defaultTextFN (filename_w, 1, getPrivateDir(), "xephemlist.txt");
 	wtip (filename_w, "Enter name of file to write");
 	XtManageChild (filename_w);
+	sr_reg (filename_w, NULL, listcategory, 1);
 
 	/* create title text area and its label */
 
@@ -303,7 +327,8 @@ lst_create_form()
 	XmStringFree (str);
 
 	n = 0;
-	title_w = XmCreateText (rc_w, "ListTitle", args, n);
+	XtSetArg (args[n], XmNcolumns, 40); n++;
+	title_w = XmCreateTextField (rc_w, "ListTitle", args, n);
 	wtip (title_w, "Enter a title to be written to the file");
 	XtManageChild (title_w);
 
@@ -395,6 +420,10 @@ XtPointer call;
 
 	case COLHDR:
 	    break;	/* toggle state is sufficient */
+
+	case LATEX:
+	    lst_latex = t->set;
+	    break;
 	}
 }
 
@@ -407,7 +436,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	XtUnmanageChild (lstform_w);
+	XtPopdown (lstshell_w);
 }
 
 /* callback from the Help
@@ -491,7 +520,7 @@ lst_try_cancel()
 static void
 lst_try_turn_on()
 {
-	char *txt = XmTextGetString (filename_w);
+	char *txt = XmTextFieldGetString (filename_w);
 
 	if (existsh (txt) == 0 && confirm()) {
 	    char *buf;
@@ -516,7 +545,7 @@ char *how;	/* fopen how argument */
 	char *txt;
 
 	/* listing is on if file opens ok */
-	txt = XmTextGetString (filename_w);
+	txt = XmTextFieldGetString (filename_w);
 	lst_fp = fopenh (txt, how);
 	if (!lst_fp) {
 	    char *buf;
@@ -544,7 +573,7 @@ lst_hdr ()
 	char *txt;
 
 	/* add a title if desired */
-	txt = XmTextGetString (title_w);
+	txt = XmTextFieldGetString (title_w);
 	if (txt[0] != '\0')
 	    (void) fprintf (lst_fp, "%c %s\n", COMMENT, txt);
 	XtFree (txt);
@@ -573,9 +602,11 @@ lst_hdr ()
 	}
 
 	if (col) {
+	    int printed_anything;
+
 	    /* print first row of column headings */
 	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++) {
-		char cmt = lp == lstflds ? COMMENT : ' ';
+		char cmt = lp == lstflds && !lst_latex ? COMMENT : ' ';
 		char *dp;
 
 		for (dp = lp->l_name; *dp && *dp != '.'; dp++)
@@ -585,21 +616,31 @@ lst_hdr ()
 						    dp-lp->l_name, lp->l_name);
 		else
 		    fprintf (lst_fp, "%c %-*s", cmt, lp->l_w, lp->l_name);
+		if (lst_latex && lp < &lstflds[nlstflds-1])
+		    fprintf (lst_fp, "&");
 	    }
+	    if (lst_latex)
+	    	fprintf (lst_fp, "\\\\");
 	    fprintf (lst_fp, "\n");
 
 	    /* print second row of column headings */
+	    printed_anything = 0;
 	    for (lp = lstflds; lp < &lstflds[nlstflds]; lp++) {
-		char cmt = lp == lstflds ? COMMENT : ' ';
+		char cmt = lp == lstflds && !lst_latex ? COMMENT : ' ';
 		char *dp;
 
 		for (dp = lp->l_name; *dp && *dp != '.'; dp++)
 		    continue;
-		if (*dp)
+		if (*dp) {
 		    fprintf (lst_fp, "%c %-*s", cmt, lp->l_w, dp+1);
-		else
+		    printed_anything = 1;
+		} else
 		    fprintf (lst_fp, "%c %-*s", cmt, lp->l_w, "");
+		if (lst_latex && printed_anything && lp < &lstflds[nlstflds-1])
+		    fprintf (lst_fp, "&");
 	    }
+	    if (lst_latex && printed_anything)
+	    	fprintf (lst_fp, "\\\\");
 	    fprintf (lst_fp, "\n");
 	}
 
@@ -607,4 +648,4 @@ lst_hdr ()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: listmenu.c,v $ $Date: 1998/04/27 21:52:44 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: listmenu.c,v $ $Date: 2000/09/29 08:05:00 $ $Revision: 1.9 $ $Name:  $"};
