@@ -31,11 +31,13 @@
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
 extern Colormap xe_cm;
+extern char marscategory[];
 
 extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
+extern XFontStruct * getXResFont P_((char *rn));
 extern char *getShareDir P_((void));
 extern double atod P_((char *buf));
 extern double delra P_((double dra));
@@ -43,11 +45,13 @@ extern int any_ison P_((void));
 extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
     double mag, ObjF **opp));
 extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
+extern int isUp P_((Widget shell));
 extern int magdiam P_((int fmag, int magstp, double scale, double mag,
     double size));
 extern int read_bdl P_((FILE *fp, double jd, double *xp, double *yp,
     double *zp, char ynot[]));
 extern int skyloc_fifo P_((Obj *op));
+extern void buttonAsButton P_((Widget w, int whether));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void f_prdec P_((Widget w, double a));
@@ -63,6 +67,7 @@ extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
 extern void register_selection P_((char *name));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void marsm_update P_((Now *np, int how_much));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
@@ -90,11 +95,12 @@ typedef enum {
 static ScreenObj *screenobj[SO_N];
 static int nscreenobj[SO_N];
 
-static void marsm_create_form_w P_((void));
+static void marsm_create_shell_w P_((void));
 static void marsm_create_tvform_w P_((void));
 static void marsm_create_msform_w P_((void));
 static void marsm_set_buttons P_((int whether));
-static void marsm_set_a_button P_((Widget pbw, int whether));
+static void marsm_sstats_close_cb P_((Widget w, XtPointer client,
+    XtPointer call));
 static void marsm_sstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void marsm_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void marsm_scale_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -103,7 +109,7 @@ static void mt_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void mt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 static void mt_track_size P_((void));
 static void mt_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
-static void marsm_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void marsm_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void marsm_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void marsm_movie_cb P_((Widget w, XtPointer client, XtPointer call));
 static void marsm_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -123,7 +129,7 @@ static ScreenObj *close_screenobj P_((SOIdx, int x, int y));
 static void sky_background P_((Drawable win, unsigned w, unsigned h, int fmag,
     double ra0, double dec0, double scale, double rad, int fliptb, int fliplr));
 
-static Widget mmform_w;		/* main form */
+static Widget mmshell_w;	/* main shell */
 static Widget msform_w;		/* statistics form */
 static Widget mmframe_w;	/* main frame */
 static Widget mtform_w;		/* top-view form */
@@ -148,7 +154,6 @@ static int flip_tb;		/* whether we want to flip top/bottom */
 static int flip_lr;		/* whether we want to flip left/right */
 static int skybkg;		/* whether we want sky background */
 static int topview;		/* whether we want the top view */
-static int uswasman;		/* whether statistics form was managed */
 
 static Widget mmpu_w;		/* main popup */
 static Widget mmpu_name_w;	/* popup name label */
@@ -183,28 +188,23 @@ static struct MoonNames {
 enum {CEV, CSV, CX, CY, CZ, CRA, CDEC, CMAG, CNum};	/* marsm_w column */
 static Widget	marsm_w[NM][CNum];		/* the data display widgets */
 
-/* called when the mars menu is activated via the main mars pulldown.
+/* called when the mars moon view is activated from the mars pulldown.
+ * then we begin a life of our own as a toplevel shell.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, just insure up.
  */
 void
 marsm_manage ()
 {
-	if (!mmform_w) {
-	    marsm_create_form_w();
+	if (!mmshell_w) {
+	    marsm_create_shell_w();
 	    marsm_create_msform_w();
+	    marsm_create_tvform_w();
 	}
 	
-	if (XtIsManaged(mmform_w))
-	    XtUnmanageChild (mmform_w);
-	else {
-	    XtManageChild (mmform_w);
-	    if (uswasman) {
-		XtManageChild (msform_w);
-		marsm_set_buttons(marsm_selecting);
-	    }
-	    /* rely on expose to do the first draw */
-	}
+	XtPopup (mmshell_w, XtGrabNone);
+	set_something (mmshell_w, XmNiconic, (XtArgVal)False);
+	marsm_set_buttons(marsm_selecting);
 }
 
 /* called when the database has changed.
@@ -222,7 +222,7 @@ int appended;
 int
 marsm_ison()
 {
-	return (mmform_w && XtIsManaged(mmform_w));
+	return (isUp(mmshell_w));
 }
 
 /* called by other menus as they want to hear from our buttons or not.
@@ -233,9 +233,12 @@ void
 marsm_selection_mode (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	marsm_selecting += whether ? 1 : -1;
+	if (whether)
+	    marsm_selecting++;
+	else if (marsm_selecting > 0)
+	    --marsm_selecting;
 
-	if (marsm_ison())
+	if (mmshell_w)
 	    if ((whether && marsm_selecting == 1)     /* first one to want on */
 		|| (!whether && marsm_selecting == 0) /* last one to want off */)
 		marsm_set_buttons (whether);
@@ -248,8 +251,8 @@ Cursor c;
 {
 	Window win;
 
-	if (mmform_w && (win = XtWindow(mmform_w)) != 0) {
-	    Display *dsp = XtDisplay(mmform_w);
+	if (mmshell_w && (win = XtWindow(mmshell_w)) != 0) {
+	    Display *dsp = XtDisplay(mmshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -263,21 +266,29 @@ Cursor c;
 	    else
 		XUndefineCursor (dsp, win);
 	}
+
+	if (mtform_w && (win = XtWindow(mtform_w)) != 0) {
+	    Display *dsp = XtDisplay(mtform_w);
+	    if (c)
+		XDefineCursor (dsp, win, c);
+	    else
+		XUndefineCursor (dsp, win);
+	}
 }
 
-/* create the main form dialog */
+/* create the main moons shell */
 static void
-marsm_create_form_w()
+marsm_create_shell_w()
 {
 	typedef struct {
-	    char *name;		/* toggle button instance name */
+	    char *name;		/* toggle button instance name, or NULL */
 	    char *label;	/* label */
 	    int *flagp;		/* pointer to global flag, or NULL if none */
 	    Widget *wp;		/* widget to save, or NULL */
 	    char *tip;		/* widget tip */
 	} Option;
 	static Option options[] = {
-	    {"TopView",	   "Top view",		&topview, &topview_w,
+	    {NULL,	   "Top view",		&topview, &topview_w,
 	    	"When on, show view looking from high above N pole"},
 	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w,
 	    	"When on, sky will include database objects and Field Stars"},
@@ -302,28 +313,32 @@ marsm_create_form_w()
 	};
 	Widget w;
 	Widget mb_w, pd_w, cb_w;
+	Widget mmform_w;
 	XmString str;
 	Arg args[20];
 	int n;
 	int i;
 
-	/* create form */
-	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	mmform_w = XmCreateFormDialog (toplevel_w, "MarsMoons", args, n);
-	set_something (mmform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (mmform_w, XmNhelpCallback, marsm_help_cb, 0);
-	XtAddCallback (mmform_w, XmNunmapCallback, marsm_unmap_cb, 0);
+	/* create form and its shell */
 
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
 	n = 0;
 	XtSetArg (args[n], XmNtitle, "xephem Mars moon view"); n++;
-	XtSetValues (XtParent(mmform_w), args, n);
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNiconName, "MarsMoon"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	mmshell_w = XtCreatePopupShell ("MarsMoons", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (mmshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (mmshell_w, XmNpopdownCallback, marsm_popdown_cb, 0);
+	sr_reg (mmshell_w, "XEphem*MarsMoons.width", marscategory, 0);
+	sr_reg (mmshell_w, "XEphem*MarsMoons.height", marscategory, 0);
+	sr_reg (mmshell_w, "XEphem*MarsMoons.x", marscategory, 0);
+	sr_reg (mmshell_w, "XEphem*MarsMoons.y", marscategory, 0);
+
+	n = 0;
+	mmform_w = XmCreateForm (mmshell_w, "MarsMoons", args, n);
+	XtAddCallback (mmform_w, XmNhelpCallback, marsm_help_cb, 0);
+	XtManageChild (mmform_w);
 
 	/* create the menu bar across the top */
 
@@ -350,7 +365,7 @@ marsm_create_form_w()
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "FS", args, n);
-	    set_xmstring (w, XmNlabelString, "Setup Field Stars ...");
+	    set_xmstring (w, XmNlabelString, "Field Stars ...");
 	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
 	    wtip (w, "Define where GSC and PPM catalogs are to be found");
 	    XtManageChild (w);
@@ -404,7 +419,8 @@ marsm_create_form_w()
 		XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 		XtSetArg (args[n], XmNmarginHeight, 0); n++;
 		XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
-		w = XmCreateToggleButton (pd_w, op->name, args, n);
+		w = XmCreateToggleButton (pd_w, op->name ? op->name : "MMTB",
+								    args, n);
 		XtAddCallback (w, XmNvalueChangedCallback, marsm_option_cb, 
 						    (XtPointer)(op->flagp));
 		set_xmstring (w, XmNlabelString, op->label);
@@ -415,6 +431,8 @@ marsm_create_form_w()
 		if (op->tip)
 		    wtip (w, op->tip);
 		XtManageChild (w);
+		if (op->name)
+		    sr_reg (w, NULL, marscategory, 1);
 	    }
 
 	    /* add a separator */
@@ -490,6 +508,7 @@ marsm_create_form_w()
 	XtAddCallback (scale_w, XmNvalueChangedCallback, marsm_scale_cb, 0);
 	wtip (scale_w, "Zoom in and out");
 	XtManageChild (scale_w);
+	sr_reg (scale_w, NULL, marscategory, 0);
 
 	/* make the limiting mag scale widget.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -512,6 +531,7 @@ marsm_create_form_w()
 	XtAddCallback (limmag_w, XmNvalueChangedCallback, marsm_scale_cb, 0);
 	wtip (limmag_w, "Adjust the brightness and limiting magnitude");
 	XtManageChild (limmag_w);
+	sr_reg (limmag_w, NULL, marscategory, 0);
 
 	/* make a frame for the drawing area.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -578,10 +598,9 @@ marsm_create_msform_w()
 	/* create form */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	msform_w = XmCreateFormDialog (toplevel_w, "MarsMoonStats", args, n);
+	msform_w = XmCreateFormDialog (mmshell_w, "MarsMoonStats", args, n);
 	set_something (msform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -627,7 +646,7 @@ marsm_create_msform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "UTag", args, n); /*PB for sz */
 		set_xmstring (w, XmNlabelString, tag ? tag : " ");
-		marsm_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -653,7 +672,7 @@ marsm_create_msform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "UName", args, n); /*PB for sz*/
 		set_xmstring (w, XmNlabelString, mnames[i].full);
-		marsm_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -738,7 +757,7 @@ marsm_create_msform_w()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 70); n++;
 	w = XmCreatePushButton (msform_w, "Close", args, n);
-	XtAddCallback (w, XmNactivateCallback, marsm_sstats_cb, 0);
+	XtAddCallback (w, XmNactivateCallback, marsm_sstats_close_cb, 0);
 	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
@@ -753,9 +772,8 @@ marsm_create_tvform_w()
 	/* create form */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	mtform_w = XmCreateFormDialog (toplevel_w, "MarsMoonsTV", args, n);
+	mtform_w = XmCreateFormDialog (mmshell_w, "MarsMoonsTV", args, n);
 	set_something (mtform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (mtform_w, XmNunmapCallback, mt_unmap_cb, 0);
 	XtAddCallback (mtform_w, XmNmapCallback, mt_map_cb, NULL);
@@ -794,87 +812,23 @@ int whether;	/* whether setting up for plotting or for not plotting */
 {
 	int i, j;
 
-	for (i = 0; i < NM; i++) {
+	for (i = 0; i < NM; i++)
 	    for (j = 0; j < CNum; j++)
-		marsm_set_a_button (marsm_w[i][j], whether);
-	}
+		buttonAsButton (marsm_w[i][j], whether);
 }
 
-/* set whether the given button looks like a label.
- */
+/* callback when the Close button is activated on the stats menu */
+/* ARGSUSED */
 static void
-marsm_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
+marsm_sstats_close_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
 {
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    Widget tmpw;
-	    int n;
-
-	    n = 0;
-	    tmpw = XmCreatePushButton (mmform_w, "tmp", args, n);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-	    XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    XtDestroyWidget (tmpw);
-	     
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
+	XtUnmanageChild (msform_w);
 }
 
-/* callback when the Close button is activated on the stats menu or when
- * the More Info button is activated. they each do the same thing.
- */
+/* callback when the the More Info button is activated */
 /* ARGSUSED */
 static void
 marsm_sstats_cb (w, client, call)
@@ -882,12 +836,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((uswasman = XtIsManaged(msform_w)) != 0)
-	    XtUnmanageChild (msform_w);
-	else {
-	    XtManageChild (msform_w);
-	    marsm_set_buttons(marsm_selecting);
-	}
+	XtManageChild (msform_w);
+	marsm_set_buttons(marsm_selecting);
 }
 
 /* callback from any of the option buttons.
@@ -993,7 +943,7 @@ XtPointer call;
 	}
 
         XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!mt_pm || nx != last_nx || ny != last_ny) {
+	if (!mt_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (mt_pm)
 		XFreePixmap (dsp, mt_pm);
 	    mt_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1045,7 +995,7 @@ mt_track_size()
 	n = 0;
 	XtSetArg (args[n], XmNx, &mfx); n++;
 	XtSetArg (args[n], XmNy, &mfy); n++;
-	XtGetValues (mmform_w, args, n);
+	XtGetValues (mmshell_w, args, n);
 	n = 0;
 	XtSetArg (args[n], XmNx, &mdx); n++;
 	XtSetArg (args[n], XmNy, &mdy); n++;
@@ -1071,25 +1021,22 @@ XtPointer call;
 	XmToggleButtonSetState (topview_w, False, True);
 }
 
-/* callback when main dialog is unmapped */
+/* callback when main shell is popped down */
 /* ARGSUSED */
 static void
-marsm_unmap_cb (w, client, call)
+marsm_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((uswasman = (msform_w && XtIsManaged(msform_w))) != 0)
-	    XtUnmanageChild (msform_w);
+	XtUnmanageChild (msform_w);
+	XtUnmanageChild (mtform_w);
+
         if (marsm_pm) {
 	    XFreePixmap (XtDisplay(mda_w), marsm_pm);
 	    marsm_pm = (Pixmap) NULL;
 	}
 
-	/* close top view */
-	if (mtform_w && XtIsManaged(mtform_w))
-	    XtUnmanageChild (mtform_w);
-
 	/* free any field stars */
 	if (fstars) {
 	    free ((void *)fstars);
@@ -1109,8 +1056,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let unmap do all the work */
-	XtUnmanageChild (mmform_w);
+	/* let popdown do all the work */
+	XtPopdown (mmshell_w);
 }
 
 /* callback from the Movie button
@@ -1176,7 +1123,7 @@ XtPointer call;
 	}
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!marsm_pm || nx != last_nx || ny != last_ny) {
+	if (!marsm_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (marsm_pm)
 		XFreePixmap (dsp, marsm_pm);
 	    marsm_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1481,10 +1428,10 @@ int how_much;
 	int i;
 
 	/* see if we should bother */
-	if (!mmform_w)
+	if (!mmshell_w)
 	    return;
 	wantstats = XtIsManaged(msform_w) || any_ison() || how_much;
-	if (!XtIsManaged(mmform_w) && !wantstats)
+	if (!isUp(mmshell_w) && !wantstats)
 	    return;
 
 	watch_cursor (1);
@@ -1520,7 +1467,7 @@ int how_much;
 	    timestamp (np, sdt_w);
 	}
 
-	if (XtIsManaged(mmform_w)) {
+	if (isUp(mmshell_w)) {
 	    marsm_draw_map (mda_w, mop, usize, md);
 	    timestamp (np, dt_w);
 	}
@@ -1528,6 +1475,18 @@ int how_much;
 	watch_cursor (0);
 }
 
+/* called when basic resources change.
+ * rebuild and redraw.
+ */
+void
+marsm_newres()
+{
+	if (!mmshell_w)
+	    return;
+	make_gcs (mda_w);
+	marsm_update (mm_get_now(), 1);
+}
+
 /* given the loc of the moons, draw a nifty little picture.
  * also save resulting screen locs of everything in the screenobj array.
  * scale of the locations is in terms of Mars radii == 1.
@@ -1539,8 +1498,8 @@ int how_much;
  */
 static void
 marsm_draw_map (w, mop, usize, md)
-Obj *mop;
 Widget w;
+Obj *mop;
 double usize;
 MoonData md[NM];
 {
@@ -1633,8 +1592,8 @@ MoonData md[NM];
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_MAIN, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1655,7 +1614,7 @@ MoonData md[NM];
 	XFillRectangle (dsp, mt_pm, marsm_bgc, 0, 0, nx, ny);
 
 	/* draw Mars in center with unit radius */
-	i = ew*64*raddeg(asin(sin(degrad(mop->s_elong))/mop->s_sdist));
+	i = (int)(ew*64*raddeg(asin(sin(degrad(mop->s_elong))/mop->s_sdist)));
 	XPSFillArc(dsp, mt_pm, marsm_fgc, nx/2-scale1, ny/2-scale1, 2*scale1-1,
 						    2*scale1-1, -i, -180*64);
 	reset_screenobj(SO_TOP);
@@ -1690,8 +1649,8 @@ MoonData md[NM];
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_TOP, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1721,12 +1680,13 @@ Widget w;
 {
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
+	XFontStruct *fsp;
 	XGCValues gcv;
 	unsigned int gcm;
 	Pixel fg, bg;
 
 	gcm = GCForeground;
-	get_something (w, XmNforeground, (XtArgVal)&fg);
+	(void) get_color_resource (w, "marsColor", &fg);
 	gcv.foreground = fg;
 	marsm_fgc = XCreateGC (dsp, win, gcm, &gcv);
 	marsm_fs = XQueryFont (dsp, XGContextFromGC (marsm_fgc));
@@ -1734,18 +1694,20 @@ Widget w;
 	marsm_ch = marsm_fs->max_bounds.ascent + marsm_fs->max_bounds.descent;
 
 	gcm = GCForeground;
-	get_something (w, XmNbackground, (XtArgVal)&bg);
+	(void) get_color_resource (w, "MarsBackground", &bg);
 	gcv.foreground = bg;
 	marsm_bgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	gcm = GCForeground | GCFunction;
+	gcm = GCForeground | GCFunction | GCFont;
+	fsp = getXResFont ("moonsFont");
 	gcv.foreground = fg ^ bg;
 	gcv.function = GXxor;
+	gcv.font = fsp->fid;
 	marsm_xgc = XCreateGC (dsp, win, gcm, &gcv);
 }
 
 /* draw a calibration line l pixels long centered at [xc,yc] marked with tag
- * with is in a bounding box tw x th.
+ * in a bounding box tw x th.
  */
 static void
 marsm_calibline (dsp, win, gc, xc, yc, tag, tw, th, l)
@@ -1759,10 +1721,17 @@ int l;
 {
 	int lx = xc - l/2;
 	int rx = lx + l;
+	int i;
 
 	XDrawLine (dsp, win, gc, lx, yc, rx, yc);
-	XDrawLine (dsp, win, gc, lx, yc-3, lx, yc+3);
-	XDrawLine (dsp, win, gc, rx, yc-3, rx, yc+3);
+
+	for (i = 0; i <= 6; i++) {
+	    int x = lx+i*l/6;
+	    int dy = 3 + ((i%6)?0:3);
+
+	    XDrawLine (dsp, win, gc, x, yc-dy, x, yc+dy);
+	}
+
 	XDrawString (dsp, win, gc, xc-tw/2, yc+th, tag, strlen(tag));
 }
 
@@ -1821,8 +1790,8 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    ddec = dec0 - op->s_dec;	/* + S */
 	    if (fabs(ddec) > rad || delra(dra)*cdec0 > rad)
 		continue;
-	    dx = dra/scale;
-	    dy = ddec/scale;
+	    dx = (int)(dra/scale+.5);
+	    dy = (int)(ddec/scale+.5);
 	    x = fliplr ? (int)w/2-dx : (int)w/2+dx;
 	    y = fliptb ? (int)h/2+dy : (int)h/2-dy;
 
@@ -2000,4 +1969,4 @@ MoonData md[NM];        /* fill md[1..NM-1].x/y/z for each moon */
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: marsmmenu.c,v $ $Date: 1999/03/03 19:06:34 $ $Revision: 1.4 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: marsmmenu.c,v $ $Date: 2000/11/26 05:43:42 $ $Revision: 1.17 $ $Name:  $"};
