@@ -60,9 +60,12 @@ extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
     double mag, ObjF **opp));
 extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
 extern int gray_ramp P_((Display *dsp, Colormap cm, Pixel **pixp));
+extern int isUp P_((Widget shell_w));
 extern int magdiam P_((int fmag, int magstp, double scale, double mag,
     double size));
 extern int openh P_((char *name, int flags, ...));
+extern void buttonAsButton P_((Widget w, int whether));
+extern void centerScrollBars P_((Widget sw_w));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void dm_newobj P_((int dbidx));
@@ -81,12 +84,14 @@ extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void get_views_font P_((Display *dsp, XFontStruct **fspp));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void llibration P_((double JD, double *llatp, double *llonp));
+extern void mm_movie P_((double stepsz));
 extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
 extern void obj_set P_((Obj *op, int dbidx));
 extern void range P_((double *v, double r));
 extern void register_selection P_((char *name));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
 extern char *syserrstr P_((void));
@@ -110,6 +115,7 @@ typedef struct {
 
 static SkyObj *skyobjs;		/* malloced list of objects in sky background */
 static int nskyobjs;		/* number of objects in skyobjs[] list */
+static char mooncategory[] = "Moon";	/* Save category */
 
 #define	MAXR	10		/* max gap when picking sky objects, pixels */	
 
@@ -119,25 +125,27 @@ static Obj *closeSkyObj P_((int x, int y));
 static void fill_skypopup P_((Obj *op));
 static void m_create_skypopup P_((void));
 
-static void m_create_form P_((void));
+static void m_create_shell P_((void));
 static void m_create_msform P_((void));
 static void m_create_esform P_((void));
 static void m_set_buttons P_((int whether));
-static void m_set_a_button P_((Widget pbw, int whether));
 static void m_eshine_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_eshineclose_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_eshineup_cb P_((Widget w, XtPointer client, XtPointer call));
+static void m_movie_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_mstats_cb P_((Widget w, XtPointer client, XtPointer call));
+static void m_mstats_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_activate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_print_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_print P_((void));
 static void m_imps P_((void));
 static void m_ps_annotate P_((void));
-static void m_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void m_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_init_gcs P_((void));
 static void m_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_helpon_cb P_((Widget w, XtPointer client, XtPointer call));
+static void m_pl_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_shrink_cb P_((Widget w, XtPointer client, XtPointer call));
 static void m_sizewidgets P_((void));
@@ -161,11 +169,10 @@ static int m_shrink P_((int f));
 static void mBWdither P_((void));
 static void m_resize P_((unsigned char *in, int nr, int nc, int f,
     unsigned char *out));
-static void m_eclipse P_((void));
+static void m_umbra P_((void));
 static void fliptb P_((void));
 static void fliplr P_((void));
 static void mi_draw P_((void));
-static void m_features P_((void));
 static void m_orientation P_((void));
 static void m_labels P_((void));
 static void m_sky P_((void));
@@ -195,6 +202,9 @@ static void m_popup P_((XEvent *ep));
 #define	BORD		200	/* extra drawing area border for labels/stars */
 #define	CLNG	degrad(-4.3)	/* longitude of image center, rads east */
 #define	CLAT	degrad(1.0)	/* latitude of image center, rads north */
+#define MOVIE_STEPSZ    (2.)	/* movie step size without stars, hours */
+#define MOVIE_SSTEPSZ   (1./60)	/* movie step size with stars, hours */
+
 static unsigned char *mimage;	/* malloced array of full-size moon image */
 static Pixel *mgray;		/* gray-scale ramp for drawing image */
 static int nmgray;		/* number of pixels usable in mgray[] */
@@ -218,8 +228,9 @@ static MoonInfo minfo;		/* stats about the current situation */
 #define	FMAG		12	/* faintest mag of sky background object */
 
 /* main's widgets */
-static Widget mform_w;		/* main moon form dialog */
+static Widget mshell_w;		/* main moon shell */
 static Widget msform_w;		/* statistics form dialog */
+static Widget msw_w;		/* main scrolled window */
 static Widget mda_w;		/* image view DrawingArea */
 static Pixmap m_pm;		/* image view staging pixmap */
 static Widget dt_w;		/* main date/time stamp widget */
@@ -236,7 +247,6 @@ static Widget sunalt_w;		/* sun altitude under cursor */
 static Widget lat_w, lng_w;	/* lat/long under cursor */
 static Widget nsrt_w, nsrd_w;	/* next sunrise time/date under cursor */
 static Widget nsst_w, nssd_w;	/* next sunset time/date under cursor */
-static int mswasman;            /* whether "More info" form was managed */
 
 /* earthshine widgets */
 static Widget esform_w;		/* main dialog */
@@ -250,6 +260,7 @@ static Widget pu_size_w;	/* popup size label */
 static Widget pu_lat_w;		/* popup latitude label */
 static Widget pu_lng_w;		/* popup longitude label */
 static Widget pu_alt_w;		/* popup sun altitude label */
+static Widget pu_pl_w;		/* popup persistent label TB */
 
 /* sky background object's widgets */
 static Widget skypu_w;		/* main popup */
@@ -269,6 +280,7 @@ static double fsdec, fsra;      /* location when field stars were loaded */
 static void m_loadfs P_((Now *np, double ra, double dec));
 
 static GC m_fgc, m_bgc, m_agc;	/* various GCs */
+static XFontStruct *m_fsp;	/* label font info */
 
 static int m_selecting;		/* set while our fields are being selected */
 
@@ -285,7 +297,7 @@ typedef enum {
 
 /* options list */
 typedef enum {
-    APOLLO_OPT, GRID_OPT, LABELS_OPT, FLIPLR_OPT, FLIPTB_OPT,
+    SPACECRAFT_OPT, GRID_OPT, LABELS_OPT, FLIPLR_OPT, FLIPTB_OPT,
     SKY_OPT, IMAGE_OPT, UMBRA_OPT, N_OPT
 } Option;
 static int option[N_OPT];
@@ -301,26 +313,16 @@ static Widget option_w[N_OPT];
 #define	FX(x)	(option[FLIPLR_OPT] ? mncols-1-(x) : (x))
 #define	FY(y)	(option[FLIPTB_OPT] ? mnrows-1-(y) : (y))
 
-/* special features list */
-typedef struct {
-    double lt, lg;		/* degrees N, E */
-    char *name;
-} LFeatures;
-static LFeatures lfeatures[] = {
-    {  0.8,  23.5, "Apollo 11"},
-    { -3.2, 336.6, "Apollo 12"},
-    { -3.7, 342.5, "Apollo (13)14"},
-    { 26.0,   3.6, "Apollo 15"},
-    { -9.0,  15.6, "Apollo 16"},
-    { 20.1,  30.8, "Apollo 17"},
-};
-
 /* lunar database */
 typedef struct {
     char *name;			/* malloced name */
     char *type;			/* malloced type (may be shared) */
     short x, y;			/* image location, X pixels coords */
-    short sz; 			/* diam without forshortening, km */
+    short sz; 			/* diam w/o forshortening, km,
+    				 * or landing date if < 0
+				 */
+    char pl;			/* whether want persistent label */
+    char rjust;			/* whether to right- else left-justify */
 } MoonDB;
 static MoonDB *moondb;		/* malloced array from file */
 static int nmoondb;		/* entries in moondb[] */
@@ -331,17 +333,17 @@ static void m_create_popup P_((void));
 static MoonDB *closeMoonDB P_((int x, int y));
 
 /* called when the moon view is activated via the main menu pulldown.
- * if first time, build everything, else just toggle whether we are mapped.
+ * if first time, build everything, else just up we go.
  * allow for retrying to read the image file each time until find it.
  */
 void
 m_manage ()
 {
-	if (!mform_w) {
+	if (!mshell_w) {
 	    /* menu one-time-only work */
 
 	    /* build dialogs */
-	    m_create_form();
+	    m_create_shell();
 	    m_create_msform();
 	    m_create_popup();
 	    m_create_skypopup();
@@ -360,22 +362,15 @@ m_manage ()
 		return;
 	}
 	
-	if (XtIsManaged(mform_w)) {
-	    XtUnmanageChild (mform_w);
-	} else {
-	    XtManageChild (mform_w);
-	    if (mswasman) {
-		XtManageChild (msform_w);
-		m_set_buttons(m_selecting);
-	    }
-	    /* the expose will do the drawing */
-	}
+	XtPopup (mshell_w, XtGrabNone);
+	set_something (mshell_w, XmNiconic, (XtArgVal)False);
+	centerScrollBars (msw_w);
 }
 
 int
 m_ison()
 {
-	return (mform_w && XtIsManaged(mform_w));
+	return (isUp(mshell_w));
 }
 
 /* commanded from main to update with a new set of circumstances */
@@ -384,18 +379,28 @@ m_update (np, how_much)
 Now *np;
 int how_much;
 {
-	if (!mform_w)
+	if (!mshell_w)
 	    return;
-	if (!XtIsManaged(mform_w) && !any_ison() && !how_much)
+	if (!isUp(mshell_w) && !any_ison() && !how_much)
 	    return;
 
 	watch_cursor (1);
-
 	m_redraw();
-
 	watch_cursor (0);
 }
 
+/* called when basic resources change.
+ * rebuild and redraw.
+ */
+void
+m_newres()
+{
+	if (!mshell_w)
+	    return;
+	m_init_gcs();
+	m_update (mm_get_now(), 1);
+}
+
 /* called when the database has changed.
  * if we are drawing background, we'd best redraw everything.
  */
@@ -403,7 +408,7 @@ void
 m_newdb (appended)
 int appended;
 {
-	if (option[SKY_OPT] && mform_w && XtIsManaged(mform_w))
+	if (option[SKY_OPT] && isUp(mshell_w))
 	    m_redraw();
 }
 
@@ -415,9 +420,12 @@ void
 m_selection_mode (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	m_selecting += whether ? 1 : -1;
+	if (whether)
+	    m_selecting++;
+	else if (m_selecting > 0)
+	    --m_selecting;
 
-	if (m_ison()) {
+	if (mshell_w) {
 	    if ((whether && m_selecting == 1)     /* first one to want on */
 		|| (!whether && m_selecting == 0) /* last one to want off */)
 		m_set_buttons (whether);
@@ -431,8 +439,8 @@ Cursor c;
 {
 	Window win;
 
-	if (mform_w && (win = XtWindow(mform_w)) != 0) {
-	    Display *dsp = XtDisplay(mform_w);
+	if (mshell_w && (win = XtWindow(mshell_w)) != 0) {
+	    Display *dsp = XtDisplay(mshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -449,7 +457,7 @@ Cursor c;
 }
 
 static void
-m_create_form()
+m_create_shell()
 {
 	typedef struct {
 	    Option o;		/* which option */
@@ -458,9 +466,9 @@ m_create_form()
 	    char *tip;		/* widget tip */
 	} OpSetup;
 	static OpSetup ops[] = {
-	    {APOLLO_OPT,	"Apollo",	"Apollo sites",
-	    	"Label each Apollo landing site"},
-	    {LABELS_OPT,	"Labels",	"More labels",
+	    {SPACECRAFT_OPT,	"Spacecraft",	"Spacecraft",
+	    	"Label each Spacecraft landing site"},
+	    {LABELS_OPT,	"Labels",	"Features",
 	    	"Label significant natural features"},
 	    {SKY_OPT,		"SkyBkg",	"Sky background",
 	    	"When on, sky will include database objects and Field Stars"},
@@ -487,8 +495,8 @@ m_create_form()
 	    {"on Scale...",	"Moon - scale"},
 	};
 	Widget mb_w, pd_w, cb_w;
-	Widget msw_w;
 	Widget w;
+	Widget mform_w;
 	char msg[1024];
 	XmString str;
 	unsigned long mask;
@@ -498,22 +506,25 @@ m_create_form()
 
 	/* create master form */
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Moon view"); n++;
+	XtSetArg (args[n], XmNiconName, "Moon"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	mshell_w = XtCreatePopupShell ("Moon", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (mshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (mshell_w, XmNpopdownCallback, m_popdown_cb, 0);
+	sr_reg (mshell_w, "XEphem*Moon.width", mooncategory, 0);
+	sr_reg (mshell_w, "XEphem*Moon.height", mooncategory, 0);
+	sr_reg (mshell_w, "XEphem*Moon.x", mooncategory, 0);
+	sr_reg (mshell_w, "XEphem*Moon.y", mooncategory, 0);
+
+	n = 0;
 	XtSetArg (args[n], XmNhorizontalSpacing, 5); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	mform_w = XmCreateFormDialog (toplevel_w, "Moon", args, n);
-	set_something (mform_w, XmNcolormap, (XtArgVal)xe_cm);
+	mform_w = XmCreateForm (mshell_w, "MoonF", args, n);
 	XtAddCallback (mform_w, XmNhelpCallback, m_help_cb, 0);
-	XtAddCallback (mform_w, XmNunmapCallback, m_unmap_cb, 0);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
-	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Moon view"); n++;
-	XtSetValues (XtParent(mform_w), args, n);
+	XtManageChild (mform_w);
 
 	/* create the menu bar across the top */
 
@@ -549,7 +560,7 @@ m_create_form()
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "FS", args, n);
-	    set_xmstring (w, XmNlabelString, "Setup Field Stars...");
+	    set_xmstring (w, XmNlabelString, "Field Stars...");
 	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
 	    wtip (w, "Define where GSC and PPM catalogs are to be found");
 	    XtManageChild (w);
@@ -563,6 +574,15 @@ m_create_form()
 	    wtip (w, "Display a dialog to set brightness of dark half of moon");
 	    XtManageChild (w);
 
+	    /* button for Movie demo */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "EMPB", args, n);
+	    XtAddCallback (w, XmNactivateCallback, m_movie_cb, 0);
+	    set_xmstring (w, XmNlabelString, "Movie Demo");
+	    wtip (w, "Start/Stop a fun time-lapse animation");
+	    XtManageChild (w);
+
 	    /* add a separator */
 	    n = 0;
 	    w = XmCreateSeparator (pd_w, "CtS", args, n);
@@ -605,6 +625,7 @@ m_create_form()
 		if (osp->tip)
 		    wtip (w, osp->tip);
 		XtManageChild (w);
+		sr_reg (w, NULL, mooncategory, 1);
 	    }
 
 	    /* add a separator */
@@ -618,7 +639,7 @@ m_create_form()
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "Stats", args, n);
 	    set_xmstring (w, XmNlabelString, "More info...");
-	    XtAddCallback (w, XmNactivateCallback, m_mstats_cb, NULL);
+	    XtAddCallback (w, XmNactivateCallback, m_mstats_cb, 0);
 	    wtip (w, "Display additional information");
 	    XtManageChild (w);
 
@@ -633,6 +654,7 @@ m_create_form()
 	    XtSetArg (args[n], XmNmnemonic, 'S'); n++;
 	    cb_w = XmCreateCascadeButton (mb_w, "ScaleCB", args, n);
 	    set_xmstring (cb_w, XmNlabelString, "Scale");
+	    wtip (cb_w, "Set the image size");
 	    XtManageChild (cb_w);
 
 	    for (i = 1; i <= MAXSHRINK; i++) {
@@ -662,13 +684,15 @@ m_create_form()
 		    } else
 			mshrink = i;
 		}
-	    }
 
-	    /* if none found to be set, default to last one -- MAXSHRINK */
-	    if (mshrink == 0) {
-		xe_msg("No Moon*Scale<n>X resource found -- defaulting to 1",0);
-		XmToggleButtonSetState (w, True, False);
-		mshrink = MAXSHRINK;
+		/* if done and none set, default to last one */
+		if (i == MAXSHRINK && mshrink == 0) {
+		    xe_msg("No Moon*Scale<n>X resource -- defaulting to 1",0);
+		    XmToggleButtonSetState (w, True, False);
+		    mshrink = MAXSHRINK;
+		}
+
+		sr_reg (w, NULL, mooncategory, 1);
 	    }
 
 	/* make the help pulldown */
@@ -784,7 +808,7 @@ m_create_msform()
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
-	msform_w = XmCreateFormDialog (toplevel_w, "MoonStats", args, n);
+	msform_w = XmCreateFormDialog (mshell_w, "MoonStats", args, n);
 	set_something (msform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -997,7 +1021,7 @@ m_create_msform()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 80); n++;
 	w = XmCreatePushButton (msform_w, "Close", args, n);
-	XtAddCallback (w, XmNactivateCallback, m_mstats_cb, NULL);
+	XtAddCallback (w, XmNactivateCallback, m_mstats_close_cb, 0);
 	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
@@ -1018,7 +1042,7 @@ m_create_esform()
 	XtSetArg (args[n], XmNfractionBase, 10); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	esform_w = XmCreateFormDialog (toplevel_w, "MoonES", args, n);
+	esform_w = XmCreateFormDialog (mshell_w, "MoonES", args, n);
 	set_something (esform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -1045,6 +1069,7 @@ m_create_esform()
 	eshine_w = XmCreateScale (esform_w, "Earthshine", args, n);
 	wtip (eshine_w, "Set to desired relative Earthshine brightness");
 	XtManageChild (eshine_w);
+	sr_reg (eshine_w, NULL, mooncategory, 1);
 
 	get_something (eshine_w, XmNminimum, (XtArgVal)&min);
 	get_something (eshine_w, XmNmaximum, (XtArgVal)&max);
@@ -1107,84 +1132,12 @@ static void
 m_set_buttons (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	m_set_a_button (srlng_w, whether);
-	m_set_a_button (sslat_w, whether);
-	m_set_a_button (llat_w, whether);
-	m_set_a_button (llong_w, whether);
-	m_set_a_button (limb_w, whether);
-	m_set_a_button (lib_w, whether);
-}
-
-/* set whether the given button looks like a label.
- */
-static void
-m_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
-{
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    Widget tmpw;
-	    int n;
-
-	    n = 0;
-	    tmpw = XmCreatePushButton (mform_w, "tmp", args, n);
-
-	    n = 0;
-            XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-            XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    XtDestroyWidget (tmpw);
-	     
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
+	buttonAsButton (srlng_w, whether);
+	buttonAsButton (sslat_w, whether);
+	buttonAsButton (llat_w, whether);
+	buttonAsButton (llong_w, whether);
+	buttonAsButton (limb_w, whether);
+	buttonAsButton (lib_w, whether);
 }
 
 /* callback from the Apply button in the earthshine dialog */
@@ -1224,8 +1177,19 @@ XtPointer call;
 }
 
 
-/* callback from the Close button is activated on the stats menu or when
- * the More Info button is activated. they each do the same thing.
+/* callback from the Close button on the stats menu.
+ */
+/* ARGSUSED */
+static void
+m_mstats_close_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XtUnmanageChild (msform_w);
+}
+
+/* callback from the More Info button on the main menu.
  */
 /* ARGSUSED */
 static void
@@ -1234,12 +1198,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((mswasman = XtIsManaged(msform_w)) != 0)
-	    XtUnmanageChild (msform_w);
-	else {
-	    XtManageChild (msform_w);
-	    m_set_buttons(m_selecting);
-	}
+	XtManageChild (msform_w);
+	m_set_buttons(m_selecting);
 }
 
 /* store intensity v into array[x,y] */
@@ -1271,8 +1231,6 @@ XtPointer call;
 static void
 m_print ()
 {
-	XFontStruct *fsp;
-
 	/* must be up */
 	if (!m_ison()) {
 	    xe_msg ("Moon must be open to print.", 1);
@@ -1286,10 +1244,6 @@ m_print ()
 	XPSXBegin (m_pm, 0, 0, mncols+2*BORD, mnrows+2*BORD, 1*72, 10*72,
 								(int)(6.5*72));
 
-        /* get and register the font to use for all gcs from obj_pickgc() */
-	get_views_font (XtD, &fsp);
-	XPSRegisterFont (fsp->fid, "Helvetica");
-
 	/* redraw everything now */
 	m_redraw();
 
@@ -1459,19 +1413,17 @@ XtPointer call;
 	}
 }
 
-/* callback from mform_w being unmapped.
+/* callback from mshell_w being popped down.
  */
 /* ARGSUSED */
 static void
-m_unmap_cb (w, client, call)
+m_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
 	XtUnmanageChild (esform_w);
-
-	if ((mswasman = (msform_w && XtIsManaged(msform_w))) != 0)
-	    XtUnmanageChild (msform_w);
+	XtUnmanageChild (msform_w);
 
 	/* freeing the pixmap will prevent any useless updates while off and
 	 * insure a fresh update on the next expose.
@@ -1487,6 +1439,9 @@ XtPointer call;
 	    fstars = NULL;
 	    nfstars = 0;
 	}
+
+	/* stop movie that might be running */
+	mm_movie (0.0);
 }
 
 /* called from Close button */
@@ -1496,9 +1451,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	XtUnmanageChild (mform_w);
-
-	/* unmapping mform_w will also unmap msform_w */
+	XtPopdown (mshell_w);
+	/* let popdown do all the work */
 }
 
 /* callback from the any of the option TBs.
@@ -1520,14 +1474,6 @@ XtPointer call;
 	option[opt] = set = XmToggleButtonGetState (w);
 
 	switch (opt) {
-	case APOLLO_OPT:
-	    if (set) {
-		m_features();
-		m_refresh (NULL);
-	    } else
-		m_redraw();
-	    break;
-
 	case GRID_OPT:
 	    if (set) {
 		m_grid();
@@ -1551,12 +1497,13 @@ XtPointer call;
 
 	case UMBRA_OPT:
 	    if (set) {
-		m_eclipse();
+		m_umbra();
 		m_refresh (NULL);
 	    } else
 		m_redraw();
 	    break;
 
+	case SPACECRAFT_OPT:	/* FALLTHRU */
 	case LABELS_OPT:
 	    if (set) {
 		m_labels();
@@ -1603,6 +1550,18 @@ XtPointer call;
 	mshrink = f;
 }
 
+
+/* callback for when the Movie button is activated. */
+/* ARGSUSED */
+static void
+m_movie_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+        mm_movie (option[SKY_OPT] ? MOVIE_SSTEPSZ : MOVIE_STEPSZ);
+}
+
 /* callback from the Help all button
  */
 /* ARGSUSED */
@@ -1885,6 +1844,7 @@ unsigned *wp, *hp, *dp;
 
 /* make the various gcs, handy pixel values and fill in mgray[].
  * N.B. just call this once.
+ * TODO: reclaim old stuff if called again.
  */
 static void
 m_init_gcs()
@@ -1894,14 +1854,12 @@ m_init_gcs()
 	Colormap cm = xe_cm;
 	XGCValues gcv;
 	unsigned int gcm;
-	XFontStruct *fsp;
 	Pixel fg;
 	Pixel p;
 
-	/* make gcs from MoonDA colors
-	 */
+	/* fg and bg */
 	get_something (mda_w, XmNforeground, (XtArgVal)&fg);
-	get_something (mda_w, XmNbackground, (XtArgVal)&mbg);
+	(void) get_color_resource (mda_w, "MoonBackground", &mbg);
 
 	gcm = GCForeground | GCBackground;
 	gcv.foreground = fg;
@@ -1912,21 +1870,13 @@ m_init_gcs()
 	gcv.background = fg;
 	m_bgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	/* make the label marker gc.
-	 * beware of monochrome screens that claim ok for any ol' color.
-	 */
-	if (get_color_resource (mda_w, "MoonAnnotColor", &p) < 0
-						    || p == mbg || p == fg) {
-	    xe_msg("Can not get moon annotation color -- using white.",0);
-	    p = WhitePixel (dsp, 0);
-	}
+	/* make the label marker gc */
+	(void) get_color_resource (mda_w, "MoonAnnotColor", &p);
 	gcm = GCForeground | GCBackground;
 	gcv.foreground = p;
 	gcv.background = mbg;
 	m_agc = XCreateGC (dsp, win, gcm, &gcv);
-
-	get_views_font (dsp, &fsp);
-	XSetFont (dsp, m_agc, fsp->fid);
+	get_views_font (dsp, &m_fsp);
 
 	/* build gray-scale ramp for image */
 	nmgray = gray_ramp (dsp, cm, &mgray);
@@ -2036,7 +1986,7 @@ mi_draw ()
 
 	/* add eclipse pen/umbra boundaries */
 	if (option[UMBRA_OPT])
-	    m_eclipse();
+	    m_umbra();
 
 	/* add the libration marker */
 	m_mark_libr ();
@@ -2047,13 +1997,8 @@ mi_draw ()
 	    m_sub ();
 	}
 
-	/* add apollo labels, if enabled */
-	if (option[APOLLO_OPT])
-	    m_features();
-
-	/* add feature labels, if enabled */
-	if (option[LABELS_OPT])
-	    m_labels();
+	/* add any desired feature labels */
+	m_labels();
 
 	/* add sky background objects */
 	resetSkyObj();
@@ -2064,50 +2009,36 @@ mi_draw ()
 	m_orientation();
 }
 
-/* draw the special features list on m_pm */
-static void
-m_features()
-{
-	Display *dsp = XtDisplay (mda_w);
-	Window win = m_pm;
-	int i;
-
-	for (i = 0; i < XtNumber(lfeatures); i++) {
-	    LFeatures *lfp = &lfeatures[i];
-	    int x, y;
-
-	    if (ll2xy (degrad(lfp->lt), degrad(lfp->lg), &x, &y) == 0) {
-		char *n = lfp->name;
-		int l = strlen(n);
-
-		x = FX(x) + BORD;
-		y = FY(y) + BORD;
-
-		XPSDrawArc (dsp, win, m_agc, x-2, y-2, 5, 5, 0, 360*64);
-		if ((!strcmp (n, "Apollo 12") && !option[FLIPLR_OPT]) ||
-			    (!strcmp(n,"Apollo (13)14") && option[FLIPLR_OPT])){
-		    XID fid = XGContextFromGC (m_agc);
-		    int dir, asc, des;
-		    XCharStruct ap;
-
-		    XQueryTextExtents (XtD, fid, n, l, &dir, &asc, &des, &ap);
-		    XPSDrawString (dsp, win, m_agc, x-ap.width-4, y-4, n, l);
-		} else
-		    XPSDrawString (dsp, win, m_agc, x+4, y-4, n, l);
-	    }
-	}
-}
-
 /* label the database entries marked for name display */
 static void
 m_labels()
 {
 	MoonDB *mp;
 
+	XSetFont (XtD, m_agc, m_fsp->fid);
+
 	for (mp = moondb; mp < &moondb[nmoondb]; mp++)
-	    if (m_wantlabel(mp))
-		XPSDrawString (XtD, m_pm, m_agc, FX(mp->x)+BORD, FY(mp->y)+BORD,
-	    					mp->name, strlen(mp->name));
+	    if (m_wantlabel(mp)) {
+		char *n = mp->name;
+		int l = strlen (n);
+		int x = FX(mp->x)+BORD;
+		int y = FY(mp->y)+BORD;
+
+		if (mp->sz < 0) {
+		    /* spacecraft */
+		    XPSDrawArc (XtD, m_pm, m_agc, x-2, y-2, 5, 5, 0, 360*64);
+		    if (!mp->rjust == option[FLIPLR_OPT]) {
+			int dir, asc, des;
+			XCharStruct ap;
+
+			XQueryTextExtents (XtD, m_fsp->fid, n, l, &dir, &asc,
+								    &des,&ap);
+			XPSDrawString (XtD, m_pm, m_agc, x-ap.width-4, y-4,n,l);
+		    } else
+			XPSDrawString (XtD, m_pm, m_agc, x+4, y-4, n, l);
+		} else
+		    XPSDrawString (XtD, m_pm, m_agc, x, y, n, l);
+	    }
 }
 
 /* add background sky objects to m_pm and to skyobjs[] list */
@@ -2244,6 +2175,21 @@ MoonDB *mp;
 	double lt, lg;
 	int ok;
 
+	/* always show if marked as persistent */
+	if (mp->pl)
+	    return (1);
+
+	/* or if want spacecraft and it appears to be one and
+	 * we are at full size or this is an Apollo site
+	 */
+	if (option[SPACECRAFT_OPT] && mp->sz < 0
+		    && (mshrink == 1 || !strncmp (mp->name, "Apollo", 6)))
+	    return (1);
+
+	/* but never show otherwise if disabled */
+	if (!option[LABELS_OPT])
+	    return (0);
+
 	/* don't show any if just too small */
 	if (mshrink > MAXSHRINK/2)
 	    return (0);
@@ -2271,24 +2217,21 @@ MoonDB *mp;
 }
 
 /* draw the N/S E/W labels on the four edges of the m_pm
- * we are showing celestial coords.
+ * we are showing ecliptic coords.
  */
 static void
 m_orientation()
 {
-	static int fw, fa, fd;
+	int fw, fa, fd;
+	int dir, asc, des;
+	XCharStruct xcs;
 
-	if (fw == 0) {
-	    XID fid = XGContextFromGC (m_agc);
-	    int dir, asc, des;
-	    XCharStruct xcs;
-
-	    XQueryTextExtents (XtD, fid, "W", 1, &dir, &asc, &des, &xcs);
-	    fw = xcs.width;
-	    fa = xcs.ascent;
-	    fd = xcs.descent;
-	}
+	XQueryTextExtents (XtD, m_fsp->fid, "W", 1, &dir, &asc, &des, &xcs);
+	fw = xcs.width;
+	fa = xcs.ascent;
+	fd = xcs.descent;
 
+	XSetFont (XtD, m_agc, m_fsp->fid);
 	XPSDrawString (XtD, m_pm, m_agc, BORD+(mncols-fw)/2, BORD-fd-LGAP,
 					    option[FLIPTB_OPT] ? "S" : "N", 1);
 	XPSDrawString (XtD, m_pm, m_agc, BORD+(mncols-fw)/2,BORD+mnrows+fa+LGAP,
@@ -2757,8 +2700,10 @@ int f;
 
 	/* size the drawing area to hold the new image plus a border.
 	 * this will give us an expose event to show the new image.
+	 * also center the scrollbars.
 	 */
 	m_sizewidgets ();
+	centerScrollBars(msw_w);
 
 	return(0);
 }
@@ -2856,19 +2801,21 @@ mBWdither()
 
 /* draw umbra and penumbra boundaries */
 static void
-m_eclipse()
+m_umbra()
 {
+	Now *np = mm_get_now();
 	Obj *mop = db_basic (MOON);
 	Obj *sop = db_basic (SUN);
 	double scale;		/* image scale, rads per pixel */
 	double mrad;		/* moon radius, rads */
-	double maxr;		/* max dst from center we want to draw, rads */
-	double ara, adec;	/* anti-solar location */
-	double cmdec;		/* cos moon's dec */
-	double dra, ddec;	/* dist from center, rads */
+	double ara, adec;	/* anti-solar loc, equatorial coords, rads */
+	double alat, alng;	/* anti-solar loc, ecliptic coords, rads */
+	double mlat, mlng;	/* moon loc, ecliptic coords, rads */
+	double dlat, dlng;	/* difference, rads */
+	double cmlat;		/* cos moon's lat */
 	double sed, med;	/* sun-earth and moon-earth dist, au */
 	double u, p;		/* umbra and penumbra radius, m at moon dist*/
-	double erad;		/* effective earth diam, m */
+	double maxr;
 	int dx, dy;
 	int x, y, r;
 
@@ -2876,33 +2823,33 @@ m_eclipse()
 	db_update (mop);
 	db_update (sop);
 	mrad = degrad(mop->s_size/3600.0)/2.0;
-	maxr = 10 * mrad;
-	cmdec = cos (mop->s_gaedec);
 	scale = mrad / moonrad;
-	adec = -sop->s_gaedec;
 	ara = sop->s_gaera + PI;
-	range (&ara, 2*PI);
-	ddec = mop->s_gaedec - adec;
-	dra = mop->s_gaera - ara;
-	med = mop->s_edist;
-	sed = sop->s_edist;
-	erad = ERAD*1.02;	/* add a little for the atmosphere */
+	adec = -sop->s_gaedec;
+	eq_ecl (mjd, ara, adec, &alat, &alng);
+	eq_ecl (mjd, mop->s_gaera, mop->s_gaedec, &mlat, &mlng);
+	dlat = mlat - alat;
+	dlng = mlng - alng;
+	cmlat = cos (mlat);
 
-	/* forget it if not close */
-	if (fabs(ddec) > maxr || delra(dra)*cmdec > maxr)
+	/* skip if obviously outside fov */
+	maxr = (sqrt((double)mnrows*mnrows + mncols*mncols) + BORD)*scale;
+	if (fabs(dlat) > maxr || delra(dlng)*cmlat > maxr)
 	    return;
 
 	/* find center of circle */
-	dx = (int)floor((dra*cmdec)/scale + 0.5);	/* + right */
-	dy = (int)floor(ddec/scale + 0.5);		/* + down */
+	dx = (int)floor((dlng*cmlat)/scale + 0.5);	/* + right */
+	dy = (int)floor(dlat/scale + 0.5);		/* + down */
 	x = FX(dx+mncols/2) + BORD;
 	y = FY(dy+mnrows/2) + BORD;
 
 	/* compute diameters at moon (based on similar triangles) */
-	u = erad - med/sed*(SRAD-erad);
+	med = mop->s_edist;
+	sed = sop->s_edist;
+	u = ERAD - med/sed*(SRAD-ERAD);
 	r = (int)floor(u/MRAD*moonrad + 0.5);
 	XPSDrawArc (XtD, m_pm, m_fgc, x-r, y-r, 2*r, 2*r, 0, 360*64);
-	p = erad + med/sed*(SRAD-erad);
+	p = ERAD + med/sed*(SRAD-ERAD);
 	r = (int)floor(p/MRAD*moonrad + 0.5);
 	XSetLineAttributes (XtD, m_fgc, 0, LineOnOffDash, CapButt, JoinMiter);
 	XPSDrawArc (XtD, m_pm, m_fgc, x-r, y-r, 2*r, 2*r, 0, 360*64);
@@ -3075,10 +3022,7 @@ Window win;
 	unsigned int gcm;
 	Pixel p;
 
-	if (get_color_resource (mda_w, "GlassBorderColor", &p) < 0) {
-	    xe_msg ("Can not get GlassBorderColor -- using White", 0);
-	    p = WhitePixel (dsp, 0);
-	}
+	(void) get_color_resource (mda_w, "GlassBorderColor", &p);
 	gcm = GCForeground;
 	gcv.foreground = p;
 	glassGC = XCreateGC (dsp, win, gcm, &gcv);
@@ -3271,13 +3215,14 @@ m_readdb()
 	    double lt, lg;
 	    int x, y;
 	    char *new;
-	    int km;
+	    int km, rjust;
 	    int n;
 	    int i;
 
 	    /* crack the line -- just skip it if not recognized */
-	    n = sscanf(buf,"%[^|]|%[^|]|%lf|%lf|%d", name, type, &lt, &lg, &km);
-	    if (n != 5)
+	    n = sscanf(buf,"%[^|]|%[^|]|%lf|%lf|%d|%d", name, type, &lt,
+							    &lg, &km, &rjust);
+	    if (n != 6)
 		continue;
 
 	    /* grow moondb if now filled */
@@ -3331,6 +3276,10 @@ m_readdb()
 	    /* radius as seen overhead, km */
 	    mp->sz = km;
 
+	    /* initialy not persistent label and set justification */
+	    mp->pl = 0;
+	    mp->rjust = rjust;
+
 	    /* all ok */
 	    nmoondb++;
 	}
@@ -3512,6 +3461,15 @@ m_create_popup()
 	pu_alt_w = XmCreateLabel (pu_w, "MPAlt", args, n);
 	wtip (pu_alt_w, "Altitude of Sun above horizon at this feature");
 	XtManageChild (pu_alt_w);
+
+	/* persistent label TB -- managed as needed */
+	n = 0;
+	XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
+	pu_pl_w = XmCreateToggleButton (pu_w, "MPPL", args, n);
+	XtAddCallback (pu_pl_w, XmNvalueChangedCallback, m_pl_cb, 0);
+	set_xmstring (pu_pl_w, XmNlabelString, "Persistent label");
+	wtip (pu_pl_w, "Whether to always label this object on the map.");
 }
 
 /* fill in popup widgets from mp. if mp == NULL use x,y and skip name.
@@ -3535,12 +3493,21 @@ int x, y;
 	    XtManageChild (pu_type_w);
 	    f_showit (pu_type_w, mp->type);
 	    XtManageChild (pu_size_w);
-	    f_double (pu_size_w, "%.0f km", (double)mp->sz);
+	    if (mp->sz < 0)
+		f_double (pu_size_w, "%.0f", -(double)mp->sz);
+	    else if (pref_get(PREF_UNITS) == PREF_ENGLISH)
+		f_double (pu_size_w, "%.0f mi", (double)mp->sz/1.609);
+	    else
+		f_double (pu_size_w, "%.0f km", (double)mp->sz);
+	    XtManageChild (pu_pl_w);
+	    XmToggleButtonSetState (pu_pl_w, mp->pl, False);
+	    set_something (pu_pl_w, XmNuserData, (XtArgVal)mp);
 	    xy2ll (mp->x, mp->y, &lt, &lg);
 	} else {
 	    XtUnmanageChild (pu_name_w);
 	    XtUnmanageChild (pu_type_w);
 	    XtUnmanageChild (pu_size_w);
+	    XtUnmanageChild (pu_pl_w);
 	    xy2ll (FX(x), FY(y), &lt, &lg);
 	}
 
@@ -3624,6 +3591,29 @@ int x, y;
 	return (smallp);
 }
 
+/* called when the persistent label TB is activated in the popup.
+ * userData is the MoonDB entry whose pl we toggle, then redraw.
+ */
+/* ARGSUSED */
+static void
+m_pl_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	MoonDB *mp;
+
+	get_something (w, XmNuserData, (XtArgVal)&mp);
+	mp->pl = XmToggleButtonGetState (w);
+
+	/* draw just labels if adding or redraw all to erase */
+	if (mp->pl) {
+	    m_labels();
+	    m_refresh (NULL);
+	} else
+	    m_redraw();
+}
+
 /* assign the object pointed to by skypu_op to ObjX/Y/Z or in Data Table,
  * depending on client.
  */
@@ -3749,4 +3739,4 @@ Obj *op;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: moonmenu.c,v $ $Date: 1999/02/19 04:16:02 $ $Revision: 1.5 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: moonmenu.c,v $ $Date: 2000/08/25 08:13:51 $ $Revision: 1.23 $ $Name:  $"};
