@@ -24,6 +24,7 @@
 extern Widget toplevel_w;
 extern Colormap xe_cm;
 
+extern int isUp P_((Widget w));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
@@ -44,8 +45,7 @@ static Widget msg_w;		/* main form dialog widget */
 static Widget txt_w;		/* scrolled text widget */
 static int txtl;		/* current length of text in txt_w */
 
-/* called to force the scrolling message window to be up or
- * bring it down if it already is.
+/* called to force the scrolling message window to be up.
  */
 void
 msg_manage()
@@ -53,10 +53,9 @@ msg_manage()
 	if (!msg_w)
 	    msg_create_w();
 
-	if (!XtIsManaged(msg_w))
-	    XtManageChild (msg_w);
-	else
-	    XtUnmanageChild (msg_w);
+	XtManageChild (msg_w);
+	if (XtWindow(msg_w))
+	    XMapRaised (XtDisplay(msg_w), XtWindow(msg_w));
 }
 
 /* ring the bellm but avoid overdoing a lot of them */
@@ -90,7 +89,7 @@ int app_modal;
 	msg_add (msg);
 
 	/* if model, display directly too, and that's all */
-	if (app_modal) {
+	if (app_modal && isUp(toplevel_w)) {
 	    xe_msg_modal (msg);
 	    return;
 	}
@@ -161,7 +160,6 @@ msg_create_w()
 	/* make the help shell form-dialog widget */
 
 	n = 0;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNfractionBase, 9); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
@@ -275,7 +273,6 @@ char *p;
 
 	    /* Create MessageBox dialog. */
 	    n = 0;
-	    XtSetArg (args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg (args[n], XmNtitle, "xephem Message");  n++;
 	    XtSetArg (args[n], XmNokLabelString, button_string);  n++;
 	    XtSetArg (args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
@@ -306,4 +303,4 @@ char *p;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: msgmenu.c,v $ $Date: 1998/02/28 01:27:38 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: msgmenu.c,v $ $Date: 2000/09/12 01:21:07 $ $Revision: 1.4 $ $Name:  $"};
