@@ -5,6 +5,7 @@
 #include <signal.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <netdb.h>
 
 #ifdef __STDC__
 #include <stdlib.h>
@@ -46,11 +47,13 @@ extern XtAppContext xe_app;
 				 */
 
 extern char *syserrstr P_((void));
+extern int isUp P_((Widget w));
 extern void XCheck P_((XtAppContext app));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void watch_cursor P_((int want));
 extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
@@ -69,7 +72,7 @@ static int connect_to P_((int sockfd, struct sockaddr *serv_addr, int addrlen));
 static int stopd_check P_((void));
 static void stopd_cb P_((Widget w, XtPointer data, XtPointer call));
 
-static Widget netform_w;	/* the main form dialog */
+static Widget netshell_w;	/* the main form dialog */
 static Widget ndir_w;		/* net direct TB */
 static Widget socks_w;		/* SOCKS option TB */
 static Widget socksh_w, socksp_w;/* SOCKS host and port TF */
@@ -90,12 +93,19 @@ static int socks_on;		/* whether SOCKS network connection is on */
 static char *socks_host;	/* SOCKS host */
 static char *socks_port;	/* SOCKS port (as a string) */
 
+/* buffer for recvlineb() */
+static char rb_linebuf[512];	/* [next .. bad-1] are good */
+static int rb_next;		/* index of next good char */
+static int rb_unk;		/* index of first unknown char */
+
+static char netcategory[] = "Network";	/* Save category */
+
 /* call to set up without actually bringing up the menus.
  */
 void
 net_create()
 {
-	if (!netform_w) {
+	if (!netshell_w) {
 	    net_create_form();
 	    (void) net_save();	/* confirming here is just annoying */
 	}
@@ -106,12 +116,11 @@ net_manage()
 {
 	net_create();
 
-	if (XtIsManaged(netform_w))
-	    XtUnmanageChild (netform_w);
-	else {
+	if (!isUp (netshell_w))
 	    net_setup();
-	    XtManageChild (netform_w);
-	}
+
+        XtPopup (netshell_w, XtGrabNone);
+	set_something (netshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* open the host, do the given GET cmd, and return a socket fd for the result.
@@ -159,6 +168,7 @@ char msg[];
 /* establish a TCP connection to the named host on the given port.
  * if ok return file descriptor, else -1 with excuse in msg[].
  * try Socks if see socks_on.
+ * reset recvlineb() readahead in case old stuff left behind.
  */
 int
 mkconnection (host, port, msg)
@@ -201,7 +211,7 @@ char msg[];	/* return diagnostic message here, if returning -1 */
 
             SocksPacket.VN = 4;		/* version 4 */
 	    SocksPacket.CD = 1;		/* Command code 1 = connect request */
-            SocksPacket.DSTPORT = htons(port);/*Taken from function parameters*/
+            SocksPacket.DSTPORT = htons((short)port);
 
             if (!hs) {
 		(void) sprintf (msg, "SOCKS: %s:\nCan not get server IP:\n%s.",
@@ -213,7 +223,7 @@ char msg[];	/* return diagnostic message here, if returning -1 */
 	    serv_addr.sin_family = AF_INET;
 	    serv_addr.sin_addr.s_addr=
                               ((struct in_addr *)(hs->h_addr_list[0]))->s_addr;
-	    serv_addr.sin_port = htons(Socks_port);
+	    serv_addr.sin_port = htons((short)Socks_port);
 	    if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
 	        (void) sprintf (msg, "SOCKS: %s/%d:\n%s", socks_host, port,
 								syserrstr());
@@ -257,7 +267,7 @@ char msg[];	/* return diagnostic message here, if returning -1 */
 	    serv_addr.sin_family = AF_INET;
 	    serv_addr.sin_addr.s_addr =
 			((struct in_addr *)(hp->h_addr_list[0]))->s_addr;
-	    serv_addr.sin_port = htons(port);
+	    serv_addr.sin_port = htons((short)port);
 	    if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
 	        (void) sprintf (msg, "%s/%d: %s", host, port, syserrstr());
 	        return (-1);
@@ -270,6 +280,10 @@ char msg[];	/* return diagnostic message here, if returning -1 */
             }
 	}
 
+	/* reset readahead in case user uses recvlineb() */
+	rb_next = rb_unk = 0;
+
+	/* ok */
 	return (sockfd);
 }
 
@@ -373,9 +387,6 @@ int sock;
 char *buf;
 int size;
 {
-	static char linebuf[512];	/* [next .. bad-1] are good */
-	static int next;		/* index of next good char */
-	static int unk;			/* index of first unknown char */
 	char *origbuf = buf;		/* save to prevent overfilling buf */
 	char c = '\0';
 	int nr = 0;
@@ -386,20 +397,20 @@ int size;
 	/* read and copy linebuf[next] to buf until buf fills or copied a \n */
 	do {
 
-	    if (next >= unk) {
+	    if (rb_next >= rb_unk) {
 		/* linebuf is empty -- refill */
 		if (tout (TOUT, sock, 0) < 0) {
 		    nr = -1;
 		    break;
 		}
-		nr = read (sock, linebuf, sizeof(linebuf));
+		nr = read (sock, rb_linebuf, sizeof(rb_linebuf));
 		if (nr <= 0)
 		    break;
-		next = 0;
-		unk = nr;
+		rb_next = 0;
+		rb_unk = nr;
 	    }
 
-	    if ((c = linebuf[next++]) != '\r')
+	    if ((c = rb_linebuf[rb_next++]) != '\r')
 		*buf++ = c;
 
 	} while (buf-origbuf < size && c != '\n');
@@ -417,13 +428,13 @@ int size;
 void
 stopd_up()
 {
+	/* create if first time */
 	if (!stopd_w) {
 	    Arg args[20];
 	    int n;
 
 	    n = 0;
 	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	    XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);  n++;
 	    XtSetArg(args[n], XmNtitle, "xephem Net Stop");  n++;
 	    stopd_w = XmCreateInformationDialog(toplevel_w, "NetStop", args, n);
@@ -433,11 +444,17 @@ stopd_up()
 	    XtUnmanageChild(XmMessageBoxGetChild(stopd_w,
 						    XmDIALOG_CANCEL_BUTTON));
 	    XtUnmanageChild(XmMessageBoxGetChild(stopd_w,XmDIALOG_HELP_BUTTON));
-	    set_xmstring(stopd_w, XmNmessageString, "Press Stop to cancel\nthe network read...");
+	    set_xmstring(stopd_w, XmNmessageString, "Press Stop to cancel...");
 	    set_xmstring(stopd_w, XmNokLabelString, "Stop");
 	}
 
+	/* get it up */
 	XtManageChild (stopd_w);
+	XFlush (XtD);
+	XmUpdateDisplay (toplevel_w);
+	XSync (XtD, 0);
+
+	/* init button not pressed */
 	stopd_stopped = 0;
 }
 
@@ -453,28 +470,30 @@ stopd_down()
 static void
 net_create_form()
 {
+	Widget netform_w;
 	Widget f, w;
 	Arg args[20];
 	int n;
 
 	/* create form */
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNtitle,"xephem Network setup");n++;
+	XtSetArg (args[n], XmNiconName, "Net"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	netshell_w = XtCreatePopupShell ("NetSetup", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (netshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	sr_reg (netshell_w, "XEphem*NetSetup.x", netcategory, 0);
+	sr_reg (netshell_w, "XEphem*NetSetup.y", netcategory, 0);
+
+	n = 0;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	netform_w = XmCreateFormDialog (toplevel_w, "NetSetup", args, n);
-	set_something (netform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (netform_w, XmNmapCallback, prompt_map_cb, NULL);
+	netform_w = XmCreateForm (netshell_w, "NetForm", args, n);
         XtAddCallback (netform_w, XmNhelpCallback, help_cb, NULL);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
-	n = 0;
-	XtSetArg (args[n], XmNtitle,"xephem Network setup");n++;
-	XtSetValues (XtParent(netform_w), args, n);
+	XtManageChild (netform_w);
 
 	/* make the title */
 
@@ -499,6 +518,7 @@ net_create_form()
 	    set_xmstring (ndir_w, XmNlabelString, "Direct connect");
 	    wtip (ndir_w, "Use direct internet connection (no proxy or SOCKS)");
 	    XtManageChild (ndir_w);
+	    sr_reg (ndir_w, NULL, netcategory, 1);
 
 	    /* make the SOCKS toggle and info (first because label is wider) */
 
@@ -513,6 +533,7 @@ net_create_form()
 							(XtPointer)NETSOCKSTB);
 	    wtip (socks_w, "Use SOCKS V4 to Internet");
 	    XtManageChild (socks_w);
+	    sr_reg (socks_w, NULL, netcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -537,6 +558,10 @@ net_create_form()
 	    wtip (socksh_w, "Name of SOCKS host");
 	    XtManageChild (socksh_w);
 
+	    defaultSOCKS();
+	    sr_reg (socksp_w, NULL, netcategory, 1);
+	    sr_reg (socksh_w, NULL, netcategory, 1);
+
 	    /* make the Proxy toggle and info */
 
 	    n = 0;
@@ -550,6 +575,7 @@ net_create_form()
 							(XtPointer)NETPROXTB);
 	    wtip (proxy_w, "Reach the Internet through a proxy");
 	    XtManageChild (proxy_w);
+	    sr_reg (proxy_w, NULL, netcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -561,6 +587,7 @@ net_create_form()
 	    proxyp_w = XmCreateTextField (netform_w, "ProxyPort", args, n);
 	    wtip (proxyp_w, "Proxy port number");
 	    XtManageChild (proxyp_w);
+	    sr_reg (proxyp_w, NULL, netcategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -573,8 +600,7 @@ net_create_form()
 	    proxyh_w = XmCreateTextField (netform_w, "ProxyHost", args, n);
 	    wtip (proxyh_w, "Name of Proxy host");
 	    XtManageChild (proxyh_w);
-
-	    defaultSOCKS();
+	    sr_reg (proxyh_w, NULL, netcategory, 1);
 
 	/* make the controls across the bottom under a separator */
 
@@ -732,7 +758,7 @@ XtPointer client;
 XtPointer call;
 {
 	if (net_save() == 0)
-	    XtUnmanageChild (netform_w);
+	    XtPopdown (netshell_w);
 }
 
 /* called from Ok */
@@ -744,7 +770,7 @@ XtPointer client;
 XtPointer call;
 {
 	/* outta here */
-	XtUnmanageChild (netform_w);
+	XtPopdown (netshell_w);
 }
 
 /* called from any of the choice toggle buttons.
@@ -854,7 +880,6 @@ herr (errmsg)
 char *errmsg;
 {
 #if defined(HOST_NOT_FOUND) && defined(TRY_AGAIN)
-	extern int h_errno;
 	switch (h_errno) {
 	case HOST_NOT_FOUND:
 	    errmsg = "Host Not Found";
@@ -881,7 +906,8 @@ int addrlen;
 #else
 #define NOBLOCK O_NDELAY
 #endif
-	int err, len;
+	unsigned int len;
+	int err;
 	int flags;
 	int ret;
 
@@ -944,4 +970,4 @@ stopd_check()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: netmenu.c,v $ $Date: 1999/10/25 21:09:17 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: netmenu.c,v $ $Date: 2000/10/08 08:09:41 $ $Revision: 1.12 $ $Name:  $"};
