@@ -4,9 +4,14 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
+
 #if defined(__STDC__)
 #include <stdlib.h>
+typedef const void * qsort_arg;
+#else
+typedef void * qsort_arg;
 #endif
+
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -34,9 +39,11 @@ extern Obj *db_scan P_((DBScan *sp));
 extern char *obj_description P_((Obj *op));
 extern int db_n P_((void));
 extern int db_set_field P_((char bp[], int id, PrefDateFormat pref, Obj *op));
+extern int isUp P_((Widget w));
 extern int obj_cir P_((Now *np, Obj *op));
 extern int skyloc_fifo P_((Obj *op));
 extern void all_newobj P_((int dbidx));
+extern void buttonAsButton P_((Widget w, int whether));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_setuserobj P_((int id, Obj *op));
 extern void f_prdec P_((Widget w, double a));
@@ -50,6 +57,7 @@ extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_id P_((Obj *op));
 extern void sv_point P_((Obj *op));
 extern void watch_cursor P_((int want));
@@ -80,7 +88,7 @@ static void obj_set_db P_((void));
 static void obj_setmenu P_((void));
 static void obj_setxyz P_((void));
 static void obj_setobjinfo P_((Field *fp, int nf));
-static void obj_create_form P_((void));
+static void obj_create_shell P_((void));
 static void obj_create_type_rc_form P_((Widget fw, Field *fp));
 static void obj_change_cb P_((Widget w, XtPointer client, XtPointer call));
 static void obj_ctl_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -95,14 +103,13 @@ static int nl_cmp P_((const void *nl1, const void *nl2));
 static void new_namelist P_((void));
 static void load_namelist P_((void));
 static void obj_setnames P_((int nli));
-static void obj_set_a_button P_((Widget pbw, int whether));
 static void prompt_ok_cb P_((Widget w, XtPointer client, XtPointer call));
 static void prompt P_((Field *fp));
 static void obj_set_button P_((Widget w, int id));
 static void epoch_as_decimal P_((Widget w, double e));
 static void epoch_as_mdy P_((Widget w, double e));
 
-static Widget objform_w;	/* overall form Dialog */
+static Widget objshell_w;	/* overall shell */
 static Widget fldrc_w;		/* overall field info rc */
 static Widget scroll_w;		/* name scroll widget */
 static Widget srch_w;		/* the search text field */
@@ -231,6 +238,8 @@ static Field esat_fields[] = {
     	"Revolutions per day^2"},
     {ES_ORBIT, "Orbit reference:", "Orbit number at epoch (integer):",
     	"Orbit number at epoch"},
+    {ES_DRAG, "Drag coefficient:", "Drag coefficient, (earth radii)^-1:",
+    	"Drag coefficient"},
 };
 
 typedef struct {
@@ -289,31 +298,29 @@ enum {
 };
 
 static int newdb;		/* set when db is updated while we are down */
+static char objcategory[] = "Date base -- Search";	/* Save category */
 
 /* called by the main menu pick.
  * create the main form, if this is the first time we've been called.
- * then we toggle each time.
+ * then we go for it.
  */
 void
 obj_manage()
 {
-	if (!objform_w) {
-	    obj_create_form();
+	if (!objshell_w) {
+	    obj_create_shell();
 	    init_working_obj();
 	    newdb = 1;	/* force a fresh list */
 	}
 	
-	if (XtIsManaged(objform_w))
-	    XtUnmanageChild (objform_w);
-	else {
-	    if (newdb) {
-		new_namelist();
-		newdb = 0;
-	    }
-	    obj_setxyz();
-	    obj_setmenu();
-	    XtManageChild (objform_w);
+	if (newdb) {
+	    new_namelist();
+	    newdb = 0;
 	}
+	obj_setxyz();
+	obj_setmenu();
+	XtPopup (objshell_w, XtGrabNone);
+	set_something (objshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* update our view if we are up */
@@ -323,7 +330,7 @@ obj_update (np, howmuch)
 Now *np;
 int howmuch;
 {
-	if (!objform_w || !XtIsManaged(objform_w))
+	if (!isUp(objshell_w))
 	    return;
 
 	obj_setmenu();
@@ -369,7 +376,7 @@ void
 obj_newdb(appended)
 int appended;
 {
-	if (!objform_w || !XtIsManaged(objform_w)) {
+	if (!isUp(objshell_w)) {
 	    newdb = 1;
 	    return;
 	}
@@ -384,8 +391,8 @@ Cursor c;
 {
 	Window win;
 
-	if (objform_w && (win = XtWindow(objform_w)) != 0) {
-	    Display *dsp = XtDisplay(objform_w);
+	if (objshell_w && (win = XtWindow(objshell_w)) != 0) {
+	    Display *dsp = XtDisplay(objshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -393,6 +400,40 @@ Cursor c;
 	}
 }
 
+/* like strcmp() but take care to sort numerics by value and ignore case.
+ */
+int
+strnncmp (s1, s2)
+char *s1, *s2;
+{
+	int p;
+	int q;
+
+	for (;;) {
+	    p = *s1++;
+	    if (islower(p))
+		p = toupper (p);
+	    q = *s2++;
+	    if (islower(q))
+		q = toupper (q);
+	    if (isdigit(p) && isdigit(q)) {
+		int np = 0, nq = 0;
+		do {
+		    np = np*10 + p - '0';
+		    p = *s1;
+		} while (isdigit(p) && s1++);
+		do {
+		    nq = nq*10 + q - '0';
+		    q = *s2;
+		} while (isdigit(q) && s2++);
+		if (np != nq)
+		    return (np - nq);
+	    }
+	    if (!p || !q || p != q)
+		return (p - q);
+	}
+}
+
 /* this was added to support backward compatability with the ability to
  * set OBJXYZ from the ephem.cfg file.
  */
@@ -414,7 +455,7 @@ init_working_obj()
 static void
 obj_unsetmenu()
 {
-	if (!objform_w)
+	if (!objshell_w)
 	    return;
 
 	XtUnmanageChild (typeinfo[objp->o_type].frc_w);
@@ -425,7 +466,7 @@ obj_unsetmenu()
 static void
 obj_setxyz()
 {
-	if (!objform_w)
+	if (!objshell_w)
 	    return;
 
 	XmToggleButtonSetState (objx_w, False, False);
@@ -474,7 +515,7 @@ obj_setmenu()
 {
 	TypeInfo *tp;
 
-	if (!objform_w)
+	if (!objshell_w)
 	    return;
 
 	tp = &typeinfo[objp->o_type];
@@ -496,14 +537,14 @@ int nf;
 	}
 }
 
-/* called once to build the basic form.
+/* called once to build the basic shell and form.
  * N.B. we could not manage to connect the right side of NameF to the left
  * side of ScrollB without complaints from the Obj form. So, to make it look
  * attached, both NameF and ScrollB are attach to the right side of the form
  * but the NameF has an offset equal to the width of ScrollB. hey -- it works!
  */
 static void
-obj_create_form ()
+obj_create_shell ()
 {
 	struct Btns {	/* to streamline creation of control buttons */
 	    int id;
@@ -529,6 +570,7 @@ obj_create_form ()
 	Widget fldrcf_w;
 	Widget xyzrbf_w, xyzrb_w;
 	Widget namerc_w, namercf_w;
+	Widget objform_w;
 	Widget ctlfr_w, ctlf_w;
 	Widget skyfr_w, skyf_w;
 	Widget srchf_w, srchrc_w;
@@ -537,22 +579,27 @@ obj_create_form ()
 	int n;
 	int i;
 
+	/* create shell */
+	n = 0;
+	XtSetArg (args[n], XmNallowShellResize, True); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Search and ObjXYZ"); n++;
+	XtSetArg (args[n], XmNiconName, "Objects"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	objshell_w = XtCreatePopupShell ("Objects", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (objshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	sr_reg (objshell_w, "XEphem*Objects.x", objcategory, 0);
+	sr_reg (objshell_w, "XEphem*Objects.y", objcategory, 0);
+
 	/* create form */
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNallowOverlap, False); n++;
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	objform_w = XmCreateFormDialog (toplevel_w, "Obj", args, n);
+	objform_w = XmCreateForm (objshell_w, "ObjF", args, n);
 	set_something (objform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (objform_w, XmNhelpCallback, obj_ctl_cb, (XtPointer)HELP);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
-	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Search and ObjXYZ"); n++;
-	XtSetValues (XtParent(objform_w), args, n);
+	XtManageChild (objform_w);
 
 	/* make the type control radio box in a frame.
 	 * we save the widget ids so we can force the type to be what
@@ -897,7 +944,7 @@ XtPointer call;
 	case OK:
 	    obj_set_db();
 	    all_newobj(objidx);
-	    XtUnmanageChild (objform_w);
+	    XtPopdown (objshell_w);
 	    break;
 	case APPLY:
 	    obj_set_db();
@@ -939,7 +986,7 @@ XtPointer call;
 		xe_msg ("No current object", 1);
 	    break;
 	case CANCEL:
-	    XtUnmanageChild (objform_w);
+	    XtPopdown (objshell_w);
 	    break;
 	case HELP:
 	    do_help();
@@ -1183,39 +1230,11 @@ char *n1, *n2;
  */
 static int
 nl_cmp (nl1, nl2)
-#if defined(__STDC__) || defined(__cplusplus)
-const
-#endif
-void *nl1, *nl2;
+qsort_arg nl1, nl2;
 {
 	char *s1 = ((NameList *)nl1)->op->o_name;
 	char *s2 = ((NameList *)nl2)->op->o_name;
-	int p;
-	int q;
-
-	for (;;) {
-	    p = *s1++;
-	    if (islower(p))
-		p = toupper (p);
-	    q = *s2++;
-	    if (islower(q))
-		q = toupper (q);
-	    if (isdigit(p) && isdigit(q)) {
-		int np = 0, nq = 0;
-		do {
-		    np = np*10 + p - '0';
-		    p = *s1;
-		} while (isdigit(p) && s1++);
-		do {
-		    nq = nq*10 + q - '0';
-		    q = *s2;
-		} while (isdigit(q) && s2++);
-		if (np != nq)
-		    return (np - nq);
-	    }
-	    if (!p || !q || p != q)
-		return (p - q);
-	}
+	return (strnncmp (s1, s2));
 }
 
 /* set up namelist for a new list of db objects.
@@ -1304,12 +1323,12 @@ int nli;
 	    if (bi < nb) {
 		namep = namelist[nli].op->o_name;
 		XtSetArg (args[n], XmNuserData, nli); n++;
-		obj_set_a_button(namepb_w[bi], 1);
+		buttonAsButton(namepb_w[bi], 1);
 		nli++;
 	    } else {
 		namep = noname;
 		XtSetArg (args[n], XmNuserData, -1); n++;
-		obj_set_a_button(namepb_w[bi], 0);
+		buttonAsButton(namepb_w[bi], 0);
 	    }
 	    str = XmStringCreateLtoR (namep, XmSTRING_DEFAULT_CHARSET);
 	    XtSetArg (args[n], XmNlabelString, str); n++;
@@ -1318,78 +1337,6 @@ int nli;
 	}
 }
 
-/* set whether the given button looks like a label.
- */
-static void
-obj_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
-{
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    Widget tmpw;
-	    int n;
-
-	    n = 0;
-	    tmpw = XmCreatePushButton (objform_w, "tmp", args, n);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-	    XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    XtDestroyWidget (tmpw);
-	     
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
-}
-
 /* user typed OK to a prompt for fp. get his new value and use it.
  * get fp from userData.
  */
@@ -1436,7 +1383,6 @@ Field *fp;
 	    title = XmStringCreate("xephem Object Prompt",
 						    XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
-	    XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);  n++;
 	    XtSetArg(args[n], XmNdialogTitle, title);  n++;
 	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
@@ -1644,9 +1590,12 @@ int id;
 	case ES_ORBIT:
 	    f_double (w, "%g", (double)objp->es_orbit);
 	    break;
+	case ES_DRAG:
+	    f_double (w, "%g", objp->es_drag);
+	    break;
 
 	default:
-	    printf ("%s: bad parabolic id: %d\n", me, id);
+	    printf ("%s: bad id: %d\n", me, id);
 	    exit (1);
 	}
 }
@@ -1673,4 +1622,4 @@ double e;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: objmenu.c,v $ $Date: 1999/02/15 23:21:12 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: objmenu.c,v $ $Date: 2000/09/26 23:32:13 $ $Revision: 1.9 $ $Name:  $"};
