@@ -32,7 +32,7 @@ static void mk_gcs P_((Widget w));
 static int x_ticks P_((int asdate, double minx, double maxx, int maxticks,
     double ticks[]));
 static int draw_x_label P_((Display *dsp, Window win, int asdate, double v,
-    int x, int y));
+    int x, int y, int w));
 
 /* maximum number of unique functions (ie tags) in the plot file.
  * to be fair, it should be at least as large as the value in plot.
@@ -59,11 +59,13 @@ int grid;		/* whether to include a grid with the tick marks */
 int xasdate;		/* whether to show x axis as dates */
 {
 #define	TL	5	/* tick mark length, in pixels */
+#define	RB	30	/* right border */
 #define	NT	10	/* rough number of tick marks on each axis */
 	Display *dsp = XtDisplay(widget);
 	Window win = XtWindow(widget);
-	double ticks[NT+2];
-	int nt;
+	double xticks[NT+2];
+	double yticks[NT+2];
+	int nxt, nyt;
 	static char fmt[] = "%c,%lf,%lf";
 	double x, y;	/* N.B. be sure these match what scanf's %lf wants*/
 	double minx=0, maxx=0, miny=0, maxy=0, xscale, yscale;
@@ -77,7 +79,7 @@ int xasdate;		/* whether to show x axis as dates */
 	int ix, iy;	/* misc X drawing coords */
 	int x0;		/* X x coord of lower left of plotting box, in pixels */
 	int y0;		/* X y coord of lower left of plotting box, in pixels */
-	int h, w;	/* width of height of plotting box, in pixels */
+	int w, h;	/* width and height of plotting box, in pixels */
 	int asc, desc;	/* font ascent and descent, in pixels */
 	int maxlblw;	/* width of longest y axis label, in pixels */
 	int i;
@@ -100,7 +102,7 @@ int xasdate;		/* whether to show x axis as dates */
 		if (nlines == MAXPLTLINES) {
 		   (void) sprintf (buf,
 		     "Plot file contains more than %d functions.", MAXPLTLINES);
-		   xe_msg (buf, 0);
+		   xe_msg (buf, 1);
 		   return(-1);
 		}
 		tags[nlines++] = c;
@@ -112,12 +114,12 @@ int xasdate;		/* whether to show x axis as dates */
 	}
 
 	if (nlines == 0) {
-	    xe_msg ("Plot file appears to be empty.", 0);
+	    xe_msg ("Plot file appears to be empty.", 1);
 	    return(-1);
 	}
 #define	SMALL	(1e-6)
 	if (fabs(minx-maxx) < SMALL || fabs(miny-maxy) < SMALL) {
-	    xe_msg ("Plot file values contain insufficient spread.", 0);
+	    xe_msg ("Plot file values contain insufficient spread.", 1);
 	    return(-1);
 	}
 
@@ -125,79 +127,64 @@ int xasdate;		/* whether to show x axis as dates */
 	if (!plt_gc)
 	    mk_gcs (widget);
 	XSetForeground (dsp, plt_gc, da_p);
-	
-	/* pick a font to use in case we are printing */
-	XPSRegisterFont (plt_fs->fid, "Helvetica");
+	XSetFont (dsp, plt_gc, plt_fs->fid);
 
-	/* add y-axis tick marks.
-	 * first compute length of longest y-axis label and other char stuff.
+	/* decide tickmarks */
+	nxt = x_ticks (xasdate, minx, maxx, NT, xticks);
+	minx = xticks[0];
+	maxx = xticks[nxt-1];
+	nyt = tickmarks (miny, maxy, NT, yticks);
+	miny = yticks[0];
+	maxy = yticks[nyt-1];
+
+	/* compute length of longest y-axis label and other char stuff.
 	 */
-	nt = tickmarks (miny, maxy, NT, ticks);
 	maxlblw = 0;
-	for (i = 0; i < nt; i++) {
+	for (i = 0; i < nyt; i++) {
 	    int dir, l;
-
-	    if (ticks[i] < miny || ticks[i] > maxy)
-		continue;
-	    (void) sprintf (buf, "%g", ticks[i]);
+	    (void) sprintf (buf, "%g", yticks[i]);
 	    l = strlen(buf);
 	    XTextExtents (plt_fs, buf, l, &dir, &asc, &desc, &overall);
 	    if (overall.width > maxlblw)
 		maxlblw = overall.width;
 	}
 
-	/* now we can compute border sizes and the scaling factors */
+	/* compute border sizes and the scaling factors */
 	x0 = maxlblw+TL+10;
-	w = nx - x0 - 30;
+	w = nx - x0 - RB;
 	y0 = ny - (asc+desc+2+2*TL);
 	h = y0 - (asc+desc+2);
 	xscale = w/(maxx-minx);
 	yscale = h/(maxy-miny);
 
-	/* now draw y axis, its labels, and optionally the horizontal grid */
-	for (i = 0; i < nt; i++) {
+	/* draw y axis, its labels, and optionally the horizontal grid */
+	for (i = 0; i < nyt; i++) {
 	    int l;
-
-	    if (ticks[i] < miny || ticks[i] > maxy)
-		continue;
-	    (void) sprintf (buf, "%g", ticks[i]);
+	    (void) sprintf (buf, "%g", yticks[i]);
 	    l = strlen(buf);
-	    iy = YCORD(ticks[i]);
+	    iy = YCORD(yticks[i]);
 	    XPSDrawLine (dsp, win, plt_gc, x0-TL, iy, x0, iy);
 	    XPSDrawString (dsp, win, plt_gc, 1, iy+(asc-desc)/2, buf, l);
 	    if (grid)
 		XPSDrawLine (dsp, win, plt_gc, x0, iy, x0+w-1, iy);
 	}
 
-	/* now draw x axis and label it's first and last tick mark.
+	/* draw x axis and label it's first and last tick mark.
 	 * if there's room, label the center tickmark too.
 	 * also grid, if requested.
 	 */
-	nt = x_ticks (xasdate, minx, maxx, NT, ticks);
 	ylblw = 0;
-	for (i = 0; i < nt; i++) {
-	    if (ticks[i] < minx || ticks[i] > maxx)
-		continue;
-	    ix = XCORD(ticks[i]);
+	for (i = 0; i < nxt; i++) {
+	    ix = XCORD(xticks[i]);
 	    XPSDrawLine (dsp, win, plt_gc, ix, y0+TL, ix, y0);
 	    if (grid)
 		XPSDrawLine (dsp, win, plt_gc, ix, y0, ix, y0-h-1);
 	}
-	for (i = 0; i < nt; i++)
-	    if (ticks[i] >= minx) {
-		ylblw += draw_x_label (dsp, win, xasdate, ticks[i],
-							XCORD(ticks[i]), y0);
-		break;
-	    }
-	for (i = nt; --i >= 0;)
-	    if (ticks[i] <= maxx) {
-		ylblw += draw_x_label (dsp, win, xasdate, ticks[i],
-							XCORD(ticks[i]), y0);
-		break;
-	    }
+	ylblw += draw_x_label (dsp, win, xasdate, minx, XCORD(minx),y0,w+x0+RB);
+	ylblw += draw_x_label (dsp, win, xasdate, maxx, XCORD(maxx),y0,w+x0+RB);
 	if (ylblw < w/2)
-	    (void) draw_x_label (dsp, win, xasdate, ticks[nt/2],
-	    						XCORD(ticks[nt/2]), y0);
+	    (void) draw_x_label (dsp, win, xasdate, xticks[nxt/2],
+					    XCORD(xticks[nxt/2]), y0, w+x0+RB);
 
 	/* draw border of actual plotting area */
 	XPSDrawLine (dsp, win, plt_gc, x0, y0-h, x0, y0);
@@ -265,7 +252,6 @@ Widget w;
 	plt_gc = XCreateGC (dsp, win, 0L, NULL);
 	XSetForeground (dsp, plt_gc, da_p);
 	get_views_font (dsp, &plt_fs);
-	XSetFont (dsp, plt_gc, plt_fs->fid);
 
 	/* fill in plt_p array with pixels to use for function plotting.
 	 * Use colors defined in the plotColor[0-9] resources, else reuse fg.
@@ -289,77 +275,75 @@ int maxticks;
 double ticks[];
 {
 	double jd, minjd, maxjd;
-	double d;
-	int m, y;
-	int nmonths;
+	double d0, d1;
+	int m0, m1, y0, y1;
 	int nt;
 	int i;
 
 	if (!asdate)
 	    return (tickmarks (minx, maxx, maxticks, ticks));
 
+	/* find spanning period in whole months */
 	year_mjd (minx, &minjd);
+	mjd_cal (minjd, &m0, &d0, &y0);
 	year_mjd (maxx, &maxjd);
+	mjd_cal (maxjd, &m1, &d1, &y1);
 
-	nmonths = (int)floor((maxjd-minjd)/28.0); /* approx is ok */
-
-	if (nmonths >= maxticks-1) {	/* counting fenceposts :-) */
+	/* use month ticks if spans well more than a month */
+	if (maxjd - minjd > 40) {
 	    /* put ticks on month boundaries */
-	    int delmonths;
+	    int dt, nm;
+	    double jd0;
 
-	    delmonths = (nmonths+maxticks-1)/maxticks; /* round up */
-	    nt = nmonths/delmonths + 1;		/* include the end one */
-	    if (nt > maxticks) nt = maxticks;	/* can't happen :-) */
-	    mjd_cal (minjd, &m, &d, &y);
-	    d = 1.0;
-	    if (++m > 12) {
-		m = 1;
-		y++;
+	    if (d1 > 1 && ++m1 > 12) {		/* next whole month */
+		m1 = 1;
+		y1++;
 	    }
-	    cal_mjd (m, d, y, &jd);
-	    mjd_year (jd, &ticks[0]);
-	    for (i = 1; i < nt; i++) {
-		m += delmonths;
-		while (m > 12) {
-		    m -= 12;
-		    y++;
+	    nm = (y1*12+m1) - (y0*12+m0);	/* period */
+	    dt = nm/maxticks + 1;		/* step size */
+	    nt = (nm+(dt-1))/dt + 1;		/* n ticks */
+	    for (i = 0; i < nt; i++) {
+		cal_mjd (m0, 1.0, y0, &jd0);
+		mjd_year (jd0, &ticks[i]);
+		m0 += dt;
+		while (m0 > 12) {
+		    m0 -= 12;
+		    y0++;
 		}
-		cal_mjd (m, d, y, &jd);
-		mjd_year (jd, &ticks[i]);
 	    }
 	} else {
 	    /* put ticks on day boundaries */
-	    int ndays, deldays;
+	    int nd, dt;
 
-	    ndays = (int)floor(maxjd-minjd);
-	    deldays = (ndays+maxticks-1)/maxticks; /* round up */
-	    nt = ndays/deldays + 1;		/* include the end one */
-	    if (nt > maxticks) nt = maxticks;	/* can't happen :-) */
-	    jd = mjd_day (minjd+0.999999);	/* round up */
+	    nd = maxjd - minjd + 1;		/* period */
+	    dt = nd/maxticks + 1;		/* round up */
+	    nt = (nd+(dt-1))/dt + 1;		/* n ticks */
+	    jd = mjd_day (minjd);
 	    for (i = 0; i < nt; i++) {
 		mjd_year (jd, &ticks[i]);
-		jd += deldays;
+		jd += dt;
 	    }
 	}
 
 	return (nt);
 }
 
-/* draw v at [x,y].
+/* draw v centered at [x,y].
  * return width of string in pixels.
  */
 static int
-draw_x_label (dsp, win, asdate, v, x, y)
+draw_x_label (dsp, win, asdate, v, x, y, w)
 Display *dsp;
 Window win;
 int asdate;
 double v;
 int x, y;
+int w;
 {
 	int dir, asc, des;
 	XCharStruct ovl;
 	char buf[128];
-	int l;
+	int l, lx;
 
 	if (asdate) {
 	    double jd;
@@ -371,11 +355,17 @@ int x, y;
 	l = strlen(buf);
 	XTextExtents (plt_fs, buf, l, &dir, &asc, &des, &ovl);
 	XSetForeground (dsp, plt_gc, da_p);
+	XSetFont (dsp, plt_gc, plt_fs->fid);
 	XPSDrawLine (dsp, win, plt_gc, x, y+TL, x, y+2*TL);
-	XPSDrawString (dsp, win, plt_gc, x-ovl.width/2, y+2*TL+asc+1, buf, l);
+	lx = x-ovl.width/2;
+	if (lx < 0)
+	    lx = 1;
+	else if (lx + ovl.width > w)
+	    lx = w - ovl.width;
+	XPSDrawString (dsp, win, plt_gc, lx, y+2*TL+asc+1, buf, l);
 
 	return (ovl.width);
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plot_aux.c,v $ $Date: 1997/04/21 23:19:45 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plot_aux.c,v $ $Date: 2000/11/30 06:05:51 $ $Revision: 1.6 $ $Name:  $"};
