@@ -14,13 +14,14 @@
 #include <Xm/Form.h>
 #include <Xm/Frame.h>
 #include <Xm/DrawingA.h>
+#include <Xm/FileSB.h>
 #include <Xm/Label.h>
 #include <Xm/CascadeB.h>
 #include <Xm/PushB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/Separator.h>
-#include <Xm/Text.h>
+#include <Xm/TextF.h>
 
 #include "P_.h"
 #include "ps.h"
@@ -30,15 +31,19 @@ extern Widget	toplevel_w;
 extern Colormap xe_cm;
 
 extern FILE *fopenh P_((char *name, char *how));
-extern char *getXRes P_((char *name, char *def));
+extern char *getPrivateDir P_((void));
+extern char *getShareDir P_((void));
 extern char *syserrstr P_((void));
 extern int confirm P_((void));
 extern int existsh P_((char *filename));
+extern int isUp P_((Widget shell));
 extern int plot_cartesian P_((FILE *pfp, Widget widget, unsigned int nx,
     unsigned int ny, int flipx, int flipy, int grid, int xasdate));
 extern void all_selection_mode P_((int whether));
+extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
 extern void f_string P_((Widget w, char *s));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
+extern void get_xmstring P_((Widget w, char *resource, char **txtp));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
@@ -46,13 +51,19 @@ extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     void (*func3)(void)));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
+extern void sr_unreg P_((Widget w));
 extern void watch_cursor P_((int want));
 extern void wtip P_((Widget w, char *string));
 extern void xe_msg P_((char *msg, int app_modal));
 
 static void plt_select P_((int whether));
-static void plot_create_form P_((void));
+static void plot_create_shell P_((void));
 static void plt_activate_cb P_((Widget w, XtPointer client, XtPointer call));
+static void plt_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
+static void plt_browse_cb P_((Widget w, XtPointer client, XtPointer call));
+static void plt_fsbcancel_cb P_((Widget w, XtPointer client, XtPointer call));
+static void plt_fsbok_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void plt_reset P_((void));
@@ -82,7 +93,8 @@ static void plt_drawall P_((Widget da_w));
 #define	MAXPLTLINES	10
 #define	MAXFLDNAM	32	/* longest allowed field name */
 
-static Widget plotform_w;
+static Widget plotshell_w;	/* main shell */
+static Widget fsb_w;		/* FSB */
 static Widget select_w, active_w, prompt_w;
 static Widget title_w, filename_w;
 static Widget table_w[MAXPLTLINES][4];	/* column indeces follow.. */
@@ -99,6 +111,8 @@ static Widget da_w_save;	/* used to set up for a print action */
 /* plt_activate_cb client values. */
 enum { SELECT, ACTIVE, SHOW};
 
+static char plotcategory[] = "Tools -- Plot";	/* Save category */
+
 /* store the name of each x and y line to track and their values.
  * we get the label straight from the Text widget in the table as needed.
  */
@@ -121,24 +135,24 @@ typedef struct {
     char *filename;	/* name of file being plotted (also malloced) */
     FILE *fp;		/* FILE pointer for the file */
     int flipx, flipy;	/* flip state for this instance */
+    Widget fx_w, fy_w;	/* widgets for sr_unreg() */
     int grid;		/* whether to include a grid */
     int xasdate;	/* whether to show x axis as dates */
+    Widget as_w, g_w;	/* widgets for sr_unreg() */
 } DrawInfo;
 
 /* called when the plot menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, just get going.
  */
 void
 plot_manage ()
 {
-	if (!plotform_w)
-	    plot_create_form();
+	if (!plotshell_w)
+	    plot_create_shell();
 	
-	if (XtIsManaged(plotform_w))
-	    XtUnmanageChild (plotform_w);
-	else
-	    XtManageChild (plotform_w);
+	XtPopup (plotshell_w, XtGrabNone);
+	set_something (plotshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* called by the other menus (data, etc) as their buttons are
@@ -150,9 +164,7 @@ char *name;
 {
 	Widget tw;
 
-	if (!plotform_w
-		|| !XtIsManaged(plotform_w)
-		|| !XmToggleButtonGetState(select_w))
+	if (!isUp(plotshell_w) || !XmToggleButtonGetState(select_w))
 	    return;
 
 	tw = table_w[npltlines][selecting_xy];
@@ -205,7 +217,7 @@ plot()
 	    /* plot in order of original selection */
 	    PltLine *plp;
 	    for (plp = pltlines; plp < &pltlines[npltlines]; plp++) {
-		char *lbl = XmTextGetString (table_w[plp-pltlines][T]);
+		char *lbl = XmTextFieldGetString (table_w[plp-pltlines][T]);
 		(void) fprintf (plt_fp, "%c,%.12g,%.12g\n", lbl[0],
 						    plp->pl_xv, plp->pl_yv);
 		XtFree (lbl);
@@ -226,8 +238,8 @@ Cursor c;
 {
 	Window win;
 
-	if (plotform_w && (win = XtWindow(plotform_w)) != 0) {
-	    Display *dsp = XtDisplay(plotform_w);
+	if (plotshell_w && (win = XtWindow(plotshell_w)) != 0) {
+	    Display *dsp = XtDisplay(plotshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -246,7 +258,7 @@ int whether;
 }
 
 static void
-plot_create_form()
+plot_create_shell()
 {
 	static struct {
 	    char *title;
@@ -256,34 +268,40 @@ plot_create_form()
 	} tbs[] = {
 	    {"Select fields", SELECT, &select_w,
 	    	"When on, data fields eligible for plotting are selectable buttons"},
-	    {"Plot to file", ACTIVE, &active_w,
+	    {"Create plot file", ACTIVE, &active_w,
 	    	"When on, selected fields are written to the named file at each main Update"},
 	    {"Show plot file", SHOW, NULL, "Display the named plot file"},
 	};
 	XmString str;
 	Widget f_w, rc_w;
+	Widget plotform_w;
 	Widget w;
-	char fname[1024];
 	Arg args[20];
 	int i, n;
 
 	/* create the main dialog */
 
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Plot Control"); n++;
+	XtSetArg (args[n], XmNiconName, "Plot"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	plotshell_w = XtCreatePopupShell ("Plot", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (plotshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (plotshell_w, XmNpopdownCallback, plt_popdown_cb, NULL);
+	sr_reg (plotshell_w, "XEphem*Plot.x", plotcategory, 0);
+	sr_reg (plotshell_w, "XEphem*Plot.y", plotcategory, 0);
+
+	n = 0;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
-	plotform_w = XmCreateFormDialog (toplevel_w, "Plot", args, n);
-	set_something (plotform_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
+	plotform_w = XmCreateForm (plotshell_w, "PlotF", args, n);
 	XtAddCallback (plotform_w, XmNhelpCallback, plt_help_cb, 0);
-
-	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Plot Control"); n++;
-	XtSetValues (XtParent(plotform_w), args, n);
+	XtManageChild (plotform_w);
 
 	/* make a RowColumn to hold everything */
 
@@ -292,7 +310,7 @@ plot_create_form()
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNspacing, 8); n++;
+	XtSetArg (args[n], XmNspacing, 6); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
 	rc_w = XmCreateRowColumn(plotform_w, "PlotRC", args, n);
@@ -326,12 +344,18 @@ plot_create_form()
 	XmStringFree (str);
 	XtManageChild (w);
 
-	(void) sprintf (fname, "%s/ephem.plt", getXRes ("PrivateDir", "work"));
 	n = 0;
-	XtSetArg (args[n], XmNvalue, fname); n++;
-	filename_w = XmCreateText (rc_w, "Filename", args, n);
+	w = XmCreatePushButton (rc_w, "Browse...", args, n);
+	XtAddCallback (w, XmNactivateCallback, plt_browse_cb, NULL);
+	wtip (w, "Open a file selection box for hunting .plt files");
+	XtManageChild (w);
+
+	n = 0;
+	filename_w = XmCreateTextField (rc_w, "Filename", args, n);
+	defaultTextFN (filename_w, 1, getShareDir(), "auxil/analemma.plt");
 	wtip (filename_w, "Enter name of file to write");
 	XtManageChild (filename_w);
+	sr_reg (filename_w, NULL, plotcategory, 1);
 
 	/* create title text area and its label */
 
@@ -344,7 +368,8 @@ plot_create_form()
 	XmStringFree (str);
 
 	n = 0;
-	title_w = XmCreateText (rc_w, "PlotTitle", args, n);
+	XtSetArg (args[n], XmNcolumns, 40); n++;
+	title_w = XmCreateTextField (rc_w, "PlotTitle", args, n);
 	wtip (title_w, "Enter a title to be written to the file");
 	XtManageChild (title_w);
 
@@ -386,7 +411,7 @@ plot_create_form()
 	    } else {
 		XtSetArg (args[n], XmNmaxLength, 1); n++;
 		XtSetArg (args[n], XmNcolumns, 1); n++;
-		table_w[i][T] = XmCreateText (f_w, "PlotTag", args, n);
+		table_w[i][T] = XmCreateTextField (f_w, "PlotTag", args, n);
 		XtManageChild (table_w[i][T]);
 	    }
 
@@ -508,6 +533,84 @@ XtPointer call;
 	}
 }
 
+/* callback from the FSB Cancel PB */
+/* ARGSUSED */
+static void
+plt_fsbcancel_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (fsb_w)
+	    XtUnmanageChild (fsb_w);
+}
+
+/* callback from the FSB Ok PB */
+/* ARGSUSED */
+static void
+plt_fsbok_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (fsb_w) {
+	    char *fn;
+
+	    get_xmstring (fsb_w, XmNdirSpec, &fn);
+	    XmTextFieldSetString (filename_w, fn);
+	    XtFree (fn);
+	    XtUnmanageChild (fsb_w);
+	}
+}
+
+/* callback from the Browse PB */
+/* ARGSUSED */
+static void
+plt_browse_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (!fsb_w) {
+	    Widget w;
+	    Arg args[20];
+	    int n;
+
+	    n = 0;
+	    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
+	    XtSetArg (args[n], XmNtitle, "Plot File browser"); n++;
+	    XtSetArg (args[n], XmNwidth, 400); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 10); n++;
+	    XtSetArg (args[n], XmNmarginWidth, 10); n++;
+	    fsb_w = XmCreateFileSelectionDialog (toplevel_w, "PFSB", args, n);
+	    XtAddCallback (fsb_w, XmNmapCallback, prompt_map_cb, NULL);
+	    XtAddCallback (fsb_w, XmNokCallback, plt_fsbok_cb, NULL);
+	    XtAddCallback (fsb_w, XmNcancelCallback, plt_fsbcancel_cb, NULL);
+	    set_xmstring (fsb_w, XmNcancelLabelString, "Close");
+
+	    /* set default dir and pattern */
+	    set_xmstring (fsb_w, XmNdirectory, getPrivateDir());
+	    set_xmstring (fsb_w, XmNpattern, "*.plt");
+
+	    w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_HELP_BUTTON);
+	    XtUnmanageChild (w);
+	}
+
+	XtManageChild (fsb_w);
+}
+
+/* callback when the main plot window is closed */
+/* ARGSUSED */
+static void
+plt_popdown_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	if (fsb_w)
+	    XtUnmanageChild (fsb_w);
+}
+
 /* callback from the Close button.
  */
 /* ARGSUSED */
@@ -517,7 +620,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	XtUnmanageChild (plotform_w);
+	XtPopdown (plotshell_w);
 }
 
 /* callback from the Help button.
@@ -582,7 +685,7 @@ init_next_row()
 	f_string (prompt_w, "Select quantity for X..");
 
 	(void) sprintf (buf, "%c", 'A' + npltlines);
-	XmTextSetString (table_w[npltlines][T], buf);
+	XmTextFieldSetString (table_w[npltlines][T], buf);
 
 	f_string (table_w[npltlines][X], " ");
 	f_string (table_w[npltlines][Y], " ");
@@ -620,11 +723,11 @@ plt_try_cancel()
 static void
 plt_try_turn_on()
 {
-	char *txt = XmTextGetString (filename_w);
+	char *txt = XmTextFieldGetString (filename_w);
 	if (existsh (txt) == 0 && confirm()) {
 	    char *buf;
 	    buf = XtMalloc (strlen(txt)+100);
-	    (void) sprintf (buf, "%s exists: Append or Overwrite?", txt);
+	    (void) sprintf (buf, "%s exists:\nAppend or Overwrite?", txt);
 	    query (toplevel_w, buf, "Append", "Overwrite", "Cancel",
 			    plt_try_append, plt_try_overwrite, plt_try_cancel);
 	    XtFree (buf);
@@ -644,13 +747,13 @@ char *how;	/* fopen how argument */
 	char *txt;
 
 	/* plotting is on if file opens ok */
-	txt = XmTextGetString (filename_w);
+	txt = XmTextFieldGetString (filename_w);
 	plt_fp = fopenh (txt, how);
 	if (!plt_fp) {
 	    char *buf;
 	    XmToggleButtonSetState (active_w, False, False);
 	    buf = XtMalloc (strlen(txt)+100);
-	    (void) sprintf (buf, "Can not open %s: %s", txt, syserrstr());
+	    (void) sprintf (buf, "%s:\n%s", txt, syserrstr());
 	    xe_msg (buf, 1);
 	    XtFree (buf);
 	}
@@ -658,7 +761,7 @@ char *how;	/* fopen how argument */
 	
 	if (plt_fp) {
 	    /* add a title if it's not null */
-	    txt = XmTextGetString (title_w);
+	    txt = XmTextFieldGetString (title_w);
 	    if (txt[0] != '\0')
 		(void) fprintf (plt_fp, "* %s\n", txt);
 	    XtFree (txt);
@@ -690,12 +793,12 @@ plt_da_manage()
 	char *fn;
 
 	/* first make sure we can open the plot file */
-	fn = XmTextGetString (filename_w);
+	fn = XmTextFieldGetString (filename_w);
 	fp = fopenh (fn, "r");
 	if (!fp) {
 	    char *buf;
 	    buf = XtMalloc (strlen(fn)+100);
-	    (void) sprintf (buf, "Can not open %s: %s", fn, syserrstr());
+	    (void) sprintf (buf, "%s:\n%s", fn, syserrstr());
 	    xe_msg (buf, 1);
 	    XtFree (buf);
 	    XtFree (fn);
@@ -705,7 +808,7 @@ plt_da_manage()
 	/* create the form dialog parent. */
 	n = 0;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	daform_w = XmCreateFormDialog (toplevel_w, "PlotDisplay", args, n);
+	daform_w = XmCreateFormDialog (plotshell_w, "PlotDisplay", args, n);
 	set_something (daform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (daform_w, XmNmapCallback, prompt_map_cb, NULL);
 
@@ -813,10 +916,14 @@ plt_da_manage()
 	    XtSetArg (args[n], XmNlabelString, str); n++;
 	    w = XmCreateToggleButton(pd_w, "FlipX", args, n);
 	    di->flipx = XmToggleButtonGetState (w);
+	    di->fx_w = w;
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_flipx_cb, da_w);
 	    wtip (w, "Flip left-to-right");
 	    XtManageChild (w);
+	    /* TODO - can not use until fix findWFB()
+	     * sr_reg (w, NULL, plotcategory, 1);
+	     */
 
 	    str = XmStringCreate("Flip Y", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
@@ -825,19 +932,27 @@ plt_da_manage()
 	    XtSetArg (args[n], XmNlabelString, str); n++;
 	    w = XmCreateToggleButton(pd_w, "FlipY", args, n);
 	    di->flipy = XmToggleButtonGetState (w);
+	    di->fy_w = w;
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_flipy_cb, da_w);
 	    wtip (w, "Flip top-to-bottom");
 	    XtManageChild (w);
+	    /* TODO - can not use until fix findWFB()
+	     * sr_reg (w, NULL, plotcategory, 1);
+	     */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
 	    w = XmCreateToggleButton(pd_w, "Grid", args, n);
 	    di->grid = XmToggleButtonGetState (w);
+	    di->g_w = w;
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_grid_cb, da_w);
 	    wtip (w, "Overlay plot with calibrated grid");
 	    XtManageChild (w);
+	    /* TODO - can not use until fix findWFB()
+	     * sr_reg (w, NULL, plotcategory, 1);
+	     */
 
 	    str = XmStringCreate("Show X-Axis as Dates",
 						    XmSTRING_DEFAULT_CHARSET);
@@ -847,10 +962,14 @@ plt_da_manage()
 	    XtSetArg (args[n], XmNlabelString, str); n++;
 	    w = XmCreateToggleButton(pd_w, "XAsDate", args, n);
 	    di->xasdate = XmToggleButtonGetState (w);
+	    di->as_w = w;
 	    XmStringFree (str);
 	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_xasdate_cb, da_w);
 	    wtip (w, "Assume ordinate values are decimal years and display as dates");
 	    XtManageChild (w);
+	    /* TODO - can not use until fix findWFB()
+	     * sr_reg (w, NULL, plotcategory, 1);
+	     */
 
 	/* go. the expose will do the actual plotting */
 	XtManageChild (daform_w);
@@ -871,6 +990,12 @@ Widget da_w;
 	(void) fclose (di->fp);
 	XtFree(di->filename);
 	XtFree((char *)di);
+	/* TODO: don't do these until fix findWFB() 
+	sr_unreg (di->fx_w);
+	sr_unreg (di->fy_w);
+	sr_unreg (di->g_w);
+	sr_unreg (di->as_w);
+	*/
 
 	/* destroy the shell, not just the form */
 	while (!XtIsWMShell (daform_w))
@@ -1191,4 +1316,4 @@ Widget da_w;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plotmenu.c,v $ $Date: 1998/04/27 21:53:04 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plotmenu.c,v $ $Date: 2000/12/03 08:33:06 $ $Revision: 1.13 $ $Name:  $"};
