@@ -22,10 +22,7 @@ extern Widget toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
 extern Colormap xe_cm;
 
-#define	PERPREC	1		/* percentage precision of display */
-#if PERPREC<1
-#define	PERPREC	1
-#endif
+#define	PERPREC	2		/* percentage precision of display */
 
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
@@ -50,17 +47,14 @@ static int p0;			/* initial percentage at this run */
 static GC pm_gc;		/* GC */
 static int nup;			/* up/down stack */
 
-/* called to manage or unmanage the progress meter */
+/* called to manage the progress meter */
 void
 pm_manage()
 {
 	if (!pm_formw)
 	    pm_create_formw();
 
-	if (XtIsManaged(pm_formw)) {
-	    XtUnmanageChild (pm_formw);
-	    nup = 0;
-	} else {
+	if (!XtIsManaged(pm_formw)) {
 	    XtManageChild (pm_formw);
 	    nup = 1;
 	}
@@ -142,9 +136,33 @@ pm_create_formw()
 	XtSetArg (args[n], XmNtitle, "xephem Progress"); n++;
 	XtSetValues (XtParent(pm_formw), args, n);
 
+	/* Close PB on the bottom */
+	n = 0;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	XtSetArg (args[n], XmNleftPosition, 30); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+	XtSetArg (args[n], XmNrightPosition, 70); n++;
+	c_w = XmCreatePushButton (pm_formw, "Close", args, n);
+	XtAddCallback (c_w, XmNactivateCallback, pm_close_cb, NULL);
+	wtip (c_w, "Close this dialog");
+	XtManageChild (c_w);
+
+	/* the progress label */
+	n = 0;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNbottomWidget, c_w); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	p_w = XmCreateLabel (pm_formw, "Progress", args, n);
+	wtip (p_w, "Percentage complete and time remaining");
+	XtManageChild (p_w);
+
 	/* the drawing area in a frame */
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNbottomWidget, p_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNmarginWidth, 5); n++;
@@ -158,30 +176,6 @@ pm_create_formw()
 	    wtip (pm_da,
 		"Progress bar moves left to right to show 0 .. 100% complete");
 	    XtManageChild (pm_da);
-
-	/* the progress label */
-	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, fr_w); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	p_w = XmCreateLabel (pm_formw, "Progress", args, n);
-	wtip (p_w, "Percentage complete and time remaining");
-	XtManageChild (p_w);
-
-	/* the Close PB */
-	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, p_w); n++;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	XtSetArg (args[n], XmNleftPosition, 30); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	XtSetArg (args[n], XmNrightPosition, 70); n++;
-	c_w = XmCreatePushButton (pm_formw, "Close", args, n);
-	XtAddCallback (c_w, XmNactivateCallback, pm_close_cb, NULL);
-	wtip (c_w, "Close this dialog");
-	XtManageChild (c_w);
 }
 
 /* called from the Close PB */
@@ -311,4 +305,4 @@ make_gc()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: progress.c,v $ $Date: 1998/12/29 17:17:41 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: progress.c,v $ $Date: 2000/07/20 04:16:29 $ $Revision: 1.5 $ $Name:  $"};
