@@ -5,6 +5,7 @@
 #include <stdio.h>
 #include <math.h>
 #include <time.h>
+#include <ctype.h>
 
 #if defined(__STDC__)
 #include <stdlib.h>
@@ -33,11 +34,12 @@ extern int unlink();
 extern Widget toplevel_w;
 extern Colormap xe_cm;
 
-extern char *getXRes P_((char *name, char *def));
+extern char *getPrivateDir P_((void));
 extern char *syserrstr P_((void));
 extern FILE *fopenh P_((char *name, char *how));
 extern int confirm P_((void));
 extern int existsh P_((char *filename));
+extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void get_xmstring P_((Widget w, char *resource, char **txtp));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
@@ -47,6 +49,11 @@ extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     void (*func3)(void)));
 extern void set_xmstring P_((Widget w, char *resource, char *txtp));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
+extern void toHSV (double r, double g, double b, double *hp, double *sp,
+    double *vp);
+extern void toRGB (double h, double s, double v, double *rp, double *gp,
+    double *bp);
 extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
 
@@ -58,31 +65,36 @@ typedef enum {
 } XPSState;
 
 typedef struct {
-    Font fid;		/* Font ID */
-    char *name;		/* name of PS font to use */
-    int h;		/* height of font, in points */
-} PSFontInfo;
+    XPSState state;		/* what's happ'nin' */
+    void (*go)();		/* user's function to call when ok to proceed */
+    FILE *fp;			/* file accumulating PS commands */
+
+    char *prfile;		/* malloced name of lpr temp file */
+    int wantpr;			/* set to print (as opposed to save) */
+    int wantcolor;		/* set when  want to print in color */
+    int overimg;		/* 0:use wantcolor; 1:white text 2:black text */
+
+    Window win;			/* window we are capturing */
+    double scale;		/* PS's points / X's pixels */
+} XPSContext;
 
 typedef struct {
-    XPSState state;	/* what's happ'nin' */
-    void (*go)();	/* user's function to call when ok to proceed */
-    FILE *fp;		/* file accumulating PS commands */
-
-    char *prfile;	/* malloced name of temp file used for printing */
-    int wantpr;		/* set when want to print (as opposed to save) */
-    int wantcolor;	/* set when  want to print in color */
-    int overimg;	/* 0: use wantcolor. 1: white text 2: black text */
-
-    Window win;		/* window we are capturing */
-    double scale;	/* PS's points / X's pixels */
-
-    PSFontInfo *fi;	/* malloced array info about registered fonts */
-    int nfi;		/* number of entries malloced in fi */
-} XPSContext;
+    Font fid;			/* Font ID */
+    char *name;			/* malloced name of PS font to use */
+    int h;			/* height of font, in points */
+} PSFontInfo;
 
 /* here is the implied state behind most of the XPS* functions */
 static XPSContext xpsc;
 
+#define	NPIXCACHE	128	/* XColors we cache for speed */
+
+/* cache of info about associating a Font with its Postscript counterpart */
+static PSFontInfo *psfi;	/* malloced array info about registered fonts */
+static int npsfi;		/* number of entries malloced in fi */
+
+static char printcategory[] = "Print";	/* Save category */
+
 /* handy reference to xpsc.fp */
 #define	FP	xpsc.fp 
 
@@ -104,6 +116,7 @@ static void doArc P_((XArc *xap, int fill));
 static void doPoly P_((XPoint xp[], int nxp, int mode, int fill));
 static void doRect P_((int x, int y, int w, int h, int fill));
 static void doSegment P_((int x1, int y1, int x2, int y2));
+static PSFontInfo * find_psfont P_((Font fid));
 static void checkState P_((char *funcname, XPSState s));
 static void printTime P_((int x, int y));
 
@@ -118,13 +131,10 @@ static void no_go P_((void));
 static void call_go P_((void));
 static void x_fill_circle P_((Display *dsp, Drawable win, GC gc, int x, int y,
     int diam));
+static void pixCache P_((XColor *xcp));
 
 #define MYMIN(a,b)      ((a) < (b) ? (a) : (b))
 #define MYMAX(a,b)      ((a) > (b) ? (a) : (b))
-static void toHSV (double r, double g, double b, double *hp, double *sp,
-    double *vp);
-static void toRGB (double h, double s, double v, double *rp, double *gp,
-    double *bp);
 
 /* ask user to save or print. if ok, call his go() after we've called XPSOpen()
  */
@@ -215,37 +225,6 @@ Cursor c;
 	}
 }
 
-/* remember the fid/fname pair in xpsc */
-void
-XPSRegisterFont (fid, fname)
-Font fid;
-char *fname;
-{
-	Display *dsp = XtDisplay(toplevel_w);
-	PSFontInfo *fi;
-	int dir, asc, des;
-	XCharStruct all;
-	int i;
-
-	/* quietly do nothing if not currently building a PS file */
-	if (xpsc.state == CLOSED)
-	    return;
-
-	/* avoid dups */
-	for (i = 0; i < xpsc.nfi; i++)
-	    if (fid == xpsc.fi[i].fid)
-		return;
-
-	/* grow fi[] and save the new info */
-	xpsc.fi = (PSFontInfo *)
-		XtRealloc ((void *)xpsc.fi, (xpsc.nfi+1)*sizeof(PSFontInfo));
-	fi = &xpsc.fi[xpsc.nfi++];
-	fi->fid = fid;
-	fi->name = XtNewString (fname);
-	XQueryTextExtents (dsp, fid, "?", 1, &dir, &asc, &des, &all);
-	fi->h = asc + des;
-}
-
 /* set up to convert subsequent X drawing calls to the given X window
  * into Postscript and send to xpsc.fp. [Xx0,Yy0] is the X coordinates of the
  * upper left corner of a window Xw pixels wide to be mapped to a rectangle Pw
@@ -265,6 +244,7 @@ int Pw;		/* width of Page rectangle */
 {
 	checkState ("Begin", OPEN);
 
+	/* remember target */
 	xpsc.win = win;
 
 	/* Postscript */
@@ -545,6 +525,7 @@ int Pw;		/* width of Page rectangle */
 	}
 	fprintf (FP, "\n");
 
+	/* enable state */
 	xpsc.state = XDRAWING;
 }
 
@@ -604,7 +585,7 @@ XPSClose()
 	fprintf (FP,"(Created by XEphem Version %s %s) 306 %d cstr\n",
 					    PATCHLEVEL, PATCHDATE, AROWY(3));
 	fprintf (FP, "(%s) 306 %d cstr\n",
-			    "(c) 1996-1999 Elwood Charles Downey", AROWY(2));
+			    "(c) 1996-2000 Elwood Charles Downey", AROWY(2));
 	fprintf (FP, "(%s) 306 %d cstr\n",
 			    "http://www.ClearSkyInstitute.com", AROWY(1));
 	printTime (306, AROWY(0));
@@ -929,35 +910,24 @@ int l;
 	unsigned long gcm;
 	XGCValues gcv;
 	PSFontInfo *fi;
-	Font fid;
-	int i;
 
 	XDrawString (dsp, win, gc, x, y, s, l);
 	if (xpsc.state != XDRAWING || xpsc.win != win)
 	    return;
 
+	/* find gc's postscript font in preregistered list */
 	gcm = GCFont;
 	XGetGCValues (dsp, gc, gcm, &gcv);
-	fid = gcv.font;
-
-	/* find gc's font in preregistered list */
-	fi = NULL;
-	for (i = 0; i < xpsc.nfi; i++) {
-	    fi = &xpsc.fi[i];
-	    if (fi->fid == fid)
-		break;
-	}
-	if (i == xpsc.nfi) {
-	    printf ("Unregistered fid. String was '%.*s'\n", l, s);
-	    exit(1);
-	}
+	fi = find_psfont (gcv.font);
+	if (!fi)
+	    return;
 
 	/* load desired font and take care not to scale the text */
 	fprintf (FP, "newpath %d %d moveto gsave\n", x, y);
 	setColor (dsp, gc, 0);
 	setLineStyle (dsp, gc);
 	fprintf (FP, "  %g %g scale /%s findfont %g scalefont setfont\n",
-		1.0/xpsc.scale, -1.0/xpsc.scale, fi->name, fi->h*xpsc.scale);
+	    1.0/xpsc.scale, -1.0/xpsc.scale, fi->name, fi->h*xpsc.scale);
 
 	/* show the string -- beware a few special characters */
 	fprintf (FP, "  (%s)\n", XPSCleanStr(s,l));
@@ -983,23 +953,16 @@ int align;
 {
 	PSFontInfo *fi;
 	char *clean;
-	int i;
 
 	XRotSetMagnification (mag);
 	XRotDrawAlignedString (dsp, fs, angle, win, gc, x, y, str, align);
 	if (xpsc.state != XDRAWING || xpsc.win != win)
 	    return;
 
-	fi = NULL;
-	for (i = 0; i < xpsc.nfi; i++) {
-	    fi = &xpsc.fi[i];
-	    if (fi->fid == fs->fid)
-		break;
-	}
-	if (i == xpsc.nfi) {
-	    printf ("Unregistered font. Rot String was '%s'\n", str);
-	    exit(1);
-	}
+	/* find fs's postscript font in preregistered list */
+	fi = find_psfont (fs->fid);
+	if (!fi)
+	    return;
 
 	clean = XPSCleanStr(str, strlen(str));
 
@@ -1007,18 +970,22 @@ int align;
 	setColor (dsp, gc, 0);
 	setLineStyle (dsp, gc);
 	fprintf (FP, "  %g %g scale /%s findfont %g scalefont setfont\n",
-	    1.0/xpsc.scale, -1.0/xpsc.scale, fi->name, fi->h*xpsc.scale*mag);
+					1.0/xpsc.scale, -1.0/xpsc.scale,
+					fi->name, fi->h*xpsc.scale*mag);
 
-	/* PS rotates about lower left corner of string -- we need to 
-	 * simulate rotating about [x,y].
+	/* [x,y] is the location of the `align' position of the string and
+	 * is the location about which we are to appear to rotate. Since PS
+	 * always rotates about the lower left corner of string we need to 
+	 * move the corner to where it will be after the rotation, then let
+	 * PS rotate from there. Remember PS +Y is up.
 	 */
 	switch (align) {
 	case MLEFT:
 	    fprintf (FP, "  (%s) dup strsz\n", clean);
-	    fprintf (FP, "  exch pop 2 div dup %g sin mul exch %g cos\n", 
-	    					angle, angle);
-	    fprintf (FP, "  mul neg rmoveto\n");
+	    fprintf (FP, "  exch pop 2 div dup %g sin mul\n", angle);
+	    fprintf (FP, "  exch %g cos mul neg rmoveto\n", angle);
 	    break;
+
 	case MRIGHT:
 	    fprintf (FP, "  (%s) dup strsz\n", clean);
 	    fprintf (FP, "  2 div dup %g sin mul 2 index %g cos mul sub\n",
@@ -1027,16 +994,32 @@ int align;
 								angle, angle);
 	    fprintf (FP, "  rmoveto\n");
 	    break;
+
+	case BLEFT:
+	    fprintf (FP, "  (%s)\n", clean);
+	    break;
+
 	case BCENTRE:
-	    /* N.B. does not support rotation */
 	    fprintf (FP, "  (%s) dup strsz\n", clean);
-	    fprintf (FP, "  pop 2 div neg 0 rmoveto\n");
+	    fprintf (FP, "  pop 2 div dup %g cos mul neg\n", angle);
+	    fprintf (FP, "  exch %g sin mul neg rmoveto\n", angle);
 	    break;
+
+	case TLEFT:
+	    fprintf (FP, "  (%s) dup strsz\n", clean);
+	    fprintf (FP, "  exch pop dup %g sin mul\n", angle);
+	    fprintf (FP, "  exch %g cos mul neg rmoveto\n", angle);
+	    break;
+
 	case TCENTRE:
-	    /* N.B. does not support rotation */
 	    fprintf (FP, "  (%s) dup strsz\n", clean);
-	    fprintf (FP, "  neg exch 2 div neg exch rmoveto\n");
+	    fprintf (FP, "  dup dup mul 3 2 roll 2 div dup dup mul\n");
+	    fprintf (FP, "  exch 4 1 roll add sqrt 3 1 roll atan\n");
+	    fprintf (FP, "  %g sub dup cos exch sin\n", angle);
+	    fprintf (FP, "  3 2 roll dup 4 1 roll mul neg 3 1 roll mul neg\n");
+	    fprintf (FP, "  rmoveto\n");
 	    break;
+
 	default:
 	    printf ("XPSRot: unsupported align: %d\n", align);
 	    exit(1);
@@ -1218,7 +1201,7 @@ int lr, tb;		/* set if want flipped left-right or top-bottom */
 		if (inv)
 		    gp = maxcm - gp;
 		gp = gp*255/maxcm;
-		fprintf (FP, "%x%x", gp>>4, gp&0xf);
+		fprintf (FP, "%02x", gp);
 		if (((++n)%36) == 0)
 		    fprintf (FP, "\n");
 	    }
@@ -1230,6 +1213,165 @@ int lr, tb;		/* set if want flipped left-right or top-bottom */
 	xpsc.overimg = inv ? 2 : 1;
 }
 
+/* print the given pixmap of size widxhei using colormap cm.
+ * if bggc, print white for any pixel matching the bggc.
+ * N.B. this also sets overimg which means all subsequent text will be drawn in
+ *   white (unless wantcolor is on).
+ */
+void
+XPSPixmap (pm, wid, hei, cm, bggc)
+Pixmap pm;		/* pixmap to print */
+unsigned int wid, hei;	/* pixmap size */
+Colormap cm;		/* pixels in pm are with respect to this colormap */
+GC bggc;		/* background pixel to not draw, or 0 */
+{
+	Display *dsp = XtDisplay(toplevel_w);
+	Pixel white = WhitePixel (dsp, DefaultScreen (dsp));
+	XImage *xim;
+	Window rootwin;
+	unsigned int b, d;
+	unsigned bgpix = 0;
+	XColor xc;
+	int x, y;
+	int n;
+
+
+	if (xpsc.state != XDRAWING || xpsc.win != pm)
+	    return;
+
+	/* get the bg pixel to avoid, if any */
+	if (bggc) {
+	    XGCValues gcv;
+	    XGetGCValues (dsp, bggc, (unsigned long)GCForeground, &gcv);
+	    bgpix = gcv.foreground;
+	}
+
+	/* suck out the pixels as an image */
+	XGetGeometry (dsp, pm, &rootwin, &x, &y, &wid, &hei, &b, &d);
+	xim = XGetImage (dsp, pm, 0, 0, wid, hei, ~0, ZPixmap);
+	if (!xim) {
+	    xe_msg ("Can not create image for printing", 1);
+	    return;
+	}
+
+	/* set up postscript */
+	fprintf (FP, "gsave /imgw %d def /imgh %d def\n", wid, hei);
+	fprintf (FP, "  %d %d translate\n", 0, 0);
+	fprintf (FP, "  imgw imgh scale\n");
+	fprintf (FP, "  imgw imgh 8 [imgw 0 0 imgh neg 0 imgh]\n");
+	fprintf (FP, "  { currentfile imgw string readhexstring pop }");
+
+	/* if color use 8 bits per pixel, RGB format, else 8 bit gray */
+	if (xpsc.wantcolor)
+	    fprintf (FP, " false 3 colorimage\n");
+	else
+	    fprintf (FP, " image\n");
+
+	/* use a little cache to reduce using XQueryColor every time */
+	pixCache (NULL);
+
+	/* postscript draws bottom-to-top so draw bottom up */
+	n = 0;
+	for (y = hei-1; y >= 0; --y) {
+	    for (x = 0; x < (int)wid; x++) {
+		xc.pixel = XGetPixel (xim, x, y);
+		if (bggc && xc.pixel == bgpix)
+		    xc.pixel = white;
+		pixCache (&xc);
+		if (xpsc.wantcolor) {
+		    fprintf (FP, "%02x%02x%02x", xc.red>>8, xc.green>>8,
+								xc.blue>>8);
+		    if (((++n)%12) == 0)
+			fprintf (FP, "\n");
+		} else {
+		    fprintf (FP, "%02x", xc.red>>8);
+		    if (((++n)%36) == 0)
+			fprintf (FP, "\n");
+		}
+	    }
+	}
+
+	fprintf (FP, "\ngrestore\n");
+
+	/* done */
+	free ((void *)xim->data);
+	xim->data = NULL;
+	XDestroyImage (xim);
+
+	/* set overimage so subsequent text is written white */
+	xpsc.overimg = 1;
+}
+
+/* RGB -> HSV, all [0..1) */
+void
+toHSV (double r, double g, double b, double *hp, double *sp, double *vp)
+{
+	double min, max, diff;
+	double h, s, v;
+
+	max = MYMAX (MYMAX (r, g), b);
+	min = MYMIN (MYMIN (r, g), b);
+	diff = max - min;
+	v = max;
+	s = max != 0 ? diff/max : 0;
+	if (s == 0)
+	    h = 0;
+	else {
+	    double rc = (max - r)/diff;
+	    double gc = (max - g)/diff;
+	    double bc = (max - b)/diff;
+	    if (r == max)
+		h = bc - gc;
+	    else if (g == max)
+		h = 2 + rc - bc;
+	    else
+		h = 4 + gc - rc;
+	    if (h < 0)
+		h += 6;
+	    h /= 6;
+	}
+
+	*hp = h;
+	*sp = s;
+	*vp = v;
+}
+
+/* HSV -> RGB, all [0..1) */
+void
+toRGB (double h, double s, double v, double *rp, double *gp, double *bp)
+{
+	double r, g, b;
+	double f, p, q, t;
+	int i;
+
+	if (v == 0) 
+	    r = g = b = 0.0;
+	else if (s == 0)
+	    r = g = b = v;
+	else {
+	    h *= 6;
+	    i = (int)floor(h);
+	    f = h - i;
+	    p = v * (1.0 - s);
+	    q = v * (1.0 - s * f);
+	    t = v * (1.0 - s * (1.0 - f));
+
+	    switch (i) {
+	    default: /* FALLTHRU */
+	    case 0: r = v; g = t; b = p; break;
+	    case 1: r = q; g = v; b = p; break;
+	    case 2: r = p; g = v; b = t; break;
+	    case 3: r = p; g = q; b = v; break;
+	    case 4: r = t; g = p; b = v; break;
+	    case 5: r = v; g = p; b = q; break;
+	    }
+	}
+
+	*rp = r;
+	*gp = g;
+	*bp = b;
+}
+
 /* check gc's LineStyle and emit "solid" or "dashed" macro.
  * TODO: check dash details.
  */
@@ -1255,7 +1397,7 @@ GC gc;
  *   dig the rgb out of the gc and put in postscript as a gray level.
  * else
  *   do nothing for color (use default black).
- * N.B. can't allow for colormap since we are often used with a pixmap.
+ * N.B. we assume xe_cm is the colormap.
  */
 static void
 setColor (dsp, gc, dogray)
@@ -1279,13 +1421,14 @@ int dogray;
 	    b = (double)xc.blue/65535.0;
 	    toHSV (r, g, b, &h, &s, &v);
 	    if (s < .2) {
-		/* not much color, so go for mid-grey */
-		v = .5;
+		/* basically gray */
+		if (v > .8)
+		    v = .8;		/* white doesn't show */
+		else if (v < .5)
+		    v = .5;		/* save some ink if black */
 	    } else {
-		/* rich color, not too bright */
+		/* rich color */
 		s = 1.0;
-		if (v > .75)
-		    v = 0.75;
 	    }
 	    toRGB (h, s, v, &r, &g, &b);
 	    fprintf (FP, "%g %g %g setrgbcolor\n", r, g, b);
@@ -1386,6 +1529,117 @@ int x1, y1, x2, y2;
 	fprintf (FP, " %4d %4d moveto %4d %4d lineto ", x1, y1, x2, y2);
 }
 
+/* ARGSUSED */
+static int IgnoreError(disp, event)
+Display *disp;
+XErrorEvent *event;
+{
+	return (0);
+}
+
+/* find the postscipt info for fid, creating new one if necessary.
+ * xe_msg() why and return NULL if trouble.
+ */
+static PSFontInfo *
+find_psfont (fid)
+Font fid;
+{
+	static Atom FAMILY, WEIGHT, SLANT, PIXELSZ;
+	Display *dsp = XtDisplay(toplevel_w);
+	int (*oldhandler)() = XSetErrorHandler (IgnoreError);
+	char family[32], weight[32], slant[32];
+	char psname[64];
+	PSFontInfo *fi;
+	XFontStruct *fsp;
+	unsigned long card32;
+	char *name;
+	int pixsz;
+	int i;
+
+	/* see if already have info */
+	for (i = 0; i < npsfi; i++)
+	    if (fid == psfi[i].fid)
+		return (&psfi[i]);
+
+	/* gather atoms property into a cache */
+	if (!FAMILY) {
+	    FAMILY = XInternAtom (dsp, "FAMILY_NAME", 1);
+	    WEIGHT = XInternAtom (dsp, "WEIGHT_NAME", 1);
+	    SLANT = XInternAtom (dsp, "SLANT", 1);
+	    PIXELSZ = XInternAtom (dsp, "PIXEL_SIZE", 1);
+	}
+
+	/* grow fi[] */
+	psfi = (PSFontInfo *) XtRealloc ((void *)psfi,
+						(npsfi+1)*sizeof(PSFontInfo));
+	fi = &psfi[npsfi];
+
+	/* build the PS name from various properties */
+	fsp = XQueryFont (dsp, fid);
+	if (!fsp) {
+	    xe_msg ("No font info!", 1);
+	    return (NULL);
+	}
+
+	if (!XGetFontProperty (fsp, FAMILY, &card32)) {
+	    xe_msg ("No FAMILY font property", 1);
+	    return (NULL);
+	}
+	name = XGetAtomName (dsp, card32);
+	strcpy (family, name);
+	if (islower(family[0]))
+	    family[0] = toupper(family[0]);
+	XtFree (name);
+
+	if (!XGetFontProperty (fsp, WEIGHT, &card32)) {
+	    xe_msg ("No WEIGHT font property", 1);
+	    return (NULL);
+	}
+	name = XGetAtomName (dsp, card32);
+	switch (name[0]) {
+	case 'b': case 'B':
+	    strcpy (weight, "-Bold");
+	    break;
+	default:
+	    weight[0] = '\0';
+	    break;
+	}
+	XtFree (name);
+
+	if (!XGetFontProperty (fsp, SLANT, &card32)) {
+	    xe_msg ("No SLANT font property", 1);
+	    return (NULL);
+	}
+	name = XGetAtomName (dsp, card32);
+	switch (name[0]) {
+	case 'i': case 'I': case 'o': case 'O':
+	    strcpy (slant, "-Italic");
+	    break;
+	default:
+	    slant[0] = '\0';
+	    break;
+	}
+
+	if (!XGetFontProperty (fsp, PIXELSZ, &card32)) {
+	    xe_msg ("No PIXEL_SIZE font property", 1);
+	    return (NULL);
+	}
+	pixsz = card32;
+	XtFree (name);
+
+	/* TODO: how to free memory but retain fid?
+	XFreeFont (dsp, fsp);
+	*/
+
+	sprintf (psname, "%s%s%s", family, slant, weight);
+	fi->name = XtNewString (psname);
+	fi->h = pixsz;
+	fi->fid = fid;
+	npsfi++;
+	XSetErrorHandler (oldhandler);
+	return (fi);
+}
+
 static void
 checkState (funcname, s)
 char *funcname;
@@ -1471,14 +1725,12 @@ create_print_w()
 	Widget w;
 	Widget f_w, rc_w;
 	Widget rb_w;
-	char fname[1024];
 	Arg args[20];
 	int n;
 
 	/* create the form dialog -- title gets set later */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
@@ -1530,6 +1782,10 @@ create_print_w()
 	    gray_w = XmCreateToggleButton (rb_w, "Grayscale", args, n);
 	    wtip (gray_w, "When on, print in gray tones only (no color)");
 	    XtManageChild (gray_w);
+	    sr_reg (gray_w, NULL, printcategory, 1);
+
+	    /* Grayscale inits both */
+	    XmToggleButtonSetState(color_w, !XmToggleButtonGetState(gray_w), 0);
 
 	/* add a separator */
 
@@ -1551,17 +1807,18 @@ create_print_w()
 	    set_xmstring (filetb_w, XmNlabelString, "Save to file:");
 	    wtip(filetb_w,"When on, plot will be saved to file named at right");
 	    XtManageChild (filetb_w);
+	    sr_reg (filetb_w, NULL, printcategory, 1);
 
-	    (void) sprintf (fname,"%s/xephem.ps", getXRes("PrivateDir","work"));
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNleftPosition, 50); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNvalue, fname); n++;
 	    filename_w = XmCreateTextField (f_w, "Filename", args, n);
+	    defaultTextFN (filename_w, 1, getPrivateDir(), "xephem.ps");
 	    wtip (filename_w, "Name of postscript file to create");
 	    XtManageChild (filename_w);
+	    sr_reg (filename_w, NULL, printcategory, 1);
 
 	/* make the fake "print" toggle button and print command text field */
 
@@ -1586,6 +1843,7 @@ create_print_w()
 	    prcmd_w = XmCreateTextField (f_w, "PrintCmd", args, n);
 	    wtip (prcmd_w, "Command which will print a postscript file given name of file as first and only argument");
 	    XtManageChild (prcmd_w);
+	    sr_reg (prcmd_w, NULL, printcategory, 1);
 
 	/* connect them together so they work like a radio box */
 
@@ -1594,6 +1852,9 @@ create_print_w()
 	XtAddCallback (prtb_w, XmNvalueChangedCallback, toggle_cb,
 							(XtPointer)filetb_w);
 
+	/* value of Save sets both */
+	XmToggleButtonSetState(prtb_w, !XmToggleButtonGetState(filetb_w),False);
+
 	/* add another separator */
 
 	n = 0;
@@ -1710,7 +1971,7 @@ XtPointer call;
 
 	    if (existsh (name) == 0 && confirm()) {
 		char buf[1024];
-		(void) sprintf (buf, "%s exists: Ok to Overwrite?", name);
+		(void) sprintf (buf, "%s exists:\nOk to Overwrite?", name);
 		query (toplevel_w, buf, "Yes -- Overwrite", "No -- Cancel",
 					    NULL, saveas_confirm, no_go, NULL);
 	    } else
@@ -1784,15 +2045,6 @@ xpsc_close()
 	xpsc.wantcolor = 0;
 	xpsc.wantpr = 0;
 	xpsc.overimg = 0;
-
-	if (xpsc.nfi > 0) {
-	    int i;
-	    for (i = 0; i < xpsc.nfi; i++)
-		XtFree ((void *)xpsc.fi[i].name);
-	    XtFree ((void *)xpsc.fi);
-	    xpsc.fi = NULL;
-	    xpsc.nfi = 0;
-	}
 }
 
 /* called when it's confirmed to try and print or save */
@@ -1909,71 +2161,58 @@ int diam;
 	XSetClipMask (dsp, gc, None); /* TODO: probably should put back old */
 }
 
+/* look up xcp->pixel in a cache.
+ * if not found, add if possible.
+ * if xcp == NULL reset the cache collection.
+ */
 static void
-toHSV (double r, double g, double b, double *hp, double *sp, double *vp)
+pixCache (xcp)
+XColor *xcp;
 {
-	double min, max, diff;
-	double h, s, v;
-
-	max = MYMAX (MYMAX (r, g), b);
-	min = MYMIN (MYMIN (r, g), b);
-	diff = max - min;
-	v = max;
-	s = max != 0 ? diff/max : 0;
-	if (s == 0)
-	    h = 0;
-	else {
-	    if (r == max)
-		h = (g - b)/diff;
-	    else if (g == max)
-		h = 2 + (b - r)/diff;
-	    else
-		h = 4 + (r - g)/diff;
-	    h /= 6;
-	    if (h < 0)
-		h += 1;
+	static XColor xc[NPIXCACHE];
+	static int nxc;
+	int t, b;
+	
+	/* just reset if no xcp */
+	if (!xcp) {
+	    nxc = 0;
+	    return;
 	}
 
-	*hp = h;
-	*sp = s;
-	*vp = v;
-}
-
-static void
-toRGB (double h, double s, double v, double *rp, double *gp, double *bp)
-{
-	double r, g, b;
-	double f, p, q, t;
-	int i;
-
-	if (v == 0) 
-	    r = g = b = 0.0;
-	else if (s == 0)
-	    r = g = b = v;
-	else {
-	    if (h >= 1)
-		h = 0;
-	    i = (int)floor(h * 6);
-	    f = h * 6.0 - i;
-	    p = v * (1.0 - s);
-	    q = v * (1.0 - s * f);
-	    t = v * (1.0 - s * (1.0 - f));
-
-	    switch (i) {
-	    case 0: r = v; g = t; b = p; break;
-	    case 1: r = q; g = v; b = p; break;
-	    case 2: r = p; g = v; b = t; break;
-	    case 3: r = p; g = q; b = v; break;
-	    case 4: r = t; g = p; b = v; break;
-	    case 5: r = v; g = p; b = q; break;
-	    default: r = g = b = 1; break;
+	/* binary search */
+	t = nxc - 1;
+	b = 0;
+	while (b <= t) {
+	    int m = (t+b)/2;
+	    XColor *mxcp = &xc[m];
+	    if (xcp->pixel == mxcp->pixel) {
+		*xcp = *mxcp;
+		return;
 	    }
+	    if (xcp->pixel < mxcp->pixel)
+		t = m-1;
+	    else
+		b = m+1;
 	}
 
-	*rp = r;
-	*gp = g;
-	*bp = b;
+	/* not found, get the r/g/b */
+	XQueryColor (XtDisplay(toplevel_w), xe_cm, xcp);
+	if (!xpsc.wantcolor) {
+	    double r = (double)xcp->red;
+	    double g = (double)xcp->green;
+	    double b = (double)xcp->blue;
+	    int gray = (int)sqrt ((r*r + g*g + b*b)/3.0);
+	    xcp->red = xcp->green = xcp->blue = gray;
+	}
+
+	/* add to cache in ascending order, if more room */
+	if (nxc < NPIXCACHE) {
+	    for (t = nxc-1; xc[t].pixel > xcp->pixel; --t)
+		xc[t+1] = xc[t];
+	    xc[t+1] = *xcp;
+	    nxc++;
+	}
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: ps.c,v $ $Date: 1999/02/24 08:33:14 $ $Revision: 1.8 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: ps.c,v $ $Date: 2000/10/30 22:14:49 $ $Revision: 1.22 $ $Name:  $"};
