@@ -261,7 +261,7 @@ void XRotSetMagnification(m)
     float m;
 {
     if(m>0.)
-	style.magnify=m;
+	style.magnify=(float)m;
 }
 
 
@@ -437,6 +437,7 @@ static int XRotPaintAlignedString(dpy, font, angle, drawable, gc, x, y, text,
 	return 0;
 
     /* manipulate angle to 0<=angle<360 degrees */
+    angle = floor(angle*100+.5)/100.;
     while(angle<0)
         angle+=360;
     
@@ -482,8 +483,8 @@ static int XRotPaintAlignedString(dpy, font, angle, drawable, gc, x, y, text,
         hot_x=(float)item->max_width/2*style.magnify;
     
     /* pre-calculate sin and cos */
-    sin_angle=sin(angle);
-    cos_angle=cos(angle);
+    sin_angle=(float)sin(angle);
+    cos_angle=(float)cos(angle);
     
     /* rotate hot_x and hot_y around bitmap centre */
     hot_xp= hot_x*cos_angle - hot_y*sin_angle;
@@ -502,10 +503,10 @@ static int XRotPaintAlignedString(dpy, font, angle, drawable, gc, x, y, text,
 	
 	/* rotate corner positions */
 	for(i=0; i<4*item->nl; i++) {
-	    xpoints[i].x=(float)x + ( (item->corners_x[i]-hot_x)*cos_angle + 
-				      (item->corners_y[i]+hot_y)*sin_angle);
-	    xpoints[i].y=(float)y + (-(item->corners_x[i]-hot_x)*sin_angle + 
-				      (item->corners_y[i]+hot_y)*cos_angle);
+	    xpoints[i].x=x + (int)( (item->corners_x[i]-hot_x)*cos_angle + 
+				    (item->corners_y[i]+hot_y)*sin_angle);
+	    xpoints[i].y=y + (int)(-(item->corners_x[i]-hot_x)*sin_angle + 
+				    (item->corners_y[i]+hot_y)*cos_angle);
 	}
 	
 	/* we want to swap foreground and background colors here;
@@ -530,8 +531,8 @@ static int XRotPaintAlignedString(dpy, font, angle, drawable, gc, x, y, text,
     }
     
     /* where should top left corner of bitmap go ? */
-    xp=(float)x-((float)item->cols_out/2 +hot_xp);
-    yp=(float)y-((float)item->rows_out/2 -hot_yp);
+    xp=x-(int)((float)item->cols_out/2 +hot_xp);
+    yp=y-(int)((float)item->rows_out/2 -hot_yp);
     
     /* by default we draw the rotated bitmap, solid */
     bitmap_to_paint=item->bitmap;
@@ -855,7 +856,7 @@ static RotatedTextItem *XRotRetrieveFromCache(dpy, font, angle, text, align)
 	    item->fid=fid;
 	}
 
-	item->angle=angle;
+	item->angle=(float)angle;
 	item->align=align;
 	item->magnify=style.magnify;
 
@@ -996,8 +997,8 @@ static RotatedTextItem *XRotCreateTextItem(dpy, font, angle, text, align)
     XSetForeground(dpy, font_gc, 1);
     
     /* pre-calculate sin and cos */
-    sin_angle=sin(angle);
-    cos_angle=cos(angle);
+    sin_angle=(float)sin(angle);
+    cos_angle=(float)cos(angle);
     
     /* text background will be drawn using XFillPolygon */
     item->corners_x=
@@ -1077,16 +1078,16 @@ static RotatedTextItem *XRotCreateTextItem(dpy, font, angle, text, align)
 
 	old_cols_in=item->cols_in;
 	old_rows_in=item->rows_in;
-	item->cols_in=(float)item->cols_in*style.magnify;
-	item->rows_in=(float)item->rows_in*style.magnify;
+	item->cols_in=(int)(item->cols_in*style.magnify);
+	item->rows_in=(int)(item->rows_in*style.magnify);
     }
 
     /* how big will rotated text be ? */
-    item->cols_out=fabs((float)item->rows_in*sin_angle) +
-	fabs((float)item->cols_in*cos_angle) +0.99999 +2;
+    item->cols_out=(int)(fabs(item->rows_in*sin_angle) +
+	                 fabs(item->cols_in*cos_angle) +0.99999) +2;
 
-    item->rows_out=fabs((float)item->rows_in*cos_angle) +
-	fabs((float)item->cols_in*sin_angle) +0.99999 +2;
+    item->rows_out=(int)(fabs(item->rows_in*cos_angle) +
+		         fabs(item->cols_in*sin_angle) +0.99999) +2;
 
     if(item->cols_out%2==0)
 	item->cols_out++;
@@ -1106,7 +1107,7 @@ static RotatedTextItem *XRotCreateTextItem(dpy, font, angle, text, align)
        a bit over-the-top */
     
     /* vertical distance from centre */
-    dj=0.5-(float)item->rows_out/2;
+    dj=(float)(0.5-(float)item->rows_out/2);
 
     /* where abouts does text actually lie in rotated image? */
     if(angle==0 || angle==M_PI/2 || 
@@ -1116,30 +1117,26 @@ static RotatedTextItem *XRotCreateTextItem(dpy, font, angle, text, align)
 	xinc=0;
     }
     else if(angle<M_PI) {
-	xl=(float)item->cols_out/2+
-	    (dj-(float)item->rows_in/(2*cos_angle))/
-		tan(angle)-2;
-	xr=(float)item->cols_out/2+
-	    (dj+(float)item->rows_in/(2*cos_angle))/
-		tan(angle)+2;
-	xinc=1./tan(angle);
+	xl=(float)(item->cols_out/2.+
+	    (dj-(float)item->rows_in/(2*cos_angle))/tan(angle)-2);
+	xr=(float)(item->cols_out/2.+
+	    (dj+(float)item->rows_in/(2*cos_angle))/tan(angle)+2);
+	xinc=(float)(1./tan(angle));
     }
     else {
-	xl=(float)item->cols_out/2+
-	    (dj+(float)item->rows_in/(2*cos_angle))/
-		tan(angle)-2;
-	xr=(float)item->cols_out/2+
-	    (dj-(float)item->rows_in/(2*cos_angle))/
-		tan(angle)+2;
+	xl=(float)(item->cols_out/2.+
+	    (dj+(float)item->rows_in/(2*cos_angle))/tan(angle)-2);
+	xr=(float)(item->cols_out/2+
+	    (dj-(float)item->rows_in/(2*cos_angle))/tan(angle)+2);
 	
-	xinc=1./tan(angle);
+	xinc=(float)(1./tan(angle));
     }
 
     /* loop through all relevent bits in rotated image */
     for(j=0; j<item->rows_out; j++) {
 	
 	/* no point re-calculating these every pass */
-	di=(float)((xl<0)?0:(int)xl)+0.5-(float)item->cols_out/2;
+	di=(float)(((xl<0)?0:(int)xl)+0.5-(float)item->cols_out/2);
 	byte_out=(item->rows_out-j-1)*byte_w_out;
 	
 	/* loop through meaningful columns */
@@ -1147,8 +1144,8 @@ static RotatedTextItem *XRotCreateTextItem(dpy, font, angle, text, align)
 	    i<((xr>=item->cols_out)?item->cols_out:(int)xr); i++) {
 	    
 	    /* rotate coordinates */
-	    it=(float)item->cols_in/2 + ( di*cos_angle + dj*sin_angle);
-	    jt=(float)item->rows_in/2 - (-di*sin_angle + dj*cos_angle);
+	    it=(int)(item->cols_in/2. + ( di*cos_angle + dj*sin_angle));
+	    jt=(int)(item->rows_in/2. - (-di*sin_angle + dj*cos_angle));
 	    
             /* set pixel if required */
             if(it>=0 && it<item->cols_in && jt>=0 && jt<item->rows_in)
@@ -1348,7 +1345,7 @@ static XImage *XRotMagnifyImage(dpy, ximage)
     int cols_in, rows_in;
     int cols_out, rows_out;
     register int i2, j2;
-    float z1, z2, z3, z4;
+    int z1, z2, z3, z4;
     int byte_width_in, byte_width_out;
     float mag_inv;
 
@@ -1357,8 +1354,8 @@ static XImage *XRotMagnifyImage(dpy, ximage)
     rows_in=ximage->height;
 
     /* size of final image */
-    cols_out=(float)cols_in*style.magnify;
-    rows_out=(float)rows_in*style.magnify;
+    cols_out=(int)(cols_in*style.magnify);
+    rows_out=(int)(rows_in*style.magnify);
 
     /* this will hold final image */
     I_out=MakeXImage(dpy, cols_out, rows_out);
@@ -1370,17 +1367,17 @@ static XImage *XRotMagnifyImage(dpy, ximage)
     byte_width_out=(cols_out-1)/8+1;
 
     /* for speed */
-    mag_inv=1./style.magnify;
+    mag_inv=(float)(1./style.magnify);
 
     y=0.;
 
     /* loop over magnified image */
     for(j2=0; j2<rows_out; j2++) {
 	x=0;
-	j=y;
+	j=(int)y;
 
 	for(i2=0; i2<cols_out; i2++) {
-	    i=x;
+	    i=(int)x;
 
 	    /* bilinear interpolation - where are we on bitmap ? */
 	    /* right edge */
@@ -1528,8 +1525,8 @@ XPoint *XRotTextExtents(dpy, font, angle, x, y, text, align)
     rows_in=nl*height;
     
     /* pre-calculate sin and cos */
-    sin_angle=sin(angle);
-    cos_angle=cos(angle);
+    sin_angle=(float)sin(angle);
+    cos_angle=(float)cos(angle);
     
     /* y position */
     if(align==TLEFT || align==TCENTRE || align==TRIGHT)
@@ -1559,22 +1556,22 @@ XPoint *XRotTextExtents(dpy, font, angle, x, y, text, align)
 	return NULL;
 
     /* bounding box when horizontal, relative to bitmap centre */
-    xp_in[0].x=-(float)cols_in*style.magnify/2-style.bbx_pad;
-    xp_in[0].y= (float)rows_in*style.magnify/2+style.bbx_pad;
-    xp_in[1].x= (float)cols_in*style.magnify/2+style.bbx_pad;
-    xp_in[1].y= (float)rows_in*style.magnify/2+style.bbx_pad;
-    xp_in[2].x= (float)cols_in*style.magnify/2+style.bbx_pad;
-    xp_in[2].y=-(float)rows_in*style.magnify/2-style.bbx_pad;
-    xp_in[3].x=-(float)cols_in*style.magnify/2-style.bbx_pad;
-    xp_in[3].y=-(float)rows_in*style.magnify/2-style.bbx_pad;
+    xp_in[0].x=-(short)(cols_in*style.magnify/2-style.bbx_pad);
+    xp_in[0].y= (short)(rows_in*style.magnify/2+style.bbx_pad);
+    xp_in[1].x= (short)(cols_in*style.magnify/2+style.bbx_pad);
+    xp_in[1].y= (short)(rows_in*style.magnify/2+style.bbx_pad);
+    xp_in[2].x= (short)(cols_in*style.magnify/2+style.bbx_pad);
+    xp_in[2].y=-(short)(rows_in*style.magnify/2-style.bbx_pad);
+    xp_in[3].x=-(short)(cols_in*style.magnify/2-style.bbx_pad);
+    xp_in[3].y=-(short)(rows_in*style.magnify/2-style.bbx_pad);
     xp_in[4].x=xp_in[0].x;
     xp_in[4].y=xp_in[0].y;
 	
     /* rotate and translate bounding box */
     for(i=0; i<5; i++) {
-	xp_out[i].x=(float)x + ( ((float)xp_in[i].x-hot_x)*cos_angle +
+	xp_out[i].x=x + (int)( ((float)xp_in[i].x-hot_x)*cos_angle +
 				 ((float)xp_in[i].y+hot_y)*sin_angle);
-	xp_out[i].y=(float)y + (-((float)xp_in[i].x-hot_x)*sin_angle +
+	xp_out[i].y=y + (int)(-((float)xp_in[i].x-hot_x)*sin_angle +
 				 ((float)xp_in[i].y+hot_y)*cos_angle);
     }
 
@@ -1586,4 +1583,4 @@ XPoint *XRotTextExtents(dpy, font, angle, x, y, text, align)
 
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: rotated.c,v $ $Date: 1996/08/26 19:42:22 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: rotated.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.3 $ $Name:  $"};
