@@ -36,16 +36,20 @@ extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
+extern XFontStruct * getXResFont P_((char *rn));
 extern char *getShareDir P_((void));
 extern double delra P_((double dra));
 extern int any_ison P_((void));
 extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
     double mag, ObjF **opp));
+extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
+extern int isUp P_((Widget shell));
 extern int magdiam P_((int fmag, int magstp, double scale, double mag,
     double size));
 extern int read_bdl P_((FILE *fp, double jd, double *xp, double *yp,
     double *zp, char ynot[]));
 extern int skyloc_fifo P_((Obj *op));
+extern void buttonAsButton P_((Widget w, int whether));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void f_prdec P_((Widget w, double a));
@@ -62,6 +66,7 @@ extern void register_selection P_((char *name));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void sm_update P_((Now *np, int how_much));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
 extern void timestamp P_((Now *np, Widget w));
@@ -88,16 +93,16 @@ typedef enum {
 static ScreenObj *screenobj[SO_N];
 static int nscreenobj[SO_N];
 
-static void sm_create_form_w P_((void));
+static void sm_create_shell_w P_((void));
 static void sm_create_tvform_w P_((void));
 static void sm_create_ssform_w P_((void));
 static void sm_set_buttons P_((int whether));
-static void sm_set_a_button P_((Widget pbw, int whether));
+static void sm_sstats_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_sstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_scale_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sm_activate_cb P_((Widget w, XtPointer client, XtPointer call));
-static void sm_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void sm_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void st_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void st_map_cb P_((Widget w, XtPointer client, XtPointer call));
 static void st_track_size P_((void));
@@ -121,7 +126,7 @@ static ScreenObj *close_screenobj P_((SOIdx, int x, int y));
 static void sky_background P_((Drawable win, unsigned w, unsigned h, int fmag,
     double ra0, double dec0, double scale, double rad, int fliptb, int fliplr));
 
-static Widget satform_w;	/* main form */
+static Widget satshell_w;	/* main shell */
 static Widget ssform_w;		/* statistics form */
 static Widget smframe_w;	/* main frame */
 static Widget stform_w;		/* top-view form */
@@ -147,7 +152,6 @@ static int flip_tb;		/* whether we want to flip top/bottom */
 static int flip_lr;		/* whether we want to flip left/right */
 static int skybkg;		/* whether we want sky background */
 static int topview;		/* whether we want the top view */
-static int sswasman;		/* whether statistics form was managed */
 
 static Widget smpu_w;		/* main popup */
 static Widget smpu_name_w;	/* popup name label */
@@ -188,31 +192,24 @@ static struct MoonNames {
 enum {CEV, CSV, CX, CY, CZ, CRA, CDEC, CMAG, CNum};	/* s_w column index */
 static Widget	s_w[NM][CNum];			/* the data display widgets */
 
+static char satcategory[] = "Saturn";		/* Save category */
 
 /* called when the saturn menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, otherwise, just get out there and do it!
  */
 void
 sm_manage ()
 {
-	if (!satform_w) {
-	    sm_create_form_w();
+	if (!satshell_w) {
+	    sm_create_shell_w();
 	    sm_create_ssform_w();
+	    sm_create_tvform_w();
 	}
 	
-	if (XtIsManaged(satform_w)) {
-	    XtUnmanageChild (satform_w);
-	    if ((sswasman = XtIsManaged(ssform_w)) != 0)
-		XtUnmanageChild(ssform_w);
-	} else {
-	    XtManageChild (satform_w);
-	    if (sswasman) {
-		XtManageChild (ssform_w);
-		sm_set_buttons(sm_selecting);
-	    }
-	    /* rely on expose to do the first draw */
-	}
+	XtPopup (satshell_w, XtGrabNone);
+	set_something (satshell_w, XmNiconic, (XtArgVal)False);
+	sm_set_buttons(sm_selecting);
 }
 
 /* called when the database has changed.
@@ -230,7 +227,7 @@ int appended;
 int
 sm_ison()
 {
-	return (satform_w && XtIsManaged(satform_w));
+	return (isUp(satshell_w));
 }
 
 /* called by other menus as they want to hear from our buttons or not.
@@ -241,9 +238,12 @@ void
 sm_selection_mode (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	sm_selecting += whether ? 1 : -1;
+	if (whether)
+	    sm_selecting++;
+	else if (sm_selecting > 0)
+	    --sm_selecting;
 
-	if (sm_ison())
+	if (satshell_w)
 	    if ((whether && sm_selecting == 1)     /* first one to want on */
 		|| (!whether && sm_selecting == 0) /* last one to want off */)
 		sm_set_buttons (whether);
@@ -256,8 +256,8 @@ Cursor c;
 {
 	Window win;
 
-	if (satform_w && (win = XtWindow(satform_w)) != 0) {
-	    Display *dsp = XtDisplay(satform_w);
+	if (satshell_w && (win = XtWindow(satshell_w)) != 0) {
+	    Display *dsp = XtDisplay(satshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -271,21 +271,29 @@ Cursor c;
 	    else
 		XUndefineCursor (dsp, win);
 	}
+
+	if (stform_w && (win = XtWindow(stform_w)) != 0) {
+	    Display *dsp = XtDisplay(stform_w);
+	    if (c)
+		XDefineCursor (dsp, win, c);
+	    else
+		XUndefineCursor (dsp, win);
+	}
 }
 
-/* create the main form dialog */
+/* create the main shell */
 static void
-sm_create_form_w()
+sm_create_shell_w()
 {
 	typedef struct {
-	    char *name;		/* toggle button instance name */
+	    char *name;		/* toggle button instance name, or NULL */
 	    char *label;	/* label */
 	    int *flagp;		/* pointer to global flag, or NULL if none */
 	    Widget *wp;		/* widget to save, or NULL */
 	    char *tip;          /* widget tip */
 	} Option;
 	static Option options[] = {
-	    {"TopView",	   "Top view",		&topview, &topview_w,
+	    {NULL,	   "Top view",		&topview, &topview_w,
 	    	"When on, show view looking from high above N pole"},
 	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w,
 	    	"When on, sky will include database objects and Field Stars"},
@@ -310,28 +318,33 @@ sm_create_form_w()
 	};
 	Widget w;
 	Widget mb_w, pd_w, cb_w;
+	Widget satform_w;
 	XmString str;
 	Arg args[20];
 	int n;
 	int i;
 
-	/* create form */
-	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	satform_w = XmCreateFormDialog (toplevel_w, "Saturn", args, n);
-	set_something (satform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (satform_w, XmNhelpCallback, sm_help_cb, 0);
-	XtAddCallback (satform_w, XmNunmapCallback, sm_unmap_cb, 0);
+	/* create form and shell */
 
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
 	n = 0;
 	XtSetArg (args[n], XmNtitle, "xephem Saturn view"); n++;
-	XtSetValues (XtParent(satform_w), args, n);
+	XtSetArg (args[n], XmNiconName, "Saturn"); n++;
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	satshell_w = XtCreatePopupShell ("Saturn", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (satshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (satshell_w, XmNpopdownCallback, sm_popdown_cb, 0);
+	sr_reg (satshell_w, "XEphem*Saturn.width", satcategory, 0);
+	sr_reg (satshell_w, "XEphem*Saturn.height", satcategory, 0);
+	sr_reg (satshell_w, "XEphem*Saturn.x", satcategory, 0);
+	sr_reg (satshell_w, "XEphem*Saturn.y", satcategory, 0);
+
+
+	n = 0;
+	satform_w = XmCreateForm (satshell_w, "SaturnF", args, n);
+	XtAddCallback (satform_w, XmNhelpCallback, sm_help_cb, 0);
+	XtManageChild (satform_w);
 
 	/* create the menu bar across the top */
 
@@ -358,7 +371,7 @@ sm_create_form_w()
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "FS", args, n);
-	    set_xmstring (w, XmNlabelString, "Setup Field Stars...");
+	    set_xmstring (w, XmNlabelString, "Field Stars...");
 	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
 	    wtip (w, "Define where GSC and PPM catalogs are to be found");
 	    XtManageChild (w);
@@ -412,7 +425,8 @@ sm_create_form_w()
 		XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 		XtSetArg (args[n], XmNmarginHeight, 0); n++;
 		XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
-		w = XmCreateToggleButton (pd_w, op->name, args, n);
+		w = XmCreateToggleButton (pd_w, op->name ? op->name : "SMTB",
+								    args, n);
 		XtAddCallback (w, XmNvalueChangedCallback, sm_option_cb, 
 						    (XtPointer)(op->flagp));
 		set_xmstring (w, XmNlabelString, op->label);
@@ -423,6 +437,8 @@ sm_create_form_w()
                 if (op->tip)
 		    wtip (w, op->tip);
 		XtManageChild (w);
+		if (op->name)
+		    sr_reg (w, NULL, satcategory, 1);
 	    }
 
 	    /* add a separator */
@@ -498,6 +514,7 @@ sm_create_form_w()
 	XtAddCallback (scale_w, XmNvalueChangedCallback, sm_scale_cb, 0);
 	wtip (scale_w, "Zoom in and out");
 	XtManageChild (scale_w);
+	sr_reg (scale_w, NULL, satcategory, 0);
 
 	/* make the limiting mag scale widget.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -520,6 +537,7 @@ sm_create_form_w()
 	XtAddCallback (limmag_w, XmNvalueChangedCallback, sm_scale_cb, 0);
 	wtip (limmag_w, "Adjust the brightness and limiting magnitude");
 	XtManageChild (limmag_w);
+	sr_reg (limmag_w, NULL, satcategory, 0);
 
 	/* make a frame for the drawing area.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -589,7 +607,7 @@ sm_create_ssform_w()
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	ssform_w = XmCreateFormDialog (toplevel_w, "SaturnStats", args, n);
+	ssform_w = XmCreateFormDialog (satshell_w, "SaturnStats", args, n);
 	set_something (ssform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -694,7 +712,7 @@ sm_create_ssform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "SatTag", args, n); /*PB for sz */
 		set_xmstring (w, XmNlabelString, tag ? tag : " ");
-		sm_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -720,7 +738,7 @@ sm_create_ssform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "SatName", args, n); /*PB for sz*/
 		set_xmstring (w, XmNlabelString, mnames[i].full);
-		sm_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -805,7 +823,7 @@ sm_create_ssform_w()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 70); n++;
 	w = XmCreatePushButton (ssform_w, "Close", args, n);
-	XtAddCallback (w, XmNactivateCallback, sm_sstats_cb, 0);
+	XtAddCallback (w, XmNactivateCallback, sm_sstats_close_cb, 0);
 	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
@@ -822,7 +840,7 @@ sm_create_tvform_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	stform_w = XmCreateFormDialog (toplevel_w, "SaturnTV", args, n);
+	stform_w = XmCreateFormDialog (satshell_w, "SaturnTV", args, n);
 	set_something (stform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (stform_w, XmNunmapCallback, st_unmap_cb, 0);
 	XtAddCallback (stform_w, XmNmapCallback, st_map_cb, NULL);
@@ -863,87 +881,24 @@ int whether;	/* whether setting up for plotting or for not plotting */
 
 	for (i = 0; i < NM; i++) {
 	    for (j = 0; j < CNum; j++)
-		sm_set_a_button (s_w[i][j], whether);
+		buttonAsButton (s_w[i][j], whether);
 	}
-	sm_set_a_button (ringet_w, whether);
-	sm_set_a_button (ringst_w, whether);
+	buttonAsButton (ringet_w, whether);
+	buttonAsButton (ringst_w, whether);
 }
 
-/* set whether the given button looks like a label.
- */
+/* callback when the Close button is activated on the stats menu */
+/* ARGSUSED */
 static void
-sm_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
+sm_sstats_close_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
 {
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    Widget tmpw;
-	    int n;
-
-	    n = 0;
-	    tmpw = XmCreatePushButton (satform_w, "tmp", args, n);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-	    XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    XtDestroyWidget (tmpw);
-	     
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
+	XtUnmanageChild (ssform_w);
 }
 
-/* callback when the Close button is activated on the stats menu or when
- * the More Info button is activated. they each do the same thing.
- */
+/* callback when the More Info button is activated */
 /* ARGSUSED */
 static void
 sm_sstats_cb (w, client, call)
@@ -951,12 +906,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((sswasman = XtIsManaged(ssform_w)) != 0)
-	    XtUnmanageChild (ssform_w);
-	else {
-	    XtManageChild (ssform_w);
-	    sm_set_buttons(sm_selecting);
-	}
+	XtManageChild (ssform_w);
+	sm_set_buttons(sm_selecting);
 }
 
 /* callback from any of the option buttons.
@@ -1062,7 +1013,7 @@ XtPointer call;
 	}
 
         XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!st_pm || nx != last_nx || ny != last_ny) {
+	if (!st_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (st_pm)
 		XFreePixmap (dsp, st_pm);
 	    st_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1114,7 +1065,7 @@ st_track_size()
 	n = 0;
 	XtSetArg (args[n], XmNx, &mfx); n++;
 	XtSetArg (args[n], XmNy, &mfy); n++;
-	XtGetValues (satform_w, args, n);
+	XtGetValues (satshell_w, args, n);
 	n = 0;
 	XtSetArg (args[n], XmNx, &mdx); n++;
 	XtSetArg (args[n], XmNy, &mdy); n++;
@@ -1140,25 +1091,22 @@ XtPointer call;
 	XmToggleButtonSetState (topview_w, False, True);
 }
 
-/* callback when main dialog is unmapped */
+/* callback when main shell is popped down */
 /* ARGSUSED */
 static void
-sm_unmap_cb (w, client, call)
+sm_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((sswasman = (ssform_w && XtIsManaged(ssform_w))) != 0)
-	    XtUnmanageChild (ssform_w);
+	XtUnmanageChild (ssform_w);
+	XtUnmanageChild (stform_w);
+
         if (sm_pm) {
 	    XFreePixmap (XtDisplay(sda_w), sm_pm);
 	    sm_pm = (Pixmap) NULL;
 	}
 
-	/* close top view */
-	if (stform_w && XtIsManaged(stform_w))
-	    XtUnmanageChild (stform_w);
-
 	/* free any field stars */
 	if (fstars) {
 	    free ((void *)fstars);
@@ -1178,8 +1126,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let unmap do all the work */
-	XtUnmanageChild (satform_w);
+	/* let pop down do all the work */
+	XtPopdown (satshell_w);
 }
 
 /* callback from the Movie button
@@ -1245,7 +1193,7 @@ XtPointer call;
 	}
 
         XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!sm_pm || nx != last_nx || ny != last_ny) {
+	if (!sm_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (sm_pm)
 		XFreePixmap (dsp, sm_pm);
 	    sm_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1551,10 +1499,10 @@ int how_much;
 	int i;
 
 	/* see if we should bother */
-	if (!satform_w)
+	if (!satshell_w)
 	    return;
 	wantstats = XtIsManaged(ssform_w) || any_ison() || how_much;
-	if (!XtIsManaged(satform_w) && !wantstats)
+	if (!isUp(satshell_w) && !wantstats)
 	    return;
 
 	watch_cursor (1);
@@ -1592,7 +1540,7 @@ int how_much;
 	    timestamp (np, sdt_w);
 	}
 
-	if (XtIsManaged(satform_w)) {
+	if (isUp(satshell_w)) {
 	    sm_draw_map (sop, etilt, satsize, md);
 	    timestamp (np, dt_w);
 	}
@@ -1600,6 +1548,18 @@ int how_much;
 	watch_cursor (0);
 }
 
+/* called when basic resources change.
+ * rebuild and redraw.
+ */
+void
+sm_newres()
+{
+	if (!satshell_w)
+	    return;
+	make_gcs (sda_w);
+	sm_update (mm_get_now(), 1);
+}
+
 /* given the loc of the moons, draw a nifty little picture.
  * also save resulting screen locs of everything in the screenobj array.
  * scale of the locations is in terms of saturn radii == 1.
@@ -1680,14 +1640,14 @@ MoonData md[NM];
 	 * positive tilt means the front rings are tilted southward.
 	 * always draw the solid s_fgc last in case we are near the ring plane.
 	 */
-	irh = MAPSCALE(2*IRR*fabs(sin(etilt)));
-	irw = (int)MAPSCALE(2*IRR);
-	orh = MAPSCALE(2*ORR*fabs(sin(etilt)));
+	irh = (int)(MAPSCALE(2*IRR*fabs(sin(etilt))));
+	irw = (int)(MAPSCALE(2*IRR));
+	orh = (int)(MAPSCALE(2*ORR*fabs(sin(etilt))));
 	orw = (int)MAPSCALE(2*ORR);
-	irx = nx/2 - MAPSCALE(IRR);
-	iry = ny/2 - MAPSCALE(IRR*fabs(sin(etilt)));
-	orx = nx/2 - MAPSCALE(ORR);
-	ory = ny/2 - MAPSCALE(ORR*fabs(sin(etilt)));
+	irx = (int)(nx/2 - MAPSCALE(IRR));
+	iry = (int)(ny/2 - MAPSCALE(IRR*fabs(sin(etilt))));
+	orx = (int)(nx/2 - MAPSCALE(ORR));
+	ory = (int)(ny/2 - MAPSCALE(ORR*fabs(sin(etilt))));
 	if (irh < RLW || orh < RLW) {
 	    /* too near the ring plane to draw a fill ellipse */
 	    XDrawLine (dsp, sm_pm, s_fgc, orx, ny/2, nx-orx, ny/2);
@@ -1740,8 +1700,8 @@ MoonData md[NM];
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_MAIN, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1762,22 +1722,22 @@ MoonData md[NM];
 	XFillRectangle (dsp, st_pm, s_bgc, 0, 0, nx, ny);
 
 	/* draw saturn in center with unit radius */
-	i = ew*64*raddeg(asin(sin(degrad(sop->s_elong))/sop->s_sdist));
+	i = (int)(ew*64*raddeg(asin(sin(degrad(sop->s_elong))/sop->s_sdist)));
 	XPSFillArc(dsp, st_pm, s_fgc, nx/2-scale1, ny/2-scale1, 2*scale1-1,
 						    2*scale1-1, -i, -180*64);
 	reset_screenobj(SO_TOP);
 	add_screenobj (SO_TOP, sop, nx/2, ny/2);
 
 	/* rings of radius IRR and ORR... simple arcs from up here */
-	irw = (int)MAPSCALE(2*IRR);
-	orw = (int)MAPSCALE(2*ORR);
-	irx = nx/2 - MAPSCALE(IRR);
-	iry = ny/2 - MAPSCALE(IRR);
-	orx = nx/2 - MAPSCALE(ORR);
-	ory = ny/2 - MAPSCALE(ORR);
-	b = 64*raddeg(acos(1./IRR));
+	irw = (int)(MAPSCALE(2*IRR));
+	orw = (int)(MAPSCALE(2*ORR));
+	irx = (int)(nx/2 - MAPSCALE(IRR));
+	iry = (int)(ny/2 - MAPSCALE(IRR));
+	orx = (int)(nx/2 - MAPSCALE(ORR));
+	ory = (int)(ny/2 - MAPSCALE(ORR));
+	b = (int)(64*raddeg(acos(1./IRR)));
 	XDrawArc (dsp, st_pm, s_xgc, irx, iry, irw, irw, b-i, -(180*64+2*b));
-	b = 64*raddeg(acos(1./ORR));
+	b = (int)(64*raddeg(acos(1./ORR)));
 	XDrawArc (dsp, st_pm, s_xgc, orx, ory, orw, orw, b-i, -(180*64+2*b));
 
 	/* draw each moon
@@ -1809,8 +1769,8 @@ MoonData md[NM];
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_TOP, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1840,12 +1800,13 @@ Widget w;
 {
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
+	XFontStruct *fsp;
 	XGCValues gcv;
 	unsigned int gcm;
 	Pixel fg, bg;
 
 	gcm = GCForeground;
-	get_something (w, XmNforeground, (XtArgVal)&fg);
+	get_color_resource (w, "saturnColor", &fg);
 	gcv.foreground = fg;
 	s_fgc = XCreateGC (dsp, win, gcm, &gcv);
 	s_fs = XQueryFont (dsp, XGContextFromGC (s_fgc));
@@ -1853,13 +1814,15 @@ Widget w;
 	s_ch = s_fs->max_bounds.ascent + s_fs->max_bounds.descent;
 
 	gcm = GCForeground;
-	get_something (w, XmNbackground, (XtArgVal)&bg);
+	get_color_resource (w, "SaturnBackground", &bg);
 	gcv.foreground = bg;
 	s_bgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	gcm = GCForeground | GCFunction;
+	gcm = GCForeground | GCFunction | GCFont;
+	fsp = getXResFont ("moonsFont");
 	gcv.foreground = fg ^ bg;
 	gcv.function = GXxor;
+	gcv.font = fsp->fid;
 	s_xgc = XCreateGC (dsp, win, gcm, &gcv);
 }
 
@@ -1940,8 +1903,8 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    ddec = dec0 - op->s_dec;	/* + S */
 	    if (fabs(ddec) > rad || delra(dra)*cdec0 > rad)
 		continue;
-	    dx = dra/scale;
-	    dy = ddec/scale;
+	    dx = (int)(dra/scale);
+	    dy = (int)(ddec/scale);
 	    x = fliplr ? (int)w/2-dx : (int)w/2+dx;
 	    y = fliptb ? (int)h/2+dy : (int)h/2-dy;
 
@@ -2393,4 +2356,4 @@ double *etiltp, *stiltp;	/* tilt from earth and sun, rads southward */
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: satmenu.c,v $ $Date: 1999/03/03 19:06:34 $ $Revision: 1.9 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: satmenu.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.20 $ $Name:  $"};
