@@ -23,14 +23,12 @@ typedef char * qsort_arg;
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
 #include <Xm/Scale.h>
-#include <Xm/Separator.h>
 
 #include "P_.h"
 #include "astro.h"
 #include "circum.h"
+#include "skytoolbar.h"
 
-extern Widget	toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
 extern Colormap xe_cm;
 
 extern Now *mm_get_now P_((void));
@@ -41,6 +39,7 @@ extern void gk_mag P_((double g, double k, double rp, double rho, double *mp));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_all P_((Now *np));
 extern void sv_amagoff P_((void));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
@@ -179,13 +178,16 @@ static Widget stmag_w, ssmag_w, dsmag_w;	/* mag scales */
 static Widget magst_w;		/* mag steps scale */
 static Widget filter_w;		/* main filter dialog */
 
+static char skyfcategory[] = "Sky View -- Filter";	/* Save category */
+
 /* create, but do not manage, the filter dialog.
  * we also apply its state info immediately to the initial resource
  * settings of the toggle buttons since they are used to set up the real filter
  * right away.
  */
 void
-svf_create()
+svf_create(shell_w)
+Widget shell_w;
 {
 	static struct {
 	    char *name;
@@ -204,7 +206,7 @@ svf_create()
 	};
 	Arg args[20];
 	XmString str;
-	Widget w, f_w, s_w;
+	Widget w, f_w;
 	Widget rc_w, lrc_w, rrc_w, mrc_w;
 	int n;
 	int i;
@@ -212,10 +214,11 @@ svf_create()
 	/* create form */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNverticalSpacing, 4); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 6); n++;
+	XtSetArg (args[n], XmNmarginWidth, 8); n++;
+	XtSetArg (args[n], XmNmarginHeight, 8); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	filter_w = XmCreateFormDialog (toplevel_w, "SkyFilter", args, n);
+	filter_w = XmCreateFormDialog (shell_w, "SkyFilter", args, n);
 	set_something (filter_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -236,7 +239,7 @@ svf_create()
 	XtSetArg (args[n], XmNadjustLast, True); n++;
 	XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
 	XtSetArg (args[n], XmNpacking, XmPACK_TIGHT); n++;
-	XtSetArg (args[n], XmNspacing, 1); n++;
+	XtSetArg (args[n], XmNspacing, 2); n++;
 	rc_w = XmCreateRowColumn (filter_w, "CRC", args, n);
 	XtManageChild (rc_w);
 
@@ -255,16 +258,6 @@ svf_create()
 	    XtAddCallback (w, XmNactivateCallback, ctls[i].cb, NULL);
 	}
 
-	/* a sep */
-
-	n = 0;
-	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, rc_w); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	s_w = XmCreateSeparator (filter_w, "Sep", args, n);
-	XtManageChild (s_w);
-
 	/* pile all but the control buttons in a main rowcolumn */
 
 	n = 0;
@@ -272,7 +265,7 @@ svf_create()
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNbottomWidget, s_w); n++;
+	XtSetArg (args[n], XmNbottomWidget, rc_w); n++;
 	XtSetArg (args[n], XmNpacking, XmPACK_COLUMN); n++;
 	XtSetArg (args[n], XmNnumColumns, 2); n++;
 	mrc_w = XmCreateRowColumn (filter_w, "MRC", args, n);
@@ -309,6 +302,7 @@ svf_create()
 	    wtip (ssmag_w, "Faint limit for solar system objects");
 	    XtManageChild (ssmag_w);
 	    XmStringFree (str);
+	    sr_reg (ssmag_w, NULL, skyfcategory, 0);
 
 	/* add mag step scale in a frame */
 
@@ -329,6 +323,7 @@ svf_create()
 	    XtManageChild (magst_w);
 	    XmStringFree (str);
 	    XmScaleGetValue (magst_w, &magstp);
+	    sr_reg (magst_w, NULL, skyfcategory, 0);
 
 	/* make the right rc */
 	n = 0;
@@ -362,6 +357,7 @@ svf_create()
 	    wtip (stmag_w, "Faint limit for stars.");
 	    XtManageChild (stmag_w);
 	    XmStringFree (str);
+	    sr_reg (stmag_w, NULL, skyfcategory, 0);
 
 	n = 0;
 	f_w = XmCreateFrame (rrc_w, "DSF", args, n);
@@ -382,6 +378,7 @@ svf_create()
 	    wtip (dsmag_w, "Faint limit for deep sky objects.");
 	    XtManageChild (dsmag_w);
 	    XmStringFree (str);
+	    sr_reg (dsmag_w, NULL, skyfcategory, 0);
 
 	/* set the real filter according to the state of the toggle buttons */
 	svf_apply();
@@ -619,6 +616,7 @@ FilterCat *fcp;
 		XmStringFree(str);
 		XtManageChild (tb_w);
 		ftbp->tbw = tb_w;
+		sr_reg (tb_w, NULL, skyfcategory, 0);
 	}
 }
 
@@ -675,6 +673,8 @@ svf_apply()
 	    }
 	}
 
+	svtb_updateCTTT(fclass_table, type_table);
+
 	XmScaleGetValue (stmag_w, &stmag);
 	XmScaleGetValue (ssmag_w, &ssmag);
 	XmScaleGetValue (dsmag_w, &dsmag);
@@ -1042,4 +1042,4 @@ AutoMag **ampp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfiltmenu.c,v $ $Date: 1999/10/06 16:18:53 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfiltmenu.c,v $ $Date: 2000/12/01 08:52:35 $ $Revision: 1.10 $ $Name:  $"};
