@@ -5,9 +5,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-
 #include <fcntl.h>
-
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,6 +32,9 @@ extern int unlink();
 #include <fcntl.h>
 #endif
 
+#ifndef X_OK
+#define	X_OK	1
+#endif
 
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
@@ -67,8 +68,11 @@ extern XtAppContext xe_app;
 #define XtD XtDisplay(toplevel_w)
 extern Colormap xe_cm;
 
+extern Widget svshell_w;
+
 extern char *expand_home P_((char *path));
 extern char *getShareDir P_((void));
+extern char *getPrivateDir P_((void));
 extern char *getXRes P_((char *name, char *def));
 extern char *syserrstr P_((void));
 extern int confirm P_((void));
@@ -78,9 +82,11 @@ extern int gray_ramp P_((Display *dsp, Colormap cm, Pixel **pixp));
 extern int openh P_((char *name, int flags, ...));
 extern int xy2RADec P_((FImage *fip, double x, double y, double *rap,
     double *decp));
+extern void defaultTextFN P_((Widget w, int cols, char *x, char *y));
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
 extern void mm_newcaldate P_((double newmjd));
+extern void msg_manage P_((void));
 extern void pm_set P_((int percentage));
 extern void pm_down P_((void));
 extern void pm_up P_((void));
@@ -90,6 +96,7 @@ extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     void (*func3)(void)));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void stopd_up P_((void));
 extern void stopd_down P_((void));
 extern void sv_getcenter P_((int *aamode, double *fov,
@@ -100,6 +107,7 @@ extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *msg, int app_modal));
 
 void sf_off P_((void));
+void sf_go_cb P_((Widget w, XtPointer client, XtPointer call));
 
 static void build_histo P_((void));
 static void build_colormap P_((void));
@@ -110,8 +118,6 @@ static void sf_newLOHI P_((void));
 static void sf_newImage P_((void));
 static void initFSB P_((Widget w));
 static void initPubShared P_((Widget rc_w, Widget fsb_w));
-static void sf_stsci_cb P_((Widget w, XtPointer client, XtPointer call));
-static void sf_eso_cb P_((Widget w, XtPointer client, XtPointer call));
 static void sf_save_cb P_((Widget w, XtPointer client, XtPointer call));
 static void save_file P_((void));
 static void sf_open_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -149,6 +155,7 @@ static int prepOpen P_((char fn[], char errmsg[]));
 static Widget sf_w;		/* main dialog */
 static Widget gunzip_w;		/* TB for whether to retrieve compressed */
 static Widget savefn_w;		/* TF for save filename */
+static Widget stsci_w;		/* TB for STScI, else ESO */
 static Widget fsb_w;		/* FSB for opening a file */
 static Widget hdr_w;		/* ScrolledText for the FITS header */
 static Widget fda_w;		/* Drawing area for the contrast map */
@@ -156,6 +163,10 @@ static Widget gamma_w;		/* gamma scale */
 static Widget autoname_w;	/* TB for whether to auto set save filename */ 
 static Widget obsdate_w;	/* label for obs date string */
 static Widget setobsdate_w;	/* PB to set main to obs date */
+static Widget live_w;		/* TB whether want live dragging */
+static Widget dss1_w;		/* TB set to use DSS 1 */
+static Widget dss2r_w;		/* TB set to use DSS 2 red */
+static Widget dss2b_w;		/* TB set to use DSS 2 blue */
 
 static FImage fim;		/* current FITS image */
 static Pixmap fpm;		/* current pixmap with image */
@@ -166,19 +177,25 @@ static int fdepth;		/* depth of image, in bits */
 static int fbpp;		/* bits per pixel in image: 1, 8, 16 or 32 */
 static XtIntervalId readId;	/* set while working on reading from socket  */
 static int lopix, hipix;	/* lo and hi image pixel values */
-static int histo[MAXCAMPIX];	/* histogram of current image */
+static int *histo;		/* malloced histogram of current image */
 static int h_mini, h_maxi;	/* histo[] index to first and last pixel used */
 static int h_peaki;		/* index of largest value in histo[] */
-static char colormap[MAXCAMPIX];/* gray[] indeces for current image */
+static char *colormap;		/* malloced gray[] indeces for current image */
 static int want_inv;		/* set when want inverse video effect */
 
+/* which survey */
+typedef enum {
+    DSS_1, DSS_2R, DSS_2B
+} Survey;
+static Survey whichSurvey P_((void));
+
 static char hxr[] = "FITSHistoColor";	/* X resource name of histogram color */
 static char mxr[] = "FITSMapColor";	/* X resource name of colormap color */
 #define	MMH	10		/* map marker height */
 #define	MMW	5		/* map marker half-width */
 
-static char fitsd[] = "PrivateDir";	/* resource name of FITS working dir */
 static char fitsp[] = "FITSpattern";	/* resource name of FITS file pattern */
+static char gcmd[] = "gunzip";	/* gunzip command */
 
 static XImage *glass_xim;       /* glass XImage -- 0 means new or can't */
 static GC glassGC;              /* GC for glass border */
@@ -191,32 +208,41 @@ typedef enum { LH_GAMMA, LH_HEQ } LowHi;
 static LowHi lowhi;		/* default is set from new TBs */
 #define	GAM_MAX		3.0	/* gamma ranges from 1/GAM_MAX .. GAM_MAX */
 
+static char skyfitscategory[] = "Sky View -- Images";	/* Save category */
+
 /* called to manage the fits dialog.
  */
 void
 sf_manage()
 {
-	if (!sf_w) {
-	    /* create the dialog */
+	if (!sf_w)
 	    sf_create();
 
-	    /* get a grayscale ramp to use */
-	    ngray = gray_ramp (XtD, xe_cm, &gray);
-	}
-
 	XtManageChild(sf_w);
 }
 
 /* called to unmanage the fits dialog.
- * we discard the image too.
+ * this is only called when the Sky View is also going away, so we
+ * can discard memory for the image, histogram and colormap too.
  */
 void
 sf_unmanage()
 {
-	if (sf_w) {
-	    sf_off();
-	    XtUnmanageChild (sf_w);
+	if (!sf_w)
+	    return;
+
+	sf_off();
+
+	if (histo) {
+	    free (histo);
+	    histo = 0;
 	}
+	if (colormap) {
+	    free (colormap);
+	    colormap = 0;
+	}
+
+	XtUnmanageChild (sf_w);
 }
 
 /* return 1 if dialog is up, else 0.
@@ -430,8 +456,17 @@ build_histo()
 	int npix = fip->sw*fip->sh;
 	int i;
 
+	/* insure memory */
+	if (!histo) {
+	    histo = (int *) malloc (MAXCAMPIX * sizeof(histo[0]));
+	    if (!histo) {
+		printf ("No memory for histogram\n");
+		exit (1);
+	    }
+	}
+
 	/* zero the histogram */
-	memset ((void *)histo, 0, sizeof(histo));
+	memset ((void *)histo, 0, MAXCAMPIX*sizeof(histo[0]));
 
 	/* find histogram */
 	for (i = 0; i < npix; i++)
@@ -460,6 +495,15 @@ build_colormap()
 	int range = hipix - lopix;
 	int total, sum, i;
 
+	/* insure memory */
+	if (!colormap) {
+	    colormap = malloc (MAXCAMPIX*sizeof(colormap[0]));
+	    if (!colormap) {
+		printf ("No memory for colormap\n");
+		exit (1);
+	    }
+	}
+
 	switch (lowhi) {
 	case LH_GAMMA:
 	    for (i = 0; i < lopix; i++)
@@ -727,13 +771,12 @@ sf_create()
 {
 	Widget tf_w, bf_w;
 	Widget heq_w, g_w, rc_w, rb_w;
+	Widget go_w;
 	Widget inv_w;
 	Widget p_w, fr_w;
 	Widget n_w, w_w;
 	Widget h_w;
 	Widget w;
-	int h, g;
-	char fn[1024];
 	EventMask mask;
 	Arg args[20];
 	int n;
@@ -741,10 +784,11 @@ sf_create()
 	/* create form */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNallowResize, True); n++;
+	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
+	XtSetArg (args[n], XmNmarginWidth, 5); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	sf_w = XmCreateFormDialog (toplevel_w, "SkyFITS", args, n);
+	sf_w = XmCreateFormDialog (svshell_w, "SkyFITS", args, n);
 	set_something (sf_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (sf_w, XmNhelpCallback, sf_help_cb, NULL);
 
@@ -770,74 +814,141 @@ sf_create()
 	/* the top form */
 
 	n = 0;
-	XtSetArg (args[n], XmNfractionBase, 16); n++;
 	tf_w = XmCreateForm (p_w, "TF", args, n);
 	XtManageChild (tf_w);
 
-	    /* buttons to fetch networked images */
+	    /* controls to fetch networked images */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNtopOffset, 6); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	    n_w = XmCreateLabel (tf_w, "Lab", args, n);
-	    set_xmstring (n_w, XmNlabelString,
-				"Retrieve Digitized Sky Survey Image from ...");
-	    XtManageChild (n_w);
+	    go_w = XmCreatePushButton (tf_w, "Get", args, n);
+	    wtip (go_w, "Retrieve image of Sky View center over Internet");
+	    XtAddCallback (go_w, XmNactivateCallback, sf_go_cb, NULL);
+	    XtManageChild (go_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNtopOffset, 8); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNleftWidget, go_w); n++;
+	    XtSetArg (args[n], XmNleftOffset, 4); n++;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    w = XmCreateLabel (tf_w, "GL", args, n);
+	    set_xmstring (w,XmNlabelString,"Digitized Sky Survey image ...");
+	    XtManageChild (w);
+
+	    /* institution selection */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNtopWidget, n_w); n++;
-	    XtSetArg (args[n], XmNtopOffset, 2); n++;
+	    XtSetArg (args[n], XmNtopWidget, go_w); n++;
+	    XtSetArg (args[n], XmNtopOffset, 3); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 1); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 5); n++;
-	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
-	    w = XmCreatePushButton (tf_w, "STScI", args, n);
-	    XtAddCallback (w, XmNactivateCallback, sf_stsci_cb, NULL);
-	    wtip (w, "Retrieve DSS image from STScI");
-	    set_xmstring (w, XmNlabelString, "STScI");
+	    XtSetArg (args[n], XmNleftPosition, 5); n++;
+	    w = XmCreateLabel (tf_w, "From:", args, n);
 	    XtManageChild (w);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNtopWidget, n_w); n++;
-	    XtSetArg (args[n], XmNtopOffset, 2); n++;
+	    XtSetArg (args[n], XmNtopWidget, go_w); n++;
+	    XtSetArg (args[n], XmNtopOffset, 1); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 6); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 10); n++;
-	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
-	    w = XmCreatePushButton (tf_w, "ESO", args, n);
-	    wtip (w, "Retrieve DSS image from ESO");
-	    XtAddCallback (w, XmNactivateCallback, sf_eso_cb, NULL);
-	    set_xmstring (w, XmNlabelString, "ESO");
+	    XtSetArg (args[n], XmNleftPosition, 25); n++;
+	    XtSetArg (args[n], XmNpacking, XmPACK_TIGHT); n++;
+	    XtSetArg (args[n], XmNspacing, 10); n++;
+	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	    rb_w = XmCreateRadioBox (tf_w, "GRB", args, n);
+	    XtManageChild (rb_w);
+
+		n = 0;
+		XtSetArg (args[n], XmNspacing, 4); n++;
+		stsci_w = XmCreateToggleButton (rb_w, "STScI", args, n);
+		wtip (stsci_w, "Get image from Maryland USA");
+		XtManageChild (stsci_w);
+		sr_reg (stsci_w, NULL, skyfitscategory, 1);
+
+		/* stsci sets logic */
+		n = 0;
+		XtSetArg (args[n], XmNspacing, 4); n++;
+		XtSetArg(args[n],XmNset,!XmToggleButtonGetState(stsci_w)); n++;
+		w = XmCreateToggleButton (rb_w, "ESO", args, n);
+		wtip (stsci_w, "Get image from Germany");
+		XtManageChild (w);
+
+	    /* survey selection */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, rb_w); n++;
+	    XtSetArg (args[n], XmNtopOffset, 3); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 5); n++;
+	    w = XmCreateLabel (tf_w, "Survey:", args, n);
 	    XtManageChild (w);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNtopWidget, n_w); n++;
-	    XtSetArg (args[n], XmNtopOffset, 2); n++;
+	    XtSetArg (args[n], XmNtopWidget, rb_w); n++;
+	    XtSetArg (args[n], XmNtopOffset, 1); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 11); n++;
-	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNrightPosition, 15); n++;
+	    XtSetArg (args[n], XmNleftPosition, 25); n++;
+	    XtSetArg (args[n], XmNpacking, XmPACK_TIGHT); n++;
+	    XtSetArg (args[n], XmNspacing, 10); n++;
+	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	    rb_w = XmCreateRadioBox (tf_w, "GRB", args, n);
+	    XtManageChild (rb_w);
+
+		n = 0;
+		XtSetArg (args[n], XmNspacing, 4); n++;
+		dss1_w = XmCreateToggleButton (rb_w, "DSS1", args, n);
+		set_xmstring (dss1_w, XmNlabelString, "DSS 1");
+		wtip (dss1_w, "Original DSS");
+		XtManageChild (dss1_w);
+		sr_reg (dss1_w, NULL, skyfitscategory, 1);
+
+		n = 0;
+		XtSetArg (args[n], XmNspacing, 4); n++;
+		dss2r_w = XmCreateToggleButton (rb_w, "DSS2R", args, n);
+		set_xmstring (dss2r_w, XmNlabelString, "DSS 2 Red");
+		wtip (dss2r_w, "DSS 2, Red band (90% complete)");
+		XtManageChild (dss2r_w);
+		sr_reg (dss2r_w, NULL, skyfitscategory, 1);
+
+		n = 0;
+		XtSetArg (args[n], XmNspacing, 4); n++;
+		dss2b_w = XmCreateToggleButton (rb_w, "DSS2B", args, n);
+		set_xmstring (dss2b_w, XmNlabelString, "DSS 2 Blue");
+		wtip (dss2b_w, "DSS 2, Blue band (50% complete)");
+		XtManageChild (dss2b_w);
+		sr_reg (dss2b_w, NULL, skyfitscategory, 1);
+
+	    /* compressed? */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, rb_w); n++;
+	    XtSetArg (args[n], XmNtopOffset, 1); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 5); n++;
 	    XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
 	    XtSetArg (args[n], XmNindicatorOn, True); n++;
+	    XtSetArg (args[n], XmNspacing, 4); n++;
 	    gunzip_w = XmCreateToggleButton (tf_w, "UseGzip", args, n);
 	    wtip(gunzip_w,"Retrieve compressed then uncompress using `gunzip'");
-	    set_xmstring (gunzip_w, XmNlabelString, "compressed");
+	    set_xmstring (gunzip_w, XmNlabelString,
+				    "Compressed (recommended if have gunzip)");
 	    XtManageChild (gunzip_w);
+	    sr_reg (gunzip_w, NULL, skyfitscategory, 1);
 
 	    /* header, with possible date */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, gunzip_w); n++;
-	    XtSetArg (args[n], XmNtopOffset, 18); n++;
+	    XtSetArg (args[n], XmNtopOffset, 10); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    h_w = XmCreateLabel (tf_w, "Lab", args, n);
@@ -847,10 +958,10 @@ sf_create()
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, gunzip_w); n++;
-	    XtSetArg (args[n], XmNtopOffset, 16); n++;
+	    XtSetArg (args[n], XmNtopOffset, 10); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    setobsdate_w = XmCreatePushButton (tf_w, "SO", args, n);
-	    set_xmstring (setobsdate_w, XmNlabelString, "Set main");
+	    set_xmstring (setobsdate_w, XmNlabelString, "Set time");
 	    XtAddCallback (setobsdate_w, XmNactivateCallback, sf_setdate_cb, 0);
 	    wtip(setobsdate_w,"Set main XEphem time to this Observation time");
 	    XtManageChild (setobsdate_w);
@@ -859,7 +970,7 @@ sf_create()
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, gunzip_w); n++;
-	    XtSetArg (args[n], XmNtopOffset, 18); n++;
+	    XtSetArg (args[n], XmNtopOffset, 10); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNleftWidget, h_w); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
@@ -896,14 +1007,6 @@ sf_create()
 
 	    /* label and support for contrast map */
 
-	    n = 0;
-	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNtopOffset, 16); n++;
-	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	    w = XmCreateLabel (bf_w, "GML", args, n);
-	    set_xmstring (w, XmNlabelString, "Gray scale:");
-	    XtManageChild (w);
-
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNtopOffset, 16); n++;
@@ -918,7 +1021,7 @@ sf_create()
 	    XtSetArg (args[n], XmNtopOffset, 16); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNrightWidget, n_w); n++;
-	    XtSetArg (args[n], XmNrightOffset, 15); n++;
+	    XtSetArg (args[n], XmNrightOffset, 10); n++;
 	    w_w = XmCreatePushButton (bf_w, "Wide", args, n);
 	    XtAddCallback (w_w, XmNactivateCallback, sf_wide_cb, 0);
 	    wtip (w_w, "Set gray scale wide open");
@@ -932,10 +1035,35 @@ sf_create()
 	    XtSetArg (args[n], XmNrightOffset, 15); n++;
 	    inv_w = XmCreateToggleButton (bf_w, "Inverse", args, n);
 	    XtAddCallback (inv_w, XmNvalueChangedCallback, sf_inv_cb, 0);
-	    set_xmstring (inv_w, XmNlabelString, "Inverse");
 	    wtip (inv_w, "When pushed in, image is displayed black-on-white");
 	    XtManageChild (inv_w);
 	    want_inv = XmToggleButtonGetState (inv_w);
+	    sr_reg (inv_w, NULL, skyfitscategory, 1);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNtopOffset, 16); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNrightWidget, inv_w); n++;
+	    XtSetArg (args[n], XmNrightOffset, 15); n++;
+	    live_w = XmCreateToggleButton (bf_w, "Live", args, n);
+	    set_xmstring (live_w, XmNlabelString, "Live dragging");
+	    wtip (live_w,
+		"When pushed in, image is updated while controls are moved");
+	    XtManageChild (live_w);
+	    sr_reg (live_w, NULL, skyfitscategory, 1);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNtopOffset, 16); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNrightWidget, live_w); n++;
+	    XtSetArg (args[n], XmNrightOffset, 15); n++;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    w = XmCreateLabel (bf_w, "GML", args, n);
+	    set_xmstring (w, XmNlabelString, "Gray scale:");
+	    XtManageChild (w);
 
 	    /* scale methods in a RB */
 
@@ -955,8 +1083,11 @@ sf_create()
 		set_xmstring (heq_w, XmNlabelString, "Hist Eq");
 		wtip (heq_w,"Assign equal number of pixels to each gray level");
 		XtManageChild (heq_w);
+		sr_reg (heq_w, NULL, skyfitscategory, 1);
 
+		/* "HEq" sets logic state */
 		n = 0;
+		XtSetArg (args[n], XmNset, !XmToggleButtonGetState(heq_w)); n++;
 		g_w = XmCreateToggleButton (rb_w, "Gamma", args, n);
 		XtAddCallback (g_w, XmNvalueChangedCallback, sf_gmlohi_cb,
 							(XtPointer)LH_GAMMA);
@@ -964,21 +1095,8 @@ sf_create()
 		wtip (g_w, "Gray scale based on (pixel)^Gamma.");
 		XtManageChild (g_w);
 
-		/* insure exactly 1 is on -- defaults to gamma */
-		h = XmToggleButtonGetState(heq_w) ? 1 : 0;
-		g = XmToggleButtonGetState(g_w) ? 1 : 0;
-		if (h + g != 1) {
-		    XmToggleButtonSetState (heq_w, False, False);
-		    h = 0;
-		    XmToggleButtonSetState (g_w, True, False);
-		    g = 1;
-		}
-
 		/* set the lowhi enum to match */
-		if (h)
-		    lowhi = LH_HEQ;
-		if (g)
-		    lowhi = LH_GAMMA;
+		lowhi = XmToggleButtonGetState(heq_w) ? LH_HEQ : LH_GAMMA;
 
 	    /* gamma scale between rb and right edge */
 
@@ -999,10 +1117,10 @@ sf_create()
 	    XtSetArg (args[n], XmNsensitive, lowhi == LH_GAMMA); n++;
 	    gamma_w = XmCreateScale (bf_w, "GammaScale", args, n);
 	    XtAddCallback (gamma_w, XmNdragCallback, sf_gamma_cb, NULL);
-	    XtAddCallback (gamma_w, XmNvalueChangedCallback, sf_gamma_cb,
-								    NULL);
+	    XtAddCallback (gamma_w, XmNvalueChangedCallback, sf_gamma_cb, NULL);
 	    wtip (gamma_w, "Select new Gamma factor");
 	    XtManageChild (gamma_w);
+	    sr_reg (gamma_w, NULL, skyfitscategory, 1);
 
 	    /* drawing area in a frame for the contrast map */
 
@@ -1056,16 +1174,16 @@ sf_create()
 	    set_xmstring (autoname_w, XmNlabelString, "Auto name");
 	    XtManageChild (autoname_w);
 	    wtip (autoname_w, "When on, automatically chooses a filename based on RA and Dec");
+	    sr_reg (autoname_w, NULL, skyfitscategory, 1);
 
-	    (void) sprintf (fn, "%s/xxx.fts", getXRes (fitsd, "fts"));
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, autoname_w); n++;
 	    XtSetArg (args[n], XmNtopOffset, 6); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	    XtSetArg (args[n], XmNvalue, fn); n++;
 	    savefn_w = XmCreateTextField (bf_w, "SaveFN", args, n);
+	    defaultTextFN (savefn_w, 0, getPrivateDir(), "xxx.fts");
 	    XtAddCallback (savefn_w, XmNactivateCallback, sf_save_cb, NULL);
 	    wtip (savefn_w, "Enter name of file to write, then press Enter");
 	    XtManageChild (savefn_w);
@@ -1104,6 +1222,9 @@ sf_create()
 	    XtManageChild (fsb_w);
 	    initFSB(fsb_w);
 	    initPubShared (rc_w, fsb_w);
+
+	/* get a grayscale ramp to use */
+	ngray = gray_ramp (XtD, xe_cm, &gray);
 }
 
 /* init the directory and pattern resources of the given FileSelectionBox.
@@ -1116,8 +1237,8 @@ Widget fsb_w;
 	Widget w;
 
 	/* set default dir and pattern */
-	set_xmstring (fsb_w, XmNdirectory, expand_home(getXRes (fitsd, "fts")));
-	set_xmstring (fsb_w, XmNpattern, getXRes (fitsp, "*.f*t[sh]"));
+	set_xmstring (fsb_w, XmNdirectory, getPrivateDir());
+	set_xmstring (fsb_w, XmNpattern, getXRes (fitsp, "*.f*t*"));
 
 	/* change some button labels.
 	 * N.B. can't add tips because these are really Gadgets.
@@ -1168,8 +1289,7 @@ XtPointer call;
 {
 	Widget fsb_w = (Widget)client;
 
-	set_xmstring (fsb_w, XmNdirectory,
-				    expand_home(getXRes("PrivateDir", "work")));
+	set_xmstring (fsb_w, XmNdirectory, getPrivateDir());
 }
 
 /* build a set of PB in RC rc_w so that they
@@ -1180,6 +1300,7 @@ initPubShared (rc_w, fsb_w)
 Widget rc_w, fsb_w;
 {
 	Arg args[20];
+	char tip[1024];
 	Widget w;
 	int n;
 
@@ -1191,46 +1312,38 @@ Widget rc_w, fsb_w;
 	n = 0;
 	w = XmCreatePushButton (rc_w, "Private", args, n);
 	XtAddCallback(w, XmNactivateCallback, privateDirCB, (XtPointer)fsb_w);
-	wtip (w,"Set Filter directory to value of XEphem.PrivateDir resource");
+	sprintf (tip, "Set directory to %s", getPrivateDir());
+	wtip (w, XtNewString(tip));
 	XtManageChild (w);
 
 	n = 0;
 	w = XmCreatePushButton (rc_w, "Shared", args, n);
 	XtAddCallback(w, XmNactivateCallback, sharedDirCB, (XtPointer)fsb_w);
-	wtip (w, "Set Filter directory to XEphem.ShareDir/fits");
+	sprintf (tip, "Set directory to %s/fits", getShareDir());
+	wtip (w, XtNewString(tip));
 	XtManageChild (w);
 }
 
-/* called when STScI is hit */
+/* called when Get PB or toolbar PB is hit */
 /* ARGSUSED */
-static void
-sf_stsci_cb (w, client, call)
+void
+sf_go_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
+	if (!sf_w)
+	    sf_create();
+
 	if (readId) {
-	    xe_msg ("Download already in progress..", 1);
+	    xe_msg ("DSS download is already in progress.", 1);
 	    return;
 	}
 
-	stsci_fits();
-}
-
-/* called when ESO is hit */
-/* ARGSUSED */
-static void
-sf_eso_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	if (readId) {
-	    xe_msg ("Download already in progress..", 1);
-	    return;
-	}
-
-	eso_fits();
+	if (XmToggleButtonGetState(stsci_w))
+	    stsci_fits();
+	else
+	    eso_fits();
 }
 
 /* called when CR is hit in the Save text field or the Save PB is hit */
@@ -1292,7 +1405,7 @@ save_file()
 	    xe_msg (buf2, 1);
 	} else {
 	    (void) sprintf (buf, "%s:\nwritten successfully", fn);
-	    xe_msg (buf, 1);
+	    xe_msg (buf, confirm());
 	}
 
 	(void) close (fd);
@@ -1425,6 +1538,8 @@ Boolean *continue_to_dispatch;
 	if (!(b1p || b1r || m1))
 	    return;
 
+	watch_cursor(1);
+
 	/* where are we? */
 	XQueryPointer (dsp, win, &root, &child, &rx, &ry, &x, &y, &mask);
 	get_something (w, XmNwidth, (XtArgVal)&wid);
@@ -1451,9 +1566,11 @@ Boolean *continue_to_dispatch;
 	build_colormap();
 	sf_drawHistogram();
 
-	/* display net result when release */
-	if (b1r)
+	/* display net result if dragging else just when release */
+	if (XmToggleButtonGetState(live_w) || b1r)
 	    sv_newfits();
+
+	watch_cursor(0);
 }
 
 /* expose callback for the gray map */
@@ -1602,11 +1719,14 @@ XtPointer call;
 	if (!fimok)
 	    return;
 
-	/* always draw new hist, but redo image when let go  */
+	watch_cursor(1);
+
 	build_colormap();
 	sf_drawHistogram();
-	if (s->reason == XmCR_VALUE_CHANGED)
+	if (XmToggleButtonGetState(live_w) || s->reason == XmCR_VALUE_CHANGED)
 	    sv_newfits();
+
+	watch_cursor(0);
 }
 
 
@@ -1644,8 +1764,9 @@ loweredge()
 	return (i);
 }
 
-/* draw the current histgram in the drawing area, and the lo/hi markers.
+/* draw histgram, if any, in the drawing area, and the lo/hi markers.
  * we stretch and only show from h_mini .. h_maxi
+ * N.B. we might be called before the first expose
  */
 static void
 sf_drawHistogram()
@@ -1655,12 +1776,17 @@ sf_drawHistogram()
 	static Pixel ip, hp, mp;
 	Display *dsp = XtDisplay(fda_w);
 	Window win = XtWindow(fda_w);
-	int h_peak = histo[h_peaki];
+	int h_peak;
 	XPoint pts[MAXPTS];
 	Dimension wid, hei;
 	int np;
 	int x;
 
+	/* might not be far enough */
+	if (!win || !histo)
+	    return;
+	h_peak = histo[h_peaki];
+
 	/* clear */
 	XClearArea (dsp, win, 0, 0, 0, 0, False);
 
@@ -1969,11 +2095,54 @@ char buf[];
 	return (-1);
 }
 
+/* return 0 if have gunzip else -1 */
+static int
+chk_gunzip()
+{
+	char *path;
+	char fn[1024];
+	char *p, *q;
+
+	if (gcmd[0] == '/')
+	    return (access (gcmd, X_OK));
+
+	path = getenv ("PATH");
+	if (!path)
+	    return (-1);
+	for (p = path; *p; ) {
+	    for (q = fn; *p && *p != ':'; )
+		*q++ = *p++;
+	    *q = '\0';
+	    strcat (fn, "/");
+	    strcat (fn, gcmd);
+	    if (access (fn, X_OK) == 0)
+		return (0);
+	    if (*p == ':')
+		p++;
+	}
+
+	return (-1);
+}
+
 /* return 1 if want to use gunzip, else 0 */
 static int
 use_gunzip()
 {
-	return (XmToggleButtonGetState (gunzip_w));
+	int want = XmToggleButtonGetState (gunzip_w);
+
+	if (!want)
+	    return (0);
+
+	if (chk_gunzip() < 0) {
+	    char msg[1024];
+	    (void) sprintf (msg,
+		    "Can not find %s.\nProceeding without compression", gcmd);
+	    XmToggleButtonSetState (gunzip_w, False, False);
+	    xe_msg (msg, 1);
+	    return (0);
+	}
+
+	return (1);
 }
 
 /* setup the pipe between gunzip and xephem to decompress the data.
@@ -1984,10 +2153,11 @@ setup_gunzip_pipe(int sockfd)
 {
 	int gzfd[2];		/* file descriptors for gunzip pipe */
 	int pid;
+	int errfd;
 
 	/* make the pipe to gunzip */
 	if (pipe(gzfd) < 0) {
-	    xe_msg ("Can not make pipe to gunzip", 0);
+	    xe_msg ("Can not make pipe for gunzip", 1);
 	    return (-1);
 	}
 
@@ -2002,12 +2172,17 @@ setup_gunzip_pipe(int sockfd)
 	    close (sockfd);	/* do not need after dup */
 	    dup2 (gzfd[1], 1);	/* write side of pipe becomes gunzip's stdout */
 	    close (gzfd[1]);	/* do not need after dup */
-	    execlp ("gunzip", "gunzip", "-c", NULL);
+	    errfd = open ("/tmp/gunzip.err", O_CREAT|O_WRONLY, 0666);
+	    if (errfd >= 0) {
+		dup2 (errfd, 2);/* err log becomes gunzip's stderr */
+		close (errfd);
+	    }
+	    execlp (gcmd, gcmd, "-c", NULL);
 	    exit (1);		/* exit in case gunzip disappeared */
-	    break;
+	    break;		/* :) */
 
 	case -1:	/* fork failed */
-	    xe_msg ("gunzip fork failed", 0);
+	    xe_msg ("Can not fork for gunzip", 1);
 	    return (-1);
 
 	default:	/* parent */
@@ -2023,6 +2198,16 @@ setup_gunzip_pipe(int sockfd)
 	return (pid);
 }
 
+static Survey
+whichSurvey()
+{
+	if (XmToggleButtonGetState(dss2r_w))
+	    return (DSS_2R);
+	if (XmToggleButtonGetState(dss2b_w))
+	    return (DSS_2B);
+	return (DSS_1);
+}
+
 /* start an input stream reading a FITS image from ESO */
 static void
 eso_fits()
@@ -2032,6 +2217,7 @@ eso_fits()
 	double fov, alt, az, ra, dec;
 	char rastr[32], *rap, decstr[32], *decp;
 	char buf[1024], msg[1024];
+	char *survey;
 	int gzpid;
 	int sockfd;
 	int aamode;
@@ -2057,6 +2243,14 @@ eso_fits()
 	if (fov < MINDSSFOV)
 	    fov = MINDSSFOV;
 
+	/* get desired survey */
+	switch (whichSurvey()) {
+	default:
+	case DSS_1:  survey = "DSS1"; break;
+	case DSS_2R: survey = "DSS2-red"; break;
+	case DSS_2B: survey = "DSS2-blue"; break;
+	}
+
 	/* format and send the request.
 	 * N.B. ESO can't tolerate leading blanks in ra dec specs
 	 */
@@ -2066,8 +2260,8 @@ eso_fits()
 	fs_sexa (decstr, dec, 3, 3600);
 	for (decp = decstr; *decp == ' '; decp++)
 	    continue;
-	(void) sprintf (buf, "/dss/dss?ra=%s&dec=%s&mime-type=%s&x=%.0f&y=%.0f HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n",
-						rap, decp,
+	(void) sprintf (buf, "/dss/dss?ra=%s&dec=%s&equinox=J2000&Sky-Survey=%s&mime-type=%s&x=%.0f&y=%.0f HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n",
+						rap, decp, survey,
 			use_gunzip() ? "display/gz-fits" : "application/x-fits",
 						fov, fov, PATCHLEVEL);
 	(void) sprintf (msg, "Command to %s:", host);
@@ -2093,14 +2287,13 @@ eso_fits()
 	/* if do not see a fits file, show what we do find */
 	if (!sawfits) {
 	    xe_msg (" ", 0);
-	    xe_msg ("Network message follows:", 0);
-	    xe_msg (" ", 0);
+	    xe_msg ("Message from server", 1);
+	    xe_msg ("------------------", 0);
 	    while (recvline (sockfd, buf, sizeof(buf)) > 0)
 		xe_msg (buf, 0);
-	    xe_msg (" ", 0);
-	    xe_msg ("End of Network message", 0);
-	    xe_msg (" ", 0);
-	    xe_msg ("Error -- see Main->File->Messages", 1);
+	    xe_msg ("------------------", 0);
+	    xe_msg ("End of Message from server", 0);
+	    msg_manage();
 	    watch_cursor (0);
 	    close (sockfd);
 	    stopd_down();
@@ -2157,6 +2350,7 @@ stsci_fits()
 	static FImage sfim, *fip = &sfim;
 	double fov, alt, az, ra, dec;
 	char buf[1024], msg[1024];
+	char *survey;
 	int gzpid;
 	int sockfd;
 	int aamode;
@@ -2182,9 +2376,17 @@ stsci_fits()
 	if (fov < MINDSSFOV)
 	    fov = MINDSSFOV;
 
+	/* get desired survey */
+	switch (whichSurvey()) {
+	default:
+	case DSS_1:  survey = "1"; break;
+	case DSS_2R: survey = "2r"; break;
+	case DSS_2B: survey = "2b"; break;
+	}
+
 	/* format and send the request */
-	(void) sprintf(buf,"/cgi-bin/dss_search?ra=%.5f&dec=%.5f&equinox=J2000&height=%.0f&width=%.0f&format=FITS&compression=%s&version=3 HTTP/1.0\nUser-Agent: xephem/%s\n\n",
-						ra, dec, fov, fov,
+	(void) sprintf(buf,"/cgi-bin/dss_search?ra=%.5f&dec=%.5f&equinox=J2000&v=%s&height=%.0f&width=%.0f&format=FITS&compression=%s&version=3 HTTP/1.0\nUser-Agent: xephem/%s\n\n",
+						ra, dec, survey, fov, fov,
 						use_gunzip() ? "gz" : "NONE",
 						PATCHLEVEL);
 	(void) sprintf (msg, "Command to %s:", host);
@@ -2209,14 +2411,13 @@ stsci_fits()
 	/* if do not see a fits file, show what we do find */
 	if (!sawfits) {
 	    xe_msg (" ", 0);
-	    xe_msg ("Network message follows:", 0);
-	    xe_msg (" ", 0);
+	    xe_msg ("Message from server", 1);
+	    xe_msg ("------------------", 0);
 	    while (recvline (sockfd, buf, sizeof(buf)) > 0)
 		xe_msg (buf, 0);
-	    xe_msg (" ", 0);
-	    xe_msg ("End of Network message", 0);
-	    xe_msg (" ", 0);
-	    xe_msg ("Error -- see Main->File->Messages", 1);
+	    xe_msg ("------------------", 0);
+	    xe_msg ("End of Message from server", 0);
+	    msg_manage();
 	    watch_cursor (0);
 	    close (sockfd);
 	    stopd_down();
@@ -2395,18 +2596,19 @@ wcsok (fip)
 FImage *fip;
 {
 	double ra, dec;
-	char buf[128];
+
+	/* if have DSS convert to WCS so we know we can go both ways.
+	 * only dss RD->xy I can find solves from xy->RD YUK!
+	 */
+	if (getRealFITS (fip, "PLTRAH", &ra) == 0) {
+	    if (dsswcs (fip) < 0)
+		return (-1);
+	}
 
 	/* now try it */
 	if (xy2RADec (fip, 0.0, 0.0, &ra, &dec) == 0)
 	    return (0);
 
-	/* only other possibility is a DSS image from STScI -- if it has
-	 * the PLTLABEL, at least give it a try.
-	 */
-	if (getStringFITS (fip, "PLTLABEL", buf) == 0 && dsswcs (fip) == 0)
-	    return (0);
-
 	/* oh well */
 	xe_msg ("No WCS fields in header", 1);
 	return (-1);
@@ -2434,14 +2636,6 @@ FImage *fip;
 	setRealFITS (fip, "CRVAL1", raddeg(a0), 10, NULL);
 	setRealFITS (fip, "CRVAL2", raddeg(d0), 10, NULL);
 
-	/* rotation is based on how center relates edge */
-	if (dssxy2RADec (fip, (double)fip->sw, fip->sh/2.0, &a1, &d1) < 0)
-	    return (-1);
-	solve_sphere (a0-a1, PI/2-d1, sin(d0), cos(d0), NULL, &rot);
-	rot = raddeg(rot-PI/2);
-	setRealFITS (fip, "CROTA2", rot, 10, NULL);
-	setRealFITS (fip, "CROTA1", rot, 10, NULL);
-
 	/* set scale */
 	if (getRealFITS (fip, "PLTSCALE", &pltscale) < 0
 			    || getRealFITS (fip, "XPIXELSZ", &px) < 0
@@ -2452,10 +2646,28 @@ FImage *fip;
 	setRealFITS (fip, "CDELT1", -pltscale*px/3600000.0, 10, NULL);
 	setRealFITS (fip, "CDELT2",  pltscale*py/3600000.0, 10, NULL);
 
-	/* and finally, the easy ones */
+	/* coord sys */
 	setStringFITS (fip, "CTYPE1", "RA---TAN", NULL);
 	setStringFITS (fip, "CTYPE2", "DEC--TAN", NULL);
 
+	/* find rotation CW to make N up.
+	 * TODO: ugly! gotta better way??
+	 */
+	if (dssxy2RADec (fip, fip->sw/2.0, 0.0, &a1, &d1) < 0)
+	    return (-1);
+	if (d0 < 0) {
+	    double A = a0-a1;
+	    double b = PI/2+d0;
+	    double c = PI/2+d1;
+	    rot = raddeg(atan2(c*sin(A), c*cos(A)-b)) - 180;
+	} else {
+	    double A = a1-a0;
+	    double b = PI/2-d0;
+	    double c = PI/2-d1;
+	    rot = raddeg(atan2(c*sin(A), c*cos(A)-b));
+	}
+	setRealFITS (fip, "CROTA2", rot, 10, "ADDED BY XEPHEM");
+
 	return (0);
 }
 
@@ -2470,7 +2682,7 @@ FImage *fip;
 double X, Y;
 double *rap, *decp;
 {
-	char buf[1024];
+	char buf[128];
 	double cnpix1, cnpix2;
 	double a[14], b[14];
 	double rh, rm, rs;
@@ -2513,7 +2725,6 @@ double *rap, *decp;
 		return (-1);
 	    }
 	}
-
 	x2y2 = x*x + y*y;
 	xi = a[1]*x + a[2]*y + a[3] + a[4]*x*x + a[5]*x*y + a[6]*y*y
 		+ a[7]*x2y2 + a[8]*x*x*x + a[9]*x*x*y + a[10]*x*y*y
@@ -2530,23 +2741,20 @@ double *rap, *decp;
 			    || getRealFITS (fip, "PLTRAS", &rs) < 0
 			    || getRealFITS (fip, "PLTDECD", &dd) < 0
 			    || getRealFITS (fip, "PLTDECM", &dm) < 0
-			    || getRealFITS (fip, "PLTDECS", &ds) < 0) {
+			    || getRealFITS (fip, "PLTDECS", &ds) < 0
+			    || getStringFITS (fip, "PLTDECSN", buf) < 0) {
 	    xe_msg ("No PLT* fields", 1);
 	    return(-1);
 	}
 	rac = rs/3600.0 + rm/60.0 + rh;
 	rac = hrrad(rac);
 	decc = ds/3600.0 + dm/60.0 + dd;
-	if (getStringFITS (fip, "PLTDECSN", buf) < 0) {
-	    xe_msg ("No PLTDECSN field", 1);
-	    return(-1);
-	}
 	if (buf[0] == '-')
 	    decc = -decc;
 	decc = degrad(decc);
 	tandecc = tan(decc);
 
-	ra = atan ((xi/cos(decc))/(1.0-nu*tandecc)) + rac;
+	ra = atan2 (xi/cos(decc), 1.0-nu*tandecc) + rac;
 	if (ra < 0)
 	    ra += 2*PI;
 	*rap = ra;
@@ -2563,38 +2771,33 @@ fitsObs(mjdp)
 double *mjdp;
 {
 	FImage *fip = sf_getFImage();
-	int mm, dd, yy;
 	char buf[128];
-	double mjd0;
+	double x;
 
 	if (!fip)
 	    return (-1);
 
-	if (getRealFITS (fip, "JD", &mjd0) == 0) {
-	    *mjdp = mjd0 - MJD0;
+	if (getRealFITS (fip, "JD", &x) == 0) {
+	    *mjdp = x - MJD0;
 	    return (0);
 	}
 
-	if (getStringFITS (fip, "DATE-OBS", buf) == 0
-			&& sscanf (buf, "%d/%d/%d", &dd, &mm, &yy) == 3) {
-	    if (yy < 20)
-		yy += 1000;	/* TODO: Y2K! */
-	    cal_mjd (mm, (double)dd, yy+1900, &mjd0);
-	    if (getStringFITS (fip, "UT", buf) == 0 
-			    || getStringFITS (fip, "TIME-OBS", buf) == 0) {
-		int h, m;
-		double s;
-		if (sscanf (buf, "%d:%d:%lf", &h, &m, &s) == 3) {
-		    *mjdp = mjd0 + (s/3600. + m/60. + h)/24.0;
-		    return (0);
-		}
-	    }
-	    *mjdp = mjd0;
-	    return (0);	/* better than nothing I suppose */
+	if (getRealFITS (fip, "EPOCH", &x) == 0) {
+	    year_mjd (x, mjdp);
+	    return (0);
 	}
 
+	if (getStringFITS (fip, "DATE-OBS", buf) == 0) {
+	    int dd, mm, yy;
+	    if (sscanf (buf, "%d%*[/-]%d%*[/-]%d", &dd, &mm, &yy) == 3) {
+		yy += (yy < 50 ? 2000 : 1900);
+		cal_mjd (mm, (double)dd, yy, mjdp);
+		return (0);
+	    }
+	}
+	
 	return (-1);
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfits.c,v $ $Date: 1999/10/25 21:09:17 $ $Revision: 1.11 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfits.c,v $ $Date: 2000/10/25 19:58:38 $ $Revision: 1.30 $ $Name:  $"};
