@@ -40,6 +40,7 @@ extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
 extern int any_ison P_((void));
 extern int execute_expr P_((double *vp, char *errbuf));
+extern int isUp P_((Widget shell));
 extern int obj_cir P_((Now *np, Obj *op));
 extern int prog_isgood P_((void));
 extern int tickmarks P_((double min, double max, int numdiv, double ticks[]));
@@ -62,6 +63,7 @@ extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
 extern void obj_set P_((Obj *op, int dbidx));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void timestamp P_((Now *np, Widget w));
 extern void watch_cursor P_((int want));
 extern void wtip P_((Widget w, char *string));
@@ -73,7 +75,7 @@ typedef struct {
     TrTS trts;		/* mjd when Obj was valid and whether to timestamp */
     int sx, sy;		/* main view window coords of object */
     int stx;		/* stereo view x coord (y is the same in both) */
-    float x, y, z;	/* heliocentric cartesian coords */
+    double x, y, z;	/* heliocentric cartesian coords */
 } HLoc;
 
 /* list of coords when "all objects" are being displayed */
@@ -82,7 +84,7 @@ typedef struct {
     short sx, stx, sy;	/* dot location on screen */
 } AllP;
 
-static void ss_create_form P_((void));
+static void ss_create_shell P_((void));
 static void st_create_form P_((void));
 static void ss_trail_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ss_activate_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -91,7 +93,7 @@ static void ss_allb_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ss_obj_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ss_changed_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ss_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ss_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void ss_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ss_print_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ss_print P_((void));
 static void ss_ps_annotate P_((Now *np));
@@ -124,13 +126,16 @@ static void ap_free P_((void));
 static int ap_add P_((Obj *op, HLoc *hp));
 static void ap_label_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ap_mkobj_cb P_((Widget w, XtPointer client, XtPointer call));
+static void mk_gcs P_((void));
 
-static Widget ssform_w;		/* main solar system form dialog */
+static Widget ssshell_w;	/* main solar system shell */
 static Widget hr_w, hlng_w, hlat_w; /* scales for heliocentric R, long, lat */
 static Widget ssda_w;		/* solar system drawring area */
 static Widget ssframe_w;	/* main's DA frame */
 static Widget dt_w;		/* date/time stamp label widget */
 static Pixmap ss_pm;		/* main view pixmap */
+static GC bgc;			/* background GC */
+static int asc;			/* max ascent in background GC font */
 
 static Widget stform_w;		/* main stereo form dialog */
 static Widget parallax_w;	/* scale to set amount of parallax */
@@ -200,38 +205,35 @@ typedef struct {
     Widget lbl_w;
     Widget assign_w;
     Widget dv_w;
-    Obj *op;		/* real or into HLoc.o if non-first trail entry */
-    AllP *ap;		/* set if picked an at-large db object */
-    HLoc *hp;		/* set if picked any member of trail(even 1st)*/
+    Obj *op;			/* real or into HLoc.o if non-first trail */
+    AllP *ap;			/* set if picked an at-large db object */
+    HLoc *hp;			/* set if picked any member of trail(even 1st)*/
 } Popup;
 static Popup pu;
 
-static char earthname[] = "Earth";
+static GC egc;			/* gc for drawing */
+static XFontStruct *efsp;	/* text info for drawing */
+
+static char earthname[] = "Earth";		/* local "Moon" alias */
+static char sscategory[] = "Solar System";	/* Save category */
 
 /* called when the solar system view is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, just go for it.
  */
 void
 ss_manage ()
 {
-	if (!ssform_w) {
-	    ss_create_form();
+	if (!ssshell_w) {
+	    ss_create_shell();
 	    ss_create_popup();
 	    st_create_form();
 	    hloc_reset(mm_get_now());
 	    timestamp (mm_get_now(), dt_w);
 	}
 	
-	if (XtIsManaged(ssform_w)) {
-	    if (XtIsManaged(stform_w))
-		XtUnmanageChild(stform_w);
-	    XtUnmanageChild (ssform_w);
-	} else {
-	    XtManageChild (ssform_w);
-	    if (stereo)
-		XtManageChild (stform_w);
-	}
+	XtPopup (ssshell_w, XtGrabNone);
+	set_something (ssshell_w, XmNiconic, (XtArgVal)False);
 }
 
 /* called when we are to update our view.
@@ -243,10 +245,7 @@ ss_update (np, how_much)
 Now *np;
 int how_much;
 {
-	int up;
-
-	up = ssform_w && XtIsManaged(ssform_w);
-	if (!up)
+	if (!isUp(ssshell_w))
 	    return;
 
 	watch_cursor (1);
@@ -263,6 +262,18 @@ int how_much;
 	watch_cursor (0);
 }
 
+/* called when basic resources change.
+ * rebuild and redraw.
+ */
+void
+ss_newres()
+{
+	if (!ssshell_w)
+	    return;
+	mk_gcs();
+	ss_update (mm_get_now(), 1);
+}
+
 /* database (beyond NOBJ) has changed, or been appended to.
  */
 void
@@ -270,7 +281,7 @@ ss_newdb (appended)
 int appended;
 {
 	/* only effects us if we are showing All db objects and we are up */
-	if (allobj && ssform_w && XtIsManaged(ssform_w))
+	if (allobj && isUp(ssshell_w))
 	    ss_all();
 }
 
@@ -287,7 +298,7 @@ int dbidx;
 	Obj *op;
 
 	/* we might get called before we are ever brought up the first time */
-	if (!ssform_w)
+	if (!ssshell_w)
 	    return;
 
 	if (dbidx >= NOBJ) {
@@ -317,7 +328,7 @@ int dbidx;
 int
 ss_ison()
 {
-	return (ssform_w && XtIsManaged(ssform_w));
+	return (isUp(ssshell_w));
 }
 
 /* called to put up or remove the watch cursor.  */
@@ -327,8 +338,8 @@ Cursor c;
 {
 	Window win;
 
-	if (ssform_w && (win = XtWindow(ssform_w)) != 0) {
-	    Display *dsp = XtDisplay(ssform_w);
+	if (ssshell_w && (win = XtWindow(ssshell_w)) != 0) {
+	    Display *dsp = XtDisplay(ssshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -336,9 +347,9 @@ Cursor c;
 	}
 }
 
-/* create the main solarsystem form */
+/* create the main solarsystem shell */
 static void
-ss_create_form()
+ss_create_shell()
 {
 	typedef struct {
 	    char *label;	/* what goes on the help label */
@@ -353,7 +364,7 @@ ss_create_form()
 	    {"Misc...",		"Solsys - misc"},
 	};
 	typedef struct {
-	    char *name;		/* name of widget */
+	    char *name;		/* name of widget, or NULL for sep */
 	    char *label;	/* label on toggle button */
 	    int id;		/* one of the toggle ids */
 	    int *state;		/* int we use to keep state */
@@ -367,35 +378,38 @@ ss_create_form()
 	    	"Display sun-centered circles to mark the ecliptic plane"},
 	    {"Labels",   "Labels",   LABELS,   &nametags, NULL, 
 	    	"Display names near the dots denoting the Objects"},
-	    {"MLegs",    "Basic Legs",BLEGS,    &blegs,    NULL,
+	    {"BasicLegs","Basic Legs",BLEGS,    &blegs,    NULL,
 	    	"Connect planets and ObjXYZ objects to the ecliptic to depict height"},
 	    {"DBLegs",   "DB Legs",  DBLEGS,   &dblegs,   NULL,
 	    	"Connect other Database objects to the ecliptic to depict height"},
 	};
 	Widget mb_w, pd_w, cb_w;
 	Widget w;
+	Widget ssform_w;
 	XmString str;
 	Arg args[20];
 	int i;
 	int n;
 
-	/* create the master form */
-	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	ssform_w = XmCreateFormDialog (toplevel_w, "SolarSystem", args, n);
-	set_something (ssform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (ssform_w, XmNhelpCallback, ss_help_cb, 0);
-	XtAddCallback (ssform_w, XmNunmapCallback, ss_unmap_cb, 0);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
+	/* create the form and shell */
 	n = 0;
 	XtSetArg (args[n], XmNtitle, "xephem Solar System view"); n++;
-	XtSetValues (XtParent(ssform_w), args, n);
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNiconName, "SolSys"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	ssshell_w = XtCreatePopupShell ("SolarSystem", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (ssshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (ssshell_w, XmNpopdownCallback, ss_popdown_cb, 0);
+	sr_reg (ssshell_w, "XEphem*SolarSystem.width", sscategory, 0);
+	sr_reg (ssshell_w, "XEphem*SolarSystem.height", sscategory, 0);
+	sr_reg (ssshell_w, "XEphem*SolarSystem.x", sscategory, 0);
+	sr_reg (ssshell_w, "XEphem*SolarSystem.y", sscategory, 0);
+
+	n = 0;
+	ssform_w = XmCreateForm (ssshell_w, "SSF", args, n);
+	XtAddCallback (ssform_w, XmNhelpCallback, ss_help_cb, 0);
+	XtManageChild (ssform_w);
 
 	/* put a menu bar across the top with the various pulldown menus */
 
@@ -460,6 +474,7 @@ ss_create_form()
 	    XtManageChild (w);
 	    set_xmstring (w, XmNlabelString, "Live Dragging");
 	    dragtoo = XmToggleButtonGetState (w);
+	    sr_reg (w, NULL, sscategory, 1);
 
 	    /* add the "stereo" toggle button */
 
@@ -520,6 +535,7 @@ ss_create_form()
 			wtip (w, cp->tip);
 		    XtManageChild (w);
 		    XmStringFree (str);
+		    sr_reg (w, NULL, sscategory, 1);
 		} else {
 		    n = 0;
 		    w = XmCreateSeparator (pd_w, "Sep", args, n);
@@ -568,6 +584,7 @@ ss_create_form()
 	    wtip (w, "Show all other Solar system objects too");
 	    XtManageChild (w);
 	    allobj = XmToggleButtonGetState(w);
+	    sr_reg (w, NULL, sscategory, 1);
 
 	    n = 0;
 	    w = XmCreateSeparator (pd_w, "Sep", args, n);
@@ -575,16 +592,25 @@ ss_create_form()
 
 	    /* make the object toggle buttons.
 	     * fill in the names we can now.
-	     * default the planets to all on.
+	     * default all solsys objects on.
 	     */
 	    for (i = 0; i < NOBJ; i++) {
 		Obj *op = db_basic(i);
+		char *tbname;
+
+		/* name TB for planet if known (or Earth for Moon) */
+		if (i < MOON)
+		    tbname = op->o_name;
+		else if (i == MOON)
+		    tbname = earthname;
+		else
+		    tbname = "ObjTB";
 
 		n = 0;
 		XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 		XtSetArg (args[n], XmNmarginHeight, 0); n++;
 		XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
-		obj_w[i] = XmCreateToggleButton (pd_w, "ObjTB", args, n);
+		obj_w[i] = XmCreateToggleButton (pd_w, tbname, args, n);
 		wtip (obj_w[i], "Whether to display this object at all");
 		XtAddCallback (obj_w[i], XmNvalueChangedCallback, ss_obj_cb,
 							    (XtPointer)i);
@@ -592,9 +618,13 @@ ss_create_form()
 		    set_xmstring (obj_w[i], XmNlabelString,
 			is_planet(op,MOON)? earthname : op->o_name);
 		    XtManageChild (obj_w[i]);
-		    XmToggleButtonSetState (obj_w[i], True, False);
+		    /* let resources determine whether basic objs are set */
+		    if (i > MOON)
+			XmToggleButtonSetState (obj_w[i], True, False);
 		}
 		obj_on[i] = XmToggleButtonGetState (obj_w[i]);
+		if (i <= MOON)
+		    sr_reg (obj_w[i], NULL, sscategory, 1);
 	    }
 
 	/* make the "help" pulldown */
@@ -650,6 +680,7 @@ ss_create_form()
 	XtAddCallback (hlng_w, XmNvalueChangedCallback, ss_changed_cb, 0);
 	wtip (hlng_w, "Set heliocentric longitude of vantage point");
 	XtManageChild (hlng_w);
+	sr_reg (hlng_w, NULL, sscategory, 0);
 
 	/* make the left scale */
 
@@ -670,6 +701,7 @@ ss_create_form()
 	XtAddCallback (hr_w, XmNvalueChangedCallback, ss_changed_cb, 0);
 	wtip (hr_w, "Zoom in and out");
 	XtManageChild (hr_w);
+	sr_reg (hr_w, NULL, sscategory, 0);
 
 	/* make the right scale */
 
@@ -690,6 +722,7 @@ ss_create_form()
 	XtAddCallback (hlat_w, XmNvalueChangedCallback, ss_changed_cb, 0);
 	wtip (hlat_w, "Set heliocentric latitude of vantage point");
 	XtManageChild (hlat_w);
+	sr_reg (hlat_w, NULL, sscategory, 0);
 
 	/* make a frame for the drawing area */
 
@@ -728,7 +761,7 @@ st_create_form()
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNnoResize, True); n++;	/* user can't resize */
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	stform_w = XmCreateFormDialog (toplevel_w, "StereoSolarSystem", args,n);
+	stform_w = XmCreateFormDialog (ssshell_w, "StereoSolarSystem", args,n);
 	set_something (stform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (stform_w, XmNmapCallback, st_map_cb, NULL);
 	XtAddCallback (stform_w, XmNunmapCallback, st_unmap_cb, 0);
@@ -949,20 +982,19 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let unmap do rest of the work */
-	XtUnmanageChild (ssform_w);
+	/* let popdown do rest of the work */
+	XtPopdown (ssshell_w);
 }
 
-/* callback from unmapping the main view */
+/* callback from popping down the main view */
 /* ARGSUSED */
 static void
-ss_unmap_cb (w, client, call)
+ss_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if (stform_w && XtIsManaged(stform_w))
-	    XtUnmanageChild (stform_w);
+	XtUnmanageChild (stform_w);
 
 	if (ss_pm) {
 	    XFreePixmap (XtD, ss_pm);
@@ -994,8 +1026,6 @@ static void
 ss_print ()
 {
 	Now *np = mm_get_now();
-	Window win = ss_pm;
-	XFontStruct *fsp;
         unsigned int nx, ny;
 	unsigned int bw, d;
 	Window root;
@@ -1009,21 +1039,17 @@ ss_print ()
 
 	watch_cursor(1);
 
-	/* get info about window size */
-        XGetGeometry(XtD, win, &root, &x, &y, &nx, &ny, &bw, &d);
+	/* get info about ss_pm size */
+        XGetGeometry(XtD, ss_pm, &root, &x, &y, &nx, &ny, &bw, &d);
 
 	/* draw in an area 6.5w x 7h centered 1in down from top */
 	if (14*nx >= 13*ny)
-	    XPSXBegin (win, 0, 0, nx, ny, 1*72, 10*72, (int)(6.5*72));
+	    XPSXBegin (ss_pm, 0, 0, nx, ny, 1*72, 10*72, (int)(6.5*72));
 	else {
 	    int pw = 72*7*nx/ny;	/* width on paper when 7 hi */
-	    XPSXBegin (win, 0, 0, nx, ny, (int)((8.5*72-pw)/2), 10*72, pw);
+	    XPSXBegin (ss_pm, 0, 0, nx, ny, (int)((8.5*72-pw)/2), 10*72, pw);
 	}
 
-	/* get and register the font to use for all gcs from obj_pickgc() */
-	get_views_font (XtD, &fsp);
-	XPSRegisterFont (fsp->fid, "Helvetica");
-
 	/* redraw the main view -- *not* the stereo view */
 	ss_ss_all ();
 
@@ -1200,6 +1226,10 @@ XtPointer call;
 	    exit(1);
 	}
 
+	/* make sure gc's are ready */
+	if (!bgc)
+	    mk_gcs();
+
 	/* make the pixmap if new or changed size */
 	XGetGeometry(dsp, win, &root, &x, &y, &ss_nx, &ss_ny, &bw, &d);
 	if (!ss_pm || ss_nx != ss_last_nx || ss_ny != ss_last_ny) {
@@ -1339,7 +1369,7 @@ st_track_size()
 	n = 0;
 	XtSetArg (args[n], XmNx, &mfx); n++;
 	XtSetArg (args[n], XmNy, &mfy); n++;
-	XtGetValues (ssform_w, args, n);
+	XtGetValues (ssshell_w, args, n);
 	n = 0;
 	XtSetArg (args[n], XmNy, &mdy); n++;
 	XtGetValues (ssframe_w, args, n);
@@ -1413,6 +1443,10 @@ XtPointer call;
 	    exit(1);
 	}
 
+	/* make sure gc's are ready */
+	if (!bgc)
+	    mk_gcs();
+
 	/* make the pixmap if new or changed size */
 	XGetGeometry(dsp, win, &root, &x, &y, &st_nx, &st_ny, &bw, &d);
 	if (!st_pm || st_nx != st_last_nx || st_ny != st_last_ny) {
@@ -1732,8 +1766,6 @@ ss_all()
 static void
 ss_ss_all()
 {
-	static GC ss_bgc;
-	static int ss_asc;
 	Display *dsp = XtDisplay(ssda_w);
 	Window ss_win = XtWindow(ssda_w);
 	unsigned int ss_nx, ss_ny;
@@ -1754,23 +1786,8 @@ ss_ss_all()
 	/* get the sizes of the windows we'll use */
 	XGetGeometry(dsp, ss_win, &root, &x, &y, &ss_nx, &ss_ny, &bw, &d);
 
-	/* make sure the gc's are created */
-	if (!ss_bgc) {
-	    XFontStruct *fs;
-	    XGCValues gcv;
-	    unsigned int gcm;
-	    Pixel p;
-
-	    gcm = GCForeground;
-	    get_something (ssda_w, XmNbackground, (XtArgVal)&p);
-	    gcv.foreground = p;
-	    ss_bgc = XCreateGC (dsp, ss_win, gcm, &gcv);
-	    fs = XQueryFont (dsp, XGContextFromGC (ss_bgc));
-	    ss_asc = fs->max_bounds.ascent;
-	}
-
 	/* clear */
-	XFillRectangle (dsp, ss_pm, ss_bgc, 0, 0, ss_nx, ss_ny);
+	XFillRectangle (dsp, ss_pm, bgc, 0, 0, ss_nx, ss_ny);
 
 	/* establish the scales */
 	XmScaleGetValue (hr_w, &sv);
@@ -1786,26 +1803,12 @@ ss_ss_all()
 
 	/* draw the ecliptic plane, if desired */
 	if (ecliptic) {
-	    static GC egc;
 	    double minau, maxau;
 	    double aus[NECLPT+2];
 	    char spacing[64];
 	    int nau, i;
 
-	    if (!egc) {
-		XGCValues gcv;
-		XFontStruct *fsp;
-		unsigned int gcm;
-		Pixel p;
-
-		gcm = GCForeground;
-		get_something (ssda_w, XmNforeground, (XtArgVal)&p);
-		gcv.foreground = p;
-		egc = XCreateGC (dsp, ss_win, gcm, &gcv);
-		get_views_font (XtD, &fsp);
-		XSetFont (XtD, egc, fsp->fid);
-		XPSRegisterFont (fsp->fid, "Helvetica");
-	    }
+	    XSetFont (XtD, egc, efsp->fid);
 
 	    minau = 0;
 	    maxau = ss_nx > ss_ny ? ss_nx/2.0/scale : ss_ny/2.0/scale;
@@ -1813,7 +1816,7 @@ ss_ss_all()
 
 	    /* draw tick mark spacing message in upper left corner */
 	    (void) sprintf (spacing, "%g AU", aus[1] - aus[0]);
-	    XPSDrawString(dsp, ss_pm, egc, 1, ss_asc+1,spacing,strlen(spacing));
+	    XPSDrawString(dsp, ss_pm, egc, 1, asc+1, spacing, strlen(spacing));
 
 	    /* draw each grid line.
 	     * the main view uses simple elipses.
@@ -1922,7 +1925,7 @@ ss_ss_all()
 	    ss_allobj (dsp, 0, scale, selt, celt, selg, celg, ss_nx, ss_ny);
 
 	/* copy the work pixmap to the screen */
-	XCopyArea (dsp, ss_pm, ss_win, ss_bgc, 0, 0, ss_nx, ss_ny, 0, 0);
+	XCopyArea (dsp, ss_pm, ss_win, bgc, 0, 0, ss_nx, ss_ny, 0, 0);
 
 	/* all finished */
 	watch_cursor (0);
@@ -1933,8 +1936,6 @@ ss_ss_all()
 static void
 ss_st_all()
 {
-	static GC st_bgc;
-	static int st_asc;
 	Display *dsp = XtDisplay(stda_w);
 	Window st_win = XtWindow(stda_w);
 	unsigned int st_nx, st_ny;
@@ -1952,23 +1953,8 @@ ss_st_all()
 	/* get the sizes of the windows we'll use */
 	XGetGeometry(dsp, st_win, &root, &x, &y, &st_nx, &st_ny, &bw, &d);
 
-	/* make sure the gc's are created */
-	if (!st_bgc) {
-	    XFontStruct *fs;
-	    XGCValues gcv;
-	    unsigned int gcm;
-	    Pixel p;
-
-	    gcm = GCForeground;
-	    get_something (stda_w, XmNbackground, (XtArgVal)&p);
-	    gcv.foreground = p;
-	    st_bgc = XCreateGC (dsp, st_win, gcm, &gcv);
-	    fs = XQueryFont (dsp, XGContextFromGC (st_bgc));
-	    st_asc = fs->max_bounds.ascent;
-	}
-
 	/* clear */
-	XFillRectangle (dsp, st_pm, st_bgc, 0, 0, st_nx, st_ny);
+	XFillRectangle (dsp, st_pm, bgc, 0, 0, st_nx, st_ny);
 
 	/* establish the scales */
 	XmScaleGetValue (hr_w, &sv);
@@ -1984,22 +1970,12 @@ ss_st_all()
 
 	/* draw the ecliptic plane, if desired */
 	if (ecliptic) {
-	    static GC egc;
 	    double minau, maxau;
 	    double aus[NECLPT+2];
 	    char spacing[64];
 	    int nau, i;
 
-	    if (!egc) {
-		XGCValues gcv;
-		unsigned int gcm;
-		Pixel p;
-
-		gcm = GCForeground;
-		get_something (ssda_w, XmNforeground, (XtArgVal)&p);
-		gcv.foreground = p;
-		egc = XCreateGC (dsp, st_win, gcm, &gcv);
-	    }
+	    XSetFont (XtD, egc, efsp->fid);
 
 	    minau = 0;
 	    maxau = st_nx > st_ny ? st_nx/2.0/scale : st_ny/2.0/scale;
@@ -2007,7 +1983,7 @@ ss_st_all()
 
 	    /* draw tick mark spacing message in upper left corner */
 	    (void) sprintf (spacing, "%g AU", aus[1] - aus[0]);
-	    XDrawString(dsp, st_pm, egc, 1, st_asc+1,spacing,strlen(spacing));
+	    XDrawString(dsp, st_pm, egc, 1, asc+1, spacing, strlen(spacing));
 
 	    /* draw each grid line.
 	     * the stereo view uses polylines drawn in world coords -- slower!
@@ -2109,7 +2085,7 @@ ss_st_all()
 	    ss_allobj (dsp, 1, scale, selt, celt, selg, celg, st_nx, st_ny);
 
 	/* copy the work pixmap to the screen */
-	XCopyArea (dsp, st_pm, st_win, st_bgc, 0, 0, st_nx, st_ny, 0, 0);
+	XCopyArea (dsp, st_pm, st_win, bgc, 0, 0, st_nx, st_ny, 0, 0);
 
 	/* all finished */
 	watch_cursor (0);
@@ -2269,8 +2245,8 @@ unsigned nx, ny;	/* size of drawing area, in pixels */
 	zpp = -yp*celt + zp*selt;
 
 	/* now, straight ortho projection */
-	hp->sx = nx/2 + xpp*scale;
-	hp->sy = ny/2 - ypp*scale;
+	hp->sx = (int)(nx/2 + xpp*scale);
+	hp->sy = (int)(ny/2 - ypp*scale);
 
 	/* back is y coord, in AU, behind which there is no parallax.
 	 * parallax is the offset of the sun (at a=0), in pixels.
@@ -2279,7 +2255,7 @@ unsigned nx, ny;	/* size of drawing area, in pixels */
 	if (zpp < back)
 	    hp->stx = hp->sx;
 	else
-	    hp->stx = hp->sx + parallax*(back-zpp)/back;
+	    hp->stx = hp->sx + (int)(parallax*(back-zpp)/back);
 }
 
 /* called by trails.c to create a new set of trails for all solar sys objects.
@@ -2333,7 +2309,7 @@ XtPointer client;
 	}
 
 	/* redraw everything to display the new trails, if we are up */
-	if (XtIsManaged(ssform_w))
+	if (isUp(ssshell_w))
 	    ss_all ();
 
 	watch_cursor (0);
@@ -2553,5 +2529,32 @@ XtPointer call;
 	dm_update (mm_get_now(), 1);
 }
 
+/* create our GCs.
+ * TODO: reclaim old stuff if called again.
+ */
+static void
+mk_gcs()
+{
+	Display *dsp = XtDisplay(ssda_w);
+	Window win = XtWindow(ssda_w);
+	XGCValues gcv;
+	unsigned int gcm;
+	XFontStruct *fs;
+	Pixel p;
+
+	gcm = GCForeground;
+	get_something (ssda_w, XmNbackground, (XtArgVal)&p);
+	gcv.foreground = p;
+	bgc = XCreateGC (dsp, win, gcm, &gcv);
+	fs = XQueryFont (dsp, XGContextFromGC (bgc));
+	asc = fs->max_bounds.ascent;
+
+	gcm = GCForeground;
+	get_something (ssda_w, XmNforeground, (XtArgVal)&p);
+	gcv.foreground = p;
+	egc = XCreateGC (dsp, win, gcm, &gcv);
+	get_views_font (XtD, &efsp);
+}
+
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: solsysmenu.c,v $ $Date: 1999/02/17 05:49:57 $ $Revision: 1.5 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: solsysmenu.c,v $ $Date: 2000/11/29 00:48:14 $ $Revision: 1.13 $ $Name:  $"};
