@@ -33,8 +33,10 @@ extern Now *mm_get_now P_((void));
 extern int any_ison P_((void));
 extern int compile_expr P_((char *ex, char *errbuf));
 extern int execute_expr P_((double *vp, char *errbuf));
+extern int isUp P_((Widget shell));
 extern int prog_isgood P_((void));
 extern void all_selection_mode P_((int whether));
+extern void buttonAsButton P_((Widget w, int whether));
 extern void compiler_log P_((char *name, double value));
 extern void f_double P_((Widget w, char *fmt, double f));
 extern void f_scansex P_((double o, char bp[], double *np));
@@ -48,14 +50,14 @@ extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
 extern void redraw_screen P_((int how_much));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void wtip P_((Widget w, char *string));
 extern void xe_msg P_((char *msg, int app_modal));
 
-static void srch_create_form P_((void));
+static void srch_create_shell P_((void));
 static int srch_isup P_((void));
 static int srching_now P_((void));
 static void srch_set_buttons P_((int whether));
-static void srch_set_a_button P_((Widget pbw, int whether));
 static void srch_use_cb P_((Widget w, XtPointer client, XtPointer call));
 static void srch_newfunc_cb P_((Widget w, XtPointer client, XtPointer call));
 static void srch_compile_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -73,7 +75,7 @@ static int srch_binary P_((double Mjd, double v, double *tmincp));
 
 
 /* the widgets we need direct access to */
-static Widget srchform_w;
+static Widget srchshell_w;
 static Widget help_w;
 static Widget acc_w;
 static Widget field_w;
@@ -87,30 +89,28 @@ static Widget on_w;	/* whether searching is active comes directly from this
 
 /* name is it appears when valu_w is selected for plotting or lising */
 static char srchvname[] = "SolveValue";
+static char solvecategory[] = "Tools -- Solve";	/* Save category */
 
 static int (*srch_f)();		/* 0 or pointer to one of the search functions*/
 static int srch_tmscalled;	/* number of iterations so far */
-static double tmlimit = 1./60.;	/* search accuracy, in hrs; def is one minute */
+static double tmlimit;		/* search accuracy, hrs */
 
 static int srch_selecting;	/* whether our value is currently selectable */
 static int srch_self_selection_mode;	/* flag to prevent self-selection */
 
 /* called when the search menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, just go for it.
  */
 void
 srch_manage ()
 {
-	if (!srchform_w)
-	    srch_create_form();
+	if (!srchshell_w)
+	    srch_create_shell();
 	
-	if (XtIsManaged(srchform_w))
-	    XtUnmanageChild (srchform_w);
-	else {
-	    XtManageChild (srchform_w);
-	    srch_set_buttons(srch_selecting);
-	}
+	XtPopup (srchshell_w, XtGrabNone);
+	set_something (srchshell_w, XmNiconic, (XtArgVal)False);
+	srch_set_buttons(srch_selecting);
 }
 
 /* called by other menus as they want to hear from our buttons or not.
@@ -126,9 +126,12 @@ int whether;	/* whether setting up for plotting or for not plotting */
 	if (srch_self_selection_mode)
 	    return;
 
-	srch_selecting += whether ? 1 : -1;
+	if (whether)
+	    srch_selecting++;
+	else if (srch_selecting > 0)
+	    --srch_selecting;
 
-	if (srchform_w && XtIsManaged(srchform_w))
+	if (isUp(srchshell_w))
 	    if ((whether && srch_selecting == 1)     /* first one to want on */
 		|| (!whether && srch_selecting == 0) /* last one to want off */)
 		srch_set_buttons (whether);
@@ -147,9 +150,7 @@ char *name;
 	int ins;
 	char *qname;
 
-	if (!srchform_w
-		|| !XtIsManaged(srchform_w)
-		|| !XmToggleButtonGetState(field_w))
+	if (!isUp(srchshell_w) || !XmToggleButtonGetState(field_w))
 	    return;
 
 	ins = XmTextGetInsertionPosition (func_w);
@@ -161,7 +162,7 @@ char *name;
 	/* move the focus right back to the search menu so the insertion point
 	 * remains visible.
 	 */
-	XSetInputFocus (XtDisplay(srchform_w), XtWindow(srchform_w),
+	XSetInputFocus (XtDisplay(srchshell_w), XtWindow(srchshell_w),
 						RevertToParent, CurrentTime);
 }
 
@@ -237,8 +238,8 @@ Cursor c;
 {
 	Window win;
 
-	if (srchform_w && (win = XtWindow(srchform_w)) != 0) {
-	    Display *dsp = XtDisplay(srchform_w);
+	if (srchshell_w && (win = XtWindow(srchshell_w)) != 0) {
+	    Display *dsp = XtDisplay(srchshell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -247,7 +248,7 @@ Cursor c;
 }
 
 static void
-srch_create_form()
+srch_create_shell()
 {
 	static struct {
 	    char *title;
@@ -262,35 +263,39 @@ srch_create_form()
 	    	"Step time along until the truth value of the Function changes"},
 	};
 	XmString str;
+	char *txt;
 	Widget w, trc_w, brc_w, f_w, rb_w;
+	Widget srchform_w;
 	Arg args[20];
 	int i, n;
 
 	/* create form dialog */
 	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
+	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNiconName, "Solve"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Solver Control"); n++;
+	srchshell_w = XtCreatePopupShell ("Solve", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (srchshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	sr_reg (srchshell_w, "XEphem*Solve.x", solvecategory, 0);
+	sr_reg (srchshell_w, "XEphem*Solve.y", solvecategory, 0);
+
+	n = 0;
+	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
-	srchform_w = XmCreateFormDialog (toplevel_w, "Srch", args, n);
-	set_something (srchform_w, XmNcolormap, (XtArgVal)xe_cm);
+	srchform_w = XmCreateForm (srchshell_w, "SrchF", args, n);
 	XtAddCallback (srchform_w, XmNhelpCallback, srch_help_cb, 0);
-
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
-	n = 0;
-	XtSetArg (args[n], XmNtitle, "xephem Solver Control"); n++;
-	XtSetValues (XtParent(srchform_w), args, n);
+	XtManageChild (srchform_w);
 
 	/* create a RowColumn to hold stuff on top of the text */
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNspacing, 8); n++;
+	XtSetArg (args[n], XmNspacing, 5); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
 	trc_w = XmCreateRowColumn (srchform_w, "TSrchRC", args, n);
@@ -342,10 +347,10 @@ srch_create_form()
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNspacing, 8); n++;
+	XtSetArg (args[n], XmNspacing, 5); n++;
 	XtSetArg (args[n], XmNisAligned, False); n++;
 	XtSetArg (args[n], XmNadjustMargin, False); n++;
-	brc_w = XmCreateRowColumn (srchform_w, "TSrchRC", args, n);
+	brc_w = XmCreateRowColumn (srchform_w, "BSrchRC", args, n);
 	XtManageChild (brc_w);
 
 	/* compiler message label */
@@ -428,11 +433,15 @@ srch_create_form()
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	    acc_w = XmCreatePushButton (f_w, "SrchAcc", args, n);
+	    acc_w = XmCreatePushButton (f_w, "Accuracy", args, n);
 	    XtAddCallback (acc_w, XmNactivateCallback, srch_acc_cb, 0);
-	    f_time (acc_w, tmlimit);
 	    wtip (acc_w, "Set desired time accuracy of solution");
 	    XtManageChild (acc_w);
+	    sr_reg (acc_w, NULL, solvecategory, 1);
+
+	    get_xmstring (acc_w, XmNlabelString, &txt);
+	    f_scansex (0.0, txt, &tmlimit);
+	    XtFree (txt);
 
 	    str = XmStringCreate("Desired accuracy:", XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
@@ -461,7 +470,7 @@ srch_create_form()
 	    XtSetArg (args[n], XmNuserData, (XtArgVal)srchvname); n++;
 	    valu_w = XmCreatePushButton (f_w, "CurValue", args, n);
 	    set_xmstring (valu_w, XmNlabelString, "0.0");
-	    srch_set_a_button (valu_w, False);
+	    buttonAsButton (valu_w, False);
 	    XtAddCallback (valu_w, XmNactivateCallback, srch_use_cb, 0);
 	    wtip (valu_w, "Value of Function evaluated at current time");
 	    XtManageChild (valu_w);
@@ -530,17 +539,18 @@ srch_create_form()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
 	XtSetArg (args[n], XmNwordWrap, True); n++;
-	func_w = XmCreateText (srchform_w, "SrchFunction", args, n);
+	func_w = XmCreateText (srchform_w, "Function", args, n);
 	XtAddCallback (func_w, XmNvalueChangedCallback, srch_newfunc_cb, NULL);
 	wtip(func_w,"Function to solve: enter fields manually, or via buttons");
 	XtManageChild (func_w);
+	sr_reg (func_w, NULL, solvecategory, 1);
 }
 
 /* return True whenever the srch control menu is up */
 static int
 srch_isup()
 {
-	return (srchform_w && XtIsManaged(srchform_w));
+	return (isUp(srchshell_w));
 }
 
 /* return True whenever we are actually in the midst of controlling a search.
@@ -558,74 +568,7 @@ static void
 srch_set_buttons (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	srch_set_a_button(valu_w, whether);
-}
-
-/* set whether the given button looks like a label.
- */
-static void
-srch_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
-{
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-	    Widget tmpw = compile_w;	/* any typical PB */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    int n;
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-	    XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
+	buttonAsButton(valu_w, whether);
 }
 
 /* callback from the value button when it is to be used for
@@ -768,7 +711,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	XtUnmanageChild (srchform_w);
+	XtPopdown (srchshell_w);
 }
 
 /* user typed OK to the accuracy prompt. get his new value and use it */
@@ -811,7 +754,6 @@ XtPointer call;
 	    title = XmStringCreate ("xephem Solver Accuracy",
 						    XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
-	    XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	    XtSetArg(args[n], XmNselectionLabelString, str);  n++;
 	    XtSetArg(args[n], XmNdialogTitle, title);  n++;
 	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
@@ -1078,4 +1020,4 @@ double *tmincp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: srchmenu.c,v $ $Date: 1999/10/25 21:01:55 $ $Revision: 1.3 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: srchmenu.c,v $ $Date: 2000/07/20 07:37:43 $ $Revision: 1.7 $ $Name:  $"};
