@@ -20,6 +20,9 @@ typedef void * qsort_arg;
 
 #include <Xm/Xm.h>	/* easy way to bring in Xt stuff is all */
 
+extern Colormap xe_cm;
+extern Widget toplevel_w;
+
 extern void get_something P_((Widget w, char *resource, XtArgVal value));
 extern char *getXRes P_((char *name, char *def));
 
@@ -31,13 +34,12 @@ typedef enum {
 } TState;
 typedef struct {
     Widget w;		/* the widget for this tip */
-    String s;		/* its tip text */
-    Pixel fg, bg;	/* colors to use */
-    Colormap cm;	/* colormap of pixels */
+    String s;		/* its text */
     TState state;	/* down, armed or up */
 } Tip;
 
-static void tip_init P_((Display *dsp));
+void wtip_init P_((void));
+
 static void tip_new P_((Widget w, String s));
 static void tip_reclaim P_((Widget w));
 static void tip_eh P_((Widget w, XtPointer c, XEvent *e, Boolean *cont));
@@ -52,15 +54,16 @@ static void tip_sort P_((void));
 
 #define	GAP		5	/* gap around text, pixels */
 #define	BW		1	/* window border width */
-#define	VOFFSET		4	/* dist from event button to window, pixels */
+#define	VOFFSET		15	/* dist from pointer to window top, pixels */
+#define	EDGE		10	/* min space to edge of screen */
 #define	TIPTO		500	/* time to wait before raising tip, ms */
 
 static Tip *tips;		/* all current tips -- w==0 when not in use */
 static int ntips;		/* total tips in tips[] */
-static Window the_tipw;		/* tips window */
-static GC the_tipgc;		/* tips gc */
-static XFontStruct *the_tipfs;	/* tips font */
-static char *tip_fgc, *tip_bgc;	/* foreground and background color names */
+static Window the_w;		/* tips window */
+static GC the_gc;		/* tips gc */
+static XFontStruct *the_fs;	/* tips font */
+static Pixel the_fg, the_bg;	/* foreground and background colors */
 static int tips_sorted;		/* flag set when tips array is sorted */
 
 /* connect the given string as the tip for the given widget */
@@ -76,8 +79,8 @@ char *tip;
 	}
 
 	/* one-time initialization */
-	if (!the_tipw)
-	    tip_init(XtDisplay(w));
+	if (!the_w)
+	    wtip_init();
 
 	/* connect handlers to this widget.
 	 * we tried using translations but it interfered too much
@@ -92,42 +95,68 @@ char *tip;
 	tip_new (w, tip);
 }
 
-/* one-time initialization stuff: create a window, font and gc */
-static void
-tip_init (dsp)
-Display *dsp;
+/* go through all the tips and make sure they are all down.
+ */
+void
+wtip_alldown()
 {
+	Tip *tp, *ltp;
+
+	for (tp = tips, ltp = &tips[ntips]; tp < ltp; tp++) {
+	    if (tp->w && tp->state != T_DOWN) {
+		tip_down(tp);
+		tp->state = T_DOWN;
+	    }
+	}
+}
+
+/* one-time initialization stuff: create a window, font and gc.
+ * TODO: reclaim old stuff when called again
+ */
+void
+wtip_init ()
+{
+	Display *dsp = XtDisplay(toplevel_w);
 	Window root = RootWindow (dsp, DefaultScreen(dsp));
 	XSetWindowAttributes wa;
 	unsigned long mask;
+	XColor defxc, dbxc;
 	XGCValues gcv;
-	unsigned long bg, fg;
+	char *cnam;
 
 	/* get tip colors */
-	tip_fgc = getXRes ("tipForeground", "black");
-	tip_bgc = getXRes ("tipBackground", "white");
+	cnam = getXRes ("tipForeground", "black");
+	if (!XAllocNamedColor (dsp, xe_cm, cnam, &defxc, &dbxc))
+	    the_fg = BlackPixel (dsp, DefaultScreen(dsp));
+	else
+	    the_fg = defxc.pixel;
+	cnam = getXRes ("tipBackground", "white");
+	if (!XAllocNamedColor (dsp, xe_cm, cnam, &defxc, &dbxc))
+	    the_bg = WhitePixel (dsp, DefaultScreen(dsp));
+	else
+	    the_bg = defxc.pixel;
 
 	/* create the tips window -- use same one for all.
 	 * any size/color will do for now since we change each time anyway.
 	 */
-	fg = BlackPixel(dsp, DefaultScreen(dsp));
-	bg = WhitePixel(dsp, DefaultScreen(dsp));
-	the_tipw = XCreateSimpleWindow (dsp, root, 0, 0, 10, 10, BW, fg, bg);
+	the_w = XCreateSimpleWindow (dsp, root, 0, 0, 10, 10, BW,
+							the_fg, the_bg);
 
 	/* prevent window manager from decorating */
-	mask = CWOverrideRedirect;
+	mask = CWOverrideRedirect | CWSaveUnder;
 	wa.override_redirect = True;
-	XChangeWindowAttributes (dsp, the_tipw, mask, &wa);
+	wa.save_under = True;
+	XChangeWindowAttributes (dsp, the_w, mask, &wa);
 
 	/* pick a font and make a GC */
-	the_tipfs = XLoadQueryFont (dsp, "fixed");
-	if (!the_tipfs) {
+	the_fs = XLoadQueryFont (dsp, getXRes ("tipFont", "fixed"));
+	if (!the_fs) {
 	    printf ("Tips: no fixed font ?!?\n");
 	    exit(1);
 	}
-	gcv.font = the_tipfs->fid;
+	gcv.font = the_fs->fid;
 	mask = GCFont;
-	the_tipgc = XCreateGC (dsp, root, mask, &gcv);
+	the_gc = XCreateGC (dsp, root, mask, &gcv);
 }
 
 /* add a new tip to tips[].
@@ -139,11 +168,7 @@ tip_new (w, s)
 Widget w;
 String s;
 {
-	Display *dsp = XtDisplay (w);
 	Tip *tp, *ltp;
-	XColor defxc, dbxc;
-	Colormap cm;
-	Pixel fg, bg;
 
 	/* find an empty spot, or grow if none */
 	for (tp = tips, ltp = &tips[ntips]; tp < ltp; tp++)
@@ -155,24 +180,6 @@ String s;
 	    tp = &tips[ntips++];
 	}
 
-	/* set desired colors */
-	get_something (w, XmNcolormap, (XtArgVal)&cm);
-	if (!tip_fgc || !XAllocNamedColor (dsp, cm, tip_fgc, &defxc, &dbxc))
-	    fg = BlackPixel (dsp, DefaultScreen(dsp));
-	else
-	    fg = defxc.pixel;
-	if (!tip_bgc || !XAllocNamedColor (dsp, cm, tip_bgc, &defxc, &dbxc))
-	    bg = WhitePixel (dsp, DefaultScreen(dsp));
-	else
-	    bg = defxc.pixel;
-	if (fg == bg) {
-	    fg = BlackPixel (dsp, DefaultScreen(dsp));
-	    bg = WhitePixel (dsp, DefaultScreen(dsp));
-	}
-	tp->cm = cm;
-	tp->fg = fg;
-	tp->bg = bg;
-
 	/* save widget */
 	tp->w = w;
 
@@ -280,47 +287,47 @@ tip_up (tp)
 Tip *tp;
 {
 	Display *dsp = XtDisplay (tp->w);
-	int dw = DisplayWidth(dsp,0);
-	int dh = DisplayHeight(dsp,0);
+	Window win = XtWindow (tp->w);
+	int scr = DefaultScreen(dsp);
+	int dw = DisplayWidth(dsp,scr);
+	int dh = DisplayHeight(dsp,scr);
 	int l = strlen (tp->s);
 	int dir, asc, des;
 	int wid, hei;
+	Window root, child;
 	XCharStruct oa;
-	Position x, y;
-	Dimension ww, wh;
-	Arg a[2];
+	int x, y, wx, wy;
+	unsigned int pqmask;
 	XSetWindowAttributes wa;
 	unsigned long mask;
 
-	/* compute size of window to wrap nicely around text.
-	 * location should be below and offset a bit, unless near screen edge.
-	 */
-	XTextExtents (the_tipfs, tp->s, l, &dir, &asc, &des, &oa);
+	/* compute size of window to wrap nicely around text */
+	XTextExtents (the_fs, tp->s, l, &dir, &asc, &des, &oa);
 	wid = oa.width + 2*GAP;
 	hei = oa.ascent + oa.descent + 2*GAP;
-	XtSetArg (a[0], XmNwidth, (XtArgVal)&ww);
-	XtSetArg (a[1], XmNheight, (XtArgVal)&wh);
-	XtGetValues (tp->w, a, XtNumber(a));
-	XtTranslateCoords (tp->w, (int)ww/3, (int)wh+VOFFSET, &x, &y);
-	if (x + wid + 2*BW > dw)
-	    x = dw - wid - 2*BW;
-	if (y + hei + 2*BW > dh)
-	    y -= VOFFSET + wh + VOFFSET + hei + 2*BW;
-	XMoveResizeWindow (dsp, the_tipw, x, y, wid, hei);
+
+	/* set position near pointer but never over it and beware screen edge */
+	XQueryPointer (dsp, win, &root, &child, &x, &y, &wx, &wy, &pqmask);
+	if (x + wid + 2*BW > dw-EDGE)
+	    x = dw-EDGE - wid - 2*BW;
+	y += VOFFSET;
+	if (y + hei + 2*BW > dh-EDGE)
+	    y -= 2*VOFFSET + hei + 2*BW;	/* never on pointer */
+	XMoveResizeWindow (dsp, the_w, x, y, wid, hei);
 
 	/* set colors to match source */
 	mask = CWBackPixel|CWBorderPixel|CWColormap;
-	wa.background_pixel = tp->bg;
-	wa.border_pixel = tp->fg;
-	wa.colormap = tp->cm;
-	XChangeWindowAttributes (dsp, the_tipw, mask, &wa);
-	XClearWindow (dsp, the_tipw);
-	XMapRaised (dsp, the_tipw);
+	wa.background_pixel = the_bg;
+	wa.border_pixel = the_fg;
+	wa.colormap = xe_cm;
+	XChangeWindowAttributes (dsp, the_w, mask, &wa);
+	XClearWindow (dsp, the_w);
+	XMapRaised (dsp, the_w);
 	XSync (dsp, 0); /* so we don't need to wait for Expose elsewhere */
 
 	/* draw the text */
-	XSetForeground (dsp, the_tipgc, tp->fg);
-	XDrawString (dsp, the_tipw, the_tipgc, GAP, oa.ascent+GAP, tp->s, l);
+	XSetForeground (dsp, the_gc, the_fg);
+	XDrawString (dsp, the_w, the_gc, GAP, oa.ascent+GAP, tp->s, l);
 }
 
 /* pop down the tip window */
@@ -328,7 +335,7 @@ static void
 tip_down(tp)
 Tip *tp;
 {
-	XUnmapWindow (XtDisplay(tp->w), the_tipw);
+	XUnmapWindow (XtDisplay(tp->w), the_w);
 }
 
 /* w is being destroyed so reclaim its tip info */
@@ -399,4 +406,4 @@ tip_sort ()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: tips.c,v $ $Date: 1997/08/10 12:57:40 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: tips.c,v $ $Date: 2000/10/19 14:45:25 $ $Revision: 1.7 $ $Name:  $"};
