@@ -4,14 +4,11 @@
 # 24 Feb 98: begin
 # 11 Mar 98: final tweaks
 
-# full path to directory containing *.edb files, ppm.xe and gsc.
-$dbdir = '/usr/local/telescope/archive/catalogs';
-
 # generally no need to change anything from here on.
 # ==================================================
 
 # set $trace to 1 to see steps in $tracefile
-$tracefile = 'xephemdbd.pl.tr';
+$tracefile = 'xephemdbd.trace';
 $trace = 0;
 
 # set to lock file location
@@ -23,7 +20,7 @@ $infifo = 'xephemdbd.in';
 # set to unique output fifo location
 $outfifo = "xephemdbd.$$";
 
-# set to xephemdbd stderr log file output
+# set to xephemdbd log file
 $logfn = "xephemdbd.log";
 
 # certainly no need to change anything from here on.
@@ -39,10 +36,15 @@ require "cgi-lib.pl";
 # autoflush output
 &afon (STDOUT);
 
+# set up log file
+open (LOG, ">>$logfn") || !print "<p>$logfn $!\n" || goto wrapup;
+&afon (LOG);
+printf LOG "xephemdbd.pl $$ started %s UTC from $ENV{'REMOTE_ADDR'}\n", gmtime()."";
+
 # set up trace file, if enabled
-!$trace || open(TR,">$tracefile") || !print "<p>$tracefile $!\n" || goto wrapup;
+!$trace || open(TR,">>$tracefile") || !print "<p>$tracefile $!\n" || goto wrapup;
 !$trace || &afon (TR);
-!$trace || print TR "xephemdbd.pl $$ started\n";
+!$trace || printf TR "xephemdbd.pl $$ started %s UTC from $ENV{'REMOTE_ADDR'}\n", gmtime()."";
 
 # Read in all the variables set by the form so we can access as $input{'name'}
 &ReadParse(*input);
@@ -56,29 +58,29 @@ print "<H2>XEphemdbd Results</H2>\n";
 
 # check versions match.
 $tmp = $input{'VERSION'};
-!$trace || print TR "checking version match";
+!$trace || print TR "  checking version match";
 if ($tmp != $ver) {
     print "<p>Wrong version: $ver <i>vs.</i> $tmp\n";
     goto wrapup;
 }
 
 # make sure we have the fifo to the xephemdbd
-!$trace || print TR "checking $infifo\n";
-if (! -p $infifo && system ("mknod $infifo p")) {
+!$trace || print TR "  checking $infifo\n";
+if (! -p $infifo && system ("mkfifo $infifo")) {
     print "<p>Can not create fifo $infifo: $!\n";
     goto wrapup;
 }
 
 # start xephemdbd -- it knows to exit if already running.
-!$trace || print TR "checking for xephemdbd\n";
-if (system ("./xephemdbd -vlic $lkfile $infifo $dbdir >> $logfn 2>&1")) {
+!$trace || print TR "  checking for xephemdbd\n";
+if (system ("./start-xephemdbd")) {
     print "<p>Can not start xephemdbd\n";
     goto wrapup;
 }
 
 # start building up the command to xephemdbd in $ecmd in the following format:
 # >file,outputmode,types,year,RA,Dec,FOV,Mag[,lat,long,elev]
-!$trace || print TR "cracking FORM args\n";
+!$trace || print TR "  cracking FORM args\n";
 $ecmd = ">$outfifo";
 
 # gather the basic options 
@@ -94,7 +96,7 @@ $obj = 0;
 $obj +=   1 if $input{'PL'};
 $obj +=  14 if $input{'SS'};
 $obj +=  16 if $input{'BS'};
-$obj += 128 if $input{'DS'};
+$obj += 224 if $input{'DS'};
 $obj += 256 if $input{'PP'};
 $obj += 512 if $input{'GS'};
 $ecmd .= ",$obj";
@@ -184,19 +186,19 @@ if ($ops & 6) {
     $ecmd .= ",$elev";
 }
 
-!$trace || print TR "command = `$ecmd'\n";
+!$trace || print TR "  command = `$ecmd'\n";
 
 # give command to xephemdbd to use outfifo and echo results
-!$trace || print TR "creating $outfifo\n";
-if (!system ("mknod $outfifo p")) {
-    !$trace || print TR "opening $infifo\n";
+!$trace || print TR "  creating $outfifo\n";
+if (!system ("mkfifo -m 0666 $outfifo")) {
+    !$trace || print TR "  opening $infifo\n";
     if (open (EIN, ">>$infifo")) {
 	&afon(EIN);
-	!$trace || print TR "sending command\n";
+	!$trace || print TR "  sending command\n";
 	if (print EIN "$ecmd\n") {
-	    !$trace || print TR "opening $outfifo\n";
+	    !$trace || print TR "  opening $outfifo\n";
 	    if (open (EOUT, "<$outfifo")) {
-		!$trace || print TR "reading $outfifo\n";
+		!$trace || print TR "  reading $outfifo\n";
 		# finally! copy result back to client
 		print "<pre>\n";
 		while (<EOUT>) {
@@ -217,17 +219,19 @@ if (!system ("mknod $outfifo p")) {
 	print "<p>Opening $infifo failed: $!\n";
 	goto wrapup;
     }
-    unlink ($outfifo);
 } else {
     print "<p>Creating $outfifo failed: $!\n";
     goto wrapup;
 }
 
-!$trace || print TR "finished successfully\n";
+!$trace || print TR "  finished successfully\n";
 
 # final boilerplate
 wrapup:
-!$trace || print TR "wrapup\n";
+!$trace || print TR "  wrapup\n";
+
+# remove temp response fifo
+unlink ($outfifo);
 
 print <<ENDOFTEXT;
 <p>Use your browser's \"Go back\" feature to modify the same request.
@@ -257,3 +261,6 @@ sub afon {
     # "bizarre and obscure", Programming Perl, page 211
     select ((select(@_[0]), $| = 1)[0]);
 }
+
+# For RCS Only -- Do Not Edit
+# @(#) $RCSfile: xephemdbd.pl,v $ $Date: 2000/10/20 19:29:58 $ $Revision: 1.4 $ $Name:  $
