@@ -29,7 +29,6 @@
 #include "circum.h"
 #include "trails.h"
 #include "ps.h"
-#include "rotated.h"
 
 extern Widget toplevel_w;
 extern Colormap xe_cm;
@@ -60,7 +59,6 @@ extern void wtip P_((Widget w, char *tip));
 
 static char fontxr[] = "trailsFont";	/* X resource naming font to use */
 static XFontStruct *tr_fs;		/* font for trails */
-static char *psfont = "Helvetica";	/* Postscript font for trails */
 
 /* context information per dialog.
  * one of these gets malloced and saved in userData of each SelectionBox.
@@ -160,9 +158,6 @@ static void close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void help_cb P_((Widget w, XtPointer client, XtPointer call));
 static void draw_stamp P_((Display *dsp, Drawable win, GC gc, TrTS *tp,
     TrState *sp, int ticklen, int lx, int ly, int x, int y));
-static int draw_tickmark P_((int ticklen, int x1, int y1, int x2, int y2,
-    XSegment *xsp));
-
 
 /* put up a dialog to ask how to make a trail. OK will call (*cb)().
  * ok to have several at once -- they keep their own context in userData.
@@ -214,9 +209,19 @@ XtPointer client;	/* saved and passed back to (*cb)() */
 	    XtManageChild (w);
 }
 
-/* draw a line from [lx,ly] to [x,y]. iff tp, draw a perpendicular tickmark of
- * halfsize ticklen pixels and draw a time stamp according to tp and sp at
- * [x,y]. iff ltp, do the same according to ltp and sp at [lx,ly].
+/* called when basic resources change.
+ * rebuild and redraw.
+ * TODO: reclaim old stuff when called again.
+ */
+void
+tr_newres()
+{
+	tr_fs = NULL;
+}
+
+/* draw a line from [lx,ly] to [x,y]. iff tp, draw a tickmark of halfsize
+ * ticklen pixels and draw a time stamp according to tp and sp at [x,y].
+ * iff ltp, do the same according to ltp and sp at [lx,ly].
  */
 void tr_draw (dsp, win, gc, ticklen, tp, ltp, sp, lx, ly, x, y)
 Display *dsp;
@@ -229,26 +234,24 @@ TrState *sp;
 int lx, ly;
 int x, y;
 {
-	XSegment xs[3];		/* a pool of XSegments */
-	XSegment *xsp = xs;
-
 	/* the base line for sure */
-	xsp->x1 = lx;  xsp->y1 = ly;  xsp->x2 = x;  xsp->y2 = y; xsp++;
+        XPSDrawLine (dsp, win, gc, lx, ly, x, y);
 
 	/* if tp, draw tick mark and stamp at [x,y] */
 	if (tp) {
-	    xsp += draw_tickmark (ticklen, lx, ly, x, y, xsp);
+	    XPSFillArc (dsp, win, gc, x-ticklen, y-ticklen, 2*ticklen+1,
+						    2*ticklen+1, 0, 360*64);
 	    draw_stamp (dsp, win, gc, tp, sp, ticklen, lx, ly, x, y);
 	}
 
 	/* same for other end if ltp */
 	if (ltp) {
-	    xsp += draw_tickmark (ticklen, x, y, lx, ly, xsp);
+	    XPSFillArc (dsp, win, gc, lx-ticklen, ly-ticklen, 2*ticklen+1,
+						    2*ticklen+1, 0, 360*64);
 	    draw_stamp (dsp, win, gc, ltp, sp, ticklen, x-2*(x-lx), y-2*(y-ly),
 									lx, ly);
 	}
 
-        XPSDrawSegments (dsp, win, gc, xs, xsp-xs);
 }
 
 /* create a trail control dialog,
@@ -287,7 +290,6 @@ XtPointer client;
 	/* make the general dialog */
 	n = 0;
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg(args[n], XmNdefaultPosition, False);  n++;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
@@ -675,9 +677,9 @@ XtPointer call;
 	/* fetch the TrContext */
 	tcp = (TrContext *)client;
 
-	/* do the work -- close too if went ok */
-	if (do_trails (tcp) == 0)
-	    XtUnmanageChild (tcp->d_w);
+	/* do the work and close regardless */
+	(void) do_trails (tcp);
+	XtUnmanageChild (tcp->d_w);
 }
 
 /* called when the Apply is hit.
@@ -696,8 +698,9 @@ XtPointer call;
 	/* fetch the TrContext */
 	tcp = (TrContext *)client;
 
-	/* do the work */
-	(void) do_trails (tcp);
+	/* do the work -- close if trouble */
+	if (do_trails (tcp) < 0)
+	    XtUnmanageChild (tcp->d_w);
 }
 
 /* perform the actual trails work.
@@ -765,7 +768,7 @@ TrContext *tcp;
 		cal_mjd (m, d, y, &t);
 		break;
 	    case TRI_YEAR:
-		mjd_cal (t, &m, &d, &y);
+		mjd_cal (mjd, &m, &d, &y);
 		d = 1;
 		m = 1;
 		y++;
@@ -862,7 +865,7 @@ TrContext *tcp;
 	free ((char *)ttp);
 
 	/* ok */
-	return (0);
+	return (cbret);
 }
 
 /* pull the options from tcp.
@@ -1030,45 +1033,6 @@ XtPointer call;
         hlp_dialog ("Trails", NULL, 0);
 }
 
-/* fill xsp with a line perpendicular to [x1,y1]-[x2,y2] at [x2,y2].
- * return 1 if really added segment else 0.
- */
-static int
-draw_tickmark (ticklen, x1, y1, x2, y2, xsp)
-int ticklen;
-int x1, y1;
-int x2, y2;
-XSegment *xsp;
-{
-	int dx = x2-x1;
-	int dy = y2-y1;
-	int dxt, dyt;
-
-	if (ticklen == 0)
-	    return (0);
-
-	if (dx == 0 && dy == 0) {
-	    /* no base line -- vertical default */
-	    dxt = 0;
-	    dyt = ticklen;
-	} else if (abs(dx) > abs(dy)) {
-	    /* base line is more horizontal than vertical */
-	    dxt = ticklen*dy/dx;
-	    dyt = ticklen;
-	} else {
-	    /* base line is more vertical than horizontal */
-	    dxt = ticklen;
-	    dyt = ticklen*dx/dy;
-	}
-
-	xsp->x1 = x2-dxt;
-	xsp->y1 = y2+dyt;
-	xsp->x2 = x2+dxt;
-	xsp->y2 = y2-dyt;
-
-	return (1);
-}
-
 /* draw a timestamp for the given trail object at x, y.
  * direction, size and content depend on tp, sp and the line endpoints.
  */
@@ -1096,7 +1060,6 @@ int x, y;
 	/* make sure the fonts are ready */
 	if (!tr_fs)
 	    tr_fs = getXResFont (fontxr);
-	XPSRegisterFont (tr_fs->fid, psfont);
 
 	/* establish character size */
 	switch (sp->s) {
@@ -1165,8 +1128,8 @@ int x, y;
 	    a += PI/2;
 	    ca = cos(a);
 	    sa = sin(a);
-	    x += ca*(gap);
-	    y -= sa*(gap);
+	    x += (int)(ca*gap);
+	    y -= (int)(sa*gap);
 	    if (a > PI/2) {
 		a -= PI;
 		align = MRIGHT;
@@ -1187,8 +1150,8 @@ int x, y;
 	    a -= PI/2;
 	    ca = cos(a);
 	    sa = sin(a);
-	    x += ca*(gap);
-	    y -= sa*(gap);
+	    x += (int)(ca*gap);
+	    y -= (int)(sa*gap);
 	    if (a > PI/2) {
 		a -= PI;
 		align = MRIGHT;
@@ -1209,4 +1172,4 @@ int x, y;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: trailmenu.c,v $ $Date: 1999/02/25 06:56:27 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: trailmenu.c,v $ $Date: 2000/10/12 21:26:31 $ $Revision: 1.9 $ $Name:  $"};
