@@ -27,7 +27,6 @@
 #include "preferences.h"
 #include "ps.h"
 
-
 extern Widget	toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
 extern Colormap xe_cm;
@@ -36,6 +35,7 @@ extern FILE *fopenh P_((char *name, char *how));
 extern Now *mm_get_now P_((void));
 extern Obj *db_basic P_((int id));
 extern Obj *db_scan P_((DBScan *sp));
+extern XFontStruct * getXResFont P_((char *rn));
 extern char *getShareDir P_((void));
 extern double atod P_((char *buf));
 extern double delra P_((double dra));
@@ -43,11 +43,13 @@ extern int any_ison P_((void));
 extern int fs_fetch P_((Now *np, double ra, double dec, double fov,
     double mag, ObjF **opp));
 extern int get_color_resource P_((Widget w, char *cname, Pixel *p));
+extern int isUp P_((Widget shell));
 extern int magdiam P_((int fmag, int magstp, double scale, double mag,
     double size));
 extern int read_bdl P_((FILE *fp, double jd, double *xp, double *yp,
     double *zp, char ynot[]));
 extern int skyloc_fifo P_((Obj *op));
+extern void buttonAsButton P_((Widget w, int whether));
 extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
 extern void db_update P_((Obj *op));
 extern void f_prdec P_((Widget w, double a));
@@ -63,10 +65,11 @@ extern void obj_pickgc P_((Obj *op, Widget w, GC *gcp));
 extern void register_selection P_((char *name));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void um_update P_((Now *np, int how_much));
+extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
 extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
     int y, int diam, int dotsonly));
 extern void timestamp P_((Now *np, Widget w));
+extern void um_update P_((Now *np, int how_much));
 extern void watch_cursor P_((int want));
 extern void wtip P_((Widget w, char *tip));
 extern void xe_msg P_((char *buf, int app_modal));
@@ -90,11 +93,11 @@ typedef enum {
 static ScreenObj *screenobj[SO_N];
 static int nscreenobj[SO_N];
 
-static void um_create_form_w P_((void));
-static void um_create_tvform_w P_((void));
+static void um_create_shell_w P_((void));
 static void um_create_usform_w P_((void));
+static void um_create_tvform_w P_((void));
 static void um_set_buttons P_((int whether));
-static void um_set_a_button P_((Widget pbw, int whether));
+static void um_sstats_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void um_sstats_cb P_((Widget w, XtPointer client, XtPointer call));
 static void um_option_cb P_((Widget w, XtPointer client, XtPointer call));
 static void um_scale_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -103,7 +106,7 @@ static void ut_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ut_map_cb P_((Widget w, XtPointer client, XtPointer call));
 static void ut_track_size P_((void));
 static void ut_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
-static void um_unmap_cb P_((Widget w, XtPointer client, XtPointer call));
+static void um_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
 static void um_close_cb P_((Widget w, XtPointer client, XtPointer call));
 static void um_movie_cb P_((Widget w, XtPointer client, XtPointer call));
 static void um_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
@@ -123,7 +126,7 @@ static ScreenObj *close_screenobj P_((SOIdx, int x, int y));
 static void sky_background P_((Drawable win, unsigned w, unsigned h, int fmag,
     double ra0, double dec0, double scale, double rad, int fliptb, int fliplr));
 
-static Widget uform_w;		/* main form */
+static Widget ushell_w;		/* main shell */
 static Widget usform_w;		/* statistics form */
 static Widget umframe_w;	/* main frame */
 static Widget utform_w;		/* top-view form */
@@ -148,7 +151,6 @@ static int flip_tb;		/* whether we want to flip top/bottom */
 static int flip_lr;		/* whether we want to flip left/right */
 static int skybkg;		/* whether we want sky background */
 static int topview;		/* whether we want the top view */
-static int uswasman;		/* whether statistics form was managed */
 
 static Widget umpu_w;		/* main popup */
 static Widget umpu_name_w;	/* popup name label */
@@ -186,28 +188,24 @@ static struct MoonNames {
 enum {CEV, CSV, CX, CY, CZ, CRA, CDEC, CMAG, CNum};	/* u_w column index */
 static Widget	u_w[NM][CNum];			/* the data display widgets */
 
+static char ucategory[] = "Uranus";		/* Save category */
+
 /* called when the uranus menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
- * form. otherwise, just toggle whether the form is managed.
+ * form. otherwise, just get out there and do it!
  */
 void
 um_manage ()
 {
-	if (!uform_w) {
-	    um_create_form_w();
+	if (!ushell_w) {
+	    um_create_shell_w();
 	    um_create_usform_w();
+	    um_create_tvform_w();
 	}
 	
-	if (XtIsManaged(uform_w))
-	    XtUnmanageChild (uform_w);
-	else {
-	    XtManageChild (uform_w);
-	    if (uswasman) {
-		XtManageChild (usform_w);
-		um_set_buttons(um_selecting);
-	    }
-	    /* rely on expose to do the first draw */
-	}
+	XtPopup (ushell_w, XtGrabNone);
+	set_something (ushell_w, XmNiconic, (XtArgVal)False);
+	um_set_buttons(um_selecting);
 }
 
 /* called when the database has changed.
@@ -225,7 +223,7 @@ int appended;
 int
 um_ison()
 {
-	return (uform_w && XtIsManaged(uform_w));
+	return (isUp(ushell_w));
 }
 
 /* called by other menus as they want to hear from our buttons or not.
@@ -236,9 +234,12 @@ void
 um_selection_mode (whether)
 int whether;	/* whether setting up for plotting or for not plotting */
 {
-	um_selecting += whether ? 1 : -1;
+	if (whether)
+	    um_selecting++;
+	else if (um_selecting > 0)
+	    --um_selecting;
 
-	if (um_ison())
+	if (ushell_w)
 	    if ((whether && um_selecting == 1)     /* first one to want on */
 		|| (!whether && um_selecting == 0) /* last one to want off */)
 		um_set_buttons (whether);
@@ -251,8 +252,8 @@ Cursor c;
 {
 	Window win;
 
-	if (uform_w && (win = XtWindow(uform_w)) != 0) {
-	    Display *dsp = XtDisplay(uform_w);
+	if (ushell_w && (win = XtWindow(ushell_w)) != 0) {
+	    Display *dsp = XtDisplay(ushell_w);
 	    if (c)
 		XDefineCursor (dsp, win, c);
 	    else
@@ -266,21 +267,29 @@ Cursor c;
 	    else
 		XUndefineCursor (dsp, win);
 	}
+
+	if (utform_w && (win = XtWindow(utform_w)) != 0) {
+	    Display *dsp = XtDisplay(utform_w);
+	    if (c)
+		XDefineCursor (dsp, win, c);
+	    else
+		XUndefineCursor (dsp, win);
+	}
 }
 
-/* create the main form dialog */
+/* create the main shell */
 static void
-um_create_form_w()
+um_create_shell_w()
 {
 	typedef struct {
-	    char *name;		/* toggle button instance name */
+	    char *name;		/* toggle button instance name, or NULL */
 	    char *label;	/* label */
 	    int *flagp;		/* pointer to global flag, or NULL if none */
 	    Widget *wp;		/* widget to save, or NULL */
 	    char *tip;		/* widget tip */
 	} Option;
 	static Option options[] = {
-	    {"TopView",	   "Top view",		&topview, &topview_w,
+	    {NULL,	   "Top view",		&topview, &topview_w,
 	    	"When on, show view looking from high above N pole"},
 	    {"SkyBkg",	   "Sky background",	&skybkg,  &skybkg_w,
 	    	"When on, sky will include database objects and Field Stars"},
@@ -305,28 +314,32 @@ um_create_form_w()
 	};
 	Widget w;
 	Widget mb_w, pd_w, cb_w;
+	Widget uform_w;
 	XmString str;
 	Arg args[20];
 	int n;
 	int i;
 
-	/* create form */
-	n = 0;
-	XtSetArg (args[n], XmNautoUnmanage, False); n++;
-	XtSetArg (args[n], XmNdefaultPosition, False); n++;
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	uform_w = XmCreateFormDialog (toplevel_w, "Uranus", args, n);
-	set_something (uform_w, XmNcolormap, (XtArgVal)xe_cm);
-	XtAddCallback (uform_w, XmNhelpCallback, um_help_cb, 0);
-	XtAddCallback (uform_w, XmNunmapCallback, um_unmap_cb, 0);
+	/* create shell and form */
 
-	/* set some stuff in the parent DialogShell.
-	 * setting XmNdialogTitle in the Form didn't work..
-	 */
 	n = 0;
 	XtSetArg (args[n], XmNtitle, "xephem Uranus view"); n++;
-	XtSetValues (XtParent(uform_w), args, n);
+	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNiconName, "Uranus"); n++;
+	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
+	ushell_w = XtCreatePopupShell ("Uranus", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	set_something (ushell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (ushell_w, XmNpopdownCallback, um_popdown_cb, 0);
+	sr_reg (ushell_w, "XEphem*Uranus.width", ucategory, 0);
+	sr_reg (ushell_w, "XEphem*Uranus.height", ucategory, 0);
+	sr_reg (ushell_w, "XEphem*Uranus.x", ucategory, 0);
+	sr_reg (ushell_w, "XEphem*Uranus.y", ucategory, 0);
+
+	n = 0;
+	uform_w = XmCreateForm (ushell_w, "UranusF", args, n);
+	XtAddCallback (uform_w, XmNhelpCallback, um_help_cb, 0);
+	XtManageChild (uform_w);
 
 	/* create the menu bar across the top */
 
@@ -353,7 +366,7 @@ um_create_form_w()
 
 	    n = 0;
 	    w = XmCreatePushButton (pd_w, "FS", args, n);
-	    set_xmstring (w, XmNlabelString, "Setup Field Stars ...");
+	    set_xmstring (w, XmNlabelString, "Field Stars ...");
 	    XtAddCallback (w, XmNactivateCallback, (XtCallbackProc)fs_manage,0);
 	    wtip (w, "Define where GSC and PPM catalogs are to be found");
 	    XtManageChild (w);
@@ -407,7 +420,8 @@ um_create_form_w()
 		XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 		XtSetArg (args[n], XmNmarginHeight, 0); n++;
 		XtSetArg (args[n], XmNindicatorType, XmN_OF_MANY); n++;
-		w = XmCreateToggleButton (pd_w, op->name, args, n);
+		w = XmCreateToggleButton (pd_w, op->name ? op->name : "UMTB",
+								    args, n);
 		XtAddCallback (w, XmNvalueChangedCallback, um_option_cb, 
 						    (XtPointer)(op->flagp));
 		set_xmstring (w, XmNlabelString, op->label);
@@ -418,6 +432,8 @@ um_create_form_w()
 		if (op->tip)
 		    wtip (w, op->tip);
 		XtManageChild (w);
+		if (op->name)
+		    sr_reg (w, NULL, ucategory, 1);
 	    }
 
 	    /* add a separator */
@@ -493,6 +509,7 @@ um_create_form_w()
 	XtAddCallback (scale_w, XmNvalueChangedCallback, um_scale_cb, 0);
 	wtip (scale_w, "Zoom in and out");
 	XtManageChild (scale_w);
+	sr_reg (scale_w, NULL, ucategory, 0);
 
 	/* make the limiting mag scale widget.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -515,6 +532,7 @@ um_create_form_w()
 	XtAddCallback (limmag_w, XmNvalueChangedCallback, um_scale_cb, 0);
 	wtip (limmag_w, "Adjust the brightness and limiting magnitude");
 	XtManageChild (limmag_w);
+	sr_reg (limmag_w, NULL, ucategory, 0);
 
 	/* make a frame for the drawing area.
 	 * attach both top and bottom so it's the one to follow resizing.
@@ -584,7 +602,7 @@ um_create_usform_w()
 	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_ANY); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 5); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	usform_w = XmCreateFormDialog (toplevel_w, "UranusStats", args, n);
+	usform_w = XmCreateFormDialog (ushell_w, "UranusStats", args, n);
 	set_something (usform_w, XmNcolormap, (XtArgVal)xe_cm);
 
 	/* set some stuff in the parent DialogShell.
@@ -630,7 +648,7 @@ um_create_usform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "UTag", args, n); /*PB for sz */
 		set_xmstring (w, XmNlabelString, tag ? tag : " ");
-		um_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -656,7 +674,7 @@ um_create_usform_w()
 		n = 0;
 		w = XmCreatePushButton (rc_w, "UName", args, n); /*PB for sz*/
 		set_xmstring (w, XmNlabelString, mnames[i].full);
-		um_set_a_button (w, False);
+		buttonAsButton (w, False);
 		XtManageChild (w);
 	    }
 
@@ -741,7 +759,7 @@ um_create_usform_w()
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	XtSetArg (args[n], XmNrightPosition, 70); n++;
 	w = XmCreatePushButton (usform_w, "Close", args, n);
-	XtAddCallback (w, XmNactivateCallback, um_sstats_cb, 0);
+	XtAddCallback (w, XmNactivateCallback, um_sstats_close_cb, 0);
 	wtip (w, "Close this dialog");
 	XtManageChild (w);
 }
@@ -758,7 +776,7 @@ um_create_tvform_w()
 	XtSetArg (args[n], XmNautoUnmanage, False); n++;
 	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	utform_w = XmCreateFormDialog (toplevel_w, "UranusTV", args, n);
+	utform_w = XmCreateFormDialog (ushell_w, "UranusTV", args, n);
 	set_something (utform_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (utform_w, XmNunmapCallback, ut_unmap_cb, 0);
 	XtAddCallback (utform_w, XmNmapCallback, ut_map_cb, NULL);
@@ -797,87 +815,23 @@ int whether;	/* whether setting up for plotting or for not plotting */
 {
 	int i, j;
 
-	for (i = 0; i < NM; i++) {
+	for (i = 0; i < NM; i++)
 	    for (j = 0; j < CNum; j++)
-		um_set_a_button (u_w[i][j], whether);
-	}
+		buttonAsButton (u_w[i][j], whether);
 }
 
-/* set whether the given button looks like a label.
- */
+/* callback when the Close button is activated on the stats menu */
+/* ARGSUSED */
 static void
-um_set_a_button(pbw, whether)
-Widget pbw;
-int whether;
+um_sstats_close_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
 {
-	static Arg look_like_button[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) True},
-	    {XmNtraversalOn, (XtArgVal) True},
-	};
-	static Arg look_like_label[] = {
-	    {XmNtopShadowColor, (XtArgVal) 0},
-	    {XmNbottomShadowColor, (XtArgVal) 0},
-            {XmNtopShadowPixmap, (XtArgVal) 0},
-            {XmNbottomShadowPixmap, (XtArgVal) 0},
-	    {XmNfillOnArm, (XtArgVal) False},
-	    {XmNtraversalOn, (XtArgVal) False},
-	};
-	static int called;
-	Arg *ap;
-	int na;
-
-	if (!called) {
-	    /* get baseline label and shadow appearances.
-	     */
-            Pixel topshadcol, botshadcol, bgcol;
-            Pixmap topshadpm, botshadpm;
-	    Arg args[20];
-	    Widget tmpw;
-	    int n;
-
-	    n = 0;
-	    tmpw = XmCreatePushButton (uform_w, "tmp", args, n);
-
-	    n = 0;
-	    XtSetArg (args[n], XmNtopShadowColor, &topshadcol); n++;
-	    XtSetArg (args[n], XmNbottomShadowColor, &botshadcol); n++;
-            XtSetArg (args[n], XmNtopShadowPixmap, &topshadpm); n++;
-            XtSetArg (args[n], XmNbottomShadowPixmap, &botshadpm); n++;
-	    XtSetArg (args[n], XmNbackground, &bgcol); n++;
-	    XtGetValues (tmpw, args, n);
-
-            look_like_button[0].value = topshadcol;
-            look_like_button[1].value = botshadcol;
-            look_like_button[2].value = topshadpm;
-            look_like_button[3].value = botshadpm;
-            look_like_label[0].value = bgcol;
-            look_like_label[1].value = bgcol;
-            look_like_label[2].value = XmUNSPECIFIED_PIXMAP;
-            look_like_label[3].value = XmUNSPECIFIED_PIXMAP;
-
-	    XtDestroyWidget (tmpw);
-	     
-	    called = 1;
-	}
-
-	if (whether) {
-	    ap = look_like_button;
-	    na = XtNumber(look_like_button);
-	} else {
-	    ap = look_like_label;
-	    na = XtNumber(look_like_label);
-	}
-
-	XtSetValues (pbw, ap, na);
+	XtUnmanageChild (usform_w);
 }
 
-/* callback when the Close button is activated on the stats menu or when
- * the More Info button is activated. they each do the same thing.
- */
+/* callback when the More Info button is activated */
 /* ARGSUSED */
 static void
 um_sstats_cb (w, client, call)
@@ -885,12 +839,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((uswasman = XtIsManaged(usform_w)) != 0)
-	    XtUnmanageChild (usform_w);
-	else {
-	    XtManageChild (usform_w);
-	    um_set_buttons(um_selecting);
-	}
+	XtManageChild (usform_w);
+	um_set_buttons(um_selecting);
 }
 
 /* callback from any of the option buttons.
@@ -996,7 +946,7 @@ XtPointer call;
 	}
 
         XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!ut_pm || nx != last_nx || ny != last_ny) {
+	if (!ut_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (ut_pm)
 		XFreePixmap (dsp, ut_pm);
 	    ut_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1048,7 +998,7 @@ ut_track_size()
 	n = 0;
 	XtSetArg (args[n], XmNx, &mfx); n++;
 	XtSetArg (args[n], XmNy, &mfy); n++;
-	XtGetValues (uform_w, args, n);
+	XtGetValues (ushell_w, args, n);
 	n = 0;
 	XtSetArg (args[n], XmNx, &mdx); n++;
 	XtSetArg (args[n], XmNy, &mdy); n++;
@@ -1074,25 +1024,22 @@ XtPointer call;
 	XmToggleButtonSetState (topview_w, False, True);
 }
 
-/* callback when main dialog is unmapped */
+/* callback when main shell is popped down */
 /* ARGSUSED */
 static void
-um_unmap_cb (w, client, call)
+um_popdown_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	if ((uswasman = (usform_w && XtIsManaged(usform_w))) != 0)
-	    XtUnmanageChild (usform_w);
+	XtUnmanageChild (usform_w);
+	XtUnmanageChild (utform_w);
+
         if (um_pm) {
 	    XFreePixmap (XtDisplay(uda_w), um_pm);
 	    um_pm = (Pixmap) NULL;
 	}
 
-	/* close top view */
-	if (utform_w && XtIsManaged(utform_w))
-	    XtUnmanageChild (utform_w);
-
 	/* free any field stars */
 	if (fstars) {
 	    free ((void *)fstars);
@@ -1112,8 +1059,8 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	/* let unmap do all the work */
-	XtUnmanageChild (uform_w);
+	/* let popdown do all the work */
+	XtPopdown (ushell_w);
 }
 
 /* callback from the Movie button
@@ -1179,7 +1126,7 @@ XtPointer call;
 	}
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
-	if (!um_pm || nx != last_nx || ny != last_ny) {
+	if (!um_pm || (int)nx != last_nx || (int)ny != last_ny) {
 	    if (um_pm)
 		XFreePixmap (dsp, um_pm);
 	    um_pm = XCreatePixmap (dsp, win, nx, ny, d);
@@ -1484,10 +1431,10 @@ int how_much;
 	int i;
 
 	/* see if we should bother */
-	if (!uform_w)
+	if (!ushell_w)
 	    return;
 	wantstats = XtIsManaged(usform_w) || any_ison() || how_much;
-	if (!XtIsManaged(uform_w) && !wantstats)
+	if (!isUp(ushell_w) && !wantstats)
 	    return;
 
 	watch_cursor (1);
@@ -1523,7 +1470,7 @@ int how_much;
 	    timestamp (np, sdt_w);
 	}
 
-	if (XtIsManaged(uform_w)) {
+	if (isUp(ushell_w)) {
 	    um_draw_map (uda_w, uop, usize, md);
 	    timestamp (np, dt_w);
 	}
@@ -1531,6 +1478,18 @@ int how_much;
 	watch_cursor (0);
 }
 
+/* called when basic resources change.
+ * rebuild and redraw.
+ */
+void
+um_newres()
+{
+	if (!ushell_w)
+	    return;
+	make_gcs (uda_w);
+	um_update (mm_get_now(), 1);
+}
+
 /* given the loc of the moons, draw a nifty little picture.
  * also save resulting screen locs of everything in the screenobj array.
  * scale of the locations is in terms of Uranus radii == 1.
@@ -1634,8 +1593,8 @@ MoonData md[NM];
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_MAIN, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1656,7 +1615,7 @@ MoonData md[NM];
 	XFillRectangle (dsp, ut_pm, u_bgc, 0, 0, nx, ny);
 
 	/* draw Uranus in center with unit radius */
-	i = ew*64*raddeg(asin(sin(degrad(uop->s_elong))/uop->s_sdist));
+	i = (int)(ew*64*raddeg(asin(sin(degrad(uop->s_elong))/uop->s_sdist)));
 	XPSFillArc(dsp, ut_pm, u_fgc, nx/2-scale1, ny/2-scale1, 2*scale1-1,
 						    2*scale1-1, -i, -180*64);
 	reset_screenobj(SO_TOP);
@@ -1691,8 +1650,8 @@ MoonData md[NM];
 	    /* add object to list of screen objects drawn */
 	    (void) strcpy (o.o_name, mnames[i].full);
 	    set_smag (&o, mag);
-	    o.s_ra = md[i].ra;
-	    o.s_dec = md[i].dec;
+	    o.s_ra = (float)md[i].ra;
+	    o.s_dec = (float)md[i].dec;
 	    add_screenobj (SO_TOP, &o, x, y);
 
 	    if (tags && mnames[i].tag)
@@ -1722,12 +1681,13 @@ Widget w;
 {
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
+	XFontStruct *fsp;
 	XGCValues gcv;
 	unsigned int gcm;
 	Pixel fg, bg;
 
 	gcm = GCForeground;
-	get_something (w, XmNforeground, (XtArgVal)&fg);
+	get_color_resource (w, "uranusColor", &fg);
 	gcv.foreground = fg;
 	u_fgc = XCreateGC (dsp, win, gcm, &gcv);
 	u_fs = XQueryFont (dsp, XGContextFromGC (u_fgc));
@@ -1735,13 +1695,15 @@ Widget w;
 	u_ch = u_fs->max_bounds.ascent + u_fs->max_bounds.descent;
 
 	gcm = GCForeground;
-	get_something (w, XmNbackground, (XtArgVal)&bg);
+	get_color_resource (w, "UranusBackground", &bg);
 	gcv.foreground = bg;
 	u_bgc = XCreateGC (dsp, win, gcm, &gcv);
 
-	gcm = GCForeground | GCFunction;
+	gcm = GCForeground | GCFunction | GCFont;
+	fsp = getXResFont ("moonsFont");
 	gcv.foreground = fg ^ bg;
 	gcv.function = GXxor;
+	gcv.font = fsp->fid;
 	u_xgc = XCreateGC (dsp, win, gcm, &gcv);
 }
 
@@ -1760,10 +1722,17 @@ int l;
 {
 	int lx = xc - l/2;
 	int rx = lx + l;
+	int i;
 
 	XDrawLine (dsp, win, gc, lx, yc, rx, yc);
-	XDrawLine (dsp, win, gc, lx, yc-3, lx, yc+3);
-	XDrawLine (dsp, win, gc, rx, yc-3, rx, yc+3);
+
+	for (i = 0; i <= 6; i++) {
+	    int x = lx+i*l/6;
+	    int dy = 3 + ((i%6)?0:3);
+
+	    XDrawLine (dsp, win, gc, x, yc-dy, x, yc+dy);
+	}
+
 	XDrawString (dsp, win, gc, xc-tw/2, yc+th, tag, strlen(tag));
 }
 
@@ -1822,8 +1791,8 @@ int fliptb, fliplr;	/* flip direction; default is S up E right */
 	    ddec = dec0 - op->s_dec;	/* + S */
 	    if (fabs(ddec) > rad || delra(dra)*cdec0 > rad)
 		continue;
-	    dx = dra/scale;
-	    dy = ddec/scale;
+	    dx = (int)(dra/scale);
+	    dy = (int)(ddec/scale);
 	    x = fliplr ? (int)w/2-dx : (int)w/2+dx;
 	    y = fliptb ? (int)h/2+dy : (int)h/2-dy;
 
@@ -2002,4 +1971,4 @@ MoonData md[NM];        /* fill md[1..NM-1].x/y/z for each moon */
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: uranusmenu.c,v $ $Date: 1999/03/03 19:06:34 $ $Revision: 1.4 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: uranusmenu.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.17 $ $Name:  $"};
