@@ -69,18 +69,23 @@ int wantgsc;
 char msg[];
 {
 	char tstname[1024];
-	FILE *fp;
+	FILE *fp = NULL;
+	int n;
 
-	/* try a typical name */
-	sprintf (tstname, "%s/zone1275.acc", cdp);
-	fp = fopenh (tstname, "r");
-	if (fp == NULL) {
-	    sprintf (msg, "%s: %s", tstname, syserrstr());
+	/* look for any legal zone*.cat file (this allows for A CDs) */
+	for (n = 0; n <= 1725; n += 75) {
+	    (void) sprintf (tstname, "%s/zone%04d.cat", cdp, n);
+	    fp = fopenh (tstname, "r");
+	    if (fp)
+		break;
+	}
+	if (!fp) {
+	    sprintf (msg, "No zone files in %s: %s", cdp, syserrstr());
 	    return (-1);
 	}
 	fclose (fp);
 
-	/* store our own copy */
+	/* store our own copy of path */
 	if (cdpath)
 	    free (cdpath);
 	cdpath = malloc (strlen(cdp) + 1);
@@ -131,7 +136,7 @@ char msg[];	/* filled with error message if return -1 */
 	d0 = raddeg(d0);
 	rov = raddeg (fov)/2;
 	if (rov >= 7.5) {
-	    strcpy (msg, "Radius must be less than 7.5 degrees");
+	    strcpy (msg, "USNO FOV must be <= 15 degrees");
 	    return (-1);
 	}
 
@@ -335,11 +340,11 @@ crackCatBuf (UC buf[CATBPR], FieldStar *fsp)
 
 	/* first 4 bytes are packed RA, big-endian */
 	ra = BEUPACK(buf)/(100.0*3600.0*15.0);
-	fsp->ra = hrrad(ra);
+	fsp->ra = (float)hrrad(ra);
 
 	/* next 4 bytes are packed Dec, big-endian */
 	dec = BEUPACK(buf+4)/(100.0*3600.0) - 90.0;
-	fsp->dec = degrad(dec);
+	fsp->dec = (float)degrad(dec);
 
 	/* last 4 bytes are packed mag info -- can lead to rejection */
 	mag = BEUPACK(buf+8);
@@ -357,9 +362,9 @@ crackCatBuf (UC buf[CATBPR], FieldStar *fsp)
 	    if (blu > 250)
 		return (-1);
 	    else
-		fsp->mag = blu/10.0;
+		fsp->mag = (float)(blu/10.);
 	} else
-	    fsp->mag = red/10.0;
+	    fsp->mag = (float)(red/10.);
 
 	return (0);
 }
@@ -410,4 +415,4 @@ addGS (StarArray *sap, FieldStar *fsp)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: usno.c,v $ $Date: 1999/10/26 04:44:33 $ $Revision: 1.6 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: usno.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.10 $ $Name:  $"};
