@@ -13,6 +13,7 @@ static int worldpos P_((double xpix, double ypix, double xref, double yref,
 static int xypix P_((double xpos, double ypos, double xref, double yref, 
       double xrefpix, double yrefpix, double xinc, double yinc, double rot,
       char *type, double *xpix, double *ypix));
+static int setWCScache P_((FImage *fip));
 
 /* given an x/y location over the given image, return ra and dec, in rads.
  * we require the C* fields in the header.
@@ -24,35 +25,16 @@ FImage *fip;
 double x, y;
 double *rap, *decp;
 {
-	double xref;	/* x reference coordinate value (deg) */
-	double yref;	/* y reference coordinate value (deg) */
-	double xrefpix;	/* x reference pixel */
-	double yrefpix;	/* y reference pixel */
-	double xinc;	/* x coordinate increment (deg) */
-	double yinc;	/* y coordinate increment (deg) */
-	double rot;	/* rotation (deg)  (from N through E) */
-	char *type;	/* projection type code e.g. "-SIN" */
-	double xpos;	/* x (RA) coordinate (deg) */
-	double ypos;	/* y (dec) coordinate (deg) */
-	int i;
-	char typestr[128];
+	double xpos;		/* x (RA) coordinate (deg) */
+	double ypos;		/* y (dec) coordinate (deg) */
 
-	if (getRealFITS (fip, "CRVAL1", &xref) < 0) return (-1);
-	if (getRealFITS (fip, "CRVAL2", &yref) < 0) return (-1);
-	if (getRealFITS (fip, "CRPIX1", &xrefpix) < 0) return (-1);
-	if (getRealFITS (fip, "CRPIX2", &yrefpix) < 0) return (-1);
-	if (getRealFITS (fip, "CDELT1", &xinc) < 0) return (-1);
-	if (getRealFITS (fip, "CDELT2", &yinc) < 0) return (-1);
-	if (getRealFITS (fip, "CROTA2", &rot) < 0) return (-1);
-	if (getStringFITS (fip, "CTYPE1", typestr) < 0) return (-1);
-	if (strncmp (typestr, "RA--", 4)) return (-1);
-	type = typestr + 4;
+	if (setWCScache (fip) < 0)
+	    return (-1);
 
 	/* CRPIXn assume pixels are 1-based */
-	i = worldpos(x+1, y+1, xref, yref, xrefpix, yrefpix, xinc, yinc,
-					      rot, type, &xpos, &ypos);
-
-	if (i != 0) return (-1);
+	if (worldpos(x+1, y+1, fip->xref, fip->yref, fip->xrefpix, fip->yrefpix,
+		fip->xinc, fip->yinc, fip->rot, fip->type, &xpos, &ypos) < 0)
+	    return (-1);
 
 	*rap = degrad (xpos);
 	*decp = degrad (ypos);
@@ -70,36 +52,18 @@ FImage *fip;
 double ra, dec;
 double *xp, *yp;
 {
-	double xpos;	/* x (RA) coordinate (deg) */
-	double ypos;	/* y (dec) coordinate (deg) */
-	double xref;	/* x reference coordinate value (deg) */
-	double yref;	/* y reference coordinate value (deg) */
-	double xrefpix;	/* x reference pixel */
-	double yrefpix;	/* y reference pixel */
-	double xinc;	/* x coordinate increment (deg) */
-	double yinc;	/* y coordinate increment (deg) */
-	double rot;	/* rotation (deg)  (from N through E) */
-	char *type;	/* projection type code e.g. "-SIN" */
-	int i;
-	char typestr[128];
+	double xpos;		/* x (RA) coordinate (deg) */
+	double ypos;		/* y (dec) coordinate (deg) */
+
+	if (setWCScache (fip) < 0)
+	    return (-1);
 
 	xpos = raddeg(ra);
 	ypos = raddeg(dec);
-	if (getRealFITS (fip, "CRVAL1", &xref) < 0) return (-1);
-	if (getRealFITS (fip, "CRVAL2", &yref) < 0) return (-1);
-	if (getRealFITS (fip, "CRPIX1", &xrefpix) < 0) return (-1);
-	if (getRealFITS (fip, "CRPIX2", &yrefpix) < 0) return (-1);
-	if (getRealFITS (fip, "CDELT1", &xinc) < 0) return (-1);
-	if (getRealFITS (fip, "CDELT2", &yinc) < 0) return (-1);
-	if (getRealFITS (fip, "CROTA2", &rot) < 0) return (-1);
-	if (getStringFITS (fip, "CTYPE1", typestr) < 0) return (-1);
-	if (strncmp (typestr, "RA--", 4)) return (-1);
-	type = typestr + 4;
 
-	i = xypix(xpos, ypos, xref, yref, xrefpix, yrefpix, xinc, yinc,
-					      rot, type, xp, yp);
-
-	if (i != 0) return (-1);
+	if (xypix(xpos, ypos, fip->xref, fip->yref, fip->xrefpix, fip->yrefpix,
+			fip->xinc, fip->yinc, fip->rot, fip->type, xp, yp) < 0)
+	    return (-1);
 
 	/* CRPIXn assume pixels are 1-based */
 	*xp -= 1;
@@ -107,6 +71,34 @@ double *xp, *yp;
 
 	return (0);
 }
+
+/* load the WCS cache if not already set up.
+ * return 0 if ok, else -1
+ */
+static int
+setWCScache (fip)
+FImage *fip;
+{
+	FITSRow typestr;
+
+	if (fip->wcsset)
+	    return (0);
+
+	if (getRealFITS (fip, "CRVAL1", &fip->xref) < 0) return (-1);
+	if (getRealFITS (fip, "CRVAL2", &fip->yref) < 0) return (-1);
+	if (getRealFITS (fip, "CRPIX1", &fip->xrefpix) < 0) return (-1);
+	if (getRealFITS (fip, "CRPIX2", &fip->yrefpix) < 0) return (-1);
+	if (getRealFITS (fip, "CDELT1", &fip->xinc) < 0) return (-1);
+	if (getRealFITS (fip, "CDELT2", &fip->yinc) < 0) return (-1);
+	if (getRealFITS (fip, "CROTA2", &fip->rot) < 0) return (-1);
+	if (getStringFITS (fip, "CTYPE1", typestr) < 0) return (-1);
+	if (strncmp (typestr, "RA--", 4)) return (-1);
+	strncpy (fip->type, typestr+4, sizeof(fip->type)-1);
+
+	fip->wcsset = 1;
+	return (0);
+}
+
 
 /*  worldpos.c -- WCS Algorithms from Classic AIPS.
     Copyright (C) 1994
@@ -581,4 +573,4 @@ double *ypix;
 }  /* end xypix */
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: wcs.c,v $ $Date: 1999/10/25 21:01:55 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: wcs.c,v $ $Date: 2000/09/21 06:02:17 $ $Revision: 1.5 $ $Name:  $"};
