@@ -1,5 +1,5 @@
 /* main() for xephem.
- * Copyright (c) 1990-1999 by Elwood Charles Downey
+ * Copyright (c) 1990-2000 by Elwood Charles Downey
  * Permission is granted to make and distribute copies of this program free of
  * charge, provided the copyright notice and this permission notice are
  * preserved on all copies.  All other rights reserved.  No representation is
@@ -13,10 +13,12 @@
 #include <stdlib.h>
 #else
 extern char *malloc();
+extern char *getenv();
 #endif
 
 #include <X11/Xlib.h>
 #include <X11/IntrinsicP.h> /* for XT_REVISION */
+#include <X11/cursorfont.h>
 
 /* define WANT_EDITRES if want to try and support X11R5's EditRes feature.
  * this will require linking with -lXmu and -lXext too.
@@ -36,6 +38,7 @@ extern char *malloc();
 #include <Xm/CascadeB.h>
 #include <Xm/Form.h>
 #include <Xm/Separator.h>
+#include <Xm/MessageB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/ToggleB.h>
 #include <Xm/Label.h>
@@ -47,6 +50,7 @@ extern char *malloc();
 
 #include "P_.h"
 #include "patchlevel.h"
+#include "preferences.h"
 
 extern String fallbacks[];
 extern char Version_resource[];
@@ -58,14 +62,18 @@ extern FILE *fopenh P_((char *name, char *how));
 extern char *getShareDir P_((void));
 extern char *getXRes P_((char *name, char *def));
 extern char *syserrstr P_((void));
+extern char *userResFile P_((void));
 extern int confirm P_((void));
-extern int explodeGIF P_((unsigned char *rawpix, int nrawpix, int *wp,
-    int *hp, unsigned char *pixap[], unsigned char ra[], unsigned char ga[],
-    unsigned char ba[], char errmsg[]));
-extern void addXPath P_((void));
+extern int gif2pm P_((Display *dsp, Colormap cm, unsigned char gif[], int ngif,
+    int *wp, int *hp, Pixmap *pmp, char why[]));
+extern int sr_autosaveon P_((void));
+extern int sr_isUp P_((void));
+extern int sr_refresh P_((void));
+extern int sr_save P_((int talk));
+extern void av_manage P_((void));
 extern void c_manage P_((void));
 extern void db_manage P_((void));
-extern void dm_create_form P_((void));
+extern void dm_create_shell P_((void));
 extern void dm_manage P_((void));
 extern void e_manage P_((void));
 extern void fs_manage P_((void));
@@ -75,30 +83,36 @@ extern void jm_manage P_((void));
 extern void lst_manage P_((void));
 extern void m_manage P_((void));
 extern void mars_manage P_((void));
-extern void mm_external P_((void));
 extern void mm_create P_((Widget mainrc));
+extern void mm_external P_((void));
 extern void mm_go_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void mm_reset P_((void));
 extern void msg_manage P_((void));
 extern void net_create P_((void));
 extern void net_manage P_((void));
+extern void ng_manage P_((void));
 extern void obj_manage P_((void));
 extern void plot_manage P_((void));
+extern void pm_manage P_((void));
 extern void pref_create_pulldown P_((Widget mb_w));
 extern void query P_((Widget tw, char *msg, char *label1, char *label2,
     char *label3, void (*func1)(void), void (*func2)(void),
     void (*func3)(void)));
-extern void pm_manage P_((void));
+extern void sah_manage P_((void));
 extern void set_something P_((Widget w, char *resource, XtArgVal value));
 extern void set_xmstring P_((Widget w, char *resource, char *txt));
 extern void sm_manage P_((void));
-extern void um_manage P_((void));
+extern void sr_addFallbacks P_((void));
+extern void sr_xmanage P_((void));
 extern void srch_manage P_((void));
 extern void ss_manage P_((void));
 extern void sv_manage P_((void));
+extern void um_manage P_((void));
 extern void version P_((void));
 extern void watch_cursor P_((int want));
+extern void wt_manage P_((void));
 extern void wtip P_((Widget w, char *tip));
+extern void wtip_alldown P_((void));
+extern void xe_msg P_((char *msg, int app_modal));
 
 /* these are used to describe and semi-automate making the main pulldown menus
  */
@@ -127,6 +141,7 @@ typedef struct {
 #define	SEPARATOR_N	"MainSep"	/* special ButtonInfo->name */
 
 
+static void addOurDBs P_((void));
 static void chk_args P_((int argc, char *argv[]));
 static void chk_version P_((void));
 static void set_title P_((void));
@@ -136,26 +151,25 @@ static void setup_icon P_((void));
 static void m_activate_cb P_((Widget w, XtPointer client, XtPointer call));
 static void make_examples_pullright P_((Widget pulldown_w, Widget cascade_w));
 static void examples_cb P_((Widget w, XtPointer client, XtPointer call));
-static void alldone P_((void));
 static void x_quit P_((void));
 
 
 /* client arg to m_activate_cb().
  */
-enum {
-    PROGRESS, QUIT, XRESET, MSGTXT, NETSETUP, EXTIN,
+typedef enum {
+    PROGRESS, QUIT, MSGTXT, NETSETUP, EXTIN,
     DATA, MOON, EARTH, MARS, JUPMOON, SATMOON, UMOON, SKYVIEW, SOLARSYS,
-    PLOT, LIST, SEARCH,
-    DB, OBJS, FIELDSTARS, CLOSEOBJS, 
+    PLOT, LIST, SEARCH, GLANCE, AAVSO, SETIAH,
+    DB, OBJS, CLOSEOBJS, FIELDSTARS, WEBTLE,
     ABOUT, REFERENCES, INTRO, MAINMENU, FILEHLP, VIEWHLP, TOOLSHLP, OBJHLP,
-	PREFHLP, OPERATION, DATETIME, NOTES
-};
+	PREFHLP, OPERATION, CONTEXT, DATETIME, NOTES
+} MBOptions;
 
 Widget toplevel_w;
 #define	XtD	XtDisplay(toplevel_w)
 Colormap xe_cm;
 XtAppContext xe_app;
-static char myclass[] = "XEphem";
+char myclass[] = "XEphem";
 
 #define xephem_width 50
 #define xephem_height 50
@@ -202,14 +216,22 @@ main(argc, argv)
 int argc;
 char *argv[];
 {
-	/* add handy things to X search path */
-	addXPath();
+	Arg args[10];
+	int n;
 
 	/* set this before using fallbacks[] */
 	(void)sprintf (Version_resource,"%s.Version: %s", myclass, PATCHLEVEL);
 
+	/* open display and gather standard resources.
+	 * we grab fallbacks[] last
+	 */
+	n = 0;
+	XtSetArg (args[n], XmNallowShellResize, True); n++;
 	toplevel_w = XtAppInitialize (&xe_app, myclass, options,
-			    XtNumber(options), &argc, argv, fallbacks, NULL, 0);
+			    XtNumber(options), &argc, argv, NULL, args, n);
+
+	/* add our resources from non-standard places */
+	addOurDBs();
 
 	chk_args (argc, argv);
 	chk_version();
@@ -251,7 +273,7 @@ char *argv[];
 	/* define the data table, but don't manage it.
 	 * we do this up front to avoid the delay on its first use later.
 	 */
-	dm_create_form();
+	dm_create_shell();
 
 	/* here we go */
 	XtRealizeWidget(toplevel_w);
@@ -276,7 +298,37 @@ Cursor c;
 		XUndefineCursor (dsp, win);
 	}
 }
-	
+
+/* merge more resource files into final db; harmless if missing.
+ * finally add any fallbacks[] not already in db.
+ */
+static void
+addOurDBs()
+{
+	XrmDatabase dspdb = XtDatabase (XtD);
+	XrmDatabase fbdb = NULL;
+	char **pp, *p;
+
+	/* check in TELHOME */
+	if ((p = getenv("TELHOME")) != NULL) {
+	    char fullp[256];
+	    sprintf (fullp, "%s/archive/config/XEphem", p);
+	    XrmCombineFileDatabase(fullp, &dspdb, True);
+	}
+
+	/* then per-user's so it has max priority */
+	XrmCombineFileDatabase(userResFile(), &dspdb, True);
+
+	/* finally add any fallbacks[] not already known */
+	for (pp = fallbacks; (p = *pp++) != NULL; )
+	    XrmPutLineResource (&fbdb, p);	/* creates fbdb if first */
+	XrmCombineDatabase (fbdb, &dspdb, False);
+
+	/* Combine evidently uses pointers.. we die if we destroy fbdb
+	XrmDestroyDatabase (fbdb);
+	*/
+}
+
 /* ARGSUSED */
 static void
 chk_args (argc, argv)
@@ -303,10 +355,10 @@ char *argv[];
 static void
 chk_version()
 {
-	char *v = getXRes ("Version", NULL);
+	char *v = getXRes ("Version", "??");
 
-	if (!v || strcmp (v, PATCHLEVEL)) {
-	    printf ("X Resources out of date. Must be for %s\n", PATCHLEVEL);
+	if (strcmp (v, PATCHLEVEL)) {
+	    printf ("Version skew: Found %s but should be %s\n", v, PATCHLEVEL);
 	    exit (1);
 	}
 }
@@ -320,84 +372,24 @@ set_title()
 	set_something (toplevel_w, XmNtitle, (XtArgVal) title);
 }
 
-/* create an XImage of size wXh.
- * return XImage * if ok else NULL and xe_msg().
- */
-static XImage *
-create_xim (w, h)
-int w, h;
-{
-	Display *dsp = XtDisplay(toplevel_w);
-	XImage *xip;
-	int mdepth;
-	int mbpp;
-	int nbytes;
-	char *data;
-
-	/* establish depth and bits per pixel */
-	get_something (toplevel_w, XmNdepth, (XtArgVal)&mdepth);
-	if (mdepth < 8) {
-	    fprintf (stderr, "Require at least 8 bit pixel depth\n");
-	    return (NULL);
-	}
-	mbpp = mdepth>=17 ? 32 : (mdepth >= 9 ? 16 : 8);
-	nbytes = w*h*mbpp/8;
-
-	/* get memory for image pixels.  */
-	data = malloc (nbytes);
-	if (!data) {
-	    fprintf(stderr,"Can not get %d bytes for logo pixels", nbytes);
-	    return (NULL);
-	}
-
-	/* create the XImage */
-	xip = XCreateImage (dsp, DefaultVisual (dsp, DefaultScreen(dsp)),
-	    /* depth */         mdepth,
-	    /* format */        ZPixmap,
-	    /* offset */        0,
-	    /* data */          data,
-	    /* width */         w,
-	    /* height */        h,
-	    /* pad */           mbpp,
-	    /* bpl */           0);
-	if (!xip) {
-	    fprintf (stderr, "Can not create logo XImage\n");
-	    free ((void *)data);
-	    return (NULL);
-	}
-
-        xip->bitmap_bit_order = LSBFirst;
-	xip->byte_order = LSBFirst;
-
-	/* ok */
-	return (xip);
-}
-
 /* connect up logo.gif */
 static void
 make_logo (rc)
 Widget rc;
 {
 	Display *dsp = XtDisplay(toplevel_w);
-	Window win = RootWindow(dsp, DefaultScreen(dsp));
 	char fn[1024];
 	unsigned char gif[200000];
-	char buf[1024];
+	char why[1024];
 	Arg args[20];
-	unsigned char *gifpix;
-	unsigned char gifr[256], gifg[256], gifb[256];
-	unsigned long xpix[256];
 	Widget f_w, l_w;
-	XImage *xip;
 	Pixmap pm;
-	GC gc;
 	FILE *fp;
 	int w, h;
 	int ngif;
-	int i, n;
-	int x, y;
+	int n;
 
-	/* open and explode the gif */
+	/* open and read the gif */
 	(void) sprintf (fn, "%s/auxil/logo.gif",  getShareDir());
 	fp = fopenh (fn, "rb");
 	if (!fp) {
@@ -414,56 +406,13 @@ Widget rc;
 	    (void) fprintf (stderr, "%s: Max size = %d\n", fn, sizeof(gif));
 	    return; /* darn again */
 	}
-	n = explodeGIF(gif, ngif, &w, &h, &gifpix, gifr, gifg, gifb, buf);
-	if (n < 0) {
-	    (void) fprintf (stderr, "%s: %s\n", fn, buf);
+
+	/* convert to pm */
+	if (gif2pm (dsp, xe_cm, gif, ngif, &w, &h, &pm, why) < 0) {
+	    fprintf (stderr, "%s: %s\n", fn, why);
 	    return; /* darn again */
 	}
 
-	/* N.B. now must free gifpix */
-
-	/* allocate colors -- don't be too fussy */
-	for (i = 0; i < 256; i++) {
-	    XColor xc;
-	    xc.red   = ((gifr[i] & 0xf8) << 8) | 0x7ff;
-	    xc.green = ((gifg[i] & 0xf8) << 8) | 0x7ff;
-	    xc.blue  = ((gifb[i] & 0xf8) << 8) | 0x7ff;
-	    if (XAllocColor (dsp, xe_cm, &xc))
-		xpix[i] = xc.pixel;
-	    else {
-		fprintf (stderr, "Can not get all logo map colors\n");
-		free ((void *)gifpix);
-		if (i > 0)
-		    XFreeColors (dsp, xe_cm, xpix, i, 0);
-		return; /* darn again */
-	    }
-	}
-
-	/* create XImage */
-	xip = create_xim (w, h);
-	if (!xip) {
-	    XFreeColors (dsp, xe_cm, xpix, 256, 0);
-	    free ((void *)gifpix);
-	    return; /* darn again */
-	}
-
-	/* N.B. now must free xip */
-
-	/* fill XImage with logo */
-	for (y = 0; y < h; y++) {
-	    int yrow = y*w;
-	    for (x = 0; x < w; x++) {
-		int gp = (int)gifpix[x + yrow];
-		unsigned long p = xpix[gp];
-		XPutPixel (xip, x, y, p);
-	    }
-	}
-
-	/* create pixmap and fill with logo */
-	pm = XCreatePixmap (dsp, win, w, h, xip->depth);
-	gc = DefaultGC (dsp, DefaultScreen(dsp));
-	XPutImage (dsp, pm, gc, xip, 0, 0, 0, 0, w, h);
-
 	/* put pm in a label and let it handle it from now on.
 	 * put label in a form so it stretches clear across the rc
 	 */
@@ -482,12 +431,6 @@ Widget rc;
 	l_w = XmCreateLabel (f_w, "LogoLabel", args, n);
 	wtip (l_w, "The file <XEphem.ShareDir>/auxil/logo.gif");
 	XtManageChild (l_w);
-
-	/* free with gifpix and xip */
-	free ((void *)gifpix);
-	free ((void *)xip->data);
-	xip->data = NULL;
-	XDestroyImage (xip);
 }
 
 /* put together the menu bar, the main form, and fill in the form with the
@@ -497,9 +440,6 @@ static void
 make_main_window ()
 {
 	static ButtonInfo file_buttons[] = {
-	    {"Restore many Main values to original settings",
-		"Reset", "Reset", 0, 0, 'R', m_activate_cb, (XtPointer)XRESET},
-	    {NULL, SEPARATOR_N},
 	    {"Display a dialog containing supporting informational messages",
 	        "Messages", "Messages...", 0, 0, 'M', m_activate_cb,
 							    (XtPointer)MSGTXT},
@@ -512,8 +452,12 @@ make_main_window ()
 		"Progress", "Progress Meter...", 0, 0, 'P', m_activate_cb,
 							   (XtPointer)PROGRESS},
 	    {NULL, SEPARATOR_N},
-	    {"Start and stop the main execution loop",
-		"Update", "Update", 0, 0, 'U', mm_go_cb, (XtPointer)NULL},
+	    {"Run or stop the main execution loop",
+		"Update", "Update", "Ctrl<Key>u", "Ctrl+u", 'U',
+						    mm_go_cb, (XtPointer)0},
+	    {"Run in reverse or stop the main execution loop",
+		"Reverse", "Update in reverse", "Ctrl<Key>r", "Ctrl+r", 'r',
+						    mm_go_cb, (XtPointer)1},
 	    {NULL, SEPARATOR_N},
 	    {"Exit xephem",
 		"Quit", "Quit...", "Ctrl<Key>d", "Ctrl+d", 'Q', m_activate_cb,
@@ -556,26 +500,41 @@ make_main_window ()
 							    (XtPointer)LIST},
 	    {"Define an equation of XEphem fields and solve for min, max or 0",
 		"Solve", "Solve equation...", 0, 0, 'S', m_activate_cb,
-							    (XtPointer)SEARCH}
+							    (XtPointer)SEARCH},
+	    {"Find all pairs of close objects in memory",
+		"CloseObjs", "Find close pairs...", 0, 0, 'c',
+					m_activate_cb, (XtPointer)CLOSEOBJS},
+	    {"Show twilight and all basic objects on a 24 hour timeline map",
+		"Glance", "Night at a glance...", 0, 0, 'N',
+					    m_activate_cb, (XtPointer)GLANCE},
+	    {"Fetch light curves from AAVSO",
+		"AAVSO", "AAVSO light curves...", 0, 0, 'A', m_activate_cb,
+							    (XtPointer)AAVSO},
+	    {"Monitor local setiathome client",
+		"SetiAtHome", "Seti@Home...", 0, 0, 'H', m_activate_cb,
+							    (XtPointer)SETIAH},
 	};
 	static ButtonInfo objs_buttons[] = {
 	    {"Load .edb files into memory, list current memory, or delete",
-		"DataBase", "Load and delete objects in memory...",0, 0, 'D',
+		"DataBase", "Load/Delete...", 0, 0, 'D',
 						m_activate_cb, (XtPointer)DB},
 	    {"Search memory for an object, view definitions, assign ObjXYZ",
-		"ObjX_Y", "Search memory and define ObjX,Y,Z...", 0, 0,'S',
+		"ObjXYZ", "Search, ObjX,Y,Z...", 0, 0,'S',
 					    m_activate_cb, (XtPointer)OBJS},
 	    {"Setup field star options and catalog locations",
-		"FieldStars", "Setup field stars...", 0, 0, 'F',
+		"FieldStars", "Field stars...", 0, 0, 'F',
 					m_activate_cb, (XtPointer)FIELDSTARS},
-	    {"Find all pairs of close objects in memory",
-		"CloseObjs", "Find close pairs of objects...", 0, 0, 'C',
-					m_activate_cb, (XtPointer)CLOSEOBJS}
+	    {"Get new Earth Satellite TLE orbital elements from the Web",
+		"WebTLE", "Update Earth Satellites...", 0, 0, 'U',
+					m_activate_cb, (XtPointer)WEBTLE},
 	};
 	static ButtonInfo help_buttons[] = {
 	    {"Overall features of xephem",
 		"Introduction", "Introduction...", 0, 0, 'I', m_activate_cb,
 							    (XtPointer)INTRO},
+	    {"Click here then roam over controls to see help tips",
+		"onContext", "on Context", 0, 0, 'x', m_activate_cb,
+							(XtPointer)CONTEXT},
 	    {"How to control xephem's running behavior, including looping",
 		"onOperation", "on Operation...", 0, 0, 'e', m_activate_cb,
 							(XtPointer)OPERATION},
@@ -584,7 +543,7 @@ make_main_window ()
 							(XtPointer)DATETIME},
 	    {NULL, SEPARATOR_N},
 	    {"Description of the overall Main xephem menu",
-		"onMainMenu", "on Main Menu...", 0, 0, 'M', m_activate_cb,
+		"onMainMenu", "on Main Window...", 0, 0, 'M', m_activate_cb,
 							(XtPointer)MAINMENU},
 	    {"Description of the options available under File",
 		"onFile", "on File...", 0, 0, 'F', m_activate_cb,
@@ -595,8 +554,8 @@ make_main_window ()
 	    {"Description of the options available under Tools",
 		"onTools", "on Tools...", 0, 0, 'T', m_activate_cb,
 							(XtPointer)TOOLSHLP},
-	    {"Description of the options available under Objects",
-		"onObjects", "on Objects...", 0, 0, 'O', m_activate_cb,
+	    {"Description of the options available under Data",
+		"onObjects", "on Data...", 0, 0, 'D', m_activate_cb,
 							(XtPointer)OBJHLP},
 	    {"Description of the options available under Preferences",
 		"onPrefs", "on Preferences...", 0, 0, 'P', m_activate_cb,
@@ -622,7 +581,7 @@ make_main_window ()
 		"Tools", 0,'T',"ctrl_pd",ctrl_buttons,XtNumber(ctrl_buttons)};
 	static PullDownMenu objs_pd =
 	    {"Add, delete and inspect objects in memory",
-		"Objects", 0,'O',"objs_pd",objs_buttons,XtNumber(objs_buttons)};
+		"Data",0,'D',"objs_pd",objs_buttons,XtNumber(objs_buttons)};
 	static PullDownMenu help_pd =
 	    {"Additional information about xephem and the Main display",
 		"Help", 0, 'H', "help_pd", help_buttons,XtNumber(help_buttons)};
@@ -645,7 +604,7 @@ make_main_window ()
 	/*	Create MenuBar in mainrc  */
 
 	n = 0;
-	mb_w = XmCreateMenuBar (mainrc, "MainMB", args, n); 
+	mb_w = XmCreateMenuBar (mainrc, "MB", args, n); 
 	XtManageChild (mb_w);
 
 	    /* create each pulldown */
@@ -769,7 +728,61 @@ setup_icon ()
 	set_something (toplevel_w, XmNiconPixmap, (XtArgVal)pm);
 }
 
+static void
+hlp_onContext()
+{
+	static Cursor qc;
+	Display *dsp = XtDisplay (toplevel_w);
+	Window root = RootWindow(dsp, DefaultScreen(dsp));
+	Window rw, cw;
+	int oldtips;
+	int rx, ry;
+	unsigned int m;
+	int x, y;
 
+	/* make query cursor */
+	if (!qc)
+	    qc = XCreateFontCursor (dsp, XC_question_arrow);
+
+	/* grab pointer and allow tips to work until press any button */
+	if (XGrabPointer (dsp, root, True, ButtonPressMask, GrabModeAsync,
+		    GrabModeSync, None, qc, CurrentTime) != GrabSuccess) {
+	    xe_msg ("Could not grab pointer", 1);
+	    return;
+	}
+	oldtips = pref_get (PREF_TIPS);
+	pref_set (PREF_TIPS, PREF_TIPSON);
+	do {
+	    if (!XQueryPointer (dsp, root, &rw, &cw, &rx, &ry, &x, &y, &m)){
+		xe_msg ("XQueryPointer error", 1);
+		break;
+	    }
+
+	    while (XtAppPending(xe_app)) {
+		XEvent e;
+		XtAppNextEvent (xe_app, &e);
+		switch (e.type) {
+		case EnterNotify:
+		case LeaveNotify:
+		case Expose:
+		    XtDispatchEvent (&e);
+		    break;
+		}
+	    }
+	} while (!(m & Button1Mask));
+
+	XUngrabPointer (dsp, CurrentTime);
+
+	/* restore to Preference */
+	if (oldtips == PREF_NOTIPS)
+	    wtip_alldown();
+	pref_set (PREF_TIPS, oldtips);
+}
+
+
+/* main menubar controls callback.
+ * client is one of MBOptions.
+ */
 /* ARGSUSED */
 static void
 m_activate_cb (w, client, call)
@@ -782,7 +795,6 @@ XtPointer call;
 	watch_cursor(1);
 
 	switch (code) {
-	case XRESET:	mm_reset(); break;
 	case MSGTXT:	msg_manage(); break;
 	case NETSETUP:	net_manage(); break;
 	case EXTIN:	mm_external(); break;
@@ -800,10 +812,14 @@ XtPointer call;
 	case PLOT:	plot_manage(); break;
 	case LIST:	lst_manage(); break;
 	case SEARCH:	srch_manage(); break;
+	case GLANCE:	ng_manage(); break;
+	case AAVSO:	av_manage(); break;
+	case SETIAH:	sah_manage(); break;
 	case OBJS:	obj_manage(); break;
 	case DB:	db_manage(); break;
 	case CLOSEOBJS:	c_manage(); break;
 	case FIELDSTARS:fs_manage(); break;
+	case WEBTLE:	wt_manage(); break;
 	case ABOUT:	version(); break;
 	case REFERENCES:hlp_dialog ("Credits", NULL, 0); break;
 	case INTRO:	hlp_dialog ("Intro", NULL, 0); break;
@@ -814,6 +830,7 @@ XtPointer call;
 	case OBJHLP:	hlp_dialog ("MainMenu -- objects", NULL, 0); break;
 	case PREFHLP:	hlp_dialog ("MainMenu -- preferences", NULL, 0); break;
 	case OPERATION:	hlp_dialog ("Operation", NULL, 0); break;
+	case CONTEXT:	hlp_onContext (); break;
 	case DATETIME:	hlp_dialog ("Date/time", NULL, 0); break;
 	case NOTES:	hlp_dialog ("Notes", NULL, 0); break;
 	default: 	printf ("Main menu bug: code=%d\n", code); exit(1);
@@ -849,6 +866,8 @@ Widget cascade_w;
 		"Ring crossing",	 "Example - ring plane"},
 	    {"Display the moon overtaking a star",
 		"Lunar occultation","Example - lunar occultation"},
+	    {"Find next viewing opportunity for ISS and show sky and ground tracks",
+		"International Space Station", "Example - ISS"},
 	};
 	Widget pd_w;
 	Widget w;
@@ -894,7 +913,7 @@ XtPointer call;
 
 /* outta here */
 static void
-alldone()
+goodbye()
 {
 	XtCloseDisplay (XtDisplay (toplevel_w));
 	exit(0);
@@ -904,14 +923,24 @@ alldone()
 static void
 x_quit()
 {
-	if (confirm())
-	    query (toplevel_w, "Quit xephem?", "Yes -- quit", "No -- resume", 0,
-								alldone, 0, 0);
-	else
-	    alldone();
+	int nchg = sr_refresh();
 
-	/* if this function returns, we resume */
+	if (confirm()) {
+	    if (!nchg || (sr_autosaveon() && !sr_save(0)) || sr_isUp()) {
+		query (toplevel_w, "Exit xephem?",
+					"  Yes  ",  "  No  ", 0,
+				        goodbye,    0,        0);
+	    } else  {
+		query (toplevel_w, "Review modified resources before exiting?",
+					"  Yes  ",  "  No  ", 0,
+					sr_xmanage, goodbye,  0);
+	    }
+	} else {
+	    if (nchg > 0 && sr_autosaveon() && sr_save(0) < 0)
+		return;	/* abort exit if save failed */
+	    goodbye();
+	}
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xephem.c,v $ $Date: 1999/02/28 05:05:38 $ $Revision: 1.9 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xephem.c,v $ $Date: 2000/12/03 08:33:51 $ $Revision: 1.42 $ $Name:  $"};
