@@ -54,6 +54,9 @@ extern void hadec_aa P_((double lat, double ha, double dec, double *alt,
 extern void ab_ecl P_((double mjd, double lsn, double *lam, double *bet));
 extern void ab_eq P_((double mjd, double lsn, double *ra, double *dec));
 
+/* airmass.c */
+extern void airmass P_((double aa, double *Xp));
+
 /* anomaly.c */
 extern void anomaly P_((double ma, double s, double *nu, double *ea));
 
@@ -114,10 +117,12 @@ extern double delra P_((double dra));
 extern void cal_mjd P_((int mn, double dy, int yr, double *mjd));
 extern void mjd_cal P_((double mjd, int *mn, double *dy, int *yr));
 extern int mjd_dow P_((double mjd, int *dow));
+extern int isleapyear P_((int year));
 extern void mjd_dpm P_((double mjd, int *ndays));
 extern void mjd_year P_((double mjd, double *yr));
 extern void year_mjd P_((double y, double *mjd));
 extern void rnd_second P_((double *t));
+extern void mjd_dayno P_((double jd, int *yr, double *dy));
 extern double mjd_day P_((double jd));
 extern double mjd_hr P_((double jd));
 extern void range P_((double *v, double r));
@@ -177,5 +182,5 @@ extern void gst_utc P_((double mjd, double gst, double *utc));
 extern int vsop87 P_((double mjd, int obj, double prec, double *ret));
 
 /* For RCS Only -- Do Not Edit
- * @(#) $RCSfile: astro.h,v $ $Date: 1998/02/26 17:56:24 $ $Revision: 1.1 $ $Name:  $
+ * @(#) $RCSfile: astro.h,v $ $Date: 2000/09/25 16:41:43 $ $Revision: 1.3 $ $Name:  $
  */
