@@ -31,6 +31,7 @@ static void cir_pos P_((Now *np, double bet, double lam, double *rho, Obj *op));
 static void elongation P_((double lam, double bet, double lsn, double *el));
 static void deflect P_((double mjd1, double lpd, double psi, double rsn,
     double lsn, double rho, double *ra, double *dec));
+static double h_albsize P_((double H));
 
 /* given a Now and an Obj, fill in the approprirate s_* fields within Obj.
  * return 0 if all ok, else -1.
@@ -93,7 +94,7 @@ Obj *op;
 	/* compute magnitude and angular size */
 	f = op->s_phase ? 5*log10(rp*rho) - 5*log10(op->s_phase/100) : 100;
 	set_smag (op, mag+f);
-	op->s_size = dia/rho;
+	op->s_size = (float)(dia/rho);
 
 	return (0);
 }
@@ -117,11 +118,11 @@ Obj *op;
 	     * user is likely to stick with this for a while.
 	     */
 	    double tra = op->f_RA, tdec = op->f_dec;
-	    float tepoch = epoch;	/* compare to float precision */
+	    float tepoch = (float)epoch;	/* compare w/float precision */
 	    precess (op->f_epoch, tepoch, &tra, &tdec);
 	    op->f_epoch = tepoch;
-	    op->f_RA = tra;
-	    op->f_dec = tdec;
+	    op->f_RA = (float)tra;
+	    op->f_dec = (float)tdec;
 	}
 
 	/* set ra/dec to astrometric @ epoch of date */
@@ -145,13 +146,13 @@ Obj *op;
 	 */
 	nut_eq(mjd, &ra, &dec);
 	ab_eq(mjd, lsn, &ra, &dec);
-	op->s_gaera = ra;
-	op->s_gaedec = dec;
+	op->s_gaera = (float)ra;
+	op->s_gaedec = (float)dec;
 
 	/* set s_ra/dec -- apparent if EOD else astrometric */
 	if (epoch == EOD) {
-	    op->s_ra = ra;
-	    op->s_dec = dec;
+	    op->s_ra = (float)ra;
+	    op->s_dec = (float)dec;
 	} else {
 	    /* annual parallax at time mjd is to be added here, too, but
 	     * technically in the frame of epoch (usually different from mjd)
@@ -163,7 +164,7 @@ Obj *op;
 	/* compute elongation from ecliptic long/lat and sun geocentric long */
 	elongation (lam, bet, lsn, &el);
 	el = raddeg(el);
-	op->s_elong = el;
+	op->s_elong = (float)el;
 
 	/* these are really the same fields ...
 	op->s_mag = op->f_mag;
@@ -272,13 +273,19 @@ Obj *op;
 	if (op->e_mag.whichm == MAG_HG) {
 	    /* the H and G parameters from the Astro. Almanac.
 	     */
-	    hg_mag (op->e_mag.m1, op->e_mag.m2, rp, rho, rsn, &mag);
+	    if (op->e_size)
+		op->s_size = (float)(op->e_size / rho);
+	    else {
+		hg_mag (op->e_mag.m1, op->e_mag.m2, rp, rho, rsn, &mag);
+		op->s_size = (float)(h_albsize (op->e_mag.m1)/rho);
+
+	    }
 	} else {
 	    /* the g/k model of comets */
 	    gk_mag (op->e_mag.m1, op->e_mag.m2, rp, rho, &mag);
+	    op->s_size = (float)(op->e_size / rho);
 	}
 	set_smag (op, mag);
-	op->s_size = op->e_size / rho;
 
 	return (0);
 }
@@ -372,7 +379,7 @@ Obj *op;
 	/* compute magnitude and size */
 	gk_mag (op->h_g, op->h_k, rp, rho, &mag);
 	set_smag (op, mag);
-	op->s_size = op->h_size / rho;
+	op->s_size = (float)(op->h_size / rho);
 
 	return (0);
 }
@@ -412,7 +419,7 @@ Obj *op;
 	/* compute magnitude and size */
 	gk_mag (op->p_g, op->p_k, rp, rho, &mag);
 	set_smag (op, mag);
-	op->s_size = op->p_size / rho;
+	op->s_size = (float)(op->p_size / rho);
 
 	return (0);
 }
@@ -436,13 +443,13 @@ Obj *op;
 	set_smag (op, -26.8);	/* TODO */
 	dhlong = lsn-PI;	/* geo- to helio- centric */
 	range (&dhlong, 2*PI);
-	op->s_hlong = dhlong;
-	op->s_hlat = -bsn;
+	op->s_hlong = (float)dhlong;
+	op->s_hlat = (float)(-bsn);
 
 	/* fill sun's ra/dec, alt/az in op */
 	cir_pos (np, bsn, lsn, &rsn, op);
-	op->s_edist = rsn;
-	op->s_size = raddeg(4.65242e-3/rsn)*3600*2;
+	op->s_edist = (float)rsn;
+	op->s_size = (float)(raddeg(4.65242e-3/rsn)*3600*2);
 
 	return (0);
 }
@@ -466,28 +473,29 @@ Obj *op;
 	moon (mjed, &lam, &bet, &edistau, &ms, &md);	/* mean ecliptic & EOD*/
 	sunpos (mjed, &lsn, &rsn, NULL);		/* mean ecliptic & EOD*/
 
-	op->s_hlong = lam;			/* save geo in helio fields */
-	op->s_hlat = bet;
+	op->s_hlong = (float)lam;		/* save geo in helio fields */
+	op->s_hlat = (float)bet;
 
 	/* find angular separation from sun */
 	elongation (lam, bet, lsn, &el);
-	op->s_elong = raddeg(el);		/* want degrees */
+	op->s_elong = (float)raddeg(el);		/* want degrees */
 
 	/* solve triangle of earth, sun, and elongation for moon-sun dist */
-	op->s_sdist= sqrt (edistau*edistau + rsn*rsn - 2.0*edistau*rsn*cos(el));
+	op->s_sdist = (float) sqrt (edistau*edistau + rsn*rsn
+						    - 2.0*edistau*rsn*cos(el));
 
 	/* TODO: improve mag; this is based on a flat moon model. */
 	set_smag (op, -12.7 + 2.5*(log10(PI) - log10(PI/2*(1+1.e-6-cos(el)))));
 
 	/* find phase -- allow for projection effects */
 	i = 0.1468*sin(el)*(1 - 0.0549*sin(md))/(1 - 0.0167*sin(ms));
-	op->s_phase = (1+cos(PI-el-degrad(i)))/2*100;
+	op->s_phase = (float)((1+cos(PI-el-degrad(i)))/2*100);
 
 	/* fill moon's ra/dec, alt/az in op and update for topo dist */
 	cir_pos (np, bet, lam, &edistau, op);
 
-	op->s_edist = edistau;
-	op->s_size = 3600*2.0*raddeg(asin(MRAD/MAU/edistau));
+	op->s_edist = (float)edistau;
+	op->s_size = (float)(3600*2.0*raddeg(asin(MRAD/MAU/edistau)));
 						/* moon angular dia, seconds */
 
 	return (0);
@@ -512,20 +520,20 @@ Obj *op;
 	/* compute elongation and phase */
 	elongation (lam, bet, lsn, &el);
 	el = raddeg(el);
-	op->s_elong = el;
+	op->s_elong = (float)el;
 	f = 0.25 * ((rp+ *rho)*(rp+ *rho) - rsn*rsn)/(rp* *rho);
-	op->s_phase = f*100.0; /* percent */
+	op->s_phase = (float)(f*100.0); /* percent */
 
 	/* set heliocentric long/lat; mean ecliptic and EOD */
-	op->s_hlong = lpd;
-	op->s_hlat = psi;
+	op->s_hlong = (float)lpd;
+	op->s_hlat = (float)psi;
 
 	/* fill solar sys body's ra/dec, alt/az in op */
 	cir_pos (np, bet, lam, rho, op);        /* updates rho */
 
 	/* set earth/planet and sun/planet distance */
-	op->s_edist = *rho;
-	op->s_sdist = rp;
+	op->s_edist = (float)(*rho);
+	op->s_sdist = (float)rp;
 }
 
 /* fill equatoreal and horizontal op-> fields; stern
@@ -588,9 +596,9 @@ Obj *op;	/* object to set s_ra/dec as per epoch */
 	/* correct ra/dec to form geocentric apparent */
 	nut_eq (mjd, &ra, &dec);
 	if (!is_planet(op,MOON))
-		ab_eq (mjd, lsn, &ra, &dec);
-	op->s_gaera = ra;
-	op->s_gaedec = dec;
+	    ab_eq (mjd, lsn, &ra, &dec);
+	op->s_gaera = (float)ra;
+	op->s_gaedec = (float)dec;
 
 	/* find parallax correction for equatoreal coords */
 	now_lst (np, &lst);
@@ -629,8 +637,8 @@ Obj *op;	/* object to set s_ra/dec as per epoch */
 	    precess (mjd, epoch, &ra, &dec);
 	}
 	range(&ra, 2*PI);
-	op->s_ra = ra;
-	op->s_dec = dec;
+	op->s_ra = (float)ra;
+	op->s_dec = (float)dec;
 }
 
 /* given geocentric ecliptic longitude and latitude, lam and bet, of some object
@@ -734,5 +742,16 @@ double *ra, *dec;	/* geocentric equatoreal */
 	cartsph(u[0], u[1], u[2], ra, dec, &rho);	/* rho thrown away */
 }
 
+/* estimate size in arc seconds @ 1AU from absolute magnitude, H, and assuming
+ * an albedo of 0.1. With this assumption an object with diameter of 1500m
+ * has an absolute mag of 18.
+ */
+static double
+h_albsize (H)
+double H;
+{
+	return (3600*raddeg(.707*1500*pow(2.51,(18-H)/2)/MAU));
+}
+
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: circum.c,v $ $Date: 1999/02/15 21:42:18 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: circum.c,v $ $Date: 2000/10/07 05:12:17 $ $Revision: 1.4 $ $Name:  $"};
