@@ -32,6 +32,9 @@ int db_chk_planet P_((char name[], Obj *op));
 static int line_candidate P_((char *buf));
 static void crack_year P_((char *bp, PrefDateFormat pref, double *p));
 static int db_get_field P_((Obj *op, int id, char *bp));
+static int tle_sum P_((char *l));
+static double tle_fld P_((char *l, int from, int thru));
+static double tle_expfld P_((char *l, int start));
 
 /* crack the given .edb database line into op.
  * if ok
@@ -160,15 +163,17 @@ char whynot[];
 	case 'E': {
 	    static int ids[] = {ES_EPOCH,ES_INC,ES_RAAN,ES_E,ES_AP,ES_M,ES_N,
 							    ES_DECAY,ES_ORBIT};
-	    if (nf != 11) {
+	    if (nf != 11 && nf != 12) {
 		if (whynot)
-		    sprintf (whynot, "E needs 11 fields: %d", nf);
+		    sprintf (whynot, "E needs 11 or 12 fields: %d", nf);
 		return (-1);
 	    }
 	    zero_mem ((void *)op, sizeof(ObjES));
 	    op->o_type = EARTHSAT;
 	    for (i = 2; i < ASIZ(ids)+2; i++)
 		(void) db_set_field (flds[i], ids[i-2], PREF_MDY, op);
+	    if (nf == 12)
+		(void) db_set_field (flds[11], ES_DRAG, PREF_MDY, op);
 	    break;
 	}
 
@@ -193,6 +198,132 @@ char whynot[];
 	return (0);
 }
 
+/* return 0 if TLE checksum is ok, else -1 */
+static int
+tle_sum (l)
+char *l;
+{
+	char *lastl = l + 68;
+	int sum;
+
+	for (sum = 0; l < lastl; ) {
+	    char c = *l++;
+	    if (c == '\0')
+		return (-1);
+	    if (isdigit(c))
+		sum += c - '0';
+	    else if (c == '-')
+		sum++;
+	}
+
+	return (*l - '0' == (sum%10) ? 0 : -1);
+}
+
+/* extract the given columns and return value.
+ * N.B. from and to are 1-based within l
+ */
+static double
+tle_fld (l, from, thru)
+char *l;
+int from, thru;
+{
+	char buf[32];
+
+	sprintf (buf, "%.*s", thru-from+1, l+from-1);
+	return (atod (buf));
+}
+
+/* extract the exponential value starting at the given column.
+ * N.B. start is 1-based within l
+ */
+static double
+tle_expfld (l, start)
+char *l;
+int start;
+{
+	char buf[32];
+	double v;
+
+	sprintf (buf, ".%.*s", 5, l+start);
+	v = atod (buf) * pow (10.0, tle_fld(l, start+6, start+7));
+	if (l[start-1] == '-')
+	    v = -v;
+	return (v);
+}
+
+/* given 3 lines, first of which is name and next 2 are TLE, fill op.
+ * we skip leading whitespace on all lines.
+ * we do /not/ assume the 2 TLE lines are 0 terminated, but we do reach out into
+ *   each as far as 69 chars.
+ * we detect nonconformance as efficiently as possible.
+ * name ends at first '\0', '\r' or '\n'.
+ * if ok return 0 else return -1
+ */
+int
+db_tle (name, l1, l2, op)
+char *name, *l1, *l2;
+Obj *op;
+{
+	double ep;
+	int i;
+
+	/* check for correct line numbers, macthing satellite numbers and
+	 * correct checksums.
+	 */
+	while (isspace(*l1))
+	    l1++;
+	if (*l1 != '1')
+	    return (-1);
+	while (isspace(*l2))
+	    l2++;
+	if (*l2 != '2')
+	    return (-1);
+	if (strncmp (l1+2, l2+2, 5))
+	    return (-1);
+	if (tle_sum (l1) < 0)
+	    return (-1);
+	if (tle_sum (l2) < 0)
+	    return (-1);
+
+	/* assume it's ok from here out */
+
+	/* fresh */
+	zero_mem ((void *)op, sizeof(ObjES));
+	op->o_type = EARTHSAT;
+
+	/* name, sans leading and trailing whitespace */
+	while (isspace(*name))
+	    name++;
+	i = strcspn (name, "\r\n");
+	while (i > 0 && name[i-1] == ' ')
+	    --i;
+	if (i == 0)
+	    return (-1);
+	if (i > MAXNM-1)
+	    i = MAXNM-1;
+	sprintf (op->o_name, "%.*s", i, name);
+
+	/* goodies from "line 1" */
+	op->es_drag = (float) tle_expfld (l1, 54);
+	i = (int) tle_fld (l1, 19, 20);
+	if (i < 57)
+	    i += 100;
+	cal_mjd (1, tle_fld(l1, 21, 32), i+1900, &ep);
+	op->es_epoch = ep;
+
+	/* goodies from "line 2" */
+	op->es_n = tle_fld (l2, 53, 63);
+	op->es_inc = (float)tle_fld (l2, 9, 16);
+	op->es_raan = (float)tle_fld (l2, 18, 25);
+	op->es_e = (float)(tle_fld (l2, 27, 33) * 1e-7);
+	op->es_ap = (float)tle_fld (l2, 35, 42);
+	op->es_M = (float)tle_fld (l2, 44, 51);
+	op->es_orbit = (int)tle_fld (l2, 64, 68);
+
+	/* yes! */
+	return (0);
+}
+
 /* write the given Obj in .edb format to lp[].
  * we do _not_ include a trailing '\n'.
  */
@@ -201,8 +332,14 @@ db_write_line (op, lp)
 Obj *op;
 char *lp;
 {
+	int priorpref;
 	int i;
 
+	/* .edb format always uses MDY.
+	 * N.B. must restore old value before returning from here!
+	 */
+	priorpref = pref_set (PREF_DATE_FORMAT, PREF_MDY);
+
 	switch (op->o_type) {
 	case FIXED: {
 	    static int ids[] = {F_CLASS, F_SPECT, F_RA, F_DEC, F_MAG, F_EPOCH,
@@ -254,7 +391,7 @@ char *lp;
 
 	case EARTHSAT: {
 	    static int ids[] = {ES_EPOCH, ES_INC, ES_RAAN, ES_E, ES_AP, ES_M,
-		ES_N, ES_DECAY,ES_ORBIT
+		ES_N, ES_DECAY,ES_ORBIT,ES_DRAG
 	    };
 
 	    sprintf (lp, "%s,E", op->o_name);
@@ -273,6 +410,9 @@ char *lp;
 	    printf ("Unknown type for %s: %d\n", op->o_name, op->o_type);
 	    exit(1);
 	}
+
+	/* restore date format preference */
+	(void) pref_set (PREF_DATE_FORMAT, priorpref);
 }
 
 /* given a text buffer and a field id, and a PREF_DATE_FORMAT,
@@ -302,24 +442,24 @@ Obj *op;
 	    break;
 	case F_RA:
 	    f_scansex (radhr(op->f_RA), bp, &tmp);
-	    op->f_RA = hrrad(tmp);
+	    op->f_RA = (float) hrrad(tmp);
 	    break;
 	case F_DEC:
 	    f_scansex (raddeg(op->f_dec), bp, &tmp);
-	    op->f_dec = degrad(tmp);
+	    op->f_dec = (float) degrad(tmp);
 	    break;
 	case F_EPOCH:
 	    tmp = op->f_epoch;
 	    crack_year (bp, pref, &tmp);
-	    op->f_epoch = tmp;
+	    op->f_epoch = (float) tmp;
 	    break;
 	case F_MAG:
 	    set_fmag (op, atod(bp));
 	    break;
 	case F_SIZE:
-	    op->f_size = atod(bp);
+	    op->f_size = (float) atod(bp);
 	    {
-		/* galaxies can have minor axis and position angle subfields */
+		/* optional minor axis and position angle subfields */
 		char *sflds[MAXFLDS];
 		int nsf = get_fields(bp, SUBFLD, sflds);
 
@@ -354,25 +494,25 @@ Obj *op;
 	}
 
 	case E_INC:
-	    op->e_inc = atod (bp);
+	    op->e_inc = (float) atod (bp);
 	    break;
 	case E_LAN:
-	    op->e_Om = atod (bp);
+	    op->e_Om = (float) atod (bp);
 	    break;
 	case E_AOP:
-	    op->e_om = atod (bp);
+	    op->e_om = (float) atod (bp);
 	    break;
 	case E_A:
-	    op->e_a = atod (bp);
+	    op->e_a = (float) atod (bp);
 	    break;
 	case E_N:
 	    /* retired */
 	    break;
 	case E_E:
-	    op->e_e = atod (bp);
+	    op->e_e = (float) atod (bp);
 	    break;
 	case E_M:
-	    op->e_M = atod (bp);
+	    op->e_M = (float) atod (bp);
 	    break;
 	case E_CEPOCH:
 	    crack_year (bp, pref, &op->e_cepoch);
@@ -394,7 +534,7 @@ Obj *op;
 		/* leave type unchanged if no or unrecognized prefix */
 		break;
 	    }
-	    op->e_mag.m1 = atod(bp);
+	    op->e_mag.m1 = (float) atod(bp);
 	    break;
 	case E_M2:
 	    switch (bp[0]) {
@@ -410,97 +550,100 @@ Obj *op;
 		/* leave type unchanged if no or unrecognized prefix */
 		break;
 	    }
-	    op->e_mag.m2 = atod(bp);
+	    op->e_mag.m2 = (float) atod(bp);
 	    break;
 	case E_SIZE:
-	    op->e_size = atod (bp);
+	    op->e_size = (float) atod (bp);
 	    break;
 
 	case H_EP:
 	    crack_year (bp, pref, &op->h_ep);
 	    break;
 	case H_INC:
-	    op->h_inc = atod (bp);
+	    op->h_inc = (float) atod (bp);
 	    break;
 	case H_LAN:
-	    op->h_Om = atod (bp);
+	    op->h_Om = (float) atod (bp);
 	    break;
 	case H_AOP:
-	    op->h_om = atod (bp);
+	    op->h_om = (float) atod (bp);
 	    break;
 	case H_E:
-	    op->h_e = atod (bp);
+	    op->h_e = (float) atod (bp);
 	    break;
 	case H_QP:
-	    op->h_qp = atod (bp);
+	    op->h_qp = (float) atod (bp);
 	    break;
 	case H_EPOCH:
 	    crack_year (bp, pref, &op->h_epoch);
 	    break;
 	case H_G:
-	    op->h_g = atod (bp);
+	    op->h_g = (float) atod (bp);
 	    break;
 	case H_K:
-	    op->h_k = atod (bp);
+	    op->h_k = (float) atod (bp);
 	    break;
 	case H_SIZE:
-	    op->h_size = atod (bp);
+	    op->h_size = (float) atod (bp);
 	    break;
 
 	case P_EP:
 	    crack_year (bp, pref, &op->p_ep);
 	    break;
 	case P_INC:
-	    op->p_inc = atod (bp);
+	    op->p_inc = (float) atod (bp);
 	    break;
 	case P_AOP:
-	    op->p_om = atod (bp);
+	    op->p_om = (float) atod (bp);
 	    break;
 	case P_QP:
-	    op->p_qp = atod (bp);
+	    op->p_qp = (float) atod (bp);
 	    break;
 	case P_LAN:
-	    op->p_Om = atod (bp);
+	    op->p_Om = (float) atod (bp);
 	    break;
 	case P_EPOCH:
 	    crack_year (bp, pref, &op->p_epoch);
 	    break;
 	case P_G:
-	    op->p_g = atod (bp);
+	    op->p_g = (float) atod (bp);
 	    break;
 	case P_K:
-	    op->p_k = atod (bp);
+	    op->p_k = (float) atod (bp);
 	    break;
 	case P_SIZE:
-	    op->p_size = atod (bp);
+	    op->p_size = (float) atod (bp);
 	    break;
 
 	case ES_EPOCH:
 	    crack_year (bp, pref, &op->es_epoch);
 	    break;
 	case ES_INC:
-	    op->es_inc = atod (bp);
+	    op->es_inc = (float) atod (bp);
 	    break;
 	case ES_RAAN:
-	    op->es_raan = atod (bp);
+	    op->es_raan = (float) atod (bp);
 	    break;
 	case ES_E:
-	    op->es_e = atod (bp);
+	    op->es_e = (float) atod (bp);
 	    break;
 	case ES_AP:
-	    op->es_ap = atod (bp);
+	    op->es_ap = (float) atod (bp);
 	    break;
 	case ES_M:
-	    op->es_M = atod (bp);
+	    op->es_M = (float) atod (bp);
 	    break;
 	case ES_N:
 	    op->es_n = atod (bp);
 	    break;
 	case ES_DECAY:
-	    op->es_decay = atod (bp);
+	    op->es_decay = (float) atod (bp);
 	    break;
 	case ES_ORBIT:
-	    op->es_orbit = (int) atod (bp);
+	    op->es_orbit = atoi (bp);
+	    break;
+	case ES_DRAG:
+	    op->es_drag = (float) atod (bp);
 	    break;
 
 	default:
@@ -856,6 +999,10 @@ char *bp;
 	    sprintf (bp, ",%d", op->es_orbit);
 	    bp += strlen(bp);
 	    break;
+	case ES_DRAG:
+	    sprintf (bp, ",%.7g", op->es_drag);
+	    bp += strlen(bp);
+	    break;
 
 	default:
 	    printf ("BUG! db_set_field: bad id: %d\n", id);
@@ -866,4 +1013,4 @@ char *bp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: dbfmt.c,v $ $Date: 1999/02/15 21:42:18 $ $Revision: 1.7 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: dbfmt.c,v $ $Date: 2000/10/07 05:12:17 $ $Revision: 1.12 $ $Name:  $"};
