@@ -324,8 +324,8 @@ gplan (J, plan)
 
 /* lblat403.c */
 
-static int lattabr[] = {-1.0};
-static int lattabb[] = {-1.0};
+static int lattabr[] = {-1};
+static int lattabb[] = {-1};
 static int lattabl[] = {
     785710,
 
@@ -1111,8 +1111,8 @@ static struct plantbl liblat = {
 };
 
 /* lblon403.c */
-static int lontabr[] = {-1.0};
-static int lontabb[] = {-1.0};
+static int lontabr[] = {-1};
+static int lontabb[] = {-1};
 static int lontabl[] = {
    -640001,
 
@@ -2222,4 +2222,4 @@ double *llonp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: libration.c,v $ $Date: 1997/05/19 18:21:21 $ $Revision: 1.1 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: libration.c,v $ $Date: 2000/10/07 05:12:17 $ $Revision: 1.2 $ $Name:  $"};
